@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -56,10 +56,13 @@ import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
 import java.net.URLDecoder;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
@@ -121,6 +124,8 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
 
     public static final String MODULE_PROJECT = "projects";
 
+    public static final String MODULE_MESSAGING = "messaging";
+
     public static final String MODULE_INFOSTORE = "infostore";
 
     public static final String MODULE_SYSTEM = "system";
@@ -202,7 +207,7 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
 
     public static final String ACTION_DETACH = "detach";
 
-    protected static final String ACTION_ATTACH = "attach";
+    public static final String ACTION_ATTACH = "attach";
 
     public static final String ACTION_REVERT = "revert";
 
@@ -227,6 +232,10 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
     public static final String ACTION_CLEAR = "clear";
 
     public static final String ACTION_KEEPALIVE = "keepalive";
+    
+    public static final String ACTION_RESOLVE_UID = "resolveuid";
+
+    public static final String ACTION_IMPORT = "import";
 
     /**
      * The parameter 'from' specifies index of starting entry in list of objects dependent on given order criteria and folder id
@@ -264,6 +273,8 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
     public static final String PARAMETER_INFOLDER = "folder";
 
     public static final String PARAMETER_MODULE = "module";
+    
+    public static final String PARAMETER_MAIL = "mail";
 
     /**
      * The parameter 'sort' specifies the field which is used as order source and can be compared to SQL'S 'Order By' statement
@@ -295,7 +306,7 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
 
     public static final String PARAMETER_TIMESTAMP = "timestamp";
 
-    public static final String PARAMETER_TIMEZONE= "timezone";
+    public static final String PARAMETER_TIMEZONE = "timezone";
 
     public static final String PARAMETER_VERSION = "version";
 
@@ -320,6 +331,9 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
     public static final String PARAMETER_USER = "user";
 
     public static final String PARAMETER_TEMPLATE = "template";
+    
+    public static final String PARAMETER_UID = "uid";
+
     /**
      * The content type if the response body contains javascript data. Set it with
      * <code>resp.setContentType(AJAXServlet.CONTENTTYPE_JAVASCRIPT)</code> .
@@ -359,7 +373,6 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
      * Error message if writing the response fails.
      */
     protected static final String RESPONSE_ERROR = "Error while writing response object.";
-    
 
     /**
      * The service method of HttpServlet is extended to catch bad exceptions and keep the AJP socket alive. Otherwise apache things in a
@@ -403,6 +416,16 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
         return (req.getParameter(name) != null);
     }
 
+    /**
+     * 8K buffer
+     */
+    private static final int BUF_SIZE = 0x2000;
+
+    /**
+     * Initialize with 16K
+     */
+    private static final int SB_SIZE = 0x4000;
+
     /**
      * Returns the complete body as a string. Be careful when getting big request bodies.
      * 
@@ -413,14 +436,17 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
     public static String getBody(final HttpServletRequest req) throws IOException {
         InputStreamReader isr = null;
         try {
+            {
+                final String characterEncoding = req.getCharacterEncoding();
+                isr =
+                    new InputStreamReader(
+                        req.getInputStream(),
+                        null == characterEncoding ? ServerConfig.getProperty(Property.DefaultEncoding) : characterEncoding);
+            }
+            final char[] c = new char[BUF_SIZE];
             int count = 0;
-            isr =
-                new InputStreamReader(
-                    req.getInputStream(),
-                    null == req.getCharacterEncoding() ? ServerConfig.getProperty(Property.DefaultEncoding) : req.getCharacterEncoding());
-            final char[] c = new char[8192]; // 8K buffer
             if ((count = isr.read(c)) > 0) {
-                final StringBuilder sb = new StringBuilder(16384); // Initialize with 16K
+                final StringBuilder sb = new StringBuilder(SB_SIZE);
                 do {
                     sb.append(c, 0, count);
                 } while ((count = isr.read(c)) > 0);
@@ -453,15 +479,16 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
     protected static String getServletSpecificURI(final HttpServletRequest req) {
         String uri;
         try {
+            final String characterEncoding = req.getCharacterEncoding();
             uri =
                 URLDecoder.decode(
                     req.getRequestURI(),
-                    req.getCharacterEncoding() == null ? ServerConfig.getProperty(ServerConfig.Property.DefaultEncoding) : req.getCharacterEncoding());
+                    characterEncoding == null ? ServerConfig.getProperty(ServerConfig.Property.DefaultEncoding) : characterEncoding);
         } catch (final UnsupportedEncodingException e) {
             LOG.error("Unsupported encoding", e);
             uri = req.getRequestURI();
         }
-        final String path = req.getContextPath() + req.getServletPath();
+        final String path = new StringBuilder(req.getContextPath()).append(req.getServletPath()).toString();
         final int pos = uri.indexOf(path);
         if (pos != -1) {
             uri = uri.substring(pos + path.length());
@@ -469,13 +496,19 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
         return uri;
     }
 
+    /**
+     * Gets the action parameter ({@link #PARAMETER_ACTION}) from specified servlet request.
+     * 
+     * @param req The servlet request
+     * @return The action parameter's value
+     * @throws OXConflictException If action parameter is missing in specified servlet request
+     */
     protected static String getAction(final HttpServletRequest req) throws OXConflictException {
         final String action = req.getParameter(PARAMETER_ACTION);
-
-        if (action != null) {
-            return action;
+        if (action == null) {
+            throw new OXConflictException(new AjaxException(AjaxException.Code.MISSING_PARAMETER, PARAMETER_ACTION));
         }
-        throw new OXConflictException(new AjaxException(AjaxException.Code.MISSING_PARAMETER, PARAMETER_ACTION));
+        return action;
     }
 
     /**
@@ -587,83 +620,155 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
 
     /* --------------------- STUFF FOR UPLOAD --------------------- */
 
+    /**
+     * Processes specified request's upload provided that request is of content type <code>multipart/*</code>.
+     * 
+     * @param req The request whose upload shall be processed
+     * @return The processed instance of {@link UploadEvent}
+     * @throws UploadException Id processing the upload fails
+     */
     public UploadEvent processUpload(final HttpServletRequest req) throws UploadException {
         return processUploadStatic(req);
     }
 
+    private static volatile ServletFileUpload servletFileUpload;
+
+    /**
+     * Gets the {@link ServletFileUpload} instance.
+     * 
+     * @return The {@link ServletFileUpload} instance
+     */
+    private static ServletFileUpload getFileUploadBase() {
+        ServletFileUpload tmp = servletFileUpload;
+        if (null == tmp) {
+            synchronized (AJAXServlet.class) {
+                tmp = servletFileUpload;
+                if (null == tmp) {
+                    tmp = servletFileUpload = newFileUploadBase();
+                }
+            }
+        }
+        return tmp;
+    }
+
+    /**
+     * 1MB threshold.
+     */
+    private static final int SIZE_THRESHOLD = 1048576;
+
+    /**
+     * Creates a new {@link ServletFileUpload} instance.
+     * 
+     * @return A new {@link ServletFileUpload} instance
+     */
+    private static ServletFileUpload newFileUploadBase() {
+        /*
+         * Create the upload event
+         */
+        final DiskFileItemFactory factory = new DiskFileItemFactory();
+        /*
+         * Set factory constraints; threshold for single files
+         */
+        factory.setSizeThreshold(SIZE_THRESHOLD);
+        factory.setRepository(new File(ServerConfig.getProperty(Property.UploadDirectory)));
+        /*
+         * Create a new file upload handler
+         */
+        final ServletFileUpload sfu = new ServletFileUpload(factory);
+        /*
+         * Set overall request size constraint
+         */
+        sfu.setSizeMax(-1);
+        return sfu;
+    }
+
+    private static final Set<String> UPLOAD_ACTIONS =
+        new HashSet<String>(Arrays.asList(ACTION_NEW, ACTION_UPLOAD, ACTION_APPEND, ACTION_UPDATE, ACTION_ATTACH, ACTION_COPY));
+
+    /**
+     * (Statically) Processes specified request's upload provided that request is of content type <code>multipart/*</code>.
+     * 
+     * @param req The request whose upload shall be processed
+     * @return The processed instance of {@link UploadEvent}
+     * @throws UploadException Id processing the upload fails
+     */
     public static final UploadEvent processUploadStatic(final HttpServletRequest req) throws UploadException {
-        final boolean isMultipart = FileUploadBase.isMultipartContent(new ServletRequestContext(req));
-        if (isMultipart) {
-            /*
-             * Create the upload event
-             */
-            final UploadEvent uploadEvent = new UploadEvent();
-            final DiskFileItemFactory factory = new DiskFileItemFactory();
-            /*
-             * Set factory constraints
-             */
-            factory.setSizeThreshold(0);
-            factory.setRepository(new File(ServerConfig.getProperty(Property.UploadDirectory)));
-            /*
-             * Create a new file upload handler
-             */
-            final ServletFileUpload upload = new ServletFileUpload(factory);
-            /*
-             * Set overall request size constraint
-             */
-            upload.setSizeMax(-1);
-            List<FileItem> items;
-            String action;
-            try {
-                action = getAction(req);
-            } catch (final OXConflictException e) {
-                throw new UploadException(UploadCode.UPLOAD_FAILED, null, e);
-            }
-            try {
-                items = upload.parseRequest(req);
-            } catch (final FileUploadException e) {
-                throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
-            }
-            if (action != null && (action.equals(ACTION_NEW) || action.equals(ACTION_UPLOAD) || action.equals(ACTION_APPEND) || action.equals(ACTION_UPDATE) || action.equals(ACTION_ATTACH) || action.equals(ACTION_COPY) || com.openexchange.groupware.importexport.Format.containsConstantName(action))) {
-                uploadEvent.setAction(action);
-                /*
-                 * Set affiliation to mail upload
-                 */
-                uploadEvent.setAffiliationId(UploadEvent.MAIL_UPLOAD);
-                /*
-                 * Fill upload event instance
-                 */
-                final int size = items.size();
-                final String charEnc =
-                    req.getCharacterEncoding() == null ? ServerConfig.getProperty(Property.DefaultEncoding) : req.getCharacterEncoding();
-                NextFileItem: for (int i = 0; i < size; i++) {
-                    final FileItem fileItem = items.get(i);
-                    if (fileItem.isFormField()) {
-                        final String fieldName = fileItem.getFieldName();
-                        try {
-                            uploadEvent.addFormField(fieldName, fileItem.getString(charEnc));
-                        } catch (final UnsupportedEncodingException e) {
-                            throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
-                        }
-                    } else {
-                        if (fileItem.getSize() == 0 && (fileItem.getName() == null || fileItem.getName().length() == 0)) {
-                            continue NextFileItem;
-                        }
-                        try {
-                            uploadEvent.addUploadFile(processUploadedFile(fileItem, ServerConfig.getProperty(Property.UploadDirectory)));
-                        } catch (final Exception e) {
-                            throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
-                        }
-                    }
-                }
-                if (uploadEvent.getAffiliationId() < 0) {
-                    throw new UploadException(UploadCode.MISSING_AFFILIATION_ID, action);
-                }
-                return uploadEvent;
-            }
+        if (!FileUploadBase.isMultipartContent(new ServletRequestContext(req))) {
+            // No multipart content
+            throw new UploadException(UploadCode.NO_MULTIPART_CONTENT, null);
+        }
+        /*
+         * Check action parameter existence
+         */
+        final String action;
+        try {
+            action = getAction(req);
+        } catch (final OXConflictException e) {
+            throw new UploadException(UploadCode.UPLOAD_FAILED, null, e);
+        }
+        /*-
+         * Check proper action value
+         * 
+         * ###########################################################################################################
+         * ######################### ENSURE YOUR ACTION IS CONTAINED IN UPLOAD_ACTIONS ! ! ! #########################
+         * ###########################################################################################################
+         */
+        if (!UPLOAD_ACTIONS.contains(action) && !com.openexchange.groupware.importexport.Format.containsConstantName(action)) {
             throw new UploadException(UploadCode.UNKNOWN_ACTION_VALUE, null, action);
         }
-        throw new UploadException(UploadCode.NO_MULTIPART_CONTENT, null);
+        /*
+         * Create file upload base
+         */
+        final ServletFileUpload upload = getFileUploadBase();
+        final List<FileItem> items;
+        try {
+            @SuppressWarnings("unchecked") final List<FileItem> tmp = upload.parseRequest(req);
+            items = tmp;
+        } catch (final FileUploadException e) {
+            throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
+        }
+        /*
+         * Create the upload event
+         */
+        final UploadEvent uploadEvent = new UploadEvent();
+        uploadEvent.setAction(action);
+        /*
+         * Set affiliation to mail upload
+         */
+        uploadEvent.setAffiliationId(UploadEvent.MAIL_UPLOAD);
+        /*
+         * Fill upload event instance
+         */
+        final String charEnc;
+        {
+            final String rce = req.getCharacterEncoding();
+            charEnc = null == rce ? ServerConfig.getProperty(Property.DefaultEncoding) : rce;
+        }
+        for (final FileItem fileItem : items) {
+            if (fileItem.isFormField()) {
+                try {
+                    uploadEvent.addFormField(fileItem.getFieldName(), fileItem.getString(charEnc));
+                } catch (final UnsupportedEncodingException e) {
+                    throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
+                }
+            } else {
+                if (fileItem.getSize() > 0 || !isEmpty(fileItem.getName())) {
+                    try {
+                        uploadEvent.addUploadFile(processUploadedFile(fileItem, ServerConfig.getProperty(Property.UploadDirectory)));
+                    } catch (final Exception e) {
+                        throw new UploadException(UploadCode.UPLOAD_FAILED, action, e);
+                    }
+                }
+            }
+        }
+        if (uploadEvent.getAffiliationId() < 0) {
+            throw new UploadException(UploadCode.MISSING_AFFILIATION_ID, action);
+        }
+        return uploadEvent;
+    }
+
+    private static final boolean isEmpty(final String str) {
+        return null == str || 0 == str.length();
     }
 
     private static final UploadFile processUploadedFile(final FileItem item, final String uploadDir) throws Exception {
@@ -793,6 +898,9 @@ public abstract class AJAXServlet extends HttpServlet implements UploadRegistry
                 moduleStr = MODULE_SYSTEM;
             }
             break;
+        case FolderObject.MESSAGING:
+            moduleStr = MODULE_MESSAGING;
+            break;
         default:
             moduleStr = "";
             break;
