@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,6 +51,7 @@ package com.openexchange.ajax;
 
 import java.io.IOException;
 import java.io.InputStream;
+import java.util.Collection;
 import java.util.Set;
 import java.util.Map.Entry;
 import javax.servlet.ServletException;
@@ -80,6 +81,19 @@ import com.openexchange.tools.servlet.http.Tools;
  */
 public abstract class MultipleAdapterServletNew extends PermissionServlet {
 
+    private static final class HTTPRequestInputStreamProvider implements AJAXRequestData.InputStreamProvider {
+
+        private final HttpServletRequest req;
+
+        HTTPRequestInputStreamProvider(final HttpServletRequest req) {
+            this.req = req;
+        }
+
+        public InputStream getInputStream() throws IOException {
+            return req.getInputStream();
+        }
+    }
+
     private static final long serialVersionUID = -8060034833311074781L;
 
     private static final Log LOG = LogFactory.getLog(MultipleAdapterServletNew.class);
@@ -124,7 +138,7 @@ public abstract class MultipleAdapterServletNew extends PermissionServlet {
      *            HTTP POST method); otherwise <code>false</code> to generate an appropriate {@link Object} from request's body
      * @throws IOException If an I/O error occurs
      */
-    protected void handle(final HttpServletRequest req, final HttpServletResponse resp, final boolean preferStream) throws IOException {
+    protected final void handle(final HttpServletRequest req, final HttpServletResponse resp, final boolean preferStream) throws IOException {
         final Response response = new Response();
         try {
             final String action = req.getParameter(PARAMETER_ACTION);
@@ -132,10 +146,13 @@ public abstract class MultipleAdapterServletNew extends PermissionServlet {
                 throw new AjaxException(AjaxException.Code.MISSING_PARAMETER, PARAMETER_ACTION);
             }
             final AJAXRequestData data = parseRequest(req, preferStream);
-            final AJAXActionService actionService = factory.createActionService(action);
-            final AJAXRequestResult result = actionService.perform(data, getSessionObject(req));
+            final AJAXRequestResult result = factory.createActionService(action).perform(data, getSessionObject(req));
             response.setData(result.getResultObject());
             response.setTimestamp(result.getTimestamp());
+            final Collection<AbstractOXException> warnings = result.getWarnings();
+            if (null != warnings && !warnings.isEmpty()) {
+                response.setWarning(warnings.iterator().next());
+            }
         } catch (final AbstractOXException e) {
             LOG.error(e.getMessage(), e);
             response.setException(e);
@@ -154,6 +171,7 @@ public abstract class MultipleAdapterServletNew extends PermissionServlet {
 
     private AJAXRequestData parseRequest(final HttpServletRequest req, final boolean preferStream) throws IOException {
         final AJAXRequestData retval = new AJAXRequestData();
+        retval.setSecure(req.isSecure());
         /*
          * Pass all parameters to AJAX request object
          */
@@ -167,7 +185,7 @@ public abstract class MultipleAdapterServletNew extends PermissionServlet {
             /*
              * Pass request's stream
              */
-            retval.setUploadStream(req.getInputStream());
+            retval.setUploadStreamProvider(new HTTPRequestInputStreamProvider(req));
         } else {
             /*
              * Guess an appropriate body object
@@ -210,8 +228,10 @@ public abstract class MultipleAdapterServletNew extends PermissionServlet {
             return startingChar == toCheck.charAt(0);
         }
         int i = 0;
-        while (i < len && Character.isWhitespace(toCheck.charAt(i))) {
-            i++;
+        if (Character.isWhitespace(toCheck.charAt(i))) {
+            do {
+                i++;
+            } while (i < len && Character.isWhitespace(toCheck.charAt(i)));
         }
         if (i >= len) {
             return false;
