@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,12 +51,9 @@ package com.openexchange.ajax.parser;
 
 import static com.openexchange.ajax.fields.TaskFields.ACTUAL_DURATION;
 import static com.openexchange.ajax.fields.TaskFields.TARGET_DURATION;
-
 import java.util.TimeZone;
-
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.ajax.fields.CalendarFields;
 import com.openexchange.ajax.fields.TaskFields;
 import com.openexchange.api.OXConflictException;
@@ -84,8 +81,8 @@ public class TaskParser extends CalendarParser {
             parseElementTask(taskobject, jsonobject);
         } catch (final OXJSONException e) {
             throw e;
-        } catch (final Exception exc) {
-            throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, exc);
+        } catch (final Exception e) {
+            throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, e, e.getMessage());
         }
     }
 
