@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,17 +49,16 @@
 
 package com.openexchange.ajax.request;
 
-import static com.openexchange.tools.Collections.newHashMap;
 import static com.openexchange.tools.TimeZoneUtils.getTimeZone;
+import gnu.trove.TIntArrayList;
+import gnu.trove.TIntIntHashMap;
+import gnu.trove.TIntObjectHashMap;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Date;
-import java.util.HashMap;
-import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.Map;
 import java.util.TimeZone;
 import java.util.regex.Pattern;
 import org.apache.commons.logging.Log;
@@ -78,6 +77,7 @@ import com.openexchange.ajax.fields.SearchFields;
 import com.openexchange.ajax.parser.AppointmentParser;
 import com.openexchange.ajax.parser.CalendarParser;
 import com.openexchange.ajax.parser.DataParser;
+import com.openexchange.ajax.parser.ParticipantParser;
 import com.openexchange.ajax.writer.AppointmentWriter;
 import com.openexchange.api.OXConflictException;
 import com.openexchange.api.OXMandatoryFieldException;
@@ -85,6 +85,7 @@ import com.openexchange.api.OXObjectNotFoundException;
 import com.openexchange.api.OXPermissionException;
 import com.openexchange.api2.AppointmentSQLInterface;
 import com.openexchange.api2.OXException;
+import com.openexchange.groupware.EnumComponent;
 import com.openexchange.groupware.calendar.AppointmentSqlFactoryService;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.groupware.calendar.CalendarDataObject;
@@ -96,7 +97,9 @@ import com.openexchange.groupware.container.CalendarObject;
 import com.openexchange.groupware.container.CommonObject;
 import com.openexchange.groupware.container.DataObject;
 import com.openexchange.groupware.container.FolderChildObject;
+import com.openexchange.groupware.container.Participant;
 import com.openexchange.groupware.container.Participants;
+import com.openexchange.groupware.container.participants.ConfirmableParticipant;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.search.AppointmentSearchObject;
@@ -131,7 +134,7 @@ public class AppointmentRequest {
         CalendarObject.START_DATE, CalendarObject.END_DATE, CalendarObject.NOTE, CalendarObject.RECURRENCE_TYPE,
         CalendarObject.RECURRENCE_CALCULATOR, CalendarObject.RECURRENCE_ID, CalendarObject.RECURRENCE_POSITION,
         CalendarObject.PARTICIPANTS, CalendarObject.USERS, Appointment.SHOWN_AS, Appointment.DELETE_EXCEPTIONS,
-        Appointment.CHANGE_EXCEPTIONS, Appointment.FULL_TIME, Appointment.COLOR_LABEL, Appointment.TIMEZONE };
+        Appointment.CHANGE_EXCEPTIONS, Appointment.FULL_TIME, Appointment.COLOR_LABEL, Appointment.TIMEZONE, Appointment.ORGANIZER, Appointment.UID, Appointment.SEQUENCE, Appointment.CONFIRMATIONS, Appointment.LAST_MODIFIED_OF_NEWEST_ATTACHMENT };
 
     private final ServerSession session;
 
@@ -201,10 +204,24 @@ public class AppointmentRequest {
             return actionFreeBusy(jsonObject);
         } else if (AJAXServlet.ACTION_COPY.equalsIgnoreCase(action)) {
             return actionCopy(jsonObject);
+        } else if (AJAXServlet.ACTION_RESOLVE_UID.equalsIgnoreCase(action)) {
+            return actionResolveUid(jsonObject);
         } else {
             throw new AjaxException(AjaxException.Code.UnknownAction, action);
         }
     }
+    
+    private JSONObject actionResolveUid(final JSONObject jsonObj) throws JSONException, OXException {
+        final AppointmentSQLInterface appointmentSql = appointmentFactory.createAppointmentSql(session);
+        final JSONObject json = new JSONObject();
+        final String uid = DataParser.parseString(jsonObj, AJAXServlet.PARAMETER_UID);
+        final int id = appointmentSql.resolveUid(uid);
+        if (id == 0) {
+            throw new OXObjectNotFoundException(OXObjectNotFoundException.Code.OBJECT_NOT_FOUND, EnumComponent.APPOINTMENT, "");
+        }
+        json.put("id", id);
+        return json;
+    }
 
     public JSONObject actionNew(final JSONObject jsonObj) throws OXMandatoryFieldException, JSONException, OXConflictException, OXException, AjaxException {
         final JSONObject jData = DataParser.checkJSONObject(jsonObj, AJAXServlet.PARAMETER_DATA);
@@ -382,7 +399,7 @@ public class AppointmentRequest {
                                 appointmentObj.setStartDate(new Date(recuResults.getRecurringResult(0).getStart()));
                                 appointmentObj.setEndDate(new Date(recuResults.getRecurringResult(0).getEnd()));
 
-                                appointmentWriter.writeArray(appointmentObj, columns, startUTC, endUTC, jsonResponseArray);
+                                appointmentWriter.writeArray(appointmentObj, columns, jsonResponseArray);
                             }
                         } else {
                             // Commented this because this is done in CalendarOperation.next():726 that calls extractRecurringInformation()
@@ -500,7 +517,7 @@ public class AppointmentRequest {
 
         SearchIterator<Appointment> it = null;
 
-        final HashMap<Integer, ArrayList<Integer>> recurrencePositionMap = newHashMap();
+        final TIntObjectHashMap<TIntArrayList> recurrencePositionMap = new TIntObjectHashMap<TIntArrayList>();
 
         final String[] sColumns = split(DataParser.checkString(jsonObj, AJAXServlet.PARAMETER_COLUMNS));
         final int[] columns = StringCollection.convertStringArray2IntArray(sColumns);
@@ -513,13 +530,13 @@ public class AppointmentRequest {
 
         final boolean bRecurrenceMaster = DataParser.parseBoolean(jsonObj, RECURRENCE_MASTER);
 
-        final HashMap<Integer, Integer> objectIdMap = newHashMap();
+        final TIntIntHashMap objectIdMap = new TIntIntHashMap();
         for (int a = 0; a < jData.length(); a++) {
             final JSONObject jObject = jData.getJSONObject(a);
             final int objectId = DataParser.checkInt(jObject, AJAXServlet.PARAMETER_ID);
             final int folderId = DataParser.checkInt(jObject, AJAXServlet.PARAMETER_FOLDERID);
 
-            objectIdMap.put(Integer.valueOf(objectId), Integer.valueOf(folderId));
+            objectIdMap.put(objectId, folderId);
 
             // for backward compatibility supporting both recurrence position parameters
             int tempRecurrencePosition = DataParser.parseInt(jObject, CalendarFields.RECURRENCE_POSITION);
@@ -533,25 +550,25 @@ public class AppointmentRequest {
 
             if (tempRecurrencePosition > 0) {
                 final int recurrencePosition = tempRecurrencePosition;
-                ArrayList<Integer> recurrencePosList = null;
-                if (recurrencePositionMap.containsKey(Integer.valueOf(objectId))) {
-                    recurrencePosList = recurrencePositionMap.get(Integer.valueOf(objectId));
+                TIntArrayList recurrencePosList = null;
+                if (recurrencePositionMap.containsKey(objectId)) {
+                    recurrencePosList = recurrencePositionMap.get(objectId);
                 } else {
-                    recurrencePosList = new ArrayList<Integer>();
+                    recurrencePosList = new TIntArrayList();
                 }
-                recurrencePosList.add(Integer.valueOf(recurrencePosition));
-                recurrencePositionMap.put(Integer.valueOf(objectId), recurrencePosList);
+                recurrencePosList.add(recurrencePosition);
+                recurrencePositionMap.put(objectId, recurrencePosList);
             }
         }
 
         final int size = objectIdMap.size();
         final int[][] objectIdAndFolderId = new int[size][2];
-
-        final Iterator<Map.Entry<Integer, Integer>> iterator = objectIdMap.entrySet().iterator();
-        for (int i = 0; i < size; i++) {
-            final Map.Entry<Integer, Integer> entry = iterator.next();
-            objectIdAndFolderId[i][0] = entry.getKey().intValue();
-            objectIdAndFolderId[i][1] = entry.getValue().intValue();
+        {
+            int i = 0;
+            for (final int objectId : objectIdMap.keys()) {
+                objectIdAndFolderId[i][0] = objectId;
+                objectIdAndFolderId[i++][1] = objectIdMap.get(objectId);
+            }
         }
 
         final AppointmentSQLInterface appointmentsql = appointmentFactory.createAppointmentSql(session);
@@ -591,17 +608,18 @@ public class AppointmentRequest {
                     } else {
                         // Commented this because this is done in CalendarOperation.next():726 that calls extractRecurringInformation()
                         // appointment.calculateRecurrence();
-                        if (recurrencePositionMap.containsKey(Integer.valueOf(appointment.getObjectID()))) {
-                            final ArrayList<Integer> recurrencePosList = recurrencePositionMap.get(Integer.valueOf(appointment.getObjectID()));
+                        if (recurrencePositionMap.containsKey(appointment.getObjectID())) {
+                            final TIntArrayList recurrencePosList = recurrencePositionMap.get(appointment.getObjectID());
 
-                            for (int a = 0; a < recurrencePosList.size(); a++) {
+                            final int listSize = recurrencePosList.size();
+                            for (int a = 0; a < listSize; a++) {
                                 appointment.setStartDate(startDate);
                                 appointment.setEndDate(endDate);
                                 final RecurringResultsInterface recuResults = recColl.calculateRecurring(
                                     appointment,
                                     0,
                                     0,
-                                    recurrencePosList.get(a).intValue());
+                                    recurrencePosList.getQuick(a));
                                 if (recuResults.size() > 0) {
                                     final RecurringResultInterface result = recuResults.getRecurringResult(0);
                                     appointment.setStartDate(new Date(result.getStart()));
@@ -732,7 +750,7 @@ public class AppointmentRequest {
                             appointment.setStartDate(new Date(recuResults.getRecurringResult(0).getStart()));
                             appointment.setEndDate(new Date(recuResults.getRecurringResult(0).getEnd()));
 
-                            writer.writeArray(appointment, columns, startUTC, endUTC, jsonResponseArray);
+                            writer.writeArray(appointment, columns, jsonResponseArray);
                         } else {
                             LOG.warn("cannot load first recurring appointment from appointment object: " + +appointment.getRecurrenceType() + " / " + appointment.getObjectID() + "\n\n\n");
                         }
@@ -854,7 +872,7 @@ public class AppointmentRequest {
                     0,
                     0,
                     recurrencePosition,
-                    recColl.MAXTC,
+                    recColl.MAX_OCCURRENCESE,
                     true);
                 if (recuResults.size() == 0) {
                     if (LOG.isWarnEnabled()) {
@@ -884,15 +902,26 @@ public class AppointmentRequest {
         final int objectId = DataParser.checkInt(jsonObj, DataFields.ID);
         final int folderId = DataParser.checkInt(jsonObj, AJAXServlet.PARAMETER_FOLDERID);
         final JSONObject jData = DataParser.checkJSONObject(jsonObj, AJAXServlet.PARAMETER_DATA);
+        //DataParser.checkInt(jData, ParticipantsFields.CONFIRMATION);
+        
+        final ConfirmableParticipant participant = new ParticipantParser().parseConfirmation(true, jData);
+        
         int userId = user.getId();
         if (jData.has(AJAXServlet.PARAMETER_ID)) {
             userId = DataParser.checkInt(jData, AJAXServlet.PARAMETER_ID);
         }
-        final String confirmMessage = jData.has(CalendarFields.CONFIRM_MESSAGE) && !jData.isNull(CalendarFields.CONFIRM_MESSAGE) ? jData.getString(CalendarFields.CONFIRM_MESSAGE) : null;
-        final int confirmStatus = DataParser.checkInt(jData, CalendarFields.CONFIRMATION);
+        final String confirmMessage = participant.getMessage();
+        final int confirmStatus = participant.getConfirm();
 
         final AppointmentSQLInterface appointmentSql = appointmentFactory.createAppointmentSql(session);
-        timestamp = appointmentSql.setUserConfirmation(objectId, folderId, userId, confirmStatus, confirmMessage);
+        timestamp = null;
+        if (participant.getType() == Participant.USER || participant.getType() == 0) {
+            timestamp = appointmentSql.setUserConfirmation(objectId, folderId, userId, confirmStatus, confirmMessage);
+        } else if (participant.getType() == Participant.EXTERNAL_USER) {
+            timestamp = appointmentSql.setExternalConfirmation(objectId, folderId, participant.getEmailAddress(), confirmStatus, confirmMessage);
+        } else {
+            throw new AjaxException(AjaxException.Code.InvalidParameterValue, AJAXServlet.PARAMETER_TYPE, jData.get(AJAXServlet.PARAMETER_TYPE));
+        }
 
         return new JSONObject();
     }
@@ -1010,13 +1039,13 @@ public class AppointmentRequest {
                                 recuResults = recColl.calculateFirstRecurring(appointment);
                             } catch (final OXException x) {
                                 LOG.error("Can not calculate recurrence " + appointment.getObjectID() + ":" + ctx.getContextId(), x);
-                                appointmentwriter.writeArray(appointment, columns, startUTC, endUTC, jsonResponseArray);
+                                appointmentwriter.writeArray(appointment, columns, jsonResponseArray);
                             }
                             if (recuResults != null && recuResults.size() == 1) {
                                 appointment.setStartDate(new Date(recuResults.getRecurringResult(0).getStart()));
                                 appointment.setEndDate(new Date(recuResults.getRecurringResult(0).getEnd()));
 
-                                appointmentwriter.writeArray(appointment, columns, startUTC, endUTC, jsonResponseArray);
+                                appointmentwriter.writeArray(appointment, columns, jsonResponseArray);
                             } else {
                                 LOG.warn("cannot load first recurring appointment from appointment object: " + +appointment.getRecurrenceType() + " / " + appointment.getObjectID() + "\n\n\n");
                             }
@@ -1303,6 +1332,7 @@ public class AppointmentRequest {
         }
 
         appointmentObj.removeObjectID();
+        appointmentObj.removeUid();
         appointmentObj.setParentFolderID(folderId);
         appointmentObj.setIgnoreConflicts(ignoreConflicts);
         final Appointment[] conflicts = appointmentSql.insertAppointmentObject(appointmentObj);
