@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,12 +49,10 @@
 
 package com.openexchange.ajax.request;
 
-import static com.openexchange.java.Autoboxing.I;
 import static com.openexchange.tools.TimeZoneUtils.getTimeZone;
+import gnu.trove.TIntArrayList;
 import java.sql.Connection;
-import java.util.ArrayList;
 import java.util.Date;
-import java.util.List;
 import java.util.TimeZone;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -476,7 +474,10 @@ public class ContactRequest {
         }
 
         final int leftHandLimit = DataParser.parseInt(jsonObj, AJAXServlet.LEFT_HAND_LIMIT);
-        final int rightHandLimit = DataParser.parseInt(jsonObj, AJAXServlet.RIGHT_HAND_LIMIT);
+        int rightHandLimit = DataParser.parseInt(jsonObj, AJAXServlet.RIGHT_HAND_LIMIT);
+        if (rightHandLimit == 0) {
+            rightHandLimit = 50000;
+        }
 
         timestamp = new Date(0);
 
@@ -492,11 +493,7 @@ public class ContactRequest {
                 session);
 
             final ContactWriter contactwriter = new ContactWriter(timeZone);
-            if (rightHandLimit == 0) {
-                it = contactInterface.getContactsInFolder(folderId, leftHandLimit, 50000, orderBy, orderDir, internalColumns);
-            } else {
-                it = contactInterface.getContactsInFolder(folderId, leftHandLimit, rightHandLimit, orderBy, orderDir, internalColumns);
-            }
+            it = contactInterface.getContactsInFolder(folderId, leftHandLimit, rightHandLimit, orderBy, orderDir, internalColumns);
 
             while (it.hasNext()) {
                 final Contact contactObj = it.next();
@@ -518,17 +515,13 @@ public class ContactRequest {
     }
 
     private int[] removeVirtual(final int[] columns) {
-        final List<Integer> helper = new ArrayList<Integer>(columns.length);
+        final TIntArrayList helper = new TIntArrayList(columns.length);
         for (final int col : columns) {
             if (col != Contact.LAST_MODIFIED_UTC) {
-                helper.add(I(col));
+                helper.add(col);
             }
         }
-        final int[] copy = new int[helper.size()];
-        for (int i = 0; i < copy.length; i++) {
-            copy[i] = helper.get(i).intValue();
-        }
-        return copy;
+        return helper.toNativeArray();
     }
 
     public JSONObject actionGet(final JSONObject jsonObj) throws OXMandatoryFieldException, JSONException, OXException, OXJSONException, AjaxException {
