@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -67,7 +67,7 @@ import com.openexchange.api.OXMandatoryFieldException;
 import com.openexchange.api.OXObjectNotFoundException;
 import com.openexchange.api2.AppointmentSQLInterface;
 import com.openexchange.api2.OXException;
-import com.openexchange.api2.ReminderSQLInterface;
+import com.openexchange.api2.ReminderService;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.EnumComponent;
 import com.openexchange.groupware.Types;
@@ -78,11 +78,8 @@ import com.openexchange.groupware.calendar.RecurringResultInterface;
 import com.openexchange.groupware.calendar.RecurringResultsInterface;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.ldap.User;
-import com.openexchange.groupware.reminder.EmptyReminderDeleteImpl;
-import com.openexchange.groupware.reminder.ReminderDeleteInterface;
 import com.openexchange.groupware.reminder.ReminderHandler;
 import com.openexchange.groupware.reminder.ReminderObject;
-import com.openexchange.groupware.tasks.ModifyThroughDependant;
 import com.openexchange.server.services.ServerServiceRegistry;
 import com.openexchange.session.Session;
 import com.openexchange.tools.TimeZoneUtils;
@@ -161,22 +158,8 @@ public final class ReminderRequest {
         final TimeZone tz = TimeZoneUtils.getTimeZone(userObj.getTimeZone());
         final JSONArray jsonArray = new JSONArray();
         try {
-            final ReminderSQLInterface reminderSql = new ReminderHandler(session.getContext());
-
+            final ReminderService reminderSql = new ReminderHandler(session.getContext());
             final ReminderObject reminder = reminderSql.loadReminder(id);
-            final ReminderDeleteInterface reminderDeleteInterface;
-            final int module = reminder.getModule();
-            switch (module) {
-            case Types.APPOINTMENT:
-                reminderDeleteInterface = ServerServiceRegistry.getInstance().getService(ReminderDeleteInterface.class);
-                break;
-            case Types.TASK:
-                reminderDeleteInterface = new ModifyThroughDependant();
-                break;
-            default:
-                reminderDeleteInterface = new EmptyReminderDeleteImpl();
-            }
-            reminderSql.setReminderDeleteInterface(reminderDeleteInterface);
 
             if (reminder.isRecurrenceAppointment()) {
                 final ReminderObject nextReminder = getNextRecurringReminder(session, tz, reminder);
@@ -214,7 +197,7 @@ public final class ReminderRequest {
         SearchIterator<?> it = null;
 
         try {
-            final ReminderSQLInterface reminderSql = new ReminderHandler(session.getContext());
+            final ReminderService reminderSql = new ReminderHandler(session.getContext());
             it = reminderSql.listModifiedReminder(userObj.getId(), timestamp);
 
             while (it.hasNext()) {
@@ -264,9 +247,9 @@ public final class ReminderRequest {
         
         final ReminderWriter reminderWriter = new ReminderWriter(timeZone);
         try {
-            final ReminderSQLInterface reminderSql = new ReminderHandler(session.getContext());
+            final ReminderService reminderSql = new ReminderHandler(session.getContext());
             final JSONArray jsonResponseArray = new JSONArray();
-            final SearchIterator<ReminderObject> it = reminderSql.listReminder(userObj.getId(), end);
+            final SearchIterator<ReminderObject> it = reminderSql.getArisingReminder(session, session.getContext(), userObj, end);
             try {
                 while (it.hasNext()) {
                     final ReminderObject reminder = it.next();
