@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,12 +49,9 @@
 
 package com.openexchange.ajax.request;
 
-import static com.openexchange.java.Autoboxing.I;
-import static com.openexchange.java.Autoboxing.I2i;
 import static com.openexchange.tools.TimeZoneUtils.getTimeZone;
-import java.util.ArrayList;
+import gnu.trove.TIntArrayList;
 import java.util.Date;
-import java.util.List;
 import java.util.TimeZone;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -440,19 +437,22 @@ public class TaskRequest {
         return jsonResponseObject;
     }
 
-    public JSONObject actionConfirm(final JSONObject jsonObj) throws OXMandatoryFieldException, OXException, AjaxException, OXJSONException, JSONException {
-        final JSONObject jData = DataParser.checkJSONObject(jsonObj, ResponseFields.DATA);
-        final Task taskObj = new Task();
-
-        final TaskParser taskParser = new TaskParser(timeZone);
-        taskParser.parse(taskObj, jData);
-
-        final TasksSQLInterface taskSql = new TasksSQLImpl(session);
-
-        final int taskId = (taskObj.containsObjectID()) ? taskObj.getObjectID() : DataParser.checkInt(jsonObj, "id");
-
-        timestamp = taskSql.setUserConfirmation(taskId, session.getUserId(), taskObj.getConfirm(), taskObj.getConfirmMessage());
-
+    public JSONObject actionConfirm(JSONObject json) throws OXMandatoryFieldException, OXException, AjaxException, OXJSONException, JSONException {
+        JSONObject data = DataParser.checkJSONObject(json, ResponseFields.DATA);
+        Task task = new Task();
+        new TaskParser(timeZone).parse(task, data);
+        TasksSQLInterface taskSql = new TasksSQLImpl(session);
+        int taskIdFromParameter = DataParser.parseInt(json, AJAXServlet.PARAMETER_ID);
+        final int taskId;
+        if (0 == taskIdFromParameter) {
+            if (!task.containsObjectID()) {
+                throw new AjaxException(AjaxException.Code.MISSING_PARAMETER, AJAXServlet.PARAMETER_ID);
+            }
+            taskId = task.getObjectID();
+        } else {
+            taskId = taskIdFromParameter;
+        }
+        timestamp = taskSql.setUserConfirmation(taskId, session.getUserId(), task.getConfirm(), task.getConfirmMessage());
         return new JSONObject();
     }
 
@@ -567,12 +567,12 @@ public class TaskRequest {
     }
 
     private int[] removeVirtualColumns(final int[] columns) {
-        final List<Integer> tmp = new ArrayList<Integer>(columns.length);
+        final TIntArrayList tmp = new TIntArrayList(columns.length);
         for (final int col : columns) {
             if (col != Task.LAST_MODIFIED_UTC) {
-                tmp.add(I(col));
+                tmp.add(col);
             }
         }
-        return I2i(tmp);
+        return tmp.toNativeArray();
     }
 }
