@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,9 +49,16 @@
 
 package com.openexchange.ajax.requesthandler;
 
+import java.io.IOException;
 import java.io.InputStream;
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 import org.json.JSONObject;
 import com.openexchange.ajax.fields.RequestConstants;
 import com.openexchange.ajax.parser.DataParser;
@@ -65,11 +72,24 @@ import com.openexchange.tools.servlet.AjaxException;
  */
 public class AJAXRequestData {
 
+    public static interface InputStreamProvider {
+
+        /**
+         * Gets this provider's input stream.
+         * 
+         * @return The input stream
+         * @throws IOException If an I/O error occurs
+         */
+        InputStream getInputStream() throws IOException;
+    }
+
     private final Map<String, String> params;
 
+    private boolean secure;
+
     private Object data;
 
-    private InputStream uploadStream;
+    private InputStreamProvider uploadStreamProvider;
 
     /**
      * Initializes a new {@link AJAXRequestData}.
@@ -87,7 +107,7 @@ public class AJAXRequestData {
      */
     public AJAXRequestData() {
         super();
-        params = new HashMap<String, String>();
+        params = new LinkedHashMap<String, String>();
     }
 
     /**
@@ -124,6 +144,39 @@ public class AJAXRequestData {
         return params.get(name);
     }
 
+    /**
+     * Gets all available parameter names wrapped by an {@link Iterator iterator}.
+     * 
+     * @return The {@link Iterator iterator} for available parameter names
+     */
+    public Iterator<String> getParameterNames() {
+        return params.keySet().iterator();
+    }
+
+    /**
+     * Gets an {@link Iterator iterator} for those parameters not matching given parameter names.
+     * 
+     * @param nonMatchingParameterNames The non-matching parameter names
+     * @return An {@link Iterator iterator} for non-matching parameters
+     */
+    public Iterator<Entry<String, String>> getNonMatchingParameters(final Collection<String> nonMatchingParameterNames) {
+        final Map<String, String> clone = new HashMap<String, String>(params);
+        clone.keySet().removeAll(nonMatchingParameterNames);
+        return clone.entrySet().iterator();
+    }
+
+    /**
+     * Gets an {@link Iterator iterator} for those parameters matching given parameter names.
+     * 
+     * @param matchingParameterNames The matching parameter names
+     * @return An {@link Iterator iterator} for matching parameters
+     */
+    public Iterator<Entry<String, String>> getMatchingParameters(final Collection<String> matchingParameterNames) {
+        final Map<String, String> clone = new HashMap<String, String>(params);
+        clone.keySet().retainAll(matchingParameterNames);
+        return clone.entrySet().iterator();
+    }
+
     /**
      * Gets the data object.
      * 
@@ -143,21 +196,63 @@ public class AJAXRequestData {
     }
 
     /**
-     * Gets the upload stream. Retrieves the body of the request as binary data as an {@link InputStream}.
+     * Whether this request has a secure connection.
      * 
-     * @return The upload stream or <code>null</code> if not available
+     * @return <code>true</code> if this request has a secure connection; otherwise <code>false</code>
      */
-    public InputStream getUploadStream() {
-        return uploadStream;
+    public boolean isSecure() {
+        return secure;
     }
 
     /**
-     * Sets the upload stream
+     * Sets whether this request has a secure connection.
      * 
-     * @param uploadStream The upload stream to set
+     * @param secure <code>true</code> if this request has a secure connection; otherwise <code>false</code>
      */
-    public void setUploadStream(final InputStream uploadStream) {
-        this.uploadStream = uploadStream;
+    public void setSecure(final boolean secure) {
+        this.secure = secure;
+    }
+
+    /**
+     * Gets the upload stream. Retrieves the body of the request as binary data as an {@link InputStream}.
+     * 
+     * @return The upload stream or <code>null</code> if not available
+     * @throws IOException If an I/O error occurs
+     */
+    public InputStream getUploadStream() throws IOException {
+        return uploadStreamProvider.getInputStream();
+    }
+
+    /**
+     * Sets the upload stream provider
+     * 
+     * @param uploadStream The upload stream provider to set
+     */
+    public void setUploadStreamProvider(final InputStreamProvider uploadStreamProvider) {
+        this.uploadStreamProvider = uploadStreamProvider;
+    }
+
+    /**
+     * Computes a list of missing parameters from a list of mandatory parameters. The typical idiom to use this would be:
+     * 
+     * <pre>
+     * List&lt;String&gt; missingParameters = requestData.getMissingParameters(&quot;param1&quot;, &quot;param2&quot;, &quot;param3&quot;);
+     * if (!missingParameters.isEmpty()) {
+     *     // Throw AbstractOXException
+     * }
+     * </pre>
+     * 
+     * @param mandatoryParameters The mandatory parameters expected.
+     * @return A list of missing parameter names
+     */
+    public List<String> getMissingParameters(final String... mandatoryParameters) {
+        final List<String> missing = new ArrayList<String>(mandatoryParameters.length);
+        for (final String paramName : mandatoryParameters) {
+            if (!params.containsKey(paramName)) {
+                missing.add(paramName);
+            }
+        }
+        return missing;
     }
 
 }
