@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,8 @@
 
 package com.openexchange.ajax.requesthandler;
 
+import java.util.Collection;
+import java.util.Collections;
 import java.util.Date;
 import java.util.Map.Entry;
 import org.json.JSONException;
@@ -61,29 +63,36 @@ import com.openexchange.tools.session.ServerSession;
 
 /**
  * {@link MultipleAdapter} maps the {@link MultipleHandler} to several {@link AJAXActionService}s. This class is not thread safe because it
- * has to remember the {@link AJAXRequestResult} between calling {@link #performRequest(String, JSONObject, ServerSession)} and
+ * has to remember the {@link AJAXRequestResult} between calling {@link #performRequest(String, JSONObject, ServerSession, boolean)} and
  * {@link #getTimestamp()} methods.
- *
+ * 
  * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
+ * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public class MultipleAdapter implements MultipleHandler {
 
     private final AJAXActionServiceFactory factory;
 
     private AJAXRequestResult result;
-  
-    public MultipleAdapter(AJAXActionServiceFactory factory) {
+
+    /**
+     * Initializes a new {@link MultipleAdapter}.
+     * 
+     * @param factory The factory used to create {@link AJAXActionService} instances
+     */
+    public MultipleAdapter(final AJAXActionServiceFactory factory) {
         super();
         this.factory = factory;
     }
 
-    public Object performRequest(String action, JSONObject jsonObject, ServerSession session) throws AbstractOXException, JSONException {
-        AJAXActionService actionService = factory.createActionService(action);
+    public Object performRequest(final String action, final JSONObject jsonObject, final ServerSession session, final boolean secure) throws AbstractOXException, JSONException {
+        final AJAXActionService actionService = factory.createActionService(action);
         if (null == actionService) {
             throw new AjaxException(AjaxException.Code.UnknownAction, action);
         }
-        AJAXRequestData request = new AJAXRequestData();
-        for (Entry<String, Object> entry : jsonObject.entrySet()) {
+        final AJAXRequestData request = new AJAXRequestData();
+        request.setSecure(secure);
+        for (final Entry<String, Object> entry : jsonObject.entrySet()) {
             if (RequestConstants.DATA.equals(entry.getKey())) {
                 request.setData(entry.getValue());
             } else {
@@ -95,10 +104,22 @@ public class MultipleAdapter implements MultipleHandler {
     }
 
     public Date getTimestamp() {
-        return null == result ? null : result.getTimestamp();
+        if (null == result) {
+            return null;
+        }
+        final Date timestamp = result.getTimestamp();
+        return null == timestamp ? null : new Date(timestamp.getTime());
     }
 
     public void close() {
         result = null;
     }
+
+    public Collection<AbstractOXException> getWarnings() {
+        if (null == result) {
+            return Collections.<AbstractOXException> emptySet();
+        }
+        return result.getWarnings();
+    }
+
 }
