@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,10 +49,8 @@
 
 package com.openexchange.ajax.writer;
 
-import java.util.Collections;
+import gnu.trove.TIntObjectHashMap;
 import java.util.Date;
-import java.util.HashMap;
-import java.util.Map;
 import java.util.TimeZone;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -64,7 +62,6 @@ import com.openexchange.groupware.calendar.CalendarCollectionService;
 import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.server.services.ServerServiceRegistry;
-import com.openexchange.tools.TimeZoneUtils;
 
 /**
  * {@link AppointmentWriter} - Writer for appointments
@@ -76,8 +73,6 @@ public class AppointmentWriter extends CalendarWriter {
 
     private static final Log LOG = LogFactory.getLog(AppointmentWriter.class);
 
-    private final TimeZone utc;
-    
     private CalendarCollectionService calColl;
 
     /**
@@ -88,17 +83,16 @@ public class AppointmentWriter extends CalendarWriter {
      */
     public AppointmentWriter(final TimeZone timeZone) {
         super(timeZone, null);
-        utc = TimeZoneUtils.getTimeZone("utc");
     }
-    
+
     public CalendarCollectionService getCalendarCollectionService(){
         if ( null == calColl ) {
             calColl = ServerServiceRegistry.getInstance().getService(CalendarCollectionService.class);
         }
         return calColl;
     }
-    
-    public void setCalendarCollectionService(CalendarCollectionService calColl){
+
+    public void setCalendarCollectionService(final CalendarCollectionService calColl){
         this.calColl = calColl;
     }
 
@@ -114,25 +108,20 @@ public class AppointmentWriter extends CalendarWriter {
         }
     }
 
-    public void writeArray(final Appointment appointmentObject, final int cols[], final JSONArray jsonArray)
-            throws JSONException {
-        final JSONArray jsonAppointmentArray = new JSONArray();
-        for (int a = 0; a < cols.length; a++) {
-            write(cols[a], appointmentObject, jsonAppointmentArray);
+    public void writeArray(final Appointment appointment, final int[] columns, final JSONArray json) throws JSONException {
+        final JSONArray array = new JSONArray();
+        for (final int column : columns) {
+            writeField(appointment, column, timeZone, array);
         }
-        jsonArray.put(jsonAppointmentArray);
+        json.put(array);
     }
 
-    public void writeAppointment(final Appointment appointmentObject, final JSONObject jsonObj)
-            throws JSONException {
-        writeCommonFields(appointmentObject, jsonObj);
-
+    public void writeAppointment(final Appointment appointmentObject, final JSONObject jsonObj) throws JSONException {
+        super.writeFields(appointmentObject, timeZone, jsonObj);
         if (appointmentObject.containsTitle()) {
             writeParameter(AppointmentFields.TITLE, appointmentObject.getTitle(), jsonObj);
         }
-
         final boolean isFullTime = appointmentObject.getFullTime();
-
         if (isFullTime) {
             writeParameter(AppointmentFields.START_DATE, appointmentObject.getStartDate(), jsonObj);
             writeParameter(AppointmentFields.END_DATE, appointmentObject.getEndDate(), jsonObj);
@@ -147,130 +136,89 @@ public class AppointmentWriter extends CalendarWriter {
                         .getEndDate(), timeZone, jsonObj);
             }
         }
-
         if (appointmentObject.containsShownAs()) {
             writeParameter(AppointmentFields.SHOW_AS, appointmentObject.getShownAs(), jsonObj);
         }
-
         if (appointmentObject.containsLocation()) {
             writeParameter(AppointmentFields.LOCATION, appointmentObject.getLocation(), jsonObj);
         }
-
         if (appointmentObject.containsNote()) {
             writeParameter(AppointmentFields.NOTE, appointmentObject.getNote(), jsonObj);
         }
-
         if (appointmentObject.containsFullTime()) {
             writeParameter(AppointmentFields.FULL_TIME, appointmentObject.getFullTime(), jsonObj);
         }
-
-        if (appointmentObject.containsCategories()) {
-            writeParameter(AppointmentFields.CATEGORIES, appointmentObject.getCategories(), jsonObj);
-        }
-
-        if (appointmentObject.containsLabel()) {
-            writeParameter(AppointmentFields.COLORLABEL, appointmentObject.getLabel(), jsonObj);
-        }
-
         if (appointmentObject.containsAlarm()) {
             writeParameter(AppointmentFields.ALARM, appointmentObject.getAlarm(), jsonObj);
         }
-
         if (appointmentObject.containsRecurrenceType()) {
             writeRecurrenceParameter(appointmentObject, jsonObj);
         }
-
         if (appointmentObject.containsRecurrenceID()) {
             writeParameter(AppointmentFields.RECURRENCE_ID, appointmentObject.getRecurrenceID(), jsonObj);
         }
-
         if (appointmentObject.containsRecurrencePosition()) {
             writeParameter(AppointmentFields.RECURRENCE_POSITION, appointmentObject.getRecurrencePosition(), jsonObj);
         }
-
         if (appointmentObject.containsRecurrenceDatePosition()) {
             writeParameter(AppointmentFields.RECURRENCE_DATE_POSITION, appointmentObject.getRecurrenceDatePosition(),
                     jsonObj);
         }
-
         if (appointmentObject.containsParticipants()) {
             jsonObj.put(AppointmentFields.PARTICIPANTS, getParticipantsAsJSONArray(appointmentObject));
         }
-
         if (appointmentObject.containsUserParticipants()) {
             jsonObj.put(AppointmentFields.USERS, getUsersAsJSONArray(appointmentObject));
         }
-
         if (appointmentObject.getIgnoreConflicts()) {
             writeParameter(AppointmentFields.IGNORE_CONFLICTS, true, jsonObj);
         }
-
         if (appointmentObject.containsTimezone()) {
             writeParameter(AppointmentFields.TIMEZONE, appointmentObject.getTimezoneFallbackUTC(), jsonObj);
         }
-
         if (appointmentObject.containsRecurringStart()) {
             writeParameter(AppointmentFields.RECURRENCE_START, appointmentObject.getRecurringStart(), jsonObj);
         }
-
         if (appointmentObject instanceof CalendarDataObject && ((CalendarDataObject) appointmentObject).isHardConflict()) {
             writeParameter(AppointmentFields.HARD_CONFLICT, true, jsonObj);
         }
     }
 
-    /**
-     * Writes given appointment field's value into specified JSON array
-     *
-     * @param field The appointment field
-     * @param appointmentObject The appointment object to take the value from
-     * @param jsonArray The JSON array to put into
-     * @throws JSONException If a JSON error occurs while putting into JSON array
-     */
-    public void write(final int field, final Appointment appointmentObject, final JSONArray jsonArray)
-            throws JSONException {
-        final AppointmentFieldWriter writer = WRITER_MAP.get(Integer.valueOf(field));
-        if (writer != null) {
-            writer.write(appointmentObject, jsonArray);
+    protected void writeField(final Appointment appointment, final int column, final TimeZone tz, final JSONArray json) throws JSONException {
+        final AppointmentFieldWriter writer = WRITER_MAP.get(column);
+        if (null != writer) {
+            writer.write(appointment, json);
+            return;
+        } else if (super.writeField(appointment, column, tz, json)) {
             return;
         }
-        /*
-         * No appropriate static writer found, write manually
-         */
-        final boolean isFullTime = appointmentObject.getFullTime();
-        switch (field) {
-        case Appointment.CREATION_DATE:
-            writeValue(appointmentObject.getCreationDate(), timeZone, jsonArray);
-            break;
-        case Appointment.LAST_MODIFIED:
-            writeValue(appointmentObject.getLastModified(), timeZone, jsonArray);
-            break;
-        case Appointment.LAST_MODIFIED_UTC:
-            writeValue(appointmentObject.getLastModified(), utc, jsonArray);
-            break;
+        // No appropriate static writer found, write manually
+        final boolean isFullTime = appointment.getFullTime();
+        switch (column) {
         case Appointment.START_DATE:
             if (isFullTime) {
-                writeValue(appointmentObject.getStartDate(), jsonArray);
+                writeValue(appointment.getStartDate(), json);
             } else {
-                if (appointmentObject.getRecurrenceType() == Appointment.NO_RECURRENCE) {
-                    writeValue(appointmentObject.getStartDate(), timeZone, jsonArray);
+                if (appointment.getRecurrenceType() == Appointment.NO_RECURRENCE) {
+                    writeValue(appointment.getStartDate(), timeZone, json);
                 } else {
-                    writeValue(appointmentObject.getStartDate(), appointmentObject.getStartDate(), timeZone, jsonArray);
+                    writeValue(appointment.getStartDate(), appointment.getStartDate(), timeZone, json);
                 }
             }
             break;
         case Appointment.END_DATE:
             if (isFullTime) {
-                writeValue(appointmentObject.getEndDate(), jsonArray);
+                writeValue(appointment.getEndDate(), json);
             } else {
-                if (appointmentObject.getRecurrenceType() == Appointment.NO_RECURRENCE) {
-                    writeValue(appointmentObject.getEndDate(), timeZone, jsonArray);
+                if (appointment.getRecurrenceType() == Appointment.NO_RECURRENCE) {
+                    writeValue(appointment.getEndDate(), timeZone, json);
                 } else {
-                    writeValue(appointmentObject.getEndDate(), appointmentObject.getEndDate(), timeZone, jsonArray);
+                    writeValue(appointment.getEndDate(), appointment.getEndDate(), timeZone, json);
                 }
             }
             break;
         default:
-            LOG.warn("missing field in mapping: " + field);
+            LOG.warn("Column " + column + " is unknown for appointment.");
         }
     }
 
@@ -295,139 +243,90 @@ public class AppointmentWriter extends CalendarWriter {
      * +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      */
 
-    private static final Map<Integer, AppointmentFieldWriter> WRITER_MAP;
+    private static final TIntObjectHashMap<AppointmentFieldWriter> WRITER_MAP;
 
     static {
-        final Map<Integer, AppointmentFieldWriter> m = new HashMap<Integer, AppointmentFieldWriter>(64);
-
-        m.put(Integer.valueOf(Appointment.OBJECT_ID), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getObjectID(), jsonArray, appointmentObject.containsObjectID());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.CREATED_BY), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getCreatedBy(), jsonArray, appointmentObject.containsCreatedBy());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.MODIFIED_BY), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getModifiedBy(), jsonArray, appointmentObject.containsModifiedBy());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.FOLDER_ID), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getParentFolderID(), jsonArray, appointmentObject.containsParentFolderID());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.TITLE), new AppointmentFieldWriter() {
+        final TIntObjectHashMap<AppointmentFieldWriter> m = new TIntObjectHashMap<AppointmentFieldWriter>(24, 1);
+        m.put(Appointment.TITLE, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getTitle(), jsonArray);
             }
         });
-
-        m.put(Integer.valueOf(Appointment.SHOWN_AS), new AppointmentFieldWriter() {
+        m.put(Appointment.SHOWN_AS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getShownAs(), jsonArray, appointmentObject.containsShownAs());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.LOCATION), new AppointmentFieldWriter() {
+        m.put(Appointment.LOCATION, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getLocation(), jsonArray);
             }
         });
-
-        m.put(Integer.valueOf(Appointment.CATEGORIES), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getCategories(), jsonArray);
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.COLOR_LABEL), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getLabel(), jsonArray, appointmentObject.containsLabel());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.PRIVATE_FLAG), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getPrivateFlag(), jsonArray, appointmentObject.containsPrivateFlag());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.FULL_TIME), new AppointmentFieldWriter() {
+        m.put(Appointment.FULL_TIME, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getFullTime(), jsonArray, appointmentObject.containsFullTime());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.NOTE), new AppointmentFieldWriter() {
+        m.put(Appointment.NOTE, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getNote(), jsonArray);
             }
         });
-
         // modification for mobility support
-        m.put(Integer.valueOf(Appointment.RECURRENCE_ID), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_ID, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurrenceID(), jsonArray, appointmentObject.containsRecurrenceID());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_TYPE), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_TYPE, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurrenceType(), jsonArray, appointmentObject.containsRecurrenceType());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.INTERVAL), new AppointmentFieldWriter() {
+        m.put(Appointment.INTERVAL, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getInterval(), jsonArray, appointmentObject.containsInterval());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.DAYS), new AppointmentFieldWriter() {
+        m.put(Appointment.DAYS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getDays(), jsonArray, appointmentObject.containsDays());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.DAY_IN_MONTH), new AppointmentFieldWriter() {
+        m.put(Appointment.DAY_IN_MONTH, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getDayInMonth(), jsonArray, appointmentObject.containsDayInMonth());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.MONTH), new AppointmentFieldWriter() {
+        m.put(Appointment.MONTH, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getMonth(), jsonArray, appointmentObject.containsMonth());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.UNTIL), new AppointmentFieldWriter() {
+        m.put(Appointment.UNTIL, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getUntil(), jsonArray);
+                if (appointmentObject.containsOccurrence()) {
+                    jsonArray.put(JSONObject.NULL);
+                } else {
+                    if (appointmentObject.containsUntil()) {
+                        writeValue(appointmentObject.getUntil(), jsonArray);
+                    } else {
+                        jsonArray.put(JSONObject.NULL);
+                    }
+                }
             }
         });
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_COUNT), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_COUNT, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getOccurrence(), jsonArray, appointmentObject.containsOccurrence());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_DATE_POSITION), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_DATE_POSITION, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurrenceDatePosition(), jsonArray);
             }
         });
-
-        m.put(Integer.valueOf(Appointment.DELETE_EXCEPTIONS), new AppointmentFieldWriter() {
+        m.put(Appointment.DELETE_EXCEPTIONS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 final JSONArray jsonDeleteExceptionArray = getExceptionAsJSONArray(appointmentObject
                         .getDeleteException());
@@ -438,8 +337,7 @@ public class AppointmentWriter extends CalendarWriter {
                 }
             }
         });
-
-        m.put(Integer.valueOf(Appointment.CHANGE_EXCEPTIONS), new AppointmentFieldWriter() {
+        m.put(Appointment.CHANGE_EXCEPTIONS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 final JSONArray jsonChangeExceptionArray = getExceptionAsJSONArray(appointmentObject
                         .getChangeException());
@@ -451,27 +349,23 @@ public class AppointmentWriter extends CalendarWriter {
             }
         });
         // end of modification for mobility support
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_POSITION), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_POSITION, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurrencePosition(), jsonArray, appointmentObject
                         .containsRecurrencePosition());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.TIMEZONE), new AppointmentFieldWriter() {
+        m.put(Appointment.TIMEZONE, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getTimezoneFallbackUTC(), jsonArray);
             }
         });
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_START), new AppointmentFieldWriter() {
+        m.put(Appointment.RECURRENCE_START, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurringStart(), jsonArray, appointmentObject.containsRecurringStart());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.PARTICIPANTS), new AppointmentFieldWriter() {
+        m.put(Appointment.PARTICIPANTS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray)
                     throws JSONException {
                 final JSONArray jsonParticipantArray = getParticipantsAsJSONArray(appointmentObject);
@@ -482,8 +376,7 @@ public class AppointmentWriter extends CalendarWriter {
                 }
             }
         });
-
-        m.put(Integer.valueOf(Appointment.USERS), new AppointmentFieldWriter() {
+        m.put(Appointment.USERS, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray)
                     throws JSONException {
                 final JSONArray jsonUserArray = getUsersAsJSONArray(appointmentObject);
@@ -494,39 +387,36 @@ public class AppointmentWriter extends CalendarWriter {
                 }
             }
         });
-
-        m.put(Integer.valueOf(Appointment.NUMBER_OF_ATTACHMENTS), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getNumberOfAttachments(), jsonArray, appointmentObject
-                        .containsNumberOfAttachments());
-            }
-        });
-
-        m.put(Integer.valueOf(Appointment.NUMBER_OF_LINKS), new AppointmentFieldWriter() {
-            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
-                writeValue(appointmentObject.getNumberOfLinks(), jsonArray, appointmentObject.containsNumberOfLinks());
-            }
-        });
-        m.put(Integer.valueOf(Appointment.ALARM), new AppointmentFieldWriter() {
+        m.put(Appointment.ALARM, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getAlarm(), jsonArray, appointmentObject.containsAlarm());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.NOTIFICATION), new AppointmentFieldWriter() {
+        m.put(Appointment.NOTIFICATION, new AppointmentFieldWriter() {
             public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getNotification(), jsonArray, appointmentObject.getNotification());
             }
         });
-
-        m.put(Integer.valueOf(Appointment.RECURRENCE_CALCULATOR), new AppointmentFieldWriter() {
-
-            public void write(Appointment appointmentObject, JSONArray jsonArray) throws JSONException {
+        m.put(Appointment.RECURRENCE_CALCULATOR, new AppointmentFieldWriter() {
+            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
                 writeValue(appointmentObject.getRecurrenceCalculator(), jsonArray);
             }
-            
         });
-        
-        WRITER_MAP = Collections.unmodifiableMap(m);
+        m.put(Appointment.ORGANIZER, new AppointmentFieldWriter() {
+            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
+                writeValue(appointmentObject.getOrganizer(), jsonArray, appointmentObject.containsOrganizer());
+            }
+        });
+        m.put(Appointment.UID, new AppointmentFieldWriter() {
+            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
+                writeValue(appointmentObject.getUid(), jsonArray, appointmentObject.containsUid());
+            }
+        });
+        m.put(Appointment.SEQUENCE, new AppointmentFieldWriter() {
+            public void write(final Appointment appointmentObject, final JSONArray jsonArray) {
+                writeValue(appointmentObject.getSequence(), jsonArray, appointmentObject.containsSequence());
+            }
+        });
+        WRITER_MAP = m;
     }
 }
