@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.ajax.writer;
 
+import gnu.trove.TIntObjectHashMap;
 import java.util.Date;
 import java.util.TimeZone;
 import org.json.JSONArray;
@@ -60,23 +61,17 @@ import com.openexchange.ajax.fields.ParticipantsFields;
 import com.openexchange.groupware.container.CalendarObject;
 import com.openexchange.groupware.container.Participant;
 import com.openexchange.groupware.container.UserParticipant;
+import com.openexchange.groupware.container.participants.ConfirmableParticipant;
 
 /**
  * {@link CalendarWriter} - Writer for calendar objects
  * 
  * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+ * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
  */
 public abstract class CalendarWriter extends CommonWriter {
 
-    /**
-     * Initializes a new {@link CalendarWriter}
-     * 
-     * @param timeZone
-     *            The user time zone
-     * @param jsonWriter
-     *            The JSON writer to write to
-     */
     protected CalendarWriter(final TimeZone timeZone, final JSONWriter jsonWriter) {
         super(timeZone, jsonWriter);
     }
@@ -86,17 +81,15 @@ public abstract class CalendarWriter extends CommonWriter {
 
         final Participant[] participants = calendarObj.getParticipants();
         if (participants != null) {
-            for (int a = 0; a < participants.length; a++) {
-                final Participant p = participants[a];
+            for (final Participant p : participants) {
                 final JSONObject jsonObj = getParticipantAsJSONObject(p);
                 jsonArray.put(jsonObj);
             }
         }
-
         return jsonArray;
     }
 
-    public static JSONArray getUsersAsJSONArray(final CalendarObject calendarObject) throws JSONException {
+    protected static JSONArray getUsersAsJSONArray(final CalendarObject calendarObject) throws JSONException {
         final JSONArray jsonArray = new JSONArray();
 
         final UserParticipant[] users = calendarObject.getUsers();
@@ -111,7 +104,7 @@ public abstract class CalendarWriter extends CommonWriter {
         return jsonArray;
     }
 
-    public static void writeRecurrenceParameter(final CalendarObject calendarObject, final JSONObject jsonObj)
+    protected static void writeRecurrenceParameter(final CalendarObject calendarObject, final JSONObject jsonObj)
             throws JSONException {
         final int recurrenceType = calendarObject.getRecurrenceType();
 
@@ -168,7 +161,7 @@ public abstract class CalendarWriter extends CommonWriter {
             writeParameter(CalendarFields.INTERVAL, calendarObject.getInterval(), jsonObj);
         }
 
-        if (calendarObject.containsUntil()) {
+        if (calendarObject.containsUntil() && !calendarObject.containsOccurrence()) {
             writeParameter(CalendarFields.UNTIL, calendarObject.getUntil(), jsonObj);
         }
 
@@ -188,19 +181,12 @@ public abstract class CalendarWriter extends CommonWriter {
         writeParameter(ParticipantsFields.TYPE, participant.getType(), jsonObj, participant.getType() > 0);
         if (Participant.USER == participant.getType()) {
             final UserParticipant userParticipant = (UserParticipant) participant;
-            writeParameter(CalendarFields.CONFIRMATION, userParticipant.getConfirm(), jsonObj, userParticipant.containsConfirm());
-            writeParameter(CalendarFields.CONFIRM_MESSAGE, userParticipant.getConfirmMessage(), jsonObj, userParticipant.containsConfirmMessage());
+            writeParameter(ParticipantsFields.CONFIRMATION, userParticipant.getConfirm(), jsonObj, userParticipant.containsConfirm());
+            writeParameter(ParticipantsFields.CONFIRM_MESSAGE, userParticipant.getConfirmMessage(), jsonObj, userParticipant.containsConfirmMessage());
         }
         return jsonObj;
     }
 
-    public static JSONObject getUserAsJSONObject(final int userId, final int status) throws JSONException {
-        final JSONObject jsonObj = new JSONObject();
-        writeParameter(ParticipantsFields.ID, userId, jsonObj);
-        writeParameter(ParticipantsFields.STATUS, status, jsonObj);
-        return jsonObj;
-    }
-
     public static JSONArray getExceptionAsJSONArray(final Date[] dateExceptions) {
         if (dateExceptions != null) {
             final JSONArray jsonArray = new JSONArray();
@@ -212,13 +198,92 @@ public abstract class CalendarWriter extends CommonWriter {
         return null;
     }
 
-    public static JSONObject getUserParticipantAsJSONObject(final UserParticipant userParticipant) throws JSONException {
+    private static JSONObject getUserParticipantAsJSONObject(final UserParticipant userParticipant) throws JSONException {
         final JSONObject jsonObj = new JSONObject();
         writeParameter(ParticipantsFields.ID, userParticipant.getIdentifier(), jsonObj);
-        writeParameter(CalendarFields.CONFIRMATION, userParticipant.getConfirm(), jsonObj);
+        writeParameter(ParticipantsFields.CONFIRMATION, userParticipant.getConfirm(), jsonObj);
         if (userParticipant.containsConfirmMessage()) {
-            writeParameter(CalendarFields.CONFIRM_MESSAGE, userParticipant.getConfirmMessage(), jsonObj);
+            writeParameter(ParticipantsFields.CONFIRM_MESSAGE, userParticipant.getConfirmMessage(), jsonObj);
         }
         return jsonObj;
     }
+
+    protected boolean writeField(final CalendarObject obj, final int column, final TimeZone tz, final JSONArray json) throws JSONException {
+        final FieldWriter<CalendarObject> writer = WRITER_MAP.get(column);
+        if (null == writer) {
+            return super.writeField(obj, column, tz, json);
+        }
+        writer.write(obj, tz, json);
+        return true;
+    }
+
+    protected void writeFields(final CalendarObject obj, final TimeZone tz, final JSONObject json) throws JSONException {
+        super.writeFields(obj, tz, json);
+        final WriterProcedure<CalendarObject> procedure = new WriterProcedure<CalendarObject>(obj, json, tz);
+        if (!WRITER_MAP.forEachValue(procedure)) {
+            final JSONException je = procedure.getError();
+            if (null != je) {
+                throw je;
+            }
+        }
+    }
+
+    private static final FieldWriter<CalendarObject> CONFIRMATIONS_WRITER = new FieldWriter<CalendarObject>() {
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONArray json) throws JSONException {
+            json.put(createConfirmationArray(obj));
+        }
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            json.put(CalendarFields.CONFIRMATIONS, createConfirmationArray(obj));
+        }
+        private JSONArray createConfirmationArray(final CalendarObject obj) throws JSONException {
+            final JSONArray confirmations = new JSONArray();
+            if (obj.containsConfirmations()) {
+                final ParticipantWriter writer = new ParticipantWriter();
+                for (final ConfirmableParticipant participant : obj.getConfirmations()) {
+                    final JSONObject jParticipant = new JSONObject();
+                    writer.write(participant, jParticipant);
+                    confirmations.put(jParticipant);
+                }
+            }
+            return confirmations;
+        }
+    };
+    
+    protected static final FieldWriter<CalendarObject> ORGANIZER_WRITER = new FieldWriter<CalendarObject>() {
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getOrganizer(), json, obj.containsOrganizer());
+        }
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(CalendarFields.ORGANIZER, obj.getOrganizer(), json, obj.containsOrganizer());
+        }
+    };
+    
+    protected static final FieldWriter<CalendarObject> UID_WRITER = new FieldWriter<CalendarObject>() {
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getUid(), json, obj.containsUid());
+        }
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(CalendarFields.UID, obj.getUid(), json, obj.containsUid());
+        }
+    };
+    
+    protected static final FieldWriter<CalendarObject> SEQUENCE_WRITER = new FieldWriter<CalendarObject>() {
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getSequence(), json, obj.containsSequence());
+        }
+        public void write(final CalendarObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(CalendarFields.SEQUENCE, obj.getSequence(), json, obj.containsSequence());
+        }
+    };
+
+    static {
+        final TIntObjectHashMap<FieldWriter<CalendarObject>> m = new TIntObjectHashMap<FieldWriter<CalendarObject>>(1, 1);
+        m.put(CalendarObject.CONFIRMATIONS, CONFIRMATIONS_WRITER);
+        m.put(CalendarObject.ORGANIZER, ORGANIZER_WRITER);
+        m.put(CalendarObject.UID, UID_WRITER);
+        m.put(CalendarObject.SEQUENCE, SEQUENCE_WRITER);
+        WRITER_MAP = m;
+    }
+
+    private static final TIntObjectHashMap<FieldWriter<CalendarObject>> WRITER_MAP;
 }
