@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.ajax.writer;
 
+import gnu.trove.TIntObjectHashMap;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.util.Date;
@@ -58,17 +59,23 @@ import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
 import org.json.JSONWriter;
+import com.openexchange.ajax.fields.DataFields;
+import com.openexchange.groupware.container.DataObject;
+import com.openexchange.tools.TimeZoneUtils;
 
 /**
  * {@link DataWriter} - Base class for all writers used throughout modules.
  *
  * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+ * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
  */
 public class DataWriter {
 
     protected TimeZone timeZone;
 
+    static final TimeZone UTC = TimeZoneUtils.getTimeZone("UTC");
+
     protected JSONWriter jsonwriter;
 
     private static final DecimalFormat floatFormat =
@@ -90,21 +97,6 @@ public class DataWriter {
         }
     }
 
-    public void writeParameter(final String name, final int value) throws JSONException {
-        jsonwriter.key(name);
-        jsonwriter.value(value);
-    }
-
-    public void writeParameter(final String name, final long value) throws JSONException {
-        jsonwriter.key(name);
-        jsonwriter.value(value);
-    }
-
-    public void writeParameter(final String name, final boolean value) throws JSONException {
-        jsonwriter.key(name);
-        jsonwriter.value(value);
-    }
-
     public void writeParameter(final String name, final Date value) throws JSONException {
         if (value != null) {
             jsonwriter.key(name);
@@ -112,92 +104,18 @@ public class DataWriter {
         }
     }
 
-    public void writeParameter(final String name, final Date value, final TimeZone timeZone) throws JSONException {
-        writeParameter(name, value, value, timeZone);
-    }
-
-    public void writeParameter(final String name, final Date value, final Date offsetDate, final TimeZone timeZone)
-            throws JSONException {
-        if (value != null) {
-            jsonwriter.key(name);
-            final int offset = timeZone.getOffset(offsetDate.getTime());
-            jsonwriter.value(value.getTime() + offset);
-        }
-    }
-
-    public void writeParameter(final String name, final byte[] value) throws JSONException {
-        if (value != null) {
-            jsonwriter.key(name);
-            jsonwriter.value(new String(value));
-        }
-    }
-
-    public void writeValue(final String value) throws JSONException {
-        if (value != null && value.length() > 0) {
-            jsonwriter.value(value);
-        } else {
-            jsonwriter.value(JSONObject.NULL);
-        }
-    }
-
-    public void writeValue(final int value) throws JSONException {
-        jsonwriter.value(value);
-    }
-
-    public void writeValue(final long value) throws JSONException {
-        jsonwriter.value(value);
-    }
-
-    public void writeValue(final boolean value) throws JSONException {
-        jsonwriter.value(value);
-    }
-
-    public void writeValue(final Date value) throws JSONException {
-        if (value == null) {
-            jsonwriter.value(JSONObject.NULL);
-        } else {
-            jsonwriter.value(value.getTime());
-        }
-    }
-
-    public void writeValue(final Date value, final TimeZone timeZone) throws JSONException {
-        writeValue(value, value, timeZone);
-    }
-
-    public void writeValue(final Date value, final Date offsetDate, final TimeZone timeZone) throws JSONException {
-        final int offset = timeZone.getOffset(offsetDate.getTime());
-        if (value == null) {
-            jsonwriter.value(JSONObject.NULL);
-        } else {
-            jsonwriter.value(value.getTime() + offset);
-        }
-    }
-
-    public void writeValue(final byte[] value) throws JSONException {
-        if (value == null) {
-            jsonwriter.value(JSONObject.NULL);
-        } else {
-            jsonwriter.value(new String(value));
-        }
-    }
-
     // new implementation
 
     /**
-     * Puts given name-<code>String</code>-pair into specified JSON object
-     * provided that <code>String</code> value is not <code>null</code> and not
-     * empty
-     *
-     * @param name
-     *            The name to which the value is bound
-     * @param value
-     *            The <code>String</code> value
-     * @param json
-     *            The JSON object to put into
-     * @throws JSONException
-     *             If putting into JSON object fails
+     * Puts given name-<code>String</code>-pair into specified JSON object provided that <code>String</code> value is not <code>null</code>
+     * and not empty
+     * 
+     * @param name The name to which the value is bound
+     * @param value The <code>String</code> value
+     * @param json The JSON object to put into
+     * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(String name, String value, JSONObject json) throws JSONException {
+    public static void writeParameter(final String name, final String value, final JSONObject json) throws JSONException {
         if (value != null && value.length() > 0) {
             json.put(name, value);
         }
@@ -209,13 +127,12 @@ public class DataWriter {
      * <code>null</code> and not empty.
      * @param name The name to which the value is bound
      * @param value The <code>String</code> value
-     * @param jsonObj The JSON object to put into
+     * @param json The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final String value,
-        final JSONObject jsonObj, final boolean condition) throws JSONException {
+    public static void writeParameter(final String name, final String value, final JSONObject json, final boolean condition) throws JSONException {
         if (condition) {
-            writeParameter(name, value, jsonObj);
+            writeParameter(name, value, json);
         }
     }
 
@@ -224,27 +141,25 @@ public class DataWriter {
      *
      * @param name The name to which the value is bound
      * @param value The <code>int</code> value
-     * @param jsonObj The JSON object to put into
+     * @param json The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final int value,
-        final JSONObject jsonObj) throws JSONException {
-        jsonObj.put(name, value);
+    public static void writeParameter(final String name, final int value, final JSONObject json) throws JSONException {
+        json.put(name, value);
     }
 
     /**
      * Conditionally puts given <code>int</code> value into specified JSON object
-     *
+     * 
      * @param name The value's name
      * @param value The <code>int</code> value
-     * @param jsonObj The JSON object to put into
+     * @param json The JSON object to put into
      * @param condition <code>true</code> to put; otherwise <code>false</code> to omit value
      * @throws JSONException If a JSON error occurs
      */
-    public static void writeParameter(final String name, final int value,
-        final JSONObject jsonObj, final boolean condition) throws JSONException {
+    public static void writeParameter(final String name, final int value, final JSONObject json, final boolean condition) throws JSONException {
         if (condition) {
-            writeParameter(name, value, jsonObj);
+            writeParameter(name, value, json);
         }
     }
 
@@ -256,25 +171,30 @@ public class DataWriter {
      * @param json The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(String name, long value, JSONObject json) throws JSONException {
+    public static void writeParameter(final String name, final long value, final JSONObject json) throws JSONException {
         // Large values of long must be written as string. See bug 11311.
         writeParameter(name, String.valueOf(value), json);
     }
 
     /**
-     * Conditionally puts given <code>long</code> value into specified JSON
-     * object.
+     * Conditionally puts given <code>long</code> value into specified JSON object.
      * @param name The value's name
      * @param value The <code>long</code> value
-     * @param jsonObj The JSON object to put into
-     * @param condition <code>true</code> to put; otherwise <code>false</code>
-     * to omit value
+     * @param json The JSON object to put into
+     * @param condition <code>true</code> to put; otherwise <code>false</code> to omit value
      * @throws JSONException If a JSON error occurs
      */
-    public static void writeParameter(final String name, final long value,
-        final JSONObject jsonObj, final boolean condition) throws JSONException {
+    public static void writeParameter(final String name, final long value, final JSONObject json, final boolean condition) throws JSONException {
         // Large values of long must be written as string. See bug 11311.
-        writeParameter(name, String.valueOf(value), jsonObj, condition);
+        writeParameter(name, Long.toString(value), json, condition);
+    }
+
+    public static void writeParameter(String name, Long value, JSONObject json, boolean condition) throws JSONException {
+        if (null == value) {
+            writeNull(name, json, condition);
+        } else {
+            writeParameter(name, String.valueOf(value), json, condition);
+        }
     }
 
     /**
@@ -282,14 +202,17 @@ public class DataWriter {
      * object.
      * @param name The value's name
      * @param value The <code>float</code> value
-     * @param jsonObj The JSON object to put into
+     * @param json The JSON object to put into
      * @param condition <code>true</code> to put; otherwise <code>false</code> to omit value
      * @throws JSONException If a JSON error occurs
      */
-    public static void writeParameter(final String name, final float value,
-        final JSONObject jsonObj, final boolean condition) throws JSONException {
+    public static void writeParameter(final String name, final Float value, final JSONObject json, final boolean condition) throws JSONException {
         // Floats must be written as strings.
-        writeParameter(name, floatFormat.format(value), jsonObj, condition);
+        if (null == value) {
+            writeNull(name, json, condition);
+        } else {
+            writeParameter(name, floatFormat.format(value), json, condition);
+        }
     }
 
     /**
@@ -300,8 +223,7 @@ public class DataWriter {
      * @param jsonObj The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final boolean value, final JSONObject jsonObj)
-            throws JSONException {
+    public static void writeParameter(final String name, final boolean value, final JSONObject jsonObj) throws JSONException {
         jsonObj.put(name, value);
     }
 
@@ -314,8 +236,7 @@ public class DataWriter {
      * @param condition <code>true</code> to put; otherwise <code>false</code> to omit value
      * @throws JSONException If a JSON error occurs
      */
-    public static void writeParameter(final String name, final boolean value, final JSONObject jsonObj,
-            final boolean condition) throws JSONException {
+    public static void writeParameter(final String name, final boolean value, final JSONObject jsonObj, final boolean condition) throws JSONException {
         if (condition) {
             jsonObj.put(name, value);
         }
@@ -330,8 +251,7 @@ public class DataWriter {
      * @param jsonObj The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final Date value, final JSONObject jsonObj)
-            throws JSONException {
+    public static void writeParameter(final String name, final Date value, final JSONObject jsonObj) throws JSONException {
         if (value != null) {
             jsonObj.put(name, value.getTime());
         }
@@ -347,11 +267,16 @@ public class DataWriter {
      * @param jsonObj The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final Date value, final TimeZone timeZone,
-            final JSONObject jsonObj) throws JSONException {
+    public static void writeParameter(final String name, final Date value, final TimeZone timeZone, final JSONObject jsonObj) throws JSONException {
         writeParameter(name, value, value, timeZone, jsonObj);
     }
 
+    static void writeParameter(final String name, final Date value, final TimeZone timeZone, final JSONObject json, final boolean condition) throws JSONException {
+        if (condition) {
+            writeParameter(name, value, timeZone, json);
+        }
+    }
+
     /**
      * Puts given name-<code>Date</code>-pair into specified JSON object with respect to time zone
      * provided that <code>Date</code> value is not <code>null</code>
@@ -363,29 +288,12 @@ public class DataWriter {
      * @param jsonObj The JSON object to put into
      * @throws JSONException If putting into JSON object fails
      */
-    public static void writeParameter(final String name, final Date value, final Date offsetDate,
-            final TimeZone timeZone, final JSONObject jsonObj) throws JSONException {
+    public static void writeParameter(final String name, final Date value, final Date offsetDate, final TimeZone timeZone, final JSONObject jsonObj) throws JSONException {
         if (value != null) {
             jsonObj.put(name, value.getTime() + timeZone.getOffset(offsetDate.getTime()));
         }
     }
 
-    /**
-     * Puts given name-<code>byte[]</code>-pair into specified JSON object
-     * provided that <code>byte[]</code> value is not <code>null</code>
-     *
-     * @param name The value's name
-     * @param value The <code>byte[]</code> value
-     * @param jsonObj The JSON object to put into
-     * @throws JSONException If putting into JSON object fails
-     */
-    public static void writeParameter(final String name, final byte[] value, final JSONObject jsonObj)
-            throws JSONException {
-        if (value != null) {
-            jsonObj.put(name, new String(value));
-        }
-    }
-
     /**
      * Puts given <code>String</code> value into specified JSON array
      * <p>
@@ -411,8 +319,7 @@ public class DataWriter {
      * @param jsonArray The JSON array to put into
      * @param condition conditionally write the value.
      */
-    public static void writeValue(final String value, final JSONArray jsonArray,
-        final boolean condition) {
+    static void writeValue(final String value, final JSONArray jsonArray, final boolean condition) {
         if (condition) {
             writeValue(value, jsonArray);
         } else {
@@ -453,21 +360,13 @@ public class DataWriter {
      * @param condition <code>true</code> to put; otherwise <code>false</code>
      * to put {@link JSONObject#NULL}
      */
-    public static void writeValue(final float value, final JSONArray jsonArray,
-        final boolean condition) {
+    public static void writeValue(Float value, final JSONArray jsonArray, final boolean condition) {
         // Floats must be written as strings
-        writeValue(floatFormat.format(value), jsonArray, condition);
-    }
-
-    /**
-     * Puts given <code>long</code> value into specified JSON array
-     *
-     * @param value The <code>long</code> value
-     * @param jsonArray The JSON array to put into
-     */
-    public static void writeValue(final long value, final JSONArray jsonArray) {
-        // Large values of long must be written as string. See bug 11311.
-        writeValue(String.valueOf(value), jsonArray);
+        if (condition) {
+            writeValue(floatFormat.format(value), jsonArray);
+        } else {
+            writeNull(jsonArray);
+        }
     }
 
     /**
@@ -477,10 +376,28 @@ public class DataWriter {
      * @param jsonArray The JSON array to put into
      * @param condition <code>true</code> to put; otherwise <code>false</code> to put {@link JSONObject#NULL}
      */
-    public static void writeValue(final long value, final JSONArray jsonArray,
-        final boolean condition) {
+    public static void writeValue(final long value, final JSONArray jsonArray, final boolean condition) {
         // Large values of long must be written as string. See bug 11311.
-        writeValue(String.valueOf(value), jsonArray, condition);
+        writeValue(Long.toString(value), jsonArray, condition);
+    }
+
+    public static void writeValue(Long value, JSONArray json, boolean condition) {
+        // Large values of long must be written as string. See bug 11311.
+        if (condition) {
+            writeValue(value.toString(), json);
+        } else {
+            writeNull(json);
+        }
+    }
+
+    protected static void writeNull(JSONArray json) {
+        json.put(JSONObject.NULL);
+    }
+
+    protected static void writeNull(String name, JSONObject json, boolean condition) throws JSONException {
+        if (condition) {
+            json.put(name, JSONObject.NULL);
+        }
     }
 
     /**
@@ -541,6 +458,14 @@ public class DataWriter {
         writeValue(value, value, timeZone, jsonArray);
     }
 
+    static void writeValue(final Date value, final TimeZone timeZone, final JSONArray json, final boolean condition) {
+        if (condition) {
+            writeValue(value, timeZone, json);
+        } else {
+            json.put(JSONObject.NULL);
+        }
+    }
+
     /**
      * Puts given <code>java.util.Date</code>'s time value into specified JSON array with respect to specified time zone
      *
@@ -559,20 +484,6 @@ public class DataWriter {
         }
     }
 
-    /**
-     * Puts given <code>byte[]</code> value into specified JSON array
-     *
-     * @param value The <code>Date</code> value
-     * @param jsonArray The JSON array to put into
-     */
-    public static void writeValue(final byte[] value, final JSONArray jsonArray) {
-        if (value == null) {
-            jsonArray.put(JSONObject.NULL);
-        } else {
-            jsonArray.put(new String(value));
-        }
-    }
-
     /**
      * Puts {@link JSONObject#NULL} into specified JSON array
      *
@@ -581,4 +492,105 @@ public class DataWriter {
     public static void writeValueNull(final JSONArray jsonArray) {
         jsonArray.put(JSONObject.NULL);
     }
+
+    protected void writeFields(final DataObject obj, final TimeZone tz, final JSONObject json) throws JSONException {
+        final WriterProcedure<DataObject> procedure = new WriterProcedure<DataObject>(obj, json, tz);
+        if (!WRITER_MAP.forEachValue(procedure)) {
+            final JSONException je = procedure.getError();
+            if (null != je) {
+                throw je;
+            }
+        }
+    }
+
+    protected boolean writeField(final DataObject obj, final int column, final TimeZone tz, final JSONArray json) throws JSONException {
+        final FieldWriter<DataObject> writer = WRITER_MAP.get(column);
+        if (null == writer) {
+            return false;
+        }
+        writer.write(obj, tz, json);
+        return true;
+    }
+
+    protected static interface FieldWriter<T> {
+
+        /**
+         * Writes this writer's value taken from specified data object to given JSON array.
+         * 
+         * @param data the data object
+         * @param json The JSON array
+         * @throws JSONException If writing to JSON array fails
+         */
+        void write(T obj, TimeZone timeZone, JSONArray json) throws JSONException;
+
+        void write(T obj, TimeZone timeZone, JSONObject json) throws JSONException;
+    }
+
+    private static final FieldWriter<DataObject> OBJECT_ID_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getObjectID(), json, obj.containsObjectID());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.ID, obj.getObjectID(), json, obj.containsObjectID());
+        }
+    };
+
+    private static final FieldWriter<DataObject> CREATED_BY_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getCreatedBy(), json, obj.containsCreatedBy());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.CREATED_BY, obj.getCreatedBy(), json, obj.containsCreatedBy());
+        }
+    };
+
+    private static final FieldWriter<DataObject> CREATION_DATE_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getCreationDate(), timeZone, json, obj.containsCreationDate());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.CREATION_DATE, obj.getCreationDate(), timeZone, json);
+        }
+    };
+
+    private static final FieldWriter<DataObject> MODIFIED_BY_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getModifiedBy(), json, obj.containsModifiedBy());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.MODIFIED_BY, obj.getModifiedBy(), json, obj.containsModifiedBy());
+        }
+    };
+
+    private static final FieldWriter<DataObject> LAST_MODIFIED_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getLastModified(), timeZone, json, obj.containsLastModified());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.LAST_MODIFIED, obj.getLastModified(), timeZone, json, obj.containsLastModified());
+        }
+    };
+
+    private static final FieldWriter<DataObject> LAST_MODIFIED_UTC_WRITER = new FieldWriter<DataObject>() {
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONArray json) {
+            writeValue(obj.getLastModified(), UTC, json, obj.containsLastModified());
+        }
+        public void write(final DataObject obj, final TimeZone timeZone, final JSONObject json) throws JSONException {
+            writeParameter(DataFields.LAST_MODIFIED_UTC, obj.getLastModified(), UTC, json, obj.containsLastModified());
+        }
+    };
+
+    static {
+        final TIntObjectHashMap<FieldWriter<DataObject>> m = new TIntObjectHashMap<FieldWriter<DataObject>>(6, 1);
+        m.put(DataObject.OBJECT_ID, OBJECT_ID_WRITER);
+        m.put(DataObject.CREATED_BY, CREATED_BY_WRITER);
+        m.put(DataObject.CREATION_DATE, CREATION_DATE_WRITER);
+        m.put(DataObject.MODIFIED_BY, MODIFIED_BY_WRITER);
+        m.put(DataObject.LAST_MODIFIED, LAST_MODIFIED_WRITER);
+        m.put(DataObject.LAST_MODIFIED_UTC, LAST_MODIFIED_UTC_WRITER);
+        WRITER_MAP = m;
+    }
+
+    private static final TIntObjectHashMap<FieldWriter<DataObject>> WRITER_MAP;
+
 }
