@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,7 @@
 
 package com.openexchange.ajp13;
 
-import static com.openexchange.tools.Collections.newHashMap;
+import gnu.trove.TIntObjectHashMap;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.net.URLDecoder;
@@ -58,7 +58,6 @@ import java.util.Arrays;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Locale;
-import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
@@ -94,9 +93,15 @@ public final class AJPv13ForwardRequest extends AJPv13Request {
             "ACL", "REPORT", "VERSION-CONTROL", "CHECKIN", "CHECKOUT", "UNCHECKOUT", "SEARCH", "MKWORKSPACE", "UPDATE", "LABEL", "MERGE",
             "BASELINE_CONTROL", "MKACTIVITY" };
 
-    private static final Map<Integer, String> httpHeaderMapping;
+    private static final int BYTE_FIRST = 0xA0;
 
-    private static final Map<Integer, String> attributeMapping;
+    private static final int BYTE_CONTENT_TYPE = 0x07;
+
+    private static final int BYTE_COOKIE = 0x09;
+
+    private static final TIntObjectHashMap<String> httpHeaderMapping;
+
+    private static final TIntObjectHashMap<String> attributeMapping;
 
     /**
      * A "set" to keep track of known JSESSIONIDs
@@ -125,37 +130,37 @@ public final class AJPv13ForwardRequest extends AJPv13Request {
     private static final int REQUEST_TERMINATOR = 0xFF;
 
     static {
-        httpHeaderMapping = newHashMap(14);
-        httpHeaderMapping.put(Integer.valueOf(0x01), "accept");
-        httpHeaderMapping.put(Integer.valueOf(0x02), "accept-charset");
-        httpHeaderMapping.put(Integer.valueOf(0x03), "accept-encoding");
-        httpHeaderMapping.put(Integer.valueOf(0x04), "accept-language");
-        httpHeaderMapping.put(Integer.valueOf(0x05), "authorization");
-        httpHeaderMapping.put(Integer.valueOf(0x06), "connection");
-        httpHeaderMapping.put(Integer.valueOf(0x07), HDR_CONTENT_TYPE);
-        httpHeaderMapping.put(Integer.valueOf(0x08), HDR_CONTENT_LENGTH);
-        httpHeaderMapping.put(Integer.valueOf(0x09), "cookie");
-        httpHeaderMapping.put(Integer.valueOf(0x0a), "cookie2");
-        httpHeaderMapping.put(Integer.valueOf(0x0b), "host");
-        httpHeaderMapping.put(Integer.valueOf(0x0c), "pragma");
-        httpHeaderMapping.put(Integer.valueOf(0x0d), "referer");
-        httpHeaderMapping.put(Integer.valueOf(0x0e), "user-agent");
+        httpHeaderMapping = new TIntObjectHashMap<String>(14);
+        httpHeaderMapping.put(0x01, "accept");
+        httpHeaderMapping.put(0x02, "accept-charset");
+        httpHeaderMapping.put(0x03, "accept-encoding");
+        httpHeaderMapping.put(0x04, "accept-language");
+        httpHeaderMapping.put(0x05, "authorization");
+        httpHeaderMapping.put(0x06, "connection");
+        httpHeaderMapping.put(BYTE_CONTENT_TYPE, HDR_CONTENT_TYPE);
+        httpHeaderMapping.put(0x08, HDR_CONTENT_LENGTH);
+        httpHeaderMapping.put(BYTE_COOKIE, "cookie");
+        httpHeaderMapping.put(0x0a, "cookie2");
+        httpHeaderMapping.put(0x0b, "host");
+        httpHeaderMapping.put(0x0c, "pragma");
+        httpHeaderMapping.put(0x0d, "referer");
+        httpHeaderMapping.put(0x0e, "user-agent");
 
-        attributeMapping = newHashMap(14);
-        attributeMapping.put(Integer.valueOf(0x01), "context");
-        attributeMapping.put(Integer.valueOf(0x02), "servlet_path");
-        attributeMapping.put(Integer.valueOf(0x03), "remote_user");
-        attributeMapping.put(Integer.valueOf(0x04), "auth_type");
-        attributeMapping.put(Integer.valueOf(0x05), ATTR_QUERY_STRING);
-        attributeMapping.put(Integer.valueOf(0x06), "jvm_route");
-        attributeMapping.put(Integer.valueOf(0x07), "ssl_cert");
-        attributeMapping.put(Integer.valueOf(0x08), "ssl_cipher");
-        attributeMapping.put(Integer.valueOf(0x09), "ssl_session");
-        attributeMapping.put(Integer.valueOf(0x0a), "req_attribute");
-        attributeMapping.put(Integer.valueOf(0x0b), ATTR_SSL_KEY_SIZE);
-        attributeMapping.put(Integer.valueOf(0x0c), "secret_attribute");
-        attributeMapping.put(Integer.valueOf(0x0d), ATTR_STORED_METHOD);
-        attributeMapping.put(Integer.valueOf(REQUEST_TERMINATOR), "are_done");
+        attributeMapping = new TIntObjectHashMap<String>(14);
+        attributeMapping.put(0x01, "context");
+        attributeMapping.put(0x02, "servlet_path");
+        attributeMapping.put(0x03, "remote_user");
+        attributeMapping.put(0x04, "auth_type");
+        attributeMapping.put(0x05, ATTR_QUERY_STRING);
+        attributeMapping.put(0x06, "jvm_route");
+        attributeMapping.put(0x07, "ssl_cert");
+        attributeMapping.put(0x08, "ssl_cipher");
+        attributeMapping.put(0x09, "ssl_session");
+        attributeMapping.put(0x0a, "req_attribute");
+        attributeMapping.put(0x0b, ATTR_SSL_KEY_SIZE);
+        attributeMapping.put(0x0c, "secret_attribute");
+        attributeMapping.put(0x0d, ATTR_STORED_METHOD);
+        attributeMapping.put(REQUEST_TERMINATOR, "are_done");
     }
 
     /**
@@ -205,7 +210,7 @@ public final class AJPv13ForwardRequest extends AJPv13Request {
             final int pos = requestURI.toLowerCase(Locale.ENGLISH).indexOf(jsessionIdURI);
             if (pos > -1) {
                 jsessionID = requestURI.substring(pos + jsessionIdURI.length());
-                requestURI = requestURI.substring(0, pos);
+//                requestURI = requestURI.substring(0, pos);
                 servletRequest.setRequestedSessionIdFromURL(true);
                 servletRequest.setRequestedSessionIdFromCookie(false);
             }
@@ -414,17 +419,17 @@ public final class AJPv13ForwardRequest extends AJPv13Request {
             {
                 final int firstByte = nextByte();
                 final int secondByte = nextByte();
-                if (firstByte == 0xA0) {
+                if (firstByte == BYTE_FIRST) {
                     /*
                      * Header name is encoded as an integer value.
                      */
-                    headerName = httpHeaderMapping.get(Integer.valueOf(secondByte));
-                    if (!contentTypeSet && (secondByte == 0x07)) {
+                    headerName = httpHeaderMapping.get(secondByte);
+                    if (!contentTypeSet && (BYTE_CONTENT_TYPE == secondByte)) {
                         servletRequest.setContentType(parseString());
                         contentTypeSet = true;
                         continue NextHeader;
                     }
-                    isCookie = (secondByte == 0x09);
+                    isCookie = (BYTE_COOKIE == secondByte);
                 } else {
                     headerName = parseString(firstByte, secondByte);
                     if (!contentTypeSet && HDR_CONTENT_TYPE.equalsIgnoreCase(headerName)) {
@@ -628,7 +633,7 @@ public final class AJPv13ForwardRequest extends AJPv13Request {
                  */
                 servletRequest.setAttribute(parseString(), parseString());
             } else {
-                final String attributeName = attributeMapping.get(Integer.valueOf(attrNum));
+                final String attributeName = attributeMapping.get(attrNum);
                 if (attributeName == null) {
                     throw new AJPv13Exception(AJPCode.NO_ATTRIBUTE_NAME, true, Integer.valueOf(attrNum));
                 }
