@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -297,6 +297,13 @@ public interface AJPv13RequestHandler {
      */
     public boolean isHttpSessionJoined();
 
+    /**
+     * Checks if headers have been sent already.
+     * 
+     * @return <code>true</code> if headers have been sent; otherwise <code>false</code>
+     */
+    public boolean isHeadersSent();
+
     /**
      * Gets the servlet path (which is not the request path). The servlet path is defined in servlet mapping configuration.
      * 
