@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,6 +60,7 @@ import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import com.openexchange.ajp13.AJPv13Config;
+import com.openexchange.ajp13.AJPv13Request;
 import com.openexchange.ajp13.AJPv13RequestHandler;
 import com.openexchange.ajp13.AJPv13Response;
 import com.openexchange.ajp13.exception.AJPv13Exception;
@@ -348,15 +349,11 @@ public class AJPv13TaskWatcher {
              */
             final AJPv13ConnectionImpl ajpConnection = task.getAJPConnection();
             if ((System.currentTimeMillis() - ajpConnection.getLastWriteAccess()) > AJPv13Config.getAJPWatcherMaxRunningTime()) {
-                if (info) {
-                    log.info(new StringBuilder(128).append("Sending KEEP-ALIVE for AJP Listener \"").append(task.getThreadName()).append(
-                        '"').toString());
-                }
                 /*
                  * Send "keep-alive" package
                  */
                 try {
-                    keepAlive(task.getAJPConnection());
+                    keepAlive();
                 } catch (final AJPv13Exception e) {
                     log.error("AJP KEEP-ALIVE failed.", e);
                 } catch (final IOException e) {
@@ -365,68 +362,95 @@ public class AJPv13TaskWatcher {
             }
         }
 
-        private void keepAlive(final AJPv13ConnectionImpl ajpConnection) throws IOException, AJPv13Exception {
+        /**
+         * Performs AJP-style keep-alive poll to web server to avoid connection timeout.
+         * 
+         * @throws IOException If an I/O error occurs
+         * @throws AJPv13Exception If an AJP error occurs
+         */
+        private void keepAlive() throws IOException, AJPv13Exception {
             /*
-             * Send "keep-alive" package; first poll connection by sending outstanding data
+             * Send "keep-alive" package depending on current request handler's state.
              */
-            final AJPv13RequestHandler ajpRequestHandler = ajpConnection.getAjpRequestHandler();
-            ajpConnection.blockOutputStream(true);
+            final AJPv13ConnectionImpl ajpConnection = task.getAJPConnection();
+            ajpConnection.block();
             try {
-                final String remoteAddress = info ? task.getSocket().getRemoteSocketAddress().toString() : null;
-                final OutputStream out = ajpConnection.getOutputStream();
-                final byte[] remainingData = ajpRequestHandler.getAndClearResponseData();
-                if (remainingData.length > 0) {
-                    /*
-                     * Send response headers first.
-                     */
-                    ajpRequestHandler.doWriteHeaders(out);
-                    /*
-                     * Send rest of data cut into MAX_BODY_CHUNK_SIZE pieces
-                     */
-                    int offset = 0;
-                    final int maxLen = AJPv13Response.MAX_SEND_BODY_CHUNK_SIZE;
-                    while (offset < remainingData.length) {
-                        final int b = (remainingData.length - offset);
-                        final int curLen = ((maxLen <= b) ? maxLen : b); // Math.min(int a, int b)
-                        out.write(AJPv13Response.getSendBodyChunkBytes(remainingData, offset, curLen));
-                        out.flush();
-                        offset += curLen;
-                    }
-                    if (info) {
-                        log.info(new StringBuilder().append("Flushed available data to socket \"").append(remoteAddress).append(
-                            "\" to initiate a KEEP-ALIVE poll."));
-                    }
-                } else {
-                    /*
-                     * No outstanding data; poll connection through requesting an empty data chunk.
-                     */
-                    ajpConnection.blockInputStream(true);
-                    try {
-                        out.write(AJPv13Response.getGetBodyChunkBytes(0));
-                        out.flush();
-                        if (info) {
-                            log.info(new StringBuilder().append("Flushed empty GET-BODY request to socket \"").append(remoteAddress).append(
-                                "\" to initiate a KEEP-ALIVE poll."));
-                        }
+                final AJPv13RequestHandler ajpRequestHandler = ajpConnection.getAjpRequestHandler();
+                ajpConnection.blockOutputStream(true);
+                try {
+                    final String remoteAddress = info ? task.getSocket().getRemoteSocketAddress().toString() : null;
+                    final OutputStream out = ajpConnection.getOutputStream();
+                    if (ajpRequestHandler.isHeadersSent()) {
                         /*
-                         * Swallow expected empty body chunk
+                         * SEND_HEADERS package already flushed to web server. Keep-Alive needs to be performed by flushing available data
+                         * or an empty SEND_BODY package.
                          */
-                        final int bodyRequestDataLength = ajpConnection.readInitialBytes(true, false);
-                        if (bodyRequestDataLength > 0 && parseInt(ajpConnection.getPayloadData(bodyRequestDataLength, true)) > 0) {
-                            log.warn("Got a non-empty data chunk from web server although an empty one was requested");
-                        } else if (info) {
-                            log.info(new StringBuilder().append("Swallowed empty REQUEST-BODY from socket \"").append(remoteAddress).append(
-                                "\" initiated by former KEEP-ALIVE poll."));
+                        final byte[] remainingData = ajpRequestHandler.getAndClearResponseData();
+                        if (remainingData.length > 0) {
+                            /*
+                             * Flush available data cut into MAX_BODY_CHUNK_SIZE chunks
+                             */
+                            keepAliveSendAvailableData(remoteAddress, out, remainingData);
+                        } else {
+                            /*
+                             * Empty SEND_BODY package.
+                             */
+                            keepAliveSendEmptyBody(remoteAddress, out);
                         }
-                    } finally {
-                        ajpConnection.blockInputStream(false);
+                    } else {
+                        /*
+                         * Pending SEND_HEADERS package. Keep-Alive needs to be performed by requesting an empty data chunk.
+                         */
+                        keepAliveGetEmptyBody(ajpConnection, remoteAddress, out);
                     }
+                } finally {
+                    ajpConnection.blockOutputStream(false);
                 }
             } finally {
-                ajpConnection.blockOutputStream(false);
+                ajpConnection.unblock();
             }
         } // End of keepAlive()
 
+        private void keepAliveSendAvailableData(final String remoteAddress, final OutputStream out, final byte[] remainingData) throws IOException, AJPv13Exception {
+            AJPv13Request.writeChunked(remainingData, out);
+            if (info) {
+                log.info(new StringBuilder().append("AJP KEEP-ALIVE: Flushed available data to socket \"").append(remoteAddress).append(
+                    "\" to initiate a KEEP-ALIVE poll."));
+            }
+        }
+
+        private void keepAliveSendEmptyBody(final String remoteAddress, final OutputStream out) throws IOException, AJPv13Exception {
+            AJPv13Request.writeEmpty(out);
+            if (info) {
+                log.info(new StringBuilder().append("AJP KEEP-ALIVE: Flushed empty SEND-BODY-CHUNK response to socket \"").append(
+                    remoteAddress).append("\" to initiate a KEEP-ALIVE poll."));
+            }
+        }
+
+        private void keepAliveGetEmptyBody(final AJPv13ConnectionImpl ajpConnection, final String remoteAddress, final OutputStream out) throws IOException, AJPv13Exception {
+            ajpConnection.blockInputStream(true);
+            try {
+                out.write(AJPv13Response.getGetBodyChunkBytes(0));
+                out.flush();
+                if (info) {
+                    log.info(new StringBuilder().append("AJP KEEP-ALIVE: Flushed empty GET-BODY request to socket \"").append(remoteAddress).append(
+                        "\" to initiate a KEEP-ALIVE poll."));
+                }
+                /*
+                 * Swallow expected empty body chunk
+                 */
+                final int bodyRequestDataLength = ajpConnection.readInitialBytes(true, false);
+                if (bodyRequestDataLength > 0 && parseInt(ajpConnection.getPayloadData(bodyRequestDataLength, true)) > 0) {
+                    log.warn("AJP KEEP-ALIVE: Got a non-empty data chunk from web server although an empty one was requested");
+                } else if (info) {
+                    log.info(new StringBuilder().append("AJP KEEP-ALIVE: Swallowed empty REQUEST-BODY from socket \"").append(remoteAddress).append(
+                        "\" initiated by former KEEP-ALIVE poll."));
+                }
+            } finally {
+                ajpConnection.blockInputStream(false);
+            }
+        }
+
         private static int parseInt(final byte[] payloadData) {
             return ((payloadData[0] & 0xff) << 8) + (payloadData[1] & 0xff);
         }
