@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -80,7 +80,7 @@ public abstract class AbstractCreateTableImpl implements CreateTableService {
         }
     }
 
-    public abstract String[] getCreateStatements();
+    protected abstract String[] getCreateStatements();
 
     protected static final String[] NO_TABLES = new String[0];
 
