@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,8 +49,25 @@
 
 package com.openexchange.database;
 
-import static com.openexchange.database.DBPoolingExceptionStrings.*;
-
+import static com.openexchange.database.DBPoolingExceptionStrings.ACTIVE_STATEMENTS_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.ALREADY_INITIALIZED_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.MISSING_CONFIGURATION_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NOT_INITIALIZED_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NOT_RESOLVED_SERVER_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NO_CONFIG_DB_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NO_CONNECTION_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NO_DRIVER_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NO_SERVER_NAME_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.NULL_CONNECTION_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.PARAMETER_PROBLEM_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.PROPERTY_MISSING_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.RESOLVE_FAILED_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.RETURN_FAILED_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.SCHEMA_FAILED_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.SQL_ERROR_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.TOO_LONG_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.TRANSACTION_MISSING_MSG;
+import static com.openexchange.database.DBPoolingExceptionStrings.UNKNOWN_POOL_MSG;
 import com.openexchange.database.internal.DBPoolingExceptionFactory;
 import com.openexchange.exceptions.OXErrorMessage;
 import com.openexchange.groupware.AbstractOXException.Category;
