@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -73,6 +73,8 @@ public abstract class AbstractFolder implements Folder {
 
     protected String id;
 
+    protected String newId;
+
     protected String name;
 
     protected String parent;
@@ -99,6 +101,8 @@ public abstract class AbstractFolder implements Folder {
 
     protected int capabilities;
 
+    protected int bits;
+
     protected boolean deefault;
 
     protected int defaultType;
@@ -115,9 +119,15 @@ public abstract class AbstractFolder implements Folder {
         unread = -1;
         deleted = -1;
         capabilities = -1;
+        bits = -1;
         defaultType = 0;
     }
 
+    @Override
+    public String toString() {
+        return new StringBuilder(32).append("{ name=").append(getName()).append(", id=").append(getID()).append('}').toString();
+    }
+
     @Override
     public Object clone() {
         try {
@@ -190,6 +200,10 @@ public abstract class AbstractFolder implements Folder {
         return id;
     }
 
+    public String getNewID() {
+        return newId;
+    }
+
     public String getLocalizedName(final Locale locale) {
         return name;
     }
@@ -226,6 +240,10 @@ public abstract class AbstractFolder implements Folder {
         this.id = id;
     }
 
+    public void setNewID(final String newId) {
+        this.newId = newId;
+    }
+
     public void setName(final String name) {
         this.name = name;
     }
@@ -330,4 +348,12 @@ public abstract class AbstractFolder implements Folder {
         this.capabilities = capabilities;
     }
 
+    public int getBits() {
+        return bits;
+    }
+
+    public void setBits(final int bits) {
+        this.bits = bits;
+    }
+
 }
