@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -165,6 +165,20 @@ public interface Folder extends Serializable, Cloneable {
      */
     void setID(String id);
 
+    /**
+     * Gets the new ID.
+     * 
+     * @return The new ID or <code>null</code> if not available
+     */
+    String getNewID();
+
+    /**
+     * Sets the new ID.
+     * 
+     * @param newId The new ID to set
+     */
+    void setNewID(String newId);
+
     /**
      * Gets the parent ID.
      * 
@@ -398,6 +412,20 @@ public interface Folder extends Serializable, Cloneable {
      */
     void setCapabilities(int capabilities);
 
+    /**
+     * Gets the bits
+     * 
+     * @return The bits
+     */
+    int getBits();
+
+    /**
+     * Sets the bits
+     * 
+     * @param bits The bits to set
+     */
+    void setBits(final int bits);
+
     /**
      * Creates and returns a copy of this object.
      * 
