@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -158,7 +158,31 @@ public enum FolderExceptionErrorMessage implements OXErrorMessage {
     /**
      * No default folder available for content type "%1$s" in tree "%2$s".
      */
-    NO_DEFAULT_FOLDER(FolderExceptionMessages.NO_DEFAULT_FOLDER_MSG, Category.CODE_ERROR, 24);
+    NO_DEFAULT_FOLDER(FolderExceptionMessages.NO_DEFAULT_FOLDER_MSG, Category.CODE_ERROR, 24),
+    /**
+     * Invalid folder identifier: %1$s.
+     */
+    INVALID_FOLDER_ID(FolderExceptionMessages.INVALID_FOLDER_ID_MSG, Category.CODE_ERROR, 25),
+    /**
+     * Folder "%1$s" must not be deleted by user "%2$s" in context "%3$s".
+     * <p>
+     * Folder identifier should be passed as first argument to not injure privacy through publishing folder name.
+     */
+    FOLDER_NOT_DELETEABLE(FolderExceptionMessages.FOLDER_NOT_DELETEABLE_MSG, Category.PERMISSION, 26),
+    /**
+     * Folder "%1$s" must not be moved by user "%2$s" in context "%3$s".
+     * <p>
+     * Folder identifier should be passed as first argument to not injure privacy through publishing folder name.
+     */
+    FOLDER_NOT_MOVEABLE(FolderExceptionMessages.FOLDER_NOT_MOVEABLE_MSG, Category.PERMISSION, 27),
+    /**
+     * A temporary error occurred. Please retry.
+     */
+    TEMPORARY_ERROR(FolderExceptionMessages.TEMPORARY_ERROR_MSG, Category.CODE_ERROR, 28),
+    /**
+     * User "%2$s" must not create subfolders below folder "%2$s" in context "%3$s".
+     */
+    NO_CREATE_SUBFOLDERS(FolderExceptionMessages.NO_CREATE_SUBFOLDERS_MSG, Category.PERMISSION, 29);
 
     private final Category category;
 
