@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -133,6 +133,21 @@ public class FolderExceptionMessages implements LocalizableStrings {
     // No default folder available for content type "%1$s" in tree "%2$s".
     public static final String NO_DEFAULT_FOLDER_MSG = "No default folder available for content type \"%1$s\" in tree \"%2$s\".";
 
+    // Invalid folder identifier: %1$s.
+    public static final String INVALID_FOLDER_ID_MSG = "Invalid folder identifier: %1$s.";
+
+    // Folder "%1$s" must not be deleted by user "%2$s" in context "%3$s".
+    public static final String FOLDER_NOT_DELETEABLE_MSG = "Folder \"%1$s\" must not be deleted by user \"%2$s\" in context \"%3$s\".";
+
+    // Folder "%1$s" must not be moved by user "%2$s" in context "%3$s".
+    public static final String FOLDER_NOT_MOVEABLE_MSG = "Folder \"%1$s\" must not be moved by user \"%2$s\" in context \"%3$s\".";
+
+    // A temporary error occurred. Please retry.
+    public static final String TEMPORARY_ERROR_MSG = "A temporary error occurred. Please retry.";
+
+    // User "%2$s" must not create subfolders below folder "%2$s" in context "%3$s".
+    public static final String NO_CREATE_SUBFOLDERS_MSG = "User \"%2$s\" must not create subfolders below folder \"%2$s\" in context \"%3$s\".";
+
     /**
      * Initializes a new {@link FolderExceptionMessages}
      */
