@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,7 @@
 package com.openexchange.folderstorage;
 
 import java.util.Date;
+import java.util.Map;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.session.Session;
@@ -61,6 +62,13 @@ import com.openexchange.session.Session;
  */
 public interface FolderService {
 
+    /**
+     * Gets a map of available content types registered by folder storages.
+     * 
+     * @return A map of available content types registered by folder storages
+     */
+    Map<Integer, ContentType> getAvailableContentTypes();
+
     /**
      * Gets the folder identified by given folder identifier and tree identifier.
      * 
@@ -97,7 +105,7 @@ public interface FolderService {
      * @return All visible folders
      * @throws FolderException If folders cannot be returned
      */
-    UserizedFolder[] getAllVisibleFolders(String treeId, FolderFilter filter, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getAllVisibleFolders(String treeId, FolderFilter filter, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets all visible folders located in given tree for given session's user.
@@ -109,7 +117,7 @@ public interface FolderService {
      * @return All visible folders
      * @throws FolderException If folders cannot be returned
      */
-    UserizedFolder[] getAllVisibleFolders(String treeId, FolderFilter filter, Session session, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getAllVisibleFolders(String treeId, FolderFilter filter, Session session, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets this storage's default folder for specified user.
@@ -206,7 +214,7 @@ public interface FolderService {
      * @return The subfolders
      * @throws FolderException If subfolders cannot be returned
      */
-    UserizedFolder[] getSubfolders(String treeId, String parentId, boolean all, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getSubfolders(String treeId, String parentId, boolean all, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets the subfolders of specified parent in given tree.
@@ -220,7 +228,7 @@ public interface FolderService {
      * @return The subfolders
      * @throws FolderException If subfolders cannot be returned
      */
-    UserizedFolder[] getSubfolders(String treeId, String parentId, boolean all, Session session, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getSubfolders(String treeId, String parentId, boolean all, Session session, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets the path from given folder to specified tree's root folder.
@@ -233,7 +241,7 @@ public interface FolderService {
      * @return The path from given folder to specified tree's root folder
      * @throws FolderException If path cannot be returned
      */
-    UserizedFolder[] getPath(String treeId, String folderId, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getPath(String treeId, String folderId, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets the path from given folder to specified tree's root folder.
@@ -245,7 +253,7 @@ public interface FolderService {
      * @return The path from given folder to specified tree's root folder
      * @throws FolderException If path cannot be returned
      */
-    UserizedFolder[] getPath(String treeId, String folderId, Session session, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[]> getPath(String treeId, String folderId, Session session, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets all new, modified and deleted folders since given time stamp.
@@ -260,7 +268,7 @@ public interface FolderService {
      * @return All new, modified and deleted folders since given time stamp
      * @throws FolderException If path cannot be returned
      */
-    UserizedFolder[][] getUpdates(String treeId, Date timeStamp, boolean ignoreDeleted, ContentType[] includeContentTypes, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[][]> getUpdates(String treeId, Date timeStamp, boolean ignoreDeleted, ContentType[] includeContentTypes, User user, Context context, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Gets all new, modified and deleted folders since given time stamp.
@@ -274,7 +282,7 @@ public interface FolderService {
      * @return All new, modified and deleted folders since given time stamp
      * @throws FolderException If path cannot be returned
      */
-    UserizedFolder[][] getUpdates(String treeId, Date timeStamp, boolean ignoreDeleted, ContentType[] includeContentTypes, Session session, FolderServiceDecorator decorator) throws FolderException;
+    FolderResponse<UserizedFolder[][]> getUpdates(String treeId, Date timeStamp, boolean ignoreDeleted, ContentType[] includeContentTypes, Session session, FolderServiceDecorator decorator) throws FolderException;
 
     /**
      * Deletes the specified folder in given tree.
