@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -79,6 +79,16 @@ public interface FolderStorage {
      */
     public static final String PRIVATE_ID = "1";
 
+    /**
+     * The reserved identifier for public folder: <code>"2"</code>.
+     */
+    public static final String PUBLIC_ID = "2";
+
+    /**
+     * The reserved identifier for shared folder: <code>"3"</code>.
+     */
+    public static final String SHARED_ID = "3";
+
     /**
      * Gets the content types supported by this folder storage.
      * <p>
@@ -109,6 +119,29 @@ public interface FolderStorage {
      */
     ContentType getDefaultContentType();
 
+    /**
+     * Checks if denoted folder is empty.
+     * 
+     * @param treeId The tree identifier
+     * @param folderId The folder identifier
+     * @param storageParameters The storage parameters
+     * @return <code>true</code> if denoted folder is empty; otherwise <code>false</code>
+     * @throws FolderException If checking folder content fails
+     */
+    boolean isEmpty(String treeId, String folderId, StorageParameters storageParameters) throws FolderException;
+
+    /**
+     * Checks if denoted folder contains user-foreign objects.
+     * 
+     * @param user The user whose default folder shall be returned
+     * @param treeId The tree identifier
+     * @param folderId The folder identifier
+     * @param storageParameters The storage parameters
+     * @return <code>true</code> if folder contains user-foreign objects; otherwise <code>false</code>
+     * @throws FolderException
+     */
+    boolean containsForeignObjects(User user, String treeId, String folderId, StorageParameters storageParameters) throws FolderException;
+
     /**
      * Checks if the folder denoted by specified folder ID exists in this folder storage.
      * 
@@ -198,6 +231,17 @@ public interface FolderStorage {
      */
     void updateFolder(Folder folder, StorageParameters storageParameters) throws FolderException;
 
+    /**
+     * Updates the last-modified time stamp of the given folder in the storage.
+     * 
+     * @param lastModified The last-modified time stamp to set
+     * @param treeId The tree identifier
+     * @param folderId The folder ID
+     * @param storageParameters The storage parameters
+     * @throws FolderException If updating last-modified time stamp fails
+     */
+    void updateLastModified(long lastModified, String treeId, String folderId, StorageParameters storageParameters) throws FolderException;
+
     /**
      * Creates the given folder on the storage.
      * <p>
@@ -252,15 +296,15 @@ public interface FolderStorage {
      * 
      * @param parameters The parameters
      * @param modify <code>true</code> if started transaction is supposed to modify storage's content; otherwise <code>false</code>
-     * @return The parameters with storage-specific extensions
+     * @return <code>true</code> if call started the transaction; otherwise <code>false</code> if transaction has already been started before
      * @throws FolderException If storage parameters cannot be returned
      */
-    StorageParameters startTransaction(StorageParameters parameters, boolean modify) throws FolderException;
+    boolean startTransaction(StorageParameters parameters, boolean modify) throws FolderException;
 
     /**
      * Publishes made changes on the storage.
      * 
-     * @param params The storage parameters.
+     * @param params The storage parameters
      * @throws FolderException If committing the made changes fails.
      */
     void commitTransaction(StorageParameters params) throws FolderException;
