@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -68,6 +68,14 @@ public interface StorageParameters {
      */
     Context getContext();
 
+    /**
+     * Convenience method to get the context identifier.
+     * 
+     * @return The context identifier
+     * @see #getContext()
+     */
+    int getContextId();
+
     /**
      * Gets the user.
      * 
@@ -75,6 +83,14 @@ public interface StorageParameters {
      */
     User getUser();
 
+    /**
+     * Convenience method to get the user identifier.
+     * 
+     * @return The user identifier
+     * @see #getUser()
+     */
+    int getUserId();
+
     /**
      * Gets the session.
      * 
@@ -119,7 +135,7 @@ public interface StorageParameters {
      * @param name The parameter name
      * @return The parameter bound to given name
      */
-    Object getParameter(FolderType folderType, String name);
+    <P> P getParameter(FolderType folderType, String name);
 
     /**
      * Puts given parameter. Any existing parameters bound to given name are replaced. A <code>null</code> value means to remove the
@@ -146,4 +162,16 @@ public interface StorageParameters {
      */
     boolean putParameterIfAbsent(FolderType folderType, String name, Object value);
 
+    /**
+     * Sets a committed marker to this storage parameters.
+     */
+    void markCommitted();
+
+    /**
+     * Gets the trace of the thread that set the committed marker to this storage parameters.
+     * 
+     * @return The trace
+     */
+    String getCommittedTrace();
+
 }
