@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,12 +53,9 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
+import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CompletionService;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.ExecutorCompletionService;
-import java.util.concurrent.Future;
-import java.util.concurrent.TimeUnit;
 import com.openexchange.caching.Cache;
 import com.openexchange.caching.CacheException;
 import com.openexchange.caching.CacheKey;
@@ -76,17 +73,18 @@ import com.openexchange.folderstorage.StoragePriority;
 import com.openexchange.folderstorage.StorageType;
 import com.openexchange.folderstorage.UserizedFolder;
 import com.openexchange.folderstorage.internal.StorageParametersImpl;
-import com.openexchange.folderstorage.internal.actions.Clear;
-import com.openexchange.folderstorage.internal.actions.Create;
-import com.openexchange.folderstorage.internal.actions.Delete;
-import com.openexchange.folderstorage.internal.actions.Update;
-import com.openexchange.folderstorage.internal.actions.Updates;
-import com.openexchange.groupware.AbstractOXException;
+import com.openexchange.folderstorage.internal.performers.ClearPerformer;
+import com.openexchange.folderstorage.internal.performers.CreatePerformer;
+import com.openexchange.folderstorage.internal.performers.DeletePerformer;
+import com.openexchange.folderstorage.internal.performers.UpdatePerformer;
+import com.openexchange.folderstorage.internal.performers.UpdatesPerformer;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.server.ServiceException;
 import com.openexchange.session.Session;
+import com.openexchange.threadpool.ThreadPoolCompletionService;
 import com.openexchange.threadpool.ThreadPoolService;
+import com.openexchange.threadpool.ThreadPools;
 import com.openexchange.tools.session.ServerSession;
 import com.openexchange.tools.session.ServerSessionAdapter;
 
@@ -97,7 +95,17 @@ import com.openexchange.tools.session.ServerSessionAdapter;
  */
 public final class CacheFolderStorage implements FolderStorage {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(CacheFolderStorage.class);
+    private static final ThreadPools.ExpectedExceptionFactory<FolderException> FACTORY =
+        new ThreadPools.ExpectedExceptionFactory<FolderException>() {
+
+            public Class<FolderException> getType() {
+                return FolderException.class;
+            }
+
+            public FolderException newUnexpectedError(final Throwable t) {
+                return FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(t, t.getMessage());
+            }
+        };
 
     private final CacheFolderStorageRegistry registry;
 
@@ -182,10 +190,10 @@ public final class CacheFolderStorage implements FolderStorage {
          */
         final String folderId;
         if (null == session) {
-            folderId = new Create(storageParameters.getUser(), storageParameters.getContext(), registry).doCreate(folder);
+            folderId = new CreatePerformer(storageParameters.getUser(), storageParameters.getContext(), registry).doCreate(folder);
         } else {
             try {
-                folderId = new Create(new ServerSessionAdapter(session), registry).doCreate(folder);
+                folderId = new CreatePerformer(new ServerSessionAdapter(session), registry).doCreate(folder);
             } catch (final ContextException e) {
                 throw new FolderException(e);
             }
@@ -199,40 +207,42 @@ public final class CacheFolderStorage implements FolderStorage {
             throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
         }
         final Folder createdFolder = loadFolder(treeId, folderId, StorageType.WORKING, storageParameters);
-        putFolder(createdFolder, treeId, storageParameters);
+        if (createdFolder.isCacheable()) {
+            putFolder(createdFolder, treeId, storageParameters);
+        }
         /*
          * Refresh parent
          */
         removeFolder(folder.getParentID(), treeId, storageParameters);
         final Folder parentFolder = loadFolder(treeId, folder.getParentID(), StorageType.WORKING, storageParameters);
-        putFolder(parentFolder, treeId, storageParameters);
+        if (parentFolder.isCacheable()) {
+            putFolder(parentFolder, treeId, storageParameters);
+        }
     }
 
     private void putFolder(final Folder folder, final String treeId, final StorageParameters storageParameters) throws FolderException {
-        if (folder.isCacheable()) {
-            /*
-             * Put to cache
-             */
-            try {
-                final CacheKey key;
-                final String id = folder.getID();
-                if (folder.isGlobalID()) {
-                    key = newCacheKey(id, treeId, storageParameters.getContext().getContextId());
-                    globalCache.put(key, folder);
-                } else {
-                    key = newCacheKey(id, treeId, storageParameters.getContext().getContextId(), storageParameters.getUser().getId());
-                    userCache.put(key, folder);
-                }
-            } catch (final CacheException e) {
-                throw new FolderException(e);
+        /*
+         * Put to cache
+         */
+        try {
+            final CacheKey key;
+            final String id = folder.getID();
+            if (folder.isGlobalID()) {
+                key = newCacheKey(id, treeId, storageParameters.getContextId());
+                globalCache.put(key, folder);
+            } else {
+                key = newCacheKey(id, treeId, storageParameters.getContextId(), storageParameters.getUserId());
+                userCache.put(key, folder);
             }
+        } catch (final CacheException e) {
+            throw new FolderException(e);
         }
     }
 
     private void removeFolder(final String id, final String treeId, final StorageParameters storageParameters) throws FolderException {
         try {
-            final int contextId = storageParameters.getContext().getContextId();
-            final int userId = storageParameters.getUser().getId();
+            final int contextId = storageParameters.getContextId();
+            final int userId = storageParameters.getUserId();
             globalCache.remove(newCacheKey(id, treeId, contextId));
             userCache.remove(newCacheKey(id, treeId, contextId, userId));
             if (!FolderStorage.REAL_TREE_ID.equals(treeId)) {
@@ -251,10 +261,10 @@ public final class CacheFolderStorage implements FolderStorage {
          * Perform clear operation via non-cache storage
          */
         if (null == session) {
-            new Clear(storageParameters.getUser(), storageParameters.getContext(), registry).doClear(treeId, folderId);
+            new ClearPerformer(storageParameters.getUser(), storageParameters.getContext(), registry).doClear(treeId, folderId);
         } else {
             try {
-                new Clear(new ServerSessionAdapter(session), registry).doClear(treeId, folderId);
+                new ClearPerformer(new ServerSessionAdapter(session), registry).doClear(treeId, folderId);
             } catch (final ContextException e) {
                 throw new FolderException(e);
             }
@@ -263,6 +273,7 @@ public final class CacheFolderStorage implements FolderStorage {
 
     public void deleteFolder(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
         final String parentId;
+        final String realParentId;
         final boolean cacheable;
         final boolean global;
         {
@@ -270,19 +281,45 @@ public final class CacheFolderStorage implements FolderStorage {
             cacheable = deleteMe.isCacheable();
             global = deleteMe.isGlobalID();
             parentId = deleteMe.getParentID();
+            if (!FolderStorage.REAL_TREE_ID.equals(treeId)) {
+                final StorageParameters parameters = newStorageParameters(storageParameters);
+                final FolderStorage folderStorage = registry.getFolderStorage(FolderStorage.REAL_TREE_ID, folderId);
+                final boolean started = folderStorage.startTransaction(parameters, false);
+                try {
+                    realParentId = folderStorage.getFolder(FolderStorage.REAL_TREE_ID, folderId, parameters).getParentID();
+                    if (started) {
+                        folderStorage.commitTransaction(parameters);
+                    }
+                } catch (final FolderException e) {
+                    if (started) {
+                        folderStorage.rollback(parameters);
+                    }
+                    throw e;
+                } catch (final Exception e) {
+                    if (started) {
+                        folderStorage.rollback(parameters);
+                    }
+                    throw FolderException.newUnexpectedException(e);
+                }
+            } else {
+                realParentId = null;
+            }
         }
         final Session session = storageParameters.getSession();
         /*
          * Perform delete
          */
         if (null == session) {
-            new Delete(storageParameters.getUser(), storageParameters.getContext(), registry).doDelete(
+            new DeletePerformer(storageParameters.getUser(), storageParameters.getContext(), registry).doDelete(
                 treeId,
                 folderId,
                 storageParameters.getTimeStamp());
         } else {
             try {
-                new Delete(new ServerSessionAdapter(session), registry).doDelete(treeId, folderId, storageParameters.getTimeStamp());
+                new DeletePerformer(new ServerSessionAdapter(session), registry).doDelete(
+                    treeId,
+                    folderId,
+                    storageParameters.getTimeStamp());
             } catch (final ContextException e) {
                 throw new FolderException(e);
             }
@@ -293,11 +330,10 @@ public final class CacheFolderStorage implements FolderStorage {
                  * Delete from cache
                  */
                 if (global) {
-                    final CacheKey key = newCacheKey(folderId, treeId, storageParameters.getContext().getContextId());
+                    final CacheKey key = newCacheKey(folderId, treeId, storageParameters.getContextId());
                     globalCache.remove(key);
                 } else {
-                    final CacheKey key =
-                        newCacheKey(folderId, treeId, storageParameters.getContext().getContextId(), storageParameters.getUser().getId());
+                    final CacheKey key = newCacheKey(folderId, treeId, storageParameters.getContextId(), storageParameters.getUserId());
                     userCache.remove(key);
                 }
             } catch (final CacheException e) {
@@ -307,10 +343,16 @@ public final class CacheFolderStorage implements FolderStorage {
         /*
          * Refresh parent
          */
+        if (null != realParentId && !FolderStorage.ROOT_ID.equals(realParentId)) {
+            removeFolder(realParentId, treeId, storageParameters);
+            removeFolder(realParentId, FolderStorage.REAL_TREE_ID, storageParameters);
+        }
         if (!FolderStorage.ROOT_ID.equals(parentId)) {
             removeFolder(parentId, treeId, storageParameters);
             final Folder parentFolder = loadFolder(treeId, parentId, StorageType.WORKING, storageParameters);
-            putFolder(parentFolder, treeId, storageParameters);
+            if (parentFolder.isCacheable()) {
+                putFolder(parentFolder, treeId, storageParameters);
+            }
         }
     }
 
@@ -320,47 +362,155 @@ public final class CacheFolderStorage implements FolderStorage {
             throw FolderExceptionErrorMessage.NO_STORAGE_FOR_CT.create(treeId, contentType);
         }
         final String folderId;
-        storage.startTransaction(storageParameters, false);
+        final boolean started = storage.startTransaction(storageParameters, false);
         try {
             folderId = storage.getDefaultFolderID(user, treeId, contentType, storageParameters);
-            storage.commitTransaction(storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
         } catch (final FolderException e) {
-            storage.rollback(storageParameters);
+            if (started) {
+                storage.rollback(storageParameters);
+            }
             throw e;
+        } catch (final Exception e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
         return folderId;
     }
 
+    public boolean containsForeignObjects(final User user, final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        /*
+         * Get folder storage
+         */
+        final FolderStorage storage = registry.getFolderStorage(treeId, folderId);
+        if (null == storage) {
+            throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
+        }
+        final boolean started = storage.startTransaction(storageParameters, false);
+        try {
+            final boolean containsForeignObjects = storage.containsForeignObjects(user, treeId, folderId, storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
+            return containsForeignObjects;
+        } catch (final FolderException e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw e;
+        } catch (final Exception e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+    }
+
+    public boolean isEmpty(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        /*
+         * Get folder storage
+         */
+        final FolderStorage storage = registry.getFolderStorage(treeId, folderId);
+        if (null == storage) {
+            throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
+        }
+        final boolean started = storage.startTransaction(storageParameters, false);
+        try {
+            final boolean isEmpty = storage.isEmpty(treeId, folderId, storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
+            return isEmpty;
+        } catch (final FolderException e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw e;
+        } catch (final Exception e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+    }
+
+    public void updateLastModified(final long lastModified, final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        final FolderStorage storage = registry.getFolderStorage(treeId, folderId);
+        if (null == storage) {
+            throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
+        }
+        final boolean started = storage.startTransaction(storageParameters, false);
+        try {
+            storage.updateLastModified(lastModified, treeId, folderId, storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
+        } catch (final FolderException e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw e;
+        } catch (final Exception e) {
+            if (started) {
+                storage.rollback(storageParameters);
+            }
+            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+        /*
+         * Invalidate cache entry
+         */
+        removeFolder(folderId, treeId, storageParameters);
+        removeFolder(folderId, REAL_TREE_ID, storageParameters);
+    }
+
     public Folder getFolder(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
         return getFolder(treeId, folderId, StorageType.WORKING, storageParameters);
     }
 
     public Folder getFolder(final String treeId, final String folderId, final StorageType storageType, final StorageParameters storageParameters) throws FolderException {
-        final int contextId = storageParameters.getContext().getContextId();
+        final int contextId = storageParameters.getContextId();
         /*
          * Try global cache key
          */
-        CacheKey key = newCacheKey(folderId, treeId, contextId);
-        Folder folder = (Folder) globalCache.get(key);
+        Folder folder = (Folder) globalCache.get(newCacheKey(folderId, treeId, contextId));
         if (null != folder) {
             /*
-             * Return a cloned version
+             * Return a cloned version from global cache
              */
             return (Folder) folder.clone();
         }
         /*
          * Try user cache key
          */
-        key = newCacheKey(folderId, treeId, contextId, storageParameters.getUser().getId());
-        folder = (Folder) userCache.get(key);
-        if (null == folder) {
-            folder = loadFolder(treeId, folderId, storageType, storageParameters);
-            putFolder(folder, treeId, storageParameters);
+        folder = (Folder) userCache.get(newCacheKey(folderId, treeId, contextId, storageParameters.getUserId()));
+        if (null != folder) {
+            /*
+             * Return a cloned version from user-bound cache
+             */
+            return (Folder) folder.clone();
         }
         /*
-         * Return a cloned version
+         * Load folder from appropriate storage
          */
-        return (Folder) folder.clone();
+        folder = loadFolder(treeId, folderId, storageType, storageParameters);
+        /*
+         * Check if folder is cacheable
+         */
+        if (folder.isCacheable()) {
+            /*
+             * Put to cache and return a cloned version
+             */
+            putFolder(folder, treeId, storageParameters);
+            return (Folder) folder.clone();
+        }
+        /*
+         * Return as-is since not cached
+         */
+        return folder;
     }
 
     public FolderType getFolderType() {
@@ -380,12 +530,30 @@ public final class CacheFolderStorage implements FolderStorage {
              * Get needed storages
              */
             final FolderStorage[] neededStorages = registry.getFolderStoragesForParent(treeId, parentId);
+            if (0 == neededStorages.length) {
+                return new SortableId[0];
+            }
             try {
-                final java.util.List<SortableId> allSubfolderIds = new ArrayList<SortableId>(neededStorages.length * 8);
-                {
+                final java.util.List<SortableId> allSubfolderIds;
+                if (1 == neededStorages.length) {
+                    final FolderStorage neededStorage = neededStorages[0];
+                    final boolean started = neededStorage.startTransaction(storageParameters, false);
+                    try {
+                        allSubfolderIds = Arrays.asList(neededStorage.getSubfolders(treeId, parentId, storageParameters));
+                        if (started) {
+                            neededStorage.commitTransaction(storageParameters);
+                        }
+                    } catch (final Exception e) {
+                        if (started) {
+                            neededStorage.rollback(storageParameters);
+                        }
+                        throw e;
+                    }
+                } else {
+                    allSubfolderIds = new ArrayList<SortableId>(neededStorages.length * 8);
                     final CompletionService<java.util.List<SortableId>> completionService =
-                        new ExecutorCompletionService<java.util.List<SortableId>>(CacheServiceRegistry.getServiceRegistry().getService(
-                            ThreadPoolService.class).getExecutor());
+                        new ThreadPoolCompletionService<java.util.List<SortableId>>(CacheServiceRegistry.getServiceRegistry().getService(
+                            ThreadPoolService.class));
                     /*
                      * Get all visible subfolders from each storage
                      */
@@ -394,14 +562,18 @@ public final class CacheFolderStorage implements FolderStorage {
 
                             public java.util.List<SortableId> call() throws Exception {
                                 final StorageParameters newParameters = newStorageParameters(storageParameters);
-                                neededStorage.startTransaction(newParameters, false);
+                                final boolean started = neededStorage.startTransaction(newParameters, false);
                                 try {
                                     final java.util.List<SortableId> l =
                                         Arrays.asList(neededStorage.getSubfolders(treeId, parentId, newParameters));
-                                    neededStorage.commitTransaction(newParameters);
+                                    if (started) {
+                                        neededStorage.commitTransaction(newParameters);
+                                    }
                                     return l;
                                 } catch (final Exception e) {
-                                    neededStorage.rollback(newParameters);
+                                    if (started) {
+                                        neededStorage.rollback(newParameters);
+                                    }
                                     throw e;
                                 }
                             }
@@ -410,32 +582,10 @@ public final class CacheFolderStorage implements FolderStorage {
                     /*
                      * Wait for completion
                      */
-                    final int maxRunningMillis = getMaxRunningMillis();
-                    try {
-                        for (int i = neededStorages.length; i > 0; i--) {
-                            final Future<java.util.List<SortableId>> f = completionService.poll(maxRunningMillis, TimeUnit.MILLISECONDS);
-                            if (null != f) {
-                                allSubfolderIds.addAll(f.get());
-                            } else if (LOG.isWarnEnabled()) {
-                                LOG.warn("Completion service's task did not complete in a timely manner!");
-                            }
-                        }
-                    } catch (final InterruptedException e) {
-                        Thread.currentThread().interrupt();
-                        throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
-                    } catch (final ExecutionException e) {
-                        final Throwable t = e.getCause();
-                        if (FolderException.class.isAssignableFrom(t.getClass())) {
-                            throw (FolderException) t;
-                        } else if (AbstractOXException.class.isAssignableFrom(t.getClass())) {
-                            throw new FolderException((AbstractOXException) t);
-                        } else if (t instanceof RuntimeException) {
-                            throw (RuntimeException) t;
-                        } else if (t instanceof Error) {
-                            throw (Error) t;
-                        } else {
-                            throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(t, t.getMessage());
-                        }
+                    final List<List<SortableId>> results =
+                        ThreadPools.pollCompletionService(completionService, neededStorages.length, getMaxRunningMillis(), FACTORY);
+                    for (final List<SortableId> result : results) {
+                        allSubfolderIds.addAll(result);
                     }
                 }
                 /*
@@ -465,8 +615,8 @@ public final class CacheFolderStorage implements FolderStorage {
         // Nothing to do
     }
 
-    public StorageParameters startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
-        return parameters;
+    public boolean startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
+        return false;
     }
 
     public void updateFolder(final Folder folder, final StorageParameters storageParameters) throws FolderException {
@@ -474,20 +624,21 @@ public final class CacheFolderStorage implements FolderStorage {
         /*
          * Perform update operation via non-cache storage
          */
+        final String parentID = folder.getParentID();
         final String oldParentId;
-        if (null != folder.getParentID()) {
+        if (null != parentID) {
             // Move
             oldParentId = getFolder(folder.getTreeID(), folder.getID(), storageParameters).getParentID();
         } else {
             oldParentId = null;
         }
         if (null == session) {
-            new Update(storageParameters.getUser(), storageParameters.getContext(), registry).doUpdate(
+            new UpdatePerformer(storageParameters.getUser(), storageParameters.getContext(), registry).doUpdate(
                 folder,
                 storageParameters.getTimeStamp());
         } else {
             try {
-                new Update(new ServerSessionAdapter(session), registry).doUpdate(folder, storageParameters.getTimeStamp());
+                new UpdatePerformer(new ServerSessionAdapter(session), registry).doUpdate(folder, storageParameters.getTimeStamp());
             } catch (final ContextException e) {
                 throw new FolderException(e);
             }
@@ -505,21 +656,14 @@ public final class CacheFolderStorage implements FolderStorage {
          * Refresh/Invalidate folder
          */
         removeFolder(folderId, treeId, storageParameters);
-        final Folder updatedFolder = loadFolder(treeId, folderId, StorageType.WORKING, storageParameters);
-        putFolder(updatedFolder, treeId, storageParameters);
         /*
          * Refresh/invalidate parent(s)
          */
-        final String parentID = updatedFolder.getParentID();
         if (!FolderStorage.ROOT_ID.equals(parentID)) {
             removeFolder(parentID, treeId, storageParameters);
-            final Folder parentFolder = loadFolder(treeId, parentID, StorageType.WORKING, storageParameters);
-            putFolder(parentFolder, treeId, storageParameters);
         }
         if (null != oldParentId && !FolderStorage.ROOT_ID.equals(oldParentId)) {
             removeFolder(oldParentId, treeId, storageParameters);
-            final Folder oldParentFolder = loadFolder(treeId, oldParentId, StorageType.WORKING, storageParameters);
-            putFolder(oldParentFolder, treeId, storageParameters);
         }
     }
 
@@ -544,15 +688,19 @@ public final class CacheFolderStorage implements FolderStorage {
         final boolean ignoreDelete = index == 0;
         if (null == session) {
             folders =
-                new Updates(storageParameters.getUser(), storageParameters.getContext(), storageParameters.getDecorator(), registry).doUpdates(
-                    treeId,
-                    timeStamp,
-                    ignoreDelete,
-                    includeContentTypes)[index];
+                new UpdatesPerformer(
+                    storageParameters.getUser(),
+                    storageParameters.getContext(),
+                    storageParameters.getDecorator(),
+                    registry).doUpdates(treeId, timeStamp, ignoreDelete, includeContentTypes)[index];
         } else {
             try {
                 folders =
-                    new Updates(new ServerSessionAdapter(session), storageParameters.getDecorator(), registry).doUpdates(treeId, timeStamp, ignoreDelete, includeContentTypes)[index];
+                    new UpdatesPerformer(new ServerSessionAdapter(session), storageParameters.getDecorator(), registry).doUpdates(
+                        treeId,
+                        timeStamp,
+                        ignoreDelete,
+                        includeContentTypes)[index];
             } catch (final ContextException e) {
                 throw new FolderException(e);
             }
@@ -572,16 +720,22 @@ public final class CacheFolderStorage implements FolderStorage {
         if (null == storage) {
             throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
         }
-        storage.startTransaction(storageParameters, true);
+        final boolean started = storage.startTransaction(storageParameters, true);
         try {
             final boolean contains = storage.containsFolder(treeId, folderId, storageType, storageParameters);
-            storage.commitTransaction(storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
             return contains;
         } catch (final FolderException e) {
-            storage.rollback(storageParameters);
+            if (started) {
+                storage.rollback(storageParameters);
+            }
             throw e;
         } catch (final Exception e) {
-            storage.rollback(storageParameters);
+            if (started) {
+                storage.rollback(storageParameters);
+            }
             throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
@@ -609,16 +763,22 @@ public final class CacheFolderStorage implements FolderStorage {
         if (null == storage) {
             throw FolderExceptionErrorMessage.NO_STORAGE_FOR_ID.create(treeId, folderId);
         }
-        storage.startTransaction(storageParameters, true);
+        final boolean started = storage.startTransaction(storageParameters, false);
         try {
             final Folder folder = storage.getFolder(treeId, folderId, storageType, storageParameters);
-            storage.commitTransaction(storageParameters);
+            if (started) {
+                storage.commitTransaction(storageParameters);
+            }
             return folder;
         } catch (final FolderException e) {
-            storage.rollback(storageParameters);
+            if (started) {
+                storage.rollback(storageParameters);
+            }
             throw e;
         } catch (final Exception e) {
-            storage.rollback(storageParameters);
+            if (started) {
+                storage.rollback(storageParameters);
+            }
             throw FolderExceptionErrorMessage.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
