@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -139,6 +139,8 @@ public class DatabaseFolder extends AbstractFolder {
             this.lastModified = null == d ? null : new Date(d.getTime());
         }
         this.subscribed = true;
+        deefault = folderObject.isDefaultFolder();
+        defaultType = deefault ? contentType.getModule() : 0;
     }
 
     @Override
