@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -156,7 +156,12 @@ public final class DatabaseFolderStorage implements FolderStorage {
             con = getParameter(Connection.class, DatabaseParameterConstants.PARAM_CONNECTION, params);
             writable = getParameter(Boolean.class, DatabaseParameterConstants.PARAM_WRITABLE, params);
         } catch (final FolderException e) {
-            LOG.error(e.getMessage(), e);
+            /*
+             * Already committed
+             */
+            if (LOG.isWarnEnabled()) {
+                LOG.warn("Storage already committed:\n" + params.getCommittedTrace(), e);
+            }
             return;
         }
         try {
@@ -176,6 +181,7 @@ public final class DatabaseFolderStorage implements FolderStorage {
             final FolderType folderType = getFolderType();
             params.putParameter(folderType, DatabaseParameterConstants.PARAM_CONNECTION, null);
             params.putParameter(folderType, DatabaseParameterConstants.PARAM_WRITABLE, null);
+            params.markCommitted();
         }
     }
 
@@ -307,6 +313,8 @@ public final class DatabaseFolderStorage implements FolderStorage {
                 throw FolderExceptionErrorMessage.MISSING_SESSION.create(new Object[0]);
             }
             final OXFolderManager folderManager = OXFolderManager.getInstance(session, con, con);
+            /*-
+             * TODO: Perform last-modified check?
             {
                 final Date clientLastModified = storageParameters.getTimeStamp();
                 if (null != clientLastModified && getFolderAccess(storageParameters, getFolderType()).getFolderLastModified(folderId).after(
@@ -314,6 +322,8 @@ public final class DatabaseFolderStorage implements FolderStorage {
                     throw FolderExceptionErrorMessage.CONCURRENT_MODIFICATION.create();
                 }
             }
+             * 
+             */
             folderManager.deleteFolder(fo, true, System.currentTimeMillis());
         } catch (final OXFolderException e) {
             throw new FolderException(e);
@@ -350,6 +360,124 @@ public final class DatabaseFolderStorage implements FolderStorage {
         }
     }
 
+    public boolean containsForeignObjects(final User user, final String treeId, final String folderIdentifier, final StorageParameters storageParameters) throws FolderException {
+        try {
+            final Connection con = getParameter(Connection.class, DatabaseParameterConstants.PARAM_CONNECTION, storageParameters);
+            final Context ctx = storageParameters.getContext();
+            /*
+             * A numeric folder identifier
+             */
+            final int folderId = getUnsignedInteger(folderIdentifier);
+            if (folderId < 0) {
+                throw new OXFolderNotFoundException(folderIdentifier, ctx);
+            }
+            if (FolderObject.SYSTEM_ROOT_FOLDER_ID == folderId) {
+                return false;
+            } else if (FolderObject.SYSTEM_SHARED_FOLDER_ID == folderId) {
+                /*
+                 * The system shared folder
+                 */
+                return false;
+            } else if (FolderObject.SYSTEM_PUBLIC_FOLDER_ID == folderId) {
+                /*
+                 * The system public folder
+                 */
+                return false;
+            } else if (FolderObject.SYSTEM_INFOSTORE_FOLDER_ID == folderId) {
+                /*
+                 * The system infostore folder
+                 */
+                return false;
+            } else if (FolderObject.SYSTEM_PRIVATE_FOLDER_ID == folderId) {
+                /*
+                 * The system private folder
+                 */
+                return false;
+            } else if (Arrays.binarySearch(VIRTUAL_IDS, folderId) >= 0) {
+                /*
+                 * A virtual database folder
+                 */
+                return true;
+            } else {
+                /*
+                 * A non-virtual database folder
+                 */
+                final FolderObject fo = FolderObject.loadFolderObjectFromDB(folderId, ctx, con);
+                return getFolderAccess(storageParameters, getFolderType()).containsForeignObjects(fo, storageParameters.getSession(), ctx);
+            }
+        } catch (final OXException e) {
+            throw new FolderException(e);
+        }
+    }
+
+    public boolean isEmpty(final String treeId, final String folderIdentifier, final StorageParameters storageParameters) throws FolderException {
+        try {
+            final Connection con = getParameter(Connection.class, DatabaseParameterConstants.PARAM_CONNECTION, storageParameters);
+            final Context ctx = storageParameters.getContext();
+            /*
+             * A numeric folder identifier
+             */
+            final int folderId = getUnsignedInteger(folderIdentifier);
+            if (folderId < 0) {
+                throw new OXFolderNotFoundException(folderIdentifier, ctx);
+            }
+            if (FolderObject.SYSTEM_ROOT_FOLDER_ID == folderId) {
+                return true;
+            } else if (FolderObject.SYSTEM_SHARED_FOLDER_ID == folderId) {
+                /*
+                 * The system shared folder
+                 */
+                return true;
+            } else if (FolderObject.SYSTEM_PUBLIC_FOLDER_ID == folderId) {
+                /*
+                 * The system public folder
+                 */
+                return true;
+            } else if (FolderObject.SYSTEM_INFOSTORE_FOLDER_ID == folderId) {
+                /*
+                 * The system infostore folder
+                 */
+                return true;
+            } else if (FolderObject.SYSTEM_PRIVATE_FOLDER_ID == folderId) {
+                /*
+                 * The system private folder
+                 */
+                return true;
+            } else if (Arrays.binarySearch(VIRTUAL_IDS, folderId) >= 0) {
+                /*
+                 * A virtual database folder
+                 */
+                return false;
+            } else {
+                /*
+                 * A non-virtual database folder
+                 */
+                final FolderObject fo = FolderObject.loadFolderObjectFromDB(folderId, ctx, con);
+                return getFolderAccess(storageParameters, getFolderType()).isEmpty(fo, storageParameters.getSession(), ctx);
+            }
+        } catch (final OXException e) {
+            throw new FolderException(e);
+        }
+    }
+
+    public void updateLastModified(final long lastModified, final String treeId, final String folderIdentifier, final StorageParameters storageParameters) throws FolderException {
+        try {
+            final Connection con = getParameter(Connection.class, DatabaseParameterConstants.PARAM_CONNECTION, storageParameters);
+            final Context ctx = storageParameters.getContext();
+            final int folderId = getUnsignedInteger(folderIdentifier);
+            if (getFolderAccess(storageParameters, getFolderType()).getFolderLastModified(folderId).after(new Date(lastModified))) {
+                throw FolderExceptionErrorMessage.CONCURRENT_MODIFICATION.create();
+            }
+            OXFolderSQL.updateLastModified(folderId, lastModified, storageParameters.getUserId(), con, ctx);
+        } catch (final DBPoolingException e) {
+            throw new FolderException(e);
+        } catch (final SQLException e) {
+            throw FolderExceptionErrorMessage.SQL_ERROR.create(e, e.getMessage());
+        } catch (final OXException e) {
+            throw new FolderException(e);
+        }
+    }
+
     public Folder getFolder(final String treeId, final String folderIdentifier, final StorageParameters storageParameters) throws FolderException {
         return getFolder(treeId, folderIdentifier, StorageType.WORKING, storageParameters);
     }
@@ -428,7 +556,7 @@ public final class DatabaseFolderStorage implements FolderStorage {
                              * owner
                              */
                             retval = new DatabaseFolder(fo);
-                            if (PrivateType.getInstance().equals(retval.getType()) && storageParameters.getUser().getId() != retval.getCreatedBy()) {
+                            if (PrivateType.getInstance().equals(retval.getType()) && storageParameters.getUserId() != retval.getCreatedBy()) {
                                 retval.setType(SharedType.getInstance());
                                 /*
                                  * A shared folder has no subfolders in real tree
@@ -673,11 +801,11 @@ public final class DatabaseFolderStorage implements FolderStorage {
         }
     }
 
-    public StorageParameters startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
+    public boolean startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
         final FolderType folderType = getFolderType();
         if (null != parameters.getParameter(folderType, DatabaseParameterConstants.PARAM_CONNECTION)) {
             // Connection already present
-            return parameters;
+            return false;
         }
         try {
             final DatabaseService databaseService = DatabaseServiceRegistry.getServiceRegistry().getService(DatabaseService.class, true);
@@ -697,7 +825,7 @@ public final class DatabaseFolderStorage implements FolderStorage {
                     databaseService.backReadOnly(context, con);
                 }
             }
-            return parameters;
+            return true;
         } catch (final ServiceException e) {
             throw new FolderException(e);
         } catch (final DBPoolingException e) {
