@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,22 +50,24 @@
 package com.openexchange.folderstorage.internal;
 
 import java.util.Date;
+import java.util.Map;
 import com.openexchange.folderstorage.ContentType;
 import com.openexchange.folderstorage.Folder;
 import com.openexchange.folderstorage.FolderException;
 import com.openexchange.folderstorage.FolderFilter;
+import com.openexchange.folderstorage.FolderResponse;
 import com.openexchange.folderstorage.FolderService;
 import com.openexchange.folderstorage.FolderServiceDecorator;
 import com.openexchange.folderstorage.UserizedFolder;
-import com.openexchange.folderstorage.internal.actions.AllVisibleFolders;
-import com.openexchange.folderstorage.internal.actions.Clear;
-import com.openexchange.folderstorage.internal.actions.Create;
-import com.openexchange.folderstorage.internal.actions.Delete;
-import com.openexchange.folderstorage.internal.actions.Get;
-import com.openexchange.folderstorage.internal.actions.List;
-import com.openexchange.folderstorage.internal.actions.Path;
-import com.openexchange.folderstorage.internal.actions.Update;
-import com.openexchange.folderstorage.internal.actions.Updates;
+import com.openexchange.folderstorage.internal.performers.AllVisibleFoldersPerformer;
+import com.openexchange.folderstorage.internal.performers.ClearPerformer;
+import com.openexchange.folderstorage.internal.performers.CreatePerformer;
+import com.openexchange.folderstorage.internal.performers.DeletePerformer;
+import com.openexchange.folderstorage.internal.performers.GetPerformer;
+import com.openexchange.folderstorage.internal.performers.ListPerformer;
+import com.openexchange.folderstorage.internal.performers.PathPerformer;
+import com.openexchange.folderstorage.internal.performers.UpdatePerformer;
+import com.openexchange.folderstorage.internal.performers.UpdatesPerformer;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.ldap.User;
@@ -73,7 +75,7 @@ import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSessionAdapter;
 
 /**
- * {@link FolderServiceImpl} - TODO Short description of this class' purpose.
+ * {@link FolderServiceImpl} - The {@link FolderService} implementation.
  * 
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
@@ -87,37 +89,37 @@ public final class FolderServiceImpl implements FolderService {
     }
 
     public void clearFolder(final String treeId, final String folderId, final User user, final Context contex) throws FolderException {
-        new Clear(user, contex).doClear(treeId, folderId);
+        new ClearPerformer(user, contex).doClear(treeId, folderId);
     }
 
     public void clearFolder(final String treeId, final String folderId, final Session session) throws FolderException {
         try {
-            new Clear(new ServerSessionAdapter(session)).doClear(treeId, folderId);
+            new ClearPerformer(new ServerSessionAdapter(session)).doClear(treeId, folderId);
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
     public String createFolder(final Folder folder, final User user, final Context context) throws FolderException {
-        return new Create(user, context).doCreate(folder);
+        return new CreatePerformer(user, context).doCreate(folder);
     }
 
     public String createFolder(final Folder folder, final Session session) throws FolderException {
         try {
-            return new Create(new ServerSessionAdapter(session)).doCreate(folder);
+            return new CreatePerformer(new ServerSessionAdapter(session)).doCreate(folder);
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
     public void deleteFolder(final String treeId, final String folderId, final Date timeStamp, final User user, final Context context) throws FolderException {
-        new Delete(user, context).doDelete(treeId, folderId, timeStamp);
+        new DeletePerformer(user, context).doDelete(treeId, folderId, timeStamp);
 
     }
 
     public void deleteFolder(final String treeId, final String folderId, final Date timeStamp, final Session session) throws FolderException {
         try {
-            new Delete(new ServerSessionAdapter(session)).doDelete(treeId, folderId, timeStamp);
+            new DeletePerformer(new ServerSessionAdapter(session)).doDelete(treeId, folderId, timeStamp);
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
@@ -134,60 +136,72 @@ public final class FolderServiceImpl implements FolderService {
     }
 
     public UserizedFolder getFolder(final String treeId, final String folderId, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
-        return new Get(user, context, decorator).doGet(treeId, folderId);
+        return new GetPerformer(user, context, decorator).doGet(treeId, folderId);
     }
 
     public UserizedFolder getFolder(final String treeId, final String folderId, final Session session, final FolderServiceDecorator decorator) throws FolderException {
         try {
-            return new Get(new ServerSessionAdapter(session), decorator).doGet(treeId, folderId);
+            return new GetPerformer(new ServerSessionAdapter(session), decorator).doGet(treeId, folderId);
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
-    public UserizedFolder[] getAllVisibleFolders(final String treeId, final FolderFilter filter, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
-        return new AllVisibleFolders(user, context, decorator).doAllVisibleFolders(treeId, filter);
+    public FolderResponse<UserizedFolder[]> getAllVisibleFolders(final String treeId, final FolderFilter filter, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
+        final AllVisibleFoldersPerformer performer = new AllVisibleFoldersPerformer(user, context, decorator);
+        return FolderResponseImpl.newFolderResponse(performer.doAllVisibleFolders(treeId, filter), performer.getWarnings());
     }
 
-    public UserizedFolder[] getAllVisibleFolders(final String treeId, final FolderFilter filter, final Session session, final FolderServiceDecorator decorator) throws FolderException {
+    public FolderResponse<UserizedFolder[]> getAllVisibleFolders(final String treeId, final FolderFilter filter, final Session session, final FolderServiceDecorator decorator) throws FolderException {
         try {
-            return new AllVisibleFolders(new ServerSessionAdapter(session), decorator).doAllVisibleFolders(treeId, filter);
+            final AllVisibleFoldersPerformer performer = new AllVisibleFoldersPerformer(new ServerSessionAdapter(session), decorator);
+            return FolderResponseImpl.newFolderResponse(performer.doAllVisibleFolders(treeId, filter), performer.getWarnings());
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
-    public UserizedFolder[] getPath(final String treeId, final String folderId, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
-        return new Path(user, context, decorator).doPath(treeId, folderId, true);
+    public FolderResponse<UserizedFolder[]> getPath(final String treeId, final String folderId, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
+        final PathPerformer performer = new PathPerformer(user, context, decorator);
+        return FolderResponseImpl.newFolderResponse(performer.doPath(treeId, folderId, true), performer.getWarnings());
     }
 
-    public UserizedFolder[] getPath(final String treeId, final String folderId, final Session session, final FolderServiceDecorator decorator) throws FolderException {
+    public FolderResponse<UserizedFolder[]> getPath(final String treeId, final String folderId, final Session session, final FolderServiceDecorator decorator) throws FolderException {
         try {
-            return new Path(new ServerSessionAdapter(session), decorator).doPath(treeId, folderId, true);
+            final PathPerformer performer = new PathPerformer(new ServerSessionAdapter(session), decorator);
+            return FolderResponseImpl.newFolderResponse(performer.doPath(treeId, folderId, true), performer.getWarnings());
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
-    public UserizedFolder[] getSubfolders(final String treeId, final String parentId, final boolean all, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
-        return new List(user, context, decorator).doList(treeId, parentId, all);
+    public FolderResponse<UserizedFolder[]> getSubfolders(final String treeId, final String parentId, final boolean all, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
+        final ListPerformer listPerformer = new ListPerformer(user, context, decorator);
+        return FolderResponseImpl.newFolderResponse(listPerformer.doList(treeId, parentId, all), listPerformer.getWarnings());
     }
 
-    public UserizedFolder[] getSubfolders(final String treeId, final String parentId, final boolean all, final Session session, final FolderServiceDecorator decorator) throws FolderException {
+    public FolderResponse<UserizedFolder[]> getSubfolders(final String treeId, final String parentId, final boolean all, final Session session, final FolderServiceDecorator decorator) throws FolderException {
         try {
-            return new List(new ServerSessionAdapter(session), decorator).doList(treeId, parentId, all);
+            final ListPerformer listPerformer = new ListPerformer(new ServerSessionAdapter(session), decorator);
+            return FolderResponseImpl.newFolderResponse(listPerformer.doList(treeId, parentId, all), listPerformer.getWarnings());
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
-    public UserizedFolder[][] getUpdates(final String treeId, final Date timeStamp, final boolean ignoreDeleted, final ContentType[] includeContentTypes, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
-        return new Updates(user, context, decorator).doUpdates(treeId, timeStamp, ignoreDeleted, includeContentTypes);
+    public FolderResponse<UserizedFolder[][]> getUpdates(final String treeId, final Date timeStamp, final boolean ignoreDeleted, final ContentType[] includeContentTypes, final User user, final Context context, final FolderServiceDecorator decorator) throws FolderException {
+        final UpdatesPerformer performer = new UpdatesPerformer(user, context, decorator);
+        return FolderResponseImpl.newFolderResponse(
+            performer.doUpdates(treeId, timeStamp, ignoreDeleted, includeContentTypes),
+            performer.getWarnings());
     }
 
-    public UserizedFolder[][] getUpdates(final String treeId, final Date timeStamp, final boolean ignoreDeleted, final ContentType[] includeContentTypes, final Session session, final FolderServiceDecorator decorator) throws FolderException {
+    public FolderResponse<UserizedFolder[][]> getUpdates(final String treeId, final Date timeStamp, final boolean ignoreDeleted, final ContentType[] includeContentTypes, final Session session, final FolderServiceDecorator decorator) throws FolderException {
         try {
-            return new Updates(new ServerSessionAdapter(session), decorator).doUpdates(treeId, timeStamp, ignoreDeleted, includeContentTypes);
+            final UpdatesPerformer performer = new UpdatesPerformer(new ServerSessionAdapter(session), decorator);
+            return FolderResponseImpl.newFolderResponse(
+                performer.doUpdates(treeId, timeStamp, ignoreDeleted, includeContentTypes),
+                performer.getWarnings());
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
@@ -214,15 +228,19 @@ public final class FolderServiceImpl implements FolderService {
     }
 
     public void updateFolder(final Folder folder, final Date timeStamp, final User user, final Context context) throws FolderException {
-        new Update(user, context).doUpdate(folder, timeStamp);
+        new UpdatePerformer(user, context).doUpdate(folder, timeStamp);
     }
 
     public void updateFolder(final Folder folder, final Date timeStamp, final Session session) throws FolderException {
         try {
-            new Update(new ServerSessionAdapter(session)).doUpdate(folder, timeStamp);
+            new UpdatePerformer(new ServerSessionAdapter(session)).doUpdate(folder, timeStamp);
         } catch (final ContextException e) {
             throw new FolderException(e);
         }
     }
 
+    public Map<Integer, ContentType> getAvailableContentTypes() {
+        return ContentTypeRegistry.getInstance().getAvailableContentTypes();
+    }
+
 }
