@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -74,17 +74,30 @@ public final class UserizedFolderImpl implements UserizedFolder {
 
     private Locale locale;
 
+    private Boolean deefault;
+
+    private Integer defaultType;
+
     /**
      * Initializes a new {@link UserizedFolderImpl} from specified folder.
      * 
      * @param folder The underlying folder
+     * @throws IllegalArgumentException If folder is <code>null</code>
      */
     public UserizedFolderImpl(final Folder folder) {
         super();
+        if (null == folder) {
+            throw new IllegalArgumentException("Folder is null.");
+        }
         // TODO: clone folder?
         this.folder = folder;
     }
 
+    @Override
+    public String toString() {
+        return new StringBuilder(32).append("{ name=").append(folder.getName()).append(", id=").append(folder.getID()).append('}').toString();
+    }
+
     @Override
     public Object clone() {
         try {
@@ -99,6 +112,14 @@ public final class UserizedFolderImpl implements UserizedFolder {
         }
     }
 
+    public int getBits() {
+        return folder.getBits();
+    }
+
+    public void setBits(final int bits) {
+        folder.setBits(bits);
+    }
+
     public int getCreatedBy() {
         return folder.getCreatedBy();
     }
@@ -260,7 +281,7 @@ public final class UserizedFolderImpl implements UserizedFolder {
     }
 
     public boolean isDefault() {
-        return folder.isDefault();
+        return null == deefault ? folder.isDefault() : deefault.booleanValue();
     }
 
     public void setCapabilities(final int capabilities) {
@@ -268,7 +289,7 @@ public final class UserizedFolderImpl implements UserizedFolder {
     }
 
     public void setDefault(final boolean deefault) {
-        folder.setDefault(deefault);
+        this.deefault = Boolean.valueOf(deefault);
     }
 
     public void setDeleted(final int deleted) {
@@ -276,11 +297,11 @@ public final class UserizedFolderImpl implements UserizedFolder {
     }
 
     public void setDefaultType(final int defaultType) {
-        folder.setDefaultType(defaultType);
+        this.defaultType = Integer.valueOf(defaultType);
     }
 
     public int getDefaultType() {
-        return folder.getDefaultType();
+        return null == defaultType ? folder.getDefaultType() : defaultType.intValue();
     }
 
     public void setNew(final int nu) {
@@ -307,4 +328,12 @@ public final class UserizedFolderImpl implements UserizedFolder {
         this.locale = locale;
     }
 
+    public String getNewID() {
+        throw new UnsupportedOperationException("UserizedFolderImpl.getNewID()");
+    }
+
+    public void setNewID(final String newId) {
+        throw new UnsupportedOperationException("UserizedFolderImpl.setNewID()");
+    }
+
 }
