@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,10 +52,11 @@ package com.openexchange.folderstorage.mail;
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
 import static com.openexchange.mail.utils.MailFolderUtility.prepareFullname;
 import static com.openexchange.mail.utils.MailFolderUtility.prepareMailFolderParam;
+import gnu.trove.TIntObjectHashMap;
+import gnu.trove.TObjectProcedure;
 import java.text.Collator;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.Date;
@@ -65,8 +66,6 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import com.openexchange.cache.OXCachingException;
 import com.openexchange.folderstorage.ContentType;
 import com.openexchange.folderstorage.Folder;
@@ -134,16 +133,15 @@ public final class MailFolderStorage implements FolderStorage {
         super();
     }
 
-    private MailAccess<?, ?> getMailAccessForAccount(final int accountId, final Session session, final ConcurrentMap<Integer, MailAccess<?, ?>> accesses) throws FolderException {
-        final Integer key = Integer.valueOf(accountId);
-        MailAccess<?, ?> ma = accesses.get(key);
+    private MailAccess<?, ?> getMailAccessForAccount(final int accountId, final Session session, final TIntObjectHashMap<MailAccess<?, ?>> accesses) throws FolderException {
+        MailAccess<?, ?> ma = accesses.get(accountId);
         if (null == ma) {
             try {
                 ma = MailAccess.getInstance(session, accountId);
             } catch (final MailException e) {
                 throw new FolderException(e);
             }
-            final MailAccess<?, ?> prev = accesses.putIfAbsent(key, ma);
+            final MailAccess<?, ?> prev = accesses.putIfAbsent(accountId, ma);
             if (null != prev) {
                 ma = prev;
             }
@@ -182,16 +180,13 @@ public final class MailFolderStorage implements FolderStorage {
     }
 
     public void commitTransaction(final StorageParameters params) throws FolderException {
-        final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-            (ConcurrentMap<Integer, MailAccess<?, ?>>) params.getParameter(
+        final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+            (TIntObjectHashMap<MailAccess<?, ?>>) params.getParameter(
                 MailFolderType.getInstance(),
                 MailParameterConstants.PARAM_MAIL_ACCESS);
         if (null != accesses) {
             try {
-                final Collection<MailAccess<?, ?>> values = accesses.values();
-                for (final MailAccess<?, ?> mailAccess : values) {
-                    mailAccess.close(true);
-                }
+                accesses.forEachValue(ACCESS_PROCEDURE);
             } finally {
                 params.putParameter(MailFolderType.getInstance(), MailParameterConstants.PARAM_MAIL_ACCESS, null);
             }
@@ -200,8 +195,8 @@ public final class MailFolderStorage implements FolderStorage {
 
     public void createFolder(final Folder folder, final StorageParameters storageParameters) throws FolderException {
         try {
-            final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -254,8 +249,8 @@ public final class MailFolderStorage implements FolderStorage {
 
     public void clearFolder(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
         try {
-            final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -278,8 +273,8 @@ public final class MailFolderStorage implements FolderStorage {
                 MailMessageCache.getInstance().removeFolderMessages(
                     arg.getAccountId(),
                     fullname,
-                    storageParameters.getUser().getId(),
-                    storageParameters.getContext().getContextId());
+                    storageParameters.getUserId(),
+                    storageParameters.getContextId());
             } catch (final OXCachingException e) {
                 LOG.error(e.getMessage(), e);
             }
@@ -290,8 +285,8 @@ public final class MailFolderStorage implements FolderStorage {
 
     public void deleteFolder(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
         try {
-            final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -314,8 +309,8 @@ public final class MailFolderStorage implements FolderStorage {
                 MailMessageCache.getInstance().removeFolderMessages(
                     arg.getAccountId(),
                     fullname,
-                    storageParameters.getUser().getId(),
-                    storageParameters.getContext().getContextId());
+                    storageParameters.getUserId(),
+                    storageParameters.getContextId());
             } catch (final OXCachingException e) {
                 LOG.error(e.getMessage(), e);
             }
@@ -333,8 +328,8 @@ public final class MailFolderStorage implements FolderStorage {
             return prepareFullname(MailAccount.DEFAULT_ID, "INBOX");
         }
         try {
-            @SuppressWarnings("unchecked") final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            @SuppressWarnings("unchecked") final TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -364,6 +359,67 @@ public final class MailFolderStorage implements FolderStorage {
         }
     }
 
+    public boolean containsForeignObjects(final User user, final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        try {
+            @SuppressWarnings("unchecked") final TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
+                    MailFolderType.getInstance(),
+                    MailParameterConstants.PARAM_MAIL_ACCESS);
+            if (null == accesses) {
+                throw new FolderException(new MailException(MailException.Code.MISSING_PARAM, MailParameterConstants.PARAM_MAIL_ACCESS));
+            }
+
+            final FullnameArgument argument = prepareMailFolderParam(folderId);
+            final int accountId = argument.getAccountId();
+            final String fullname = argument.getFullname();
+
+            final MailAccess<?, ?> mailAccess = getMailAccessForAccount(accountId, storageParameters.getSession(), accesses);
+
+            openMailAccess(mailAccess);
+            if (!MailFolder.DEFAULT_FOLDER_ID.equals(fullname)) {
+                openMailAccess(mailAccess);
+                if (!mailAccess.getFolderStorage().exists(fullname)) {
+                    throw new MailException(MailException.Code.FOLDER_NOT_FOUND, fullname);
+                }
+            }
+            return false;
+        } catch (final MailException e) {
+            throw new FolderException(e);
+        }
+    }
+
+    public boolean isEmpty(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        try {
+            @SuppressWarnings("unchecked") final TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
+                    MailFolderType.getInstance(),
+                    MailParameterConstants.PARAM_MAIL_ACCESS);
+            if (null == accesses) {
+                throw new FolderException(new MailException(MailException.Code.MISSING_PARAM, MailParameterConstants.PARAM_MAIL_ACCESS));
+            }
+            final FullnameArgument argument = prepareMailFolderParam(folderId);
+            final int accountId = argument.getAccountId();
+            final String fullname = argument.getFullname();
+
+            final MailAccess<?, ?> mailAccess = getMailAccessForAccount(accountId, storageParameters.getSession(), accesses);
+
+            if (MailFolder.DEFAULT_FOLDER_ID.equals(fullname)) {
+                return 0 == mailAccess.getRootFolder().getMessageCount();
+            }
+            /*
+             * Non-root folder
+             */
+            openMailAccess(mailAccess);
+            return 0 == mailAccess.getFolderStorage().getFolder(fullname).getMessageCount();
+        } catch (final MailException e) {
+            throw new FolderException(e);
+        }
+    }
+
+    public void updateLastModified(final long lastModified, final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
+        // Nothing to do
+    }
+
     public Folder getFolder(final String treeId, final String folderId, final StorageParameters storageParameters) throws FolderException {
         return getFolder(treeId, folderId, StorageType.WORKING, storageParameters);
     }
@@ -373,8 +429,8 @@ public final class MailFolderStorage implements FolderStorage {
             throw FolderExceptionErrorMessage.UNSUPPORTED_STORAGE_TYPE.create(storageType);
         }
         try {
-            @SuppressWarnings("unchecked") final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            @SuppressWarnings("unchecked") final TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -407,10 +463,7 @@ public final class MailFolderStorage implements FolderStorage {
                         final MailAccountStorageService storageService =
                             MailServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
                         final MailAccount mailAccount =
-                            storageService.getMailAccount(
-                                accountId,
-                                storageParameters.getUser().getId(),
-                                storageParameters.getContext().getContextId());
+                            storageService.getMailAccount(accountId, storageParameters.getUserId(), storageParameters.getContextId());
                         if (!UnifiedINBOXManagement.PROTOCOL_UNIFIED_INBOX.equals(mailAccount.getMailProtocol())) {
                             retval.setName(mailAccount.getName());
                         }
@@ -580,8 +633,8 @@ public final class MailFolderStorage implements FolderStorage {
             }
 
             // A mail folder denoted by fullname
-            @SuppressWarnings("unchecked") final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            @SuppressWarnings("unchecked") final TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -661,23 +714,28 @@ public final class MailFolderStorage implements FolderStorage {
     }
 
     public void rollback(final StorageParameters params) {
-        final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-            (ConcurrentMap<Integer, MailAccess<?, ?>>) params.getParameter(
+        final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+            (TIntObjectHashMap<MailAccess<?, ?>>) params.getParameter(
                 MailFolderType.getInstance(),
                 MailParameterConstants.PARAM_MAIL_ACCESS);
         if (null != accesses) {
             try {
-                final Collection<MailAccess<?, ?>> values = accesses.values();
-                for (final MailAccess<?, ?> mailAccess : values) {
-                    mailAccess.close(true);
-                }
+                accesses.forEachValue(ACCESS_PROCEDURE);
             } finally {
                 params.putParameter(MailFolderType.getInstance(), MailParameterConstants.PARAM_MAIL_ACCESS, null);
             }
         }
     }
 
-    public StorageParameters startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
+    private static final TObjectProcedure<MailAccess<?, ?>> ACCESS_PROCEDURE = new TObjectProcedure<MailAccess<?, ?>>() {
+
+        public boolean execute(final MailAccess<?, ?> access) {
+            access.close(true);
+            return true;
+        }
+    };
+
+    public boolean startTransaction(final StorageParameters parameters, final boolean modify) throws FolderException {
         /*
          * Ensure session is present
          */
@@ -687,11 +745,10 @@ public final class MailFolderStorage implements FolderStorage {
         /*
          * Put map
          */
-        parameters.putParameterIfAbsent(
+        return parameters.putParameterIfAbsent(
             MailFolderType.getInstance(),
             MailParameterConstants.PARAM_MAIL_ACCESS,
-            new ConcurrentHashMap<Integer, MailAccess<?, ?>>());
-        return parameters;
+            new TIntObjectHashMap<MailAccess<?, ?>>());
     }
 
     public StoragePriority getStoragePriority() {
@@ -707,8 +764,8 @@ public final class MailFolderStorage implements FolderStorage {
             return false;
         }
         try {
-            final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -716,10 +773,17 @@ public final class MailFolderStorage implements FolderStorage {
             }
 
             final FullnameArgument argument = prepareMailFolderParam(folderId);
+            final String fullname = argument.getFullname();
+            if (MailFolder.DEFAULT_FOLDER_ID.equals(fullname)) {
+                /*
+                 * The default folder always exists
+                 */
+                return true;
+            }
             final MailAccess<?, ?> mailAccess = getMailAccessForAccount(argument.getAccountId(), storageParameters.getSession(), accesses);
             openMailAccess(mailAccess);
 
-            return mailAccess.getFolderStorage().exists(folderId);
+            return mailAccess.getFolderStorage().exists(fullname);
         } catch (final MailException e) {
             throw new FolderException(e);
         }
@@ -748,8 +812,8 @@ public final class MailFolderStorage implements FolderStorage {
 
     public void updateFolder(final Folder folder, final StorageParameters storageParameters) throws FolderException {
         try {
-            final ConcurrentMap<Integer, MailAccess<?, ?>> accesses =
-                (ConcurrentMap<Integer, MailAccess<?, ?>>) storageParameters.getParameter(
+            final @SuppressWarnings("unchecked") TIntObjectHashMap<MailAccess<?, ?>> accesses =
+                (TIntObjectHashMap<MailAccess<?, ?>>) storageParameters.getParameter(
                     MailFolderType.getInstance(),
                     MailParameterConstants.PARAM_MAIL_ACCESS);
             if (null == accesses) {
@@ -842,6 +906,7 @@ public final class MailFolderStorage implements FolderStorage {
                     }
                     if (!newParent.equals(oldParent)) { // move & rename
                         fullname = mailAccess.getFolderStorage().moveFolder(fullname, newFullname.toString());
+                        folder.setID(prepareFullname(accountId, fullname));
                         movePerformed = true;
                     }
                 } else {
@@ -873,7 +938,7 @@ public final class MailFolderStorage implements FolderStorage {
                                 otherAccess,
                                 newParent,
                                 p.getSeparator(),
-                                storageParameters.getUser().getId(),
+                                storageParameters.getUserId(),
                                 otherAccess.getMailConfig().getCapabilities().hasPermissions());
                         // Delete source
                         mailAccess.getFolderStorage().deleteFolder(fullname, true);
@@ -891,6 +956,7 @@ public final class MailFolderStorage implements FolderStorage {
                 final String newName = mfd.getName();
                 if (!newName.equals(oldName)) { // rename
                     fullname = mailAccess.getFolderStorage().renameFolder(fullname, newName);
+                    folder.setID(prepareFullname(accountId, fullname));
                 }
             }
             /*
