@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -97,11 +97,12 @@ public final class FolderStorageActivator implements BundleActivator {
     public void start(final BundleContext context) throws Exception {
         try {
             // Register error component
-            componentRegistration = new ComponentRegistration(
-                context,
-                EnumComponent.FOLDER,
-                "com.openexchange.folderstorage",
-                FolderExceptionFactory.getInstance());
+            componentRegistration =
+                new ComponentRegistration(
+                    context,
+                    EnumComponent.FOLDER,
+                    "com.openexchange.folderstorage",
+                    FolderExceptionFactory.getInstance());
             // Register services
             serviceRegistrations = new ArrayList<ServiceRegistration>(4);
             // Register folder service
@@ -121,6 +122,7 @@ public final class FolderStorageActivator implements BundleActivator {
             activators = new ArrayList<BundleActivator>(4);
             activators.add(new DatabaseFolderStorageActivator()); // Database impl
             activators.add(new MailFolderStorageActivator()); // Mail impl
+            // activators.add(new MessagingFolderStorageActivator()); // Messaging impl
             activators.add(new CacheFolderStorageActivator()); // Cache impl
             activators.add(new OutlookFolderStorageActivator()); // MS Outlook storage activator
             // activators.add(new VirtualFolderStorageActivator()); // Virtual storage activator
