@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -86,6 +86,8 @@ public final class VirtualFolder implements Folder {
 
     private boolean subscribed;
 
+    private String newId;
+
     /**
      * Initializes a {@link VirtualFolder} with specified real folder.
      * 
@@ -125,6 +127,14 @@ public final class VirtualFolder implements Folder {
         }
     }
 
+    public String getNewID() {
+        return newId;
+    }
+
+    public void setNewID(final String newId) {
+        this.newId = newId;
+    }
+
     public int getCreatedBy() {
         return realFolder.getCreatedBy();
     }
@@ -325,6 +335,14 @@ public final class VirtualFolder implements Folder {
         // Nothing to do
     }
 
+    public int getBits() {
+        return realFolder.getBits();
+    }
+
+    public void setBits(final int bits) {
+        // Nothing to do
+    }
+
     private static Date cloneDate(final Date d) {
         if (null == d) {
             return null;
