@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,9 @@
 
 package com.openexchange.groupware.contact.helpers;
 
+import com.openexchange.ajax.fields.CommonFields;
+import com.openexchange.ajax.fields.ContactFields;
+import com.openexchange.ajax.fields.DataFields;
 import com.openexchange.groupware.contact.ContactException;
 import com.openexchange.groupware.contact.mappers.ContactFieldMapper;
 import com.openexchange.groupware.contact.mappers.EnglishOutlookMapper;
@@ -66,7 +69,7 @@ import com.openexchange.groupware.contact.mappers.GermanOutlookMapper;
  * different names and IDs, then write a Switcher which will perform an
  * action (on one or more ContactObjects in most cases). 
  * 
- * The design pattern used has no widely acknowledged name (to my the best
+ * The design pattern used has no widely acknowledged name (to the best
  * of my knowledge) but is used in Hibernate under the name Switcher.
  * 
  * Note: This class was mostly generated automatically.
@@ -76,118 +79,118 @@ import com.openexchange.groupware.contact.mappers.GermanOutlookMapper;
  */
 public enum ContactField{  
 
-	DISPLAY_NAME (500 , "field01" , "DISPLAY_NAME" , "Display name"  , "display_name"),
-	SUR_NAME (502 , "field02" , "SUR_NAME" , "Sur name" , ""),
-	GIVEN_NAME (501 , "field03" , "GIVEN_NAME" , "Given name" , ""),
-	MIDDLE_NAME (503 , "field04" , "MIDDLE_NAME" , "Middle name" , ""),
-	SUFFIX (504 , "field05" , "SUFFIX" , "Suffix" , "suffix"),
-	TITLE (505 , "field06" , "TITLE" , "Title" , "title"),
-	STREET_HOME (506 , "field07" , "STREET_HOME" , "Street home" , "street_home"),
-	POSTAL_CODE_HOME (507 , "field08" , "POSTAL_CODE_HOME" , "Postal code home" , "postal_code_home"),
-	CITY_HOME (508 , "field09" , "CITY_HOME" , "City home" , "city_home"),
-	STATE_HOME (509 , "field10" , "STATE_HOME" , "State home" , "state_home"),
-	COUNTRY_HOME (510 , "field11" , "COUNTRY_HOME" , "Country home" , "country_home"),
-	MARITAL_STATUS (512 , "field12" , "MARITAL_STATUS" , "Martial status"  , "marital_status"),
-	NUMBER_OF_CHILDREN (513 , "field13" , "NUMBER_OF_CHILDREN" , "Number of children" , "number_of_children"),
-	PROFESSION (514 , "field14" , "PROFESSION" , "Profession" , "profession"),
-	NICKNAME (515 , "field15" , "NICKNAME" , "Nickname"  , "nickname"),
-	SPOUSE_NAME (516 , "field16" , "SPOUSE_NAME" , "Spouse name" , "spouse_name"),
-	NOTE (518 , "field17" , "NOTE" , "Note" , "note"),
-	COMPANY (569 , "field18" , "COMPANY" , "Company" , "company"),
-	DEPARTMENT (519 , "field19" , "DEPARTMENT" , "Department" , "department"),
-	POSITION (520 , "field20" , "POSITION" , "Position"  , "position"),
-	EMPLOYEE_TYPE (521 , "field21" , "EMPLOYEE_TYPE" , "Employee type"  , "employee_type"),
-	ROOM_NUMBER (522 , "field22" , "ROOM_NUMBER" , "Room number"  , "room_number"),
-	STREET_BUSINESS (523 , "field23" , "STREET_BUSINESS" , "Street business" , "street_business"),
-	POSTAL_CODE_BUSINESS (525 , "field24" , "POSTAL_CODE_BUSINESS" , "Postal code business" , "postal_code_business"),
-	CITY_BUSINESS (526 , "field25" , "CITY_BUSINESS" , "City business" , "city_business"),
-	STATE_BUSINESS (527 , "field26" , "STATE_BUSINESS" , "State business" , "state_business"),
-	COUNTRY_BUSINESS (528 , "field27" , "COUNTRY_BUSINESS" , "Country business" , "country_business"),
-	NUMBER_OF_EMPLOYEE (529 , "field28" , "NUMBER_OF_EMPLOYEE" , "Number of employee"  , "number_of_employees"),
-	SALES_VOLUME (530 , "field29" , "SALES_VOLUME" , "Sales volume"  , "sales_volume"),
-	TAX_ID (531 , "field30" , "TAX_ID" , "Tax id"  , "tax_id"),
-	COMMERCIAL_REGISTER (532 , "field31" , "COMMERCIAL_REGISTER" , "Commercial register"  , "commercial_register"),
-	BRANCHES (533 , "field32" , "BRANCHES" , "Branches"  , "branches"),
-	BUSINESS_CATEGORY (534 , "field33" , "BUSINESS_CATEGORY" , "Business category"  , "busines_categorie"),
-	INFO (535 , "field34" , "INFO" , "Info"  , "info"),
-	MANAGER_NAME (536 , "field35" , "MANAGER_NAME" , "Manager's name" , "manager_name"),
-	ASSISTANT_NAME (537 , "field36" , "ASSISTANT_NAME" , "Assistant's name"  , "assistant_name"),
-	STREET_OTHER (538 , "field37" , "STREET_OTHER" , "Street other" , "street_other"),
-	POSTAL_CODE_OTHER (540 , "field38" , "POSTAL_CODE_OTHER" , "Postal code other" , "postal_code_other"),
-	CITY_OTHER (539 , "field39" , "CITY_OTHER" , "City other" , "city_other"),
-	STATE_OTHER (598 , "field40" , "STATE_OTHER" , "State other" , "state_other"),
-	COUNTRY_OTHER (541 , "field41" , "COUNTRY_OTHER" , "Country other" , "country_other"),
-	TELEPHONE_ASSISTANT (568 , "field42" , "TELEPHONE_ASSISTANT" , "Telephone assistant" , "telephone_assistant"),
-	TELEPHONE_BUSINESS1 (542 , "field43" , "TELEPHONE_BUSINESS1" , "Telephone business 1" , "telephone_business1"),
-	TELEPHONE_BUSINESS2 (543 , "field44" , "TELEPHONE_BUSINESS2" , "Telephone business 2" , "telephone_business2"),
-	FAX_BUSINESS (544 , "field45" , "FAX_BUSINESS" , "FAX business" , "fax_business"),
-	TELEPHONE_CALLBACK (545 , "field46" , "TELEPHONE_CALLBACK" , "Telephone callback" , "telephone_callback"),
-	TELEPHONE_CAR (546 , "field47" , "TELEPHONE_CAR" , "Telephone car" , "telephone_car"),
-	TELEPHONE_COMPANY (547 , "field48" , "TELEPHONE_COMPANY" , "Telephone company" , "telephone_company"),
-	TELEPHONE_HOME1 (548 , "field49" , "TELEPHONE_HOME1" , "Telephone home 1" , "telephone_home1"),
-	TELEPHONE_HOME2 (549 , "field50" , "TELEPHONE_HOME2" , "Telephone home 2" , "telephone_home2"),
-	FAX_HOME (550 , "field51" , "FAX_HOME" , "FAX home" , "fax_home"),
-	TELEPHONE_ISDN (559 , "field52" , "TELEPHONE_ISDN" , "Telephone ISDN" , "telephone_isdn"),
-	CELLULAR_TELEPHONE1 (551 , "field53" , "CELLULAR_TELEPHONE1" , "Cellular telephone 1" , "cellular_telephone1"),
-	CELLULAR_TELEPHONE2 (552 , "field54" , "CELLULAR_TELEPHONE2" , "Cellular telephone 2"  , "cellular_telephone2"),
-	TELEPHONE_OTHER (553 , "field55" , "TELEPHONE_OTHER" , "Telephone other" , "telephone_other"),
-	FAX_OTHER (554 , "field56" , "FAX_OTHER" , "FAX other" , "fax_other"),
-	TELEPHONE_PAGER (560 , "field57" , "TELEPHONE_PAGER" , "Telephone pager" , "telephone_pager"),
-	TELEPHONE_PRIMARY (561 , "field58" , "TELEPHONE_PRIMARY" , "Telephone primary" , "telephone_primary"),
-	TELEPHONE_RADIO (562 , "field59" , "TELEPHONE_RADIO" , "Telephone radio" , "telephone_radio"),
-	TELEPHONE_TELEX (563 , "field60" , "TELEPHONE_TELEX" , "Telephone telex" , "telephone_telex"),
-	TELEPHONE_TTYTDD (564 , "field61" , "TELEPHONE_TTYTDD" , "Telephone TTY/TDD" , "telephone_ttytdd"),
-	INSTANT_MESSENGER1 (565 , "field62" , "INSTANT_MESSENGER1" , "Instantmessenger 1"  , "instant_messenger1"),
-	INSTANT_MESSENGER2 (566 , "field63" , "INSTANT_MESSENGER2" , "Instantmessenger 2"  , "instant_messenger2"),
-	TELEPHONE_IP (567 , "field64" , "TELEPHONE_IP" , "Telephone IP"  , "telephone_ip"),
-	EMAIL1 (555 , "field65" , "EMAIL1" , "Email 1" , "email1"),
-	EMAIL2 (556 , "field66" , "EMAIL2" , "Email 2" , "email2"),
-	EMAIL3 (557 , "field67" , "EMAIL3" , "Email 3" , "email3"),
-	URL (558 , "field68" , "URL" , "URL"  , "url"),
-	CATEGORIES (100 , "field69" , "CATEGORIES" , "Categories"  , ""),
-	USERFIELD01 (571 , "field70" , "USERFIELD01" , "Dynamic Field 1" , "userfield01"),
-	USERFIELD02 (572 , "field71" , "USERFIELD02" , "Dynamic Field 2" , "userfield02"),
-	USERFIELD03 (573 , "field72" , "USERFIELD03" , "Dynamic Field 3" , "userfield03"),
-	USERFIELD04 (574 , "field73" , "USERFIELD04" , "Dynamic Field 4" , "userfield04"),
-	USERFIELD05 (575 , "field74" , "USERFIELD05" , "Dynamic Field 5"  , "userfield05"),
-	USERFIELD06 (576 , "field75" , "USERFIELD06" , "Dynamic Field 6"  , "userfield06"),
-	USERFIELD07 (577 , "field76" , "USERFIELD07" , "Dynamic Field 7"  , "userfield07"),
-	USERFIELD08 (578 , "field77" , "USERFIELD08" , "Dynamic Field 8"  , "userfield08"),
-	USERFIELD09 (579 , "field78" , "USERFIELD09" , "Dynamic Field 9"  , "userfield09"),
-	USERFIELD10 (580 , "field79" , "USERFIELD10" , "Dynamic Field 10"  , "userfield10"),
-	USERFIELD11 (581 , "field80" , "USERFIELD11" , "Dynamic Field 11"  , "userfield11"),
-	USERFIELD12 (582 , "field81" , "USERFIELD12" , "Dynamic Field 12"  , "userfield12"),
-	USERFIELD13 (583 , "field82" , "USERFIELD13" , "Dynamic Field 13"  , "userfield13"),
-	USERFIELD14 (584 , "field83" , "USERFIELD14" , "Dynamic Field 14"  , "userfield14"),
-	USERFIELD15 (585 , "field84" , "USERFIELD15" , "Dynamic Field 15"  , "userfield15"),
-	USERFIELD16 (586 , "field85" , "USERFIELD16" , "Dynamic Field 16"  , "userfield16"),
-	USERFIELD17 (587 , "field86" , "USERFIELD17" , "Dynamic Field 17"  , "userfield17"),
-	USERFIELD18 (588 , "field87" , "USERFIELD18" , "Dynamic Field 18"  , "userfield18"),
-	USERFIELD19 (589 , "field88" , "USERFIELD19" , "Dynamic Field 19"  , "userfield19"),
-	USERFIELD20 (590 , "field89" , "USERFIELD20" , "Dynamic Field 20"  , "userfield20"),
-	OBJECT_ID (1 , "intfield01" , "OBJECT_ID" , "Object id"  , ""),
-	NUMBER_OF_DISTRIBUTIONLIST (594 , "intfield02" , "NUMBER_OF_DISTRIBUTIONLIST" , "Number of distributionlists"  , "number_of_distribution_list"),
-	NUMBER_OF_LINKS (103 , "intfield03" , "NUMBER_OF_LINKS" , "Number of links"  , "number_of_links"),
-	DISTRIBUTIONLIST (592 , "intfield02" , "DISTRIBUTIONLIST" , ""  , "distribution_list"),
-	LINKS (591 , "intfield03" , "LINKS" , ""  , "links"),
-	FOLDER_ID (20 , "fid" , "FOLDER_ID" , "Folder id"  , ""),
-	CONTEXTID (593 , "cid" , "CONTEXTID" , "Context id"  , ""),
-	PRIVATE_FLAG (101 , "pflag" , "PRIVATE_FLAG" , "private"  , "private"),
-	CREATED_BY (2 , "created_from" , "CREATED_BY" , "Created by"  , ""),
-	MODIFIED_BY (3 , "changed_from" , "MODIFIED_BY" , "Modified by"  , ""),
-	CREATION_DATE (4 , "creating_date" , "CREATION_DATE" , "Creation date"  , ""),
-	LAST_MODIFIED (5 , "changing_date" , "LAST_MODIFIED" , "Changing date"  , ""),
-	BIRTHDAY (511 , "timestampfield01" , "BIRTHDAY" , "Birthday" , "birthday"),
-	ANNIVERSARY (517 , "timestampfield02" , "ANNIVERSARY" , "Anniversay" , "anniversary"),
-	IMAGE1 (570 , "intfield04" , "IMAGE1" , ""  , "image1"),
-	IMAGE_LAST_MODIFIED (597 , "intfield04" , "IMAGE_LAST_MODIFIED" , ""  , ""),
-	IMAGE1_CONTENT_TYPE (601 , "intfield04" , "IMAGE1_CONTENT_TYPE" , ""  , ""),
-	INTERNAL_USERID (524 , "userid" , "INTERNAL_USERID" , ""  , ""),
-	COLOR_LABEL (102 , "intfield05" , "COLOR_LABEL" , ""  , ""),
-	FILE_AS (599 , "field90" , "FILE_AS" , ""  , ""),
-	DEFAULT_ADDRESS (605 , "intfield06" , "DEFAULT_ADDRESS" , "Default address"  , "default_address"),
-	MARK_AS_DISTRIBUTIONLIST (602 , "intfield07" , "MARK_AS_DISTRIBUTIONLIST" , ""  , ""),
-	NUMBER_OF_ATTACHMENTS (104 , "intfield08" , "NUMBER_OF_ATTACHMENTS" , ""  , "");
+    DISPLAY_NAME (500 , "field01" , "DISPLAY_NAME" , "Display name"  , ContactFields.DISPLAY_NAME),
+    SUR_NAME (502 , "field02" , "SUR_NAME" , "Sur name" , ContactFields.LAST_NAME),
+    GIVEN_NAME (501 , "field03" , "GIVEN_NAME" , "Given name" , ContactFields.FIRST_NAME),
+    MIDDLE_NAME (503 , "field04" , "MIDDLE_NAME" , "Middle name" , ContactFields.SECOND_NAME),
+    SUFFIX (504 , "field05" , "SUFFIX" , "Suffix" , ContactFields.SUFFIX),
+    TITLE (505 , "field06" , "TITLE" , "Title" , ContactFields.TITLE),
+    STREET_HOME (506 , "field07" , "STREET_HOME" , "Street home" , ContactFields.STREET_HOME),
+    POSTAL_CODE_HOME (507 , "field08" , "POSTAL_CODE_HOME" , "Postal code home" , ContactFields.POSTAL_CODE_HOME),
+    CITY_HOME (508 , "field09" , "CITY_HOME" , "City home" , ContactFields.CITY_HOME),
+    STATE_HOME (509 , "field10" , "STATE_HOME" , "State home" , ContactFields.STATE_HOME),
+    COUNTRY_HOME (510 , "field11" , "COUNTRY_HOME" , "Country home" , ContactFields.COUNTRY_HOME),
+    MARITAL_STATUS (512 , "field12" , "MARITAL_STATUS" , "Martial status"  , ContactFields.MARITAL_STATUS),
+    NUMBER_OF_CHILDREN (513 , "field13" , "NUMBER_OF_CHILDREN" , "Number of children" , ContactFields.NUMBER_OF_CHILDREN),
+    PROFESSION (514 , "field14" , "PROFESSION" , "Profession" , ContactFields.PROFESSION),
+    NICKNAME (515 , "field15" , "NICKNAME" , "Nickname"  , ContactFields.NICKNAME),
+    SPOUSE_NAME (516 , "field16" , "SPOUSE_NAME" , "Spouse name" , ContactFields.SPOUSE_NAME),
+    NOTE (518 , "field17" , "NOTE" , "Note" , ContactFields.NOTE),
+    COMPANY (569 , "field18" , "COMPANY" , "Company" , ContactFields.COMPANY),
+    DEPARTMENT (519 , "field19" , "DEPARTMENT" , "Department" , ContactFields.DEPARTMENT),
+    POSITION (520 , "field20" , "POSITION" , "Position"  , ContactFields.POSITION),
+    EMPLOYEE_TYPE (521 , "field21" , "EMPLOYEE_TYPE" , "Employee type"  , ContactFields.EMPLOYEE_TYPE),
+    ROOM_NUMBER (522 , "field22" , "ROOM_NUMBER" , "Room number"  , ContactFields.ROOM_NUMBER),
+    STREET_BUSINESS (523 , "field23" , "STREET_BUSINESS" , "Street business" , ContactFields.STREET_BUSINESS),
+    POSTAL_CODE_BUSINESS (525 , "field24" , "POSTAL_CODE_BUSINESS" , "Postal code business" , ContactFields.POSTAL_CODE_BUSINESS),
+    CITY_BUSINESS (526 , "field25" , "CITY_BUSINESS" , "City business" , ContactFields.CITY_BUSINESS),
+    STATE_BUSINESS (527 , "field26" , "STATE_BUSINESS" , "State business" , ContactFields.STATE_BUSINESS),
+    COUNTRY_BUSINESS (528 , "field27" , "COUNTRY_BUSINESS" , "Country business" , ContactFields.COUNTRY_BUSINESS),
+    NUMBER_OF_EMPLOYEE (529 , "field28" , "NUMBER_OF_EMPLOYEE" , "Number of employee"  , ContactFields.NUMBER_OF_EMPLOYEE),
+    SALES_VOLUME (530 , "field29" , "SALES_VOLUME" , "Sales volume"  , ContactFields.SALES_VOLUME),
+    TAX_ID (531 , "field30" , "TAX_ID" , "Tax id"  , ContactFields.TAX_ID),
+    COMMERCIAL_REGISTER (532 , "field31" , "COMMERCIAL_REGISTER" , "Commercial register"  , ContactFields.COMMERCIAL_REGISTER),
+    BRANCHES (533 , "field32" , "BRANCHES" , "Branches"  , ContactFields.BRANCHES),
+    BUSINESS_CATEGORY (534 , "field33" , "BUSINESS_CATEGORY" , "Business category"  , ContactFields.BUSINESS_CATEGORY),
+    INFO (535 , "field34" , "INFO" , "Info"  , ContactFields.INFO),
+    MANAGER_NAME (536 , "field35" , "MANAGER_NAME" , "Manager's name" , ContactFields.MANAGER_NAME),
+    ASSISTANT_NAME (537 , "field36" , "ASSISTANT_NAME" , "Assistant's name"  , ContactFields.ASSISTANT_NAME),
+    STREET_OTHER (538 , "field37" , "STREET_OTHER" , "Street other" , ContactFields.STREET_OTHER),
+    POSTAL_CODE_OTHER (540 , "field38" , "POSTAL_CODE_OTHER" , "Postal code other" , ContactFields.POSTAL_CODE_OTHER),
+    CITY_OTHER (539 , "field39" , "CITY_OTHER" , "City other" , ContactFields.CITY_OTHER),
+    STATE_OTHER (598 , "field40" , "STATE_OTHER" , "State other" , ContactFields.STATE_OTHER),
+    COUNTRY_OTHER (541 , "field41" , "COUNTRY_OTHER" , "Country other" , ContactFields.COUNTRY_OTHER),
+    TELEPHONE_ASSISTANT (568 , "field42" , "TELEPHONE_ASSISTANT" , "Telephone assistant" , ContactFields.TELEPHONE_ASSISTANT),
+    TELEPHONE_BUSINESS1 (542 , "field43" , "TELEPHONE_BUSINESS1" , "Telephone business 1" , ContactFields.TELEPHONE_BUSINESS1),
+    TELEPHONE_BUSINESS2 (543 , "field44" , "TELEPHONE_BUSINESS2" , "Telephone business 2" , ContactFields.TELEPHONE_BUSINESS2),
+    FAX_BUSINESS (544 , "field45" , "FAX_BUSINESS" , "FAX business" , ContactFields.FAX_BUSINESS),
+    TELEPHONE_CALLBACK (545 , "field46" , "TELEPHONE_CALLBACK" , "Telephone callback" , ContactFields.TELEPHONE_CALLBACK),
+    TELEPHONE_CAR (546 , "field47" , "TELEPHONE_CAR" , "Telephone car" , ContactFields.TELEPHONE_CAR),
+    TELEPHONE_COMPANY (547 , "field48" , "TELEPHONE_COMPANY" , "Telephone company" , ContactFields.TELEPHONE_COMPANY),
+    TELEPHONE_HOME1 (548 , "field49" , "TELEPHONE_HOME1" , "Telephone home 1" , ContactFields.TELEPHONE_HOME1),
+    TELEPHONE_HOME2 (549 , "field50" , "TELEPHONE_HOME2" , "Telephone home 2" , ContactFields.TELEPHONE_HOME2),
+    FAX_HOME (550 , "field51" , "FAX_HOME" , "FAX home" , ContactFields.FAX_HOME),
+    TELEPHONE_ISDN (559 , "field52" , "TELEPHONE_ISDN" , "Telephone ISDN" , ContactFields.TELEPHONE_ISDN),
+    CELLULAR_TELEPHONE1 (551 , "field53" , "CELLULAR_TELEPHONE1" , "Cellular telephone 1" , ContactFields.CELLULAR_TELEPHONE1),
+    CELLULAR_TELEPHONE2 (552 , "field54" , "CELLULAR_TELEPHONE2" , "Cellular telephone 2"  , ContactFields.CELLULAR_TELEPHONE2),
+    TELEPHONE_OTHER (553 , "field55" , "TELEPHONE_OTHER" , "Telephone other" , ContactFields.TELEPHONE_OTHER),
+    FAX_OTHER (554 , "field56" , "FAX_OTHER" , "FAX other" , ContactFields.FAX_OTHER),
+    TELEPHONE_PAGER (560 , "field57" , "TELEPHONE_PAGER" , "Telephone pager" , ContactFields.TELEPHONE_PAGER),
+    TELEPHONE_PRIMARY (561 , "field58" , "TELEPHONE_PRIMARY" , "Telephone primary" , ContactFields.TELEPHONE_PRIMARY),
+    TELEPHONE_RADIO (562 , "field59" , "TELEPHONE_RADIO" , "Telephone radio" , ContactFields.TELEPHONE_RADIO),
+    TELEPHONE_TELEX (563 , "field60" , "TELEPHONE_TELEX" , "Telephone telex" , ContactFields.TELEPHONE_TELEX),
+    TELEPHONE_TTYTDD (564 , "field61" , "TELEPHONE_TTYTDD" , "Telephone TTY/TDD" , ContactFields.TELEPHONE_TTYTDD),
+    INSTANT_MESSENGER1 (565 , "field62" , "INSTANT_MESSENGER1" , "Instantmessenger 1"  , ContactFields.INSTANT_MESSENGER1),
+    INSTANT_MESSENGER2 (566 , "field63" , "INSTANT_MESSENGER2" , "Instantmessenger 2"  , ContactFields.INSTANT_MESSENGER2),
+    TELEPHONE_IP (567 , "field64" , "TELEPHONE_IP" , "Telephone IP"  , ContactFields.TELEPHONE_IP),
+    EMAIL1 (555 , "field65" , "EMAIL1" , "Email 1" , ContactFields.EMAIL1),
+    EMAIL2 (556 , "field66" , "EMAIL2" , "Email 2" , ContactFields.EMAIL2),
+    EMAIL3 (557 , "field67" , "EMAIL3" , "Email 3" , ContactFields.EMAIL3),
+    URL (558 , "field68" , "URL" , "URL"  , ContactFields.URL),
+    CATEGORIES (100 , "field69" , "CATEGORIES" , "Categories"  , ContactFields.CATEGORIES),
+    USERFIELD01 (571 , "field70" , "USERFIELD01" , "Dynamic Field 1" , ContactFields.USERFIELD01),
+    USERFIELD02 (572 , "field71" , "USERFIELD02" , "Dynamic Field 2" , ContactFields.USERFIELD02),
+    USERFIELD03 (573 , "field72" , "USERFIELD03" , "Dynamic Field 3" , ContactFields.USERFIELD03),
+    USERFIELD04 (574 , "field73" , "USERFIELD04" , "Dynamic Field 4" , ContactFields.USERFIELD04),
+    USERFIELD05 (575 , "field74" , "USERFIELD05" , "Dynamic Field 5"  , ContactFields.USERFIELD05),
+    USERFIELD06 (576 , "field75" , "USERFIELD06" , "Dynamic Field 6"  , ContactFields.USERFIELD06),
+    USERFIELD07 (577 , "field76" , "USERFIELD07" , "Dynamic Field 7"  , ContactFields.USERFIELD07),
+    USERFIELD08 (578 , "field77" , "USERFIELD08" , "Dynamic Field 8"  , ContactFields.USERFIELD08),
+    USERFIELD09 (579 , "field78" , "USERFIELD09" , "Dynamic Field 9"  , ContactFields.USERFIELD09),
+    USERFIELD10 (580 , "field79" , "USERFIELD10" , "Dynamic Field 10"  , ContactFields.USERFIELD10),
+    USERFIELD11 (581 , "field80" , "USERFIELD11" , "Dynamic Field 11"  , ContactFields.USERFIELD11),
+    USERFIELD12 (582 , "field81" , "USERFIELD12" , "Dynamic Field 12"  , ContactFields.USERFIELD12),
+    USERFIELD13 (583 , "field82" , "USERFIELD13" , "Dynamic Field 13"  , ContactFields.USERFIELD13),
+    USERFIELD14 (584 , "field83" , "USERFIELD14" , "Dynamic Field 14"  , ContactFields.USERFIELD14),
+    USERFIELD15 (585 , "field84" , "USERFIELD15" , "Dynamic Field 15"  , ContactFields.USERFIELD15),
+    USERFIELD16 (586 , "field85" , "USERFIELD16" , "Dynamic Field 16"  , ContactFields.USERFIELD16),
+    USERFIELD17 (587 , "field86" , "USERFIELD17" , "Dynamic Field 17"  , ContactFields.USERFIELD17),
+    USERFIELD18 (588 , "field87" , "USERFIELD18" , "Dynamic Field 18"  , ContactFields.USERFIELD18),
+    USERFIELD19 (589 , "field88" , "USERFIELD19" , "Dynamic Field 19"  , ContactFields.USERFIELD19),
+    USERFIELD20 (590 , "field89" , "USERFIELD20" , "Dynamic Field 20"  , ContactFields.USERFIELD20),
+    OBJECT_ID (1 , "intfield01" , "OBJECT_ID" , "Object id"  , DataFields.ID),
+    NUMBER_OF_DISTRIBUTIONLIST (594 , "intfield02" , "NUMBER_OF_DISTRIBUTIONLIST" , "Number of distributionlists"  , ContactFields.NUMBER_OF_DISTRIBUTIONLIST),
+    NUMBER_OF_LINKS (103 , "intfield03" , "NUMBER_OF_LINKS" , "Number of links"  , ContactFields.NUMBER_OF_LINKS),
+    DISTRIBUTIONLIST (592 , "intfield02" , "DISTRIBUTIONLIST" , ""  , ContactFields.DISTRIBUTIONLIST),
+    LINKS (591 , "intfield03" , "LINKS" , ""  , ContactFields.LINKS),
+    FOLDER_ID (20 , "fid" , "FOLDER_ID" , "Folder id"  , ContactFields.FOLDER_ID),
+    CONTEXTID (593 , "cid" , "CONTEXTID" , "Context id"  , ""),
+    PRIVATE_FLAG (101 , "pflag" , "PRIVATE_FLAG" , "private"  , ContactFields.PRIVATE_FLAG),
+    CREATED_BY (2 , "created_from" , "CREATED_BY" , "Created by"  , ContactFields.CREATED_BY),
+    MODIFIED_BY (3 , "changed_from" , "MODIFIED_BY" , "Modified by"  , ContactFields.MODIFIED_BY),
+    CREATION_DATE (4 , "creating_date" , "CREATION_DATE" , "Creation date"  , ContactFields.CREATION_DATE),
+    LAST_MODIFIED (5 , "changing_date" , "LAST_MODIFIED" , "Changing date"  , ContactFields.LAST_MODIFIED),
+    BIRTHDAY (511 , "timestampfield01" , "BIRTHDAY" , "Birthday" , ContactFields.BIRTHDAY),
+    ANNIVERSARY (517 , "timestampfield02" , "ANNIVERSARY" , "Anniversay" , ContactFields.ANNIVERSARY),
+    IMAGE1 (570 , "intfield04" , "IMAGE1" , ""  , ContactFields.IMAGE1),
+    IMAGE_LAST_MODIFIED (597 , "intfield04" , "IMAGE_LAST_MODIFIED" , ""  , ""),
+    IMAGE1_CONTENT_TYPE (601 , "intfield04" , "IMAGE1_CONTENT_TYPE" , ""  , ""),
+    INTERNAL_USERID (524 , "userid" , "INTERNAL_USERID" , ""  , ""),
+    COLOR_LABEL (102 , "intfield05" , "COLOR_LABEL" , ""  , CommonFields.COLORLABEL),
+    FILE_AS (599 , "field90" , "FILE_AS" , ""  , ""),
+    DEFAULT_ADDRESS (605 , "intfield06" , "DEFAULT_ADDRESS" , "Default address"  , ContactFields.DEFAULT_ADDRESS),
+    MARK_AS_DISTRIBUTIONLIST (602 , "intfield07" , "MARK_AS_DISTRIBUTIONLIST" , ""  , ContactFields.MARK_AS_DISTRIBUTIONLIST),
+    NUMBER_OF_ATTACHMENTS (104 , "intfield08" , "NUMBER_OF_ATTACHMENTS" , ""  , "");
 
 
 
