@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -264,6 +264,8 @@ public class FolderObject extends FolderChildObject implements Cloneable, Serial
 
     public static final int INFOSTORE = 8;
 
+    public static final int MESSAGING = 13;
+
     // Types
     public static final int PRIVATE = 1;
 
@@ -1522,11 +1524,12 @@ public class FolderObject extends FolderChildObject implements Cloneable, Serial
                 folderObj.setLastModified(new Date(rs.getLong(7)));
                 folderObj.setModifiedBy(parseStringValue(rs.getString(8), ctx));
                 folderObj.setPermissionFlag(rs.getInt(9));
-                int defaultFolder = rs.getInt(11);
+                final int defaultFolder = rs.getInt(11);
                 if (rs.wasNull()) {
-                    defaultFolder = 0;
+                    folderObj.setDefaultFolder(false);
+                } else {
+                    folderObj.setDefaultFolder(defaultFolder > 0);
                 }
-                folderObj.setDefaultFolder(defaultFolder > 0);
                 if (loadSubfolderList) {
                     final ArrayList<Integer> subfolderList = getSubfolderIds(folderId, ctx, readCon, table);
                     folderObj.setSubfolderIds(subfolderList);
