@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -113,7 +113,7 @@ public class MailObject {
 
     private String subject;
 
-    private String text;
+    private Object text;
 
     private String contentType;
 
@@ -380,7 +380,7 @@ public class MailObject {
              * Examine message's content type
              */
             if (!"text".equalsIgnoreCase(ct.getPrimaryType())) {
-                throw new MailException(MailException.Code.UNSUPPORTED_MIME_TYPE, ct.toString());
+                //throw new MailException(MailException.Code.UNSUPPORTED_MIME_TYPE, ct.toString());
             }
             /*
              * Set content and its type
@@ -390,10 +390,12 @@ public class MailObject {
                     msg.setContent(text, ct.toString());
                 } else if ("plain".equalsIgnoreCase(ct.getSubType()) || "enriched".equalsIgnoreCase(ct.getSubType())) {
                     if (!ct.containsCharsetParameter()) {
-                        msg.setText(text);
+                        msg.setText((String) text);
                     } else {
-                        msg.setText(text, ct.getCharsetParameter());
+                        msg.setText((String) text, ct.getCharsetParameter());
                     }
+                } else if (ct.startsWith("multipart/")) {
+                    msg.setContent((Multipart) text);
                 } else {
                     throw new MailException(MailException.Code.UNSUPPORTED_MIME_TYPE, ct.toString());
                 }
@@ -403,10 +405,12 @@ public class MailObject {
                     textPart.setContent(text, ct.toString());
                 } else if ("plain".equalsIgnoreCase(ct.getSubType()) || "enriched".equalsIgnoreCase(ct.getSubType())) {
                     if (!ct.containsCharsetParameter()) {
-                        textPart.setText(text);
+                        textPart.setText((String) text);
                     } else {
-                        textPart.setText(text, ct.getCharsetParameter());
+                        textPart.setText((String) text, ct.getCharsetParameter());
                     }
+                } else if (ct.startsWith("multipart/")) {
+                    textPart.setContent((Multipart) text);
                 } else {
                     throw new MailException(MailException.Code.UNSUPPORTED_MIME_TYPE, ct.toString());
                 }
@@ -441,9 +445,8 @@ public class MailObject {
                     final Context ctx = ContextStorage.getStorageContext(session);
                     final ContactInterface contactInterface = ServerServiceRegistry.getInstance().getService(
                         ContactInterfaceDiscoveryService.class).newContactInterface(FolderObject.SYSTEM_LDAP_FOLDER_ID, session);
-                    final Contact c = contactInterface.getObjectById(
-                        UserStorage.getInstance().getUser(ctx.getMailadmin(), ctx).getContactId(),
-                        FolderObject.SYSTEM_LDAP_FOLDER_ID);
+                    final Contact c = contactInterface.getUserById(ctx.getMailadmin(),
+                        false);
                     if (null != c && c.getCompany() != null && c.getCompany().length() > 0) {
                         session.setParameter(MailSessionParameterNames.PARAM_ORGANIZATION_HDR, c.getCompany());
                         msg.setHeader(HEADER_ORGANIZATION, c.getCompany());
@@ -547,11 +550,11 @@ public class MailObject {
         this.fromAddr = fromAddr;
     }
 
-    public String getText() {
+    public Object getText() {
         return text;
     }
 
-    public void setText(final String text) {
+    public void setText(final Object text) {
         this.text = text;
     }
 
