@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,14 +51,12 @@ package com.openexchange.groupware.contexts.impl;
 
 import static com.openexchange.java.Autoboxing.I;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
-
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.List;
-
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextException.Code;
@@ -130,7 +128,7 @@ public class RdbContextStorage extends ContextStorage {
         return contextId;
     }
 
-    private int getMailadmin(final Context ctx) throws ContextException {
+    private int getAdmin(final Context ctx) throws ContextException {
         Connection con = null;
         try {
             con = DBPool.pickup(ctx);
@@ -188,7 +186,7 @@ public class RdbContextStorage extends ContextStorage {
     @Override
     public ContextExtended loadContext(int contextId) throws ContextException {
         ContextImpl context = loadContextData(contextId);
-        context.setMailadmin(getMailadmin(context));
+        context.setMailadmin(getAdmin(context));
         context.setLoginInfo(getLoginInfos(context));
         return context;
     }
@@ -269,9 +267,11 @@ public class RdbContextStorage extends ContextStorage {
 
     @Override
     protected void shutDown() {
+        // Nothing to do.
     }
 
     @Override
     protected void startUp() {
+        // Nothing to do.
     }
 }
