@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -70,6 +70,7 @@ import com.openexchange.groupware.EnumComponent;
 import com.openexchange.groupware.OXExceptionSource;
 import com.openexchange.groupware.OXThrowsMultiple;
 import com.openexchange.groupware.AbstractOXException.Category;
+import com.openexchange.groupware.contact.ContactException;
 import com.openexchange.groupware.contact.ContactInterface;
 import com.openexchange.groupware.contact.ContactInterfaceDiscoveryService;
 import com.openexchange.groupware.contact.helpers.ContactField;
@@ -90,8 +91,10 @@ import com.openexchange.groupware.importexport.exceptions.ImportExportExceptionF
 import com.openexchange.groupware.userconfiguration.UserConfigurationStorage;
 import com.openexchange.server.impl.EffectivePermission;
 import com.openexchange.server.services.ServerServiceRegistry;
+import com.openexchange.tools.Collections;
 import com.openexchange.tools.TimeZoneUtils;
 import com.openexchange.tools.session.ServerSession;
+import com.openexchange.webdav.xml.fields.ContactFields;
 
 /**
  * Importer for OX own CSV file format - this format is able to represent a
@@ -112,9 +115,11 @@ import com.openexchange.tools.session.ServerSession;
         Category.WARNING,
         Category.USER_INPUT,
         Category.USER_INPUT,
-        Category.PERMISSION},
-    desc={"", "", "", "", "", "", ""},
-    exceptionId={0,1,2,3,4,5,6},
+        Category.PERMISSION,
+        Category.USER_INPUT,
+        Category.USER_INPUT},
+    desc={"", "", "", "", "", "", "", "", ""},
+    exceptionId={0,1,2,3,4,5,6,7,8},
     msg={
         "Can only import into one folder at a time.",
         "Cannot import this kind of data. Use method canImport() first.",
@@ -122,7 +127,9 @@ import com.openexchange.tools.session.ServerSession;
         "Could not find the following fields %s",
         "Could not translate a single column title. Is this a valid CSV file?",
         "Could not translate a single field of information, did not insert entry %s.",
-        "Module Contacts not enabled for user, cannot import contacts"
+        "Module Contacts not enabled for user, cannot import contacts",
+        "No field can be found that could be used to name contacts in this file: no name, no company nor e-mail.",
+        "No field was set that might give the contact in line %s a display name: no name, no company nor e-mail."
         })
 public class CSVContactImporter extends AbstractImporter {
 
@@ -208,11 +215,15 @@ public class CSVContactImporter extends AbstractImporter {
         if (!checkFields(fields)) {
             throw EXCEPTIONS.create(4);
         }
+        if(! passesSanityTestForDisplayName(fields)){
+            throw EXCEPTIONS.create(7);
+            
+        }
+            
 
         //reading entries...
         final List<ImportResult> results = new LinkedList<ImportResult>();
         final ContactSwitcher conSet = getContactSwitcher();
-        //final ContactSQLInterface contactsql = new RdbContactSQLInterface(sessObj, sessObj.getContext());
         int lineNumber = 1;
         while (iter.hasNext()) {
             //...and writing them
@@ -234,7 +245,31 @@ public class CSVContactImporter extends AbstractImporter {
         }
         return false;
     }
+    
+    protected boolean passesSanityTestForDisplayName(final List<String> headers){
+        return Collections.any(headers, 
+            ContactField.DISPLAY_NAME.getReadableName(),
+            ContactField.SUR_NAME.getReadableName(),
+            ContactField.GIVEN_NAME.getReadableName(),
+            ContactField.EMAIL1.getReadableName(),
+            ContactField.EMAIL2.getReadableName(),
+            ContactField.EMAIL3.getReadableName(),
+            ContactField.COMPANY.getReadableName(),
+            ContactField.NICKNAME.getReadableName(),
+            ContactField.MIDDLE_NAME.getReadableName());
+    }
 
+    protected boolean passesSanityTestForDisplayName(Contact con){
+        return con.contains(ContactField.DISPLAY_NAME.getNumber())
+        || con.contains(ContactField.SUR_NAME.getNumber())
+        || con.contains(ContactField.GIVEN_NAME.getNumber())
+        || con.contains(ContactField.EMAIL1.getNumber())
+        || con.contains(ContactField.EMAIL2.getNumber())
+        || con.contains(ContactField.EMAIL3.getNumber())
+        || con.contains(ContactField.COMPANY.getNumber())
+        || con.contains(ContactField.NICKNAME.getNumber())
+        || con.contains(ContactField.MIDDLE_NAME.getNumber());
+    }
 
     /**
      *
@@ -248,31 +283,18 @@ public class CSVContactImporter extends AbstractImporter {
      */
     protected ImportResult writeEntry(final List<String> fields, final List<String> entry, final String folder, final ContactSwitcher conSet, final int lineNumber, final ServerSession session){
         final ImportResult result = new ImportResult();
-        final Contact contactObj = new Contact();
         result.setFolder(folder);
         try{
-            final List<String> wrongFields = new LinkedList<String>();
-            boolean atLeastOneFieldWithWrongName = false;
-            boolean atLeastOneFieldInserted = false;
-            for(int i = 0; i < fields.size(); i++){
-                final ContactField currField = getRelevantField(fields.get(i));
-                if(currField == null){
-                    atLeastOneFieldWithWrongName = true;
-                    wrongFields.add(fields.get(i));
-                } else {
-                    final String currEntry = entry.get(i);
-                    if(! currEntry.equals("")){
-                        currField.doSwitch(conSet, contactObj, currEntry);
-                    }
-                    atLeastOneFieldInserted = true;
-                }
-            }
-            if(atLeastOneFieldWithWrongName){
-                result.setException(EXCEPTIONS.create(3, wrongFields.toString()));
-                addErrorInformation(result, lineNumber , fields);
+            boolean[] atLeastOneFieldInserted = new boolean[]{false};
+            final Contact contactObj= convertCsvToContact(fields, entry, conSet, lineNumber, result, atLeastOneFieldInserted);
+            if(! contactObj.canFormDisplayName()){
+                result.setException(EXCEPTIONS.create(8, Integer.valueOf(lineNumber)));
+                result.setDate(new Date());
+                return result;
             }
+                
             contactObj.setParentFolderID(Integer.parseInt( folder.trim() ));
-            if(atLeastOneFieldInserted){
+            if(atLeastOneFieldInserted[0]){
                 final ContactInterface contactInterface = ServerServiceRegistry.getInstance().getService(
                     ContactInterfaceDiscoveryService.class).newContactInterface(contactObj.getParentFolderID(), session);
                 contactInterface.insertContactObject(contactObj);
@@ -282,7 +304,6 @@ public class CSVContactImporter extends AbstractImporter {
                 result.setException(EXCEPTIONS.create(5, Integer.valueOf(lineNumber)));
                 result.setDate(new Date());
             }
-
         } catch (OXException e) {
             e =    handleDataTruncation(e);
             result.setException(e);
@@ -291,6 +312,35 @@ public class CSVContactImporter extends AbstractImporter {
         return result;
     }
 
+
+    public Contact convertCsvToContact(final List<String> fields, final List<String> entry, final ContactSwitcher conSet, final int lineNumber, final ImportResult result, boolean[] atLeastOneFieldInserted) throws ContactException {
+        final Contact contactObj = new Contact();        
+        final List<String> wrongFields = new LinkedList<String>();
+        boolean atLeastOneFieldWithWrongName = false;
+        for(int i = 0; i < fields.size(); i++){
+            final String fieldName = fields.get(i);
+            final String currEntry = entry.get(i);
+            final ContactField currField = getRelevantField(fieldName);
+            if(currField == null){
+                boolean worked = conSet._unknownfield(contactObj, fieldName, currEntry);
+                if(worked)
+                    continue;
+                atLeastOneFieldWithWrongName = true;
+                wrongFields.add(fields.get(i));
+            } else {
+                if(! currEntry.equals("")){
+                    currField.doSwitch(conSet, contactObj, currEntry);
+                }
+                atLeastOneFieldInserted[0] = true;
+            }
+        }
+        if(atLeastOneFieldWithWrongName){
+            result.setException(EXCEPTIONS.create(3, wrongFields.toString()));
+            addErrorInformation(result, lineNumber , fields);
+        }
+        return contactObj;
+    }
+
     /**
      * Adds error information to a given ImportResult
      * @param result ImportResult to be written into.
@@ -345,4 +395,5 @@ public class CSVContactImporter extends AbstractImporter {
         }
         return field.getReadableName();
     }
-}
+    
+}
\ No newline at end of file
