@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,7 @@
 package com.openexchange.groupware.importexport.importers;
 
 import java.text.SimpleDateFormat;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import com.openexchange.api2.OXException;
@@ -63,7 +64,9 @@ import com.openexchange.groupware.contact.helpers.ContactSetter;
 import com.openexchange.groupware.contact.helpers.ContactSwitcher;
 import com.openexchange.groupware.contact.helpers.ContactSwitcherForBooleans;
 import com.openexchange.groupware.contact.helpers.ContactSwitcherForSimpleDateFormat;
+import com.openexchange.groupware.contact.helpers.SplitBirthdayFieldsSetter;
 import com.openexchange.groupware.contact.mappers.ContactFieldMapper;
+import com.openexchange.groupware.contact.mappers.DutchOutlookMapper;
 import com.openexchange.groupware.contact.mappers.EnglishOutlookMapper;
 import com.openexchange.groupware.contact.mappers.FrenchOutlookMapper;
 import com.openexchange.groupware.contact.mappers.GermanOutlookMapper;
@@ -71,6 +74,7 @@ import com.openexchange.groupware.importexport.Format;
 import com.openexchange.groupware.importexport.csv.CSVParser;
 import com.openexchange.groupware.importexport.exceptions.ImportExportExceptionClasses;
 import com.openexchange.server.services.ServerServiceRegistry;
+import com.openexchange.tools.Collections;
 
 @OXExceptionSource(
 	classId=ImportExportExceptionClasses.OUTLOOKCSVCONTACTIMPORTER, 
@@ -90,12 +94,47 @@ import com.openexchange.server.services.ServerServiceRegistry;
  *
  */
 public class OutlookCSVContactImporter extends CSVContactImporter {
+	private List<ContactFieldMapper> fieldMappers;
+	private ContactFieldMapper fieldMapper;
+	private boolean isUsingFallback = false;
+    
+	public boolean isUsingFallback(){
+	    return isUsingFallback;
+	}
 	
-	protected ContactFieldMapper fieldMapper;
-	
-	@Override
+    public List<ContactFieldMapper> getFieldMappers() {
+        if(fieldMappers == null){ //default
+            fieldMappers = new LinkedList<ContactFieldMapper>();
+            fieldMappers.add( new GermanOutlookMapper()); 
+            fieldMappers.add( new FrenchOutlookMapper());
+            fieldMappers.add( new EnglishOutlookMapper());
+            fieldMappers.add( new DutchOutlookMapper());
+            isUsingFallback = true;
+        }
+        return fieldMappers;
+    }
+
+    public void setFieldMappers(List<ContactFieldMapper> fieldMappers) {
+        this.fieldMappers = fieldMappers;
+    }
+    
+    public ContactFieldMapper getFieldMapper() {
+        return fieldMapper;
+    }
+    
+    public void setFieldMapper(ContactFieldMapper fieldMapper) {
+        this.fieldMapper = fieldMapper;
+    }
+    
+    public void addFieldMappers(ContactFieldMapper newMapper) {
+        if(fieldMappers == null)
+            fieldMappers = new LinkedList<ContactFieldMapper>();
+        fieldMappers.add(newMapper);
+    }
+
+    @Override
 	protected ContactField getRelevantField(final String name) {
-		return fieldMapper.getFieldByName(name);
+		return getFieldMapper().getFieldByName(name);
 	}
 
 	@Override
@@ -128,29 +167,22 @@ public class OutlookCSVContactImporter extends CSVContactImporter {
 	 */
 	@Override
 	protected boolean checkFields(final List<String> fields) {
-		int de = 0, fr = 0, en = 0;
-		final ContactFieldMapper 	deMap = new GermanOutlookMapper(), 
-									frMap = new FrenchOutlookMapper(), 
-									enMap = new EnglishOutlookMapper();
-		for(final String name: fields){
-			if(deMap.getFieldByName(name) != null){
-				de++;
-			}
-			if(enMap.getFieldByName(name) != null){
-				en++;
-			}
-			if(frMap.getFieldByName(name) != null){
-				fr++;
-			}
+		int highestAmountOfMappedFields = 0;
+
+		for(ContactFieldMapper mapper : getFieldMappers()){
+		    int mappedFields = 0;
+		    for(final String name: fields){
+		        if(mapper.getFieldByName(name) != null){
+		            mappedFields++;
+		        }
+		    }
+		    if(mappedFields > highestAmountOfMappedFields){
+		        fieldMapper = mapper;
+		        highestAmountOfMappedFields = mappedFields;
+		    }
+		    
 		}
-		fieldMapper = enMap;
-		if(de > en){
-			fieldMapper = deMap;
-		}
-		if(fr > de){
-			fieldMapper = frMap;
-		}
-		return de > 0 || fr > 0 || en > 0;
+		return fieldMapper != null;
 	}
 
 	@Override
@@ -159,9 +191,14 @@ public class OutlookCSVContactImporter extends CSVContactImporter {
 		dateSwitcher.addDateFormat( getGermanDateNotation());
 		dateSwitcher.addDateFormat( getAmericanDateNotation());
 		dateSwitcher.setDelegate(new ContactSetter());
+
 		final ContactSwitcherForBooleans boolSwitcher = new ContactSwitcherForBooleans();
 		boolSwitcher.setDelegate(dateSwitcher);
-		return boolSwitcher;
+
+		final SplitBirthdayFieldsSetter bdaySwitcher = new SplitBirthdayFieldsSetter();
+		bdaySwitcher.setDelegate(boolSwitcher);
+
+		return bdaySwitcher;
 	}
 	
 	public static final SimpleDateFormat getGermanDateNotation(){
@@ -182,6 +219,24 @@ public class OutlookCSVContactImporter extends CSVContactImporter {
 		if(field == null){
 			return String.valueOf( id );
 		}
-		return fieldMapper.getNameOfField(field);
+		return getFieldMapper().getNameOfField(field);
 	}
+
+    @Override
+    protected boolean passesSanityTestForDisplayName(List<String> headers) {
+        ContactFieldMapper mpr = getFieldMapper();
+        
+        return Collections.any(headers, 
+            mpr.getNameOfField(ContactField.DISPLAY_NAME),
+            mpr.getNameOfField(ContactField.SUR_NAME),
+            mpr.getNameOfField(ContactField.GIVEN_NAME),
+            mpr.getNameOfField(ContactField.EMAIL1),
+            mpr.getNameOfField(ContactField.EMAIL2),
+            mpr.getNameOfField(ContactField.EMAIL3),
+            mpr.getNameOfField(ContactField.COMPANY),
+            mpr.getNameOfField(ContactField.NICKNAME),
+            mpr.getNameOfField(ContactField.MIDDLE_NAME));
+    }
+	
+	
 }
