@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,8 +51,10 @@ package com.openexchange.groupware.ldap;
 
 import static com.openexchange.java.Autoboxing.I;
 import static com.openexchange.java.Autoboxing.I2i;
+import static com.openexchange.tools.sql.DBUtils.autocommit;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
 import static com.openexchange.tools.sql.DBUtils.getIN;
+import static com.openexchange.tools.sql.DBUtils.rollback;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -67,6 +69,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.Map.Entry;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.groupware.EnumComponent;
 import com.openexchange.groupware.contexts.Context;
@@ -81,6 +85,8 @@ import com.openexchange.tools.Collections.SmartIntArray;
  */
 public class RdbUserStorage extends UserStorage {
 
+    private static final Log LOG = LogFactory.getLog(RdbUserStorage.class);
+
     private static final String SELECT_ALL_USER = "SELECT id,userPassword,mailEnabled,imapServer,imapLogin,smtpServer,mailDomain," +
         "shadowLastChange,mail,timeZone,preferredLanguage,passwordMech,contactId FROM user WHERE user.cid=?";
 
@@ -125,16 +131,10 @@ public class RdbUserStorage extends UserStorage {
             if (result.next()) {
                 userId = result.getInt(1);
             } else {
-                throw new LdapException(EnumComponent.USER, Code.USER_NOT_FOUND,
-                    uid, Integer.valueOf(context.getContextId()));
-            }
-            if (result.next()) {
-                throw new LdapException(EnumComponent.USER, Code.USER_CONFLICT,
-                    uid, Integer.valueOf(context.getContextId()));
+                throw new LdapException(EnumComponent.USER, Code.USER_NOT_FOUND, uid, I(context.getContextId()));
             }
         } catch (final SQLException e) {
-            throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e,
-                e.getMessage());
+            throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
             DBPool.closeReaderSilent(context, con);
@@ -147,35 +147,40 @@ public class RdbUserStorage extends UserStorage {
         final Connection con;
         try {
             con = DBPool.pickup(context);
-        } catch (DBPoolingException e) {
+        } catch (final DBPoolingException e) {
             throw new LdapException(EnumComponent.USER, Code.NO_CONNECTION, e);
         }
         try {
             return getUser(context, con, new int[] { userId })[0];
-        } catch (UserException e) {
+        } catch (final UserException e) {
             throw new LdapException(e);
         } finally {
             DBPool.closeReaderSilent(context, con);
         }
     }
 
-    private User[] getUser(Context ctx, Connection con, int[] userIds) throws UserException {
+    @Override
+    public User getUser(Context ctx, int userId, Connection con) throws UserException {
+        return getUser(ctx, con, new int[] { userId })[0];
+    }
+
+    private User[] getUser(final Context ctx, final Connection con, final int[] userIds) throws UserException {
         if (0 == userIds.length) {
             return new User[0];
         }
-        Map<Integer, UserImpl> users = new HashMap<Integer, UserImpl>();
+        final Map<Integer, UserImpl> users = new HashMap<Integer, UserImpl>();
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
             stmt = con.prepareStatement(getIN(SELECT_USER, userIds.length));
             int pos = 1;
             stmt.setInt(pos++, ctx.getContextId());
-            for (int userId : userIds) {
+            for (final int userId : userIds) {
                 stmt.setInt(pos++, userId);
             }
             result = stmt.executeQuery();
             while (result.next()) {
-                UserImpl user = new UserImpl();
+                final UserImpl user = new UserImpl();
                 pos = 1;
                 user.setId(result.getInt(pos++));
                 user.setUserPassword(result.getString(pos++));
@@ -195,12 +200,12 @@ public class RdbUserStorage extends UserStorage {
                 user.setContactId(result.getInt(pos++));
                 users.put(I(user.getId()), user);
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
         }
-        for (int userId : userIds) {
+        for (final int userId : userIds) {
             if (!users.containsKey(I(userId))) {
                 throw new UserException(UserException.Code.USER_NOT_FOUND, I(userId), I(ctx.getContextId()));
             }
@@ -213,11 +218,11 @@ public class RdbUserStorage extends UserStorage {
     }
 
     @Override
-    public User[] getUser(Context ctx) throws UserException {
+    public User[] getUser(final Context ctx) throws UserException {
         final Connection con;
         try {
             con = DBPool.pickup(ctx);
-        } catch (DBPoolingException e) {
+        } catch (final DBPoolingException e) {
             throw new UserException(e);
         }
         try {
@@ -228,14 +233,14 @@ public class RdbUserStorage extends UserStorage {
     }
 
     @Override
-    public User[] getUser(Context ctx, int[] userIds) throws UserException {
+    public User[] getUser(final Context ctx, final int[] userIds) throws UserException {
         if (0 == userIds.length) {
             return new User[0];
         }
         final Connection con;
         try {
             con = DBPool.pickup(ctx);
-        } catch (DBPoolingException e) {
+        } catch (final DBPoolingException e) {
             throw new UserException(e);
         }
         try {
@@ -245,97 +250,97 @@ public class RdbUserStorage extends UserStorage {
         }
     }
 
-    private void loadLoginInfo(Context context, Connection con, Map<Integer, UserImpl> users) throws UserException {
+    private void loadLoginInfo(final Context context, final Connection con, final Map<Integer, UserImpl> users) throws UserException {
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
-            String sql = getIN(SELECT_LOGIN, users.size());
+            final String sql = getIN(SELECT_LOGIN, users.size());
             stmt = con.prepareStatement(sql);
             int pos = 1;
             stmt.setInt(pos++, context.getContextId());
-            for (Integer userId : users.keySet()) {
+            for (final Integer userId : users.keySet()) {
                 stmt.setInt(pos++, userId.intValue());
             }
             result = stmt.executeQuery();
             while (result.next()) {
                 users.get(I(result.getInt(1))).setLoginInfo(result.getString(2));
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
         }
     }
 
-    private void loadContact(Context ctx, Connection con, Map<Integer, UserImpl> users) throws UserException {
+    private void loadContact(final Context ctx, final Connection con, final Map<Integer, UserImpl> users) throws UserException {
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
             stmt = con.prepareStatement(getIN(SELECT_CONTACT, users.size()));
             int pos = 1;
             stmt.setInt(pos++, ctx.getContextId());
-            Map<Integer, UserImpl> userByContactId = new HashMap<Integer, UserImpl>(users.size(), 1);
-            for (UserImpl user : users.values()) {
+            final Map<Integer, UserImpl> userByContactId = new HashMap<Integer, UserImpl>(users.size(), 1);
+            for (final UserImpl user : users.values()) {
                 stmt.setInt(pos++, user.getContactId());
                 userByContactId.put(I(user.getContactId()), user);
             }
             result = stmt.executeQuery();
             while (result.next()) {
                 pos = 1;
-                UserImpl user = userByContactId.get(I(result.getInt(pos++)));
+                final UserImpl user = userByContactId.get(I(result.getInt(pos++)));
                 user.setGivenName(result.getString(pos++));
                 user.setSurname(result.getString(pos++));
                 user.setDisplayName(result.getString(pos++));
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
         }
     }
 
-    private void loadGroups(Context context, Connection con, Map<Integer, UserImpl> users) throws UserException {
-        Map<Integer, List<Integer>> tmp = new HashMap<Integer, List<Integer>>(users.size(), 1);
-        for (User user : users.values()) {
-            List<Integer> userGroups = new ArrayList<Integer>();
+    private void loadGroups(final Context context, final Connection con, final Map<Integer, UserImpl> users) throws UserException {
+        final Map<Integer, List<Integer>> tmp = new HashMap<Integer, List<Integer>>(users.size(), 1);
+        for (final User user : users.values()) {
+            final List<Integer> userGroups = new ArrayList<Integer>();
             userGroups.add(I(0));
             tmp.put(I(user.getId()), userGroups);
         }
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
-            String sql = getIN("SELECT member,id FROM groups_member WHERE cid=? AND member IN (", users.size());
+            final String sql = getIN("SELECT member,id FROM groups_member WHERE cid=? AND member IN (", users.size());
             stmt = con.prepareStatement(sql);
             int pos = 1;
             stmt.setInt(pos++, context.getContextId());
-            for (User user : users.values()) {
+            for (final User user : users.values()) {
                 stmt.setInt(pos++, user.getId());
             }
             result = stmt.executeQuery();
             while (result.next()) {
                 tmp.get(I(result.getInt(1))).add(I(result.getInt(2)));
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
         }
-        for (UserImpl user : users.values()) {
+        for (final UserImpl user : users.values()) {
             user.setGroups(I2i(tmp.get(I(user.getId()))));
         }
     }
     
     private static final String STR_ALIAS = "alias";
     
-    private void loadAttributes(Context context, Connection con, Map<Integer, UserImpl> users) throws UserException {
-        Map<Integer, Map<String, Set<String>>> usersAttrs = new HashMap<Integer, Map<String, Set<String>>>();
+    private void loadAttributes(final Context context, final Connection con, final Map<Integer, UserImpl> users) throws UserException {
+        final Map<Integer, Map<String, Set<String>>> usersAttrs = new HashMap<Integer, Map<String, Set<String>>>();
         PreparedStatement stmt = null;
         ResultSet result = null;
         try {
             stmt = con.prepareStatement(getIN(SELECT_ATTRS, users.size()));
             int pos = 1;
             stmt.setInt(pos++, context.getContextId());
-            for (User user : users.values()) {
+            for (final User user : users.values()) {
                 stmt.setInt(pos++, user.getId());
                 usersAttrs.put(I(user.getId()), new HashMap<String, Set<String>>());
             }
@@ -344,7 +349,7 @@ public class RdbUserStorage extends UserStorage {
              * Gather attributes
              */
             while (result.next()) {
-                Map<String, Set<String>> attrs = usersAttrs.get(I(result.getInt(1)));
+                final Map<String, Set<String>> attrs = usersAttrs.get(I(result.getInt(1)));
                 final String name = result.getString(2);
                 Set<String> set = attrs.get(name);
                 if (null == set) {
@@ -353,18 +358,18 @@ public class RdbUserStorage extends UserStorage {
                 }
                 set.add(result.getString(3));
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
         }
-        for (UserImpl user : users.values()) {
-            Map<String, Set<String>> attrs = usersAttrs.get(I(user.getId()));
+        for (final UserImpl user : users.values()) {
+            final Map<String, Set<String>> attrs = usersAttrs.get(I(user.getId()));
             /*
              * Check for aliases
              */
             {
-                Set<String> aliases = attrs.get(STR_ALIAS);
+                final Set<String> aliases = attrs.get(STR_ALIAS);
                 if (aliases == null) {
                     user.setAliases(new String[0]);
                 } else {
@@ -374,7 +379,7 @@ public class RdbUserStorage extends UserStorage {
             /*
              * Apply attributes
              */
-            for (Map.Entry<String, Set<String>> entry : attrs.entrySet()) {
+            for (final Map.Entry<String, Set<String>> entry : attrs.entrySet()) {
                 entry.setValue(Collections.unmodifiableSet(entry.getValue()));
             }
             user.setAttributes(Collections.unmodifiableMap(attrs));
@@ -383,100 +388,103 @@ public class RdbUserStorage extends UserStorage {
 
     @Override
     public void updateUser(final User user, final Context context) throws LdapException {
+        int contextId = context.getContextId();
+        int userId = user.getId();
+        String timeZone = user.getTimeZone();
+        String preferredLanguage = user.getPreferredLanguage();
         final Connection con;
         try {
             con = DBPool.pickupWriteable(context);
-        } catch (final Exception e) {
+        } catch (DBPoolingException e) {
             throw new LdapException(EnumComponent.USER, Code.NO_CONNECTION, e);
         }
         try {
-            final int contextId = context.getContextId();
-            final int id = user.getId();
-            /*
-             * Update time zone and language
-             */
-            PreparedStatement stmt = null;
-            try {
-                final String sql = "UPDATE user SET " + TIMEZONE + "=?," + LANGUAGE
-                    + "=? WHERE cid=? AND id=?";
-                stmt = con.prepareStatement(sql);
-                int pos = 1;
-                stmt.setString(pos++, user.getTimeZone());
-                stmt.setString(pos++, user.getPreferredLanguage());
-                stmt.setInt(pos++, contextId);
-                stmt.setInt(pos++, id);
-                stmt.execute();
-            } catch (final SQLException e) {
-                throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e,
-                    e.getMessage());
-            } finally {
-                closeSQLStuff(null, stmt);
-            }
-            /*
-             * Check if attributes are set
-             */
-            final Map<String, Set<String>> attributes = user.getAttributes();
-            if (null != attributes) {
-                /*
-                 * Update attributes
-                 */
+            con.setAutoCommit(false);
+            // Update time zone and language
+            if (null != timeZone && null != preferredLanguage) {
+                PreparedStatement stmt = null;
                 try {
-                    /*
-                     * Clear all attributes
-                     */
-                    stmt = con.prepareStatement("DELETE FROM user_attribute WHERE cid=? AND " + IDENTIFIER + "=?");
+                    String sql = "UPDATE user SET " + TIMEZONE + "=?," + LANGUAGE + "=? WHERE cid=? AND id=?";
+                    stmt = con.prepareStatement(sql);
                     int pos = 1;
+                    stmt.setString(pos++, timeZone);
+                    stmt.setString(pos++, preferredLanguage);
                     stmt.setInt(pos++, contextId);
-                    stmt.setInt(pos++, id);
-                    stmt.executeUpdate();
-                    closeSQLStuff(null, stmt);
-                    /*
-                     * Insert new ones
-                     */
-                    if (!attributes.isEmpty()) {
-                        /*
-                         * Batch update them
-                         */
-                        stmt = con.prepareStatement("INSERT INTO user_attribute (cid, " + IDENTIFIER + ", name, value) VALUES (?, ?, ?, ?)");
-                        for (final Entry<String, Set<String>> entry : attributes.entrySet()) {
-                            final String name = entry.getKey();
-                            for (final String value : entry.getValue()) {
-                                pos = 1;
-                                stmt.setInt(pos++, contextId);
-                                stmt.setInt(pos++, id);
-                                stmt.setString(pos++, name);
-                                stmt.setString(pos++, value);
-                                stmt.addBatch();
-                            }
-                        }
-                        stmt.executeBatch();
-                    }
-                } catch (final SQLException e) {
-                    throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e,
-                        e.getMessage());
+                    stmt.setInt(pos++, userId);
+                    stmt.execute();
                 } finally {
-                    closeSQLStuff(null, stmt);
+                    closeSQLStuff(stmt);
                 }
             }
+            if (null != user.getAttributes()) {
+                updateAttributes(context, user, con);
+            }
+            con.commit();
+        } catch (SQLException e) {
+            rollback(con);
+            throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e, e.getMessage());
         } finally {
+            autocommit(con);
             DBPool.closeWriterSilent(context, con);
         }
     }
 
+    private void updateAttributes(Context ctx, User user, Connection con) throws SQLException {
+        // Update attributes
+        int contextId = ctx.getContextId();
+        int userId = user.getId();
+        Map<String, Set<String>> attributes = user.getAttributes();
+        // Clear all attributes
+        PreparedStatement stmt = null;
+        try {
+            stmt = con.prepareStatement("DELETE FROM user_attribute WHERE cid=? AND " + IDENTIFIER + "=?");
+            int pos = 1;
+            stmt.setInt(pos++, contextId);
+            stmt.setInt(pos++, userId);
+            stmt.executeUpdate();
+        } finally {
+            closeSQLStuff(stmt);
+        }
+        // Insert new ones
+        if (!attributes.isEmpty()) {
+            // Batch update them
+            stmt = null;
+            try {
+                stmt = con.prepareStatement("INSERT INTO user_attribute (cid," + IDENTIFIER + ",name,value) VALUES (?,?,?,?)");
+                for (final Entry<String, Set<String>> entry : attributes.entrySet()) {
+                    for (final String value : entry.getValue()) {
+                        int pos = 1;
+                        stmt.setInt(pos++, contextId);
+                        stmt.setInt(pos++, userId);
+                        stmt.setString(pos++, entry.getKey());
+                        stmt.setString(pos++, value);
+                        stmt.addBatch();
+                    }
+                }
+                stmt.executeBatch();
+            } finally {
+                closeSQLStuff(stmt);
+            }
+        } else {
+            UserException e = new UserException(UserException.Code.ERASED_ATTRIBUTES, I(contextId), I(userId));
+            LOG.warn(e.getMessage(), e);
+        }
+    }
+
     /**
      * {@inheritDoc}
      */
     @Override
-    public User searchUser(String email, Context context) throws LdapException {
+    public User searchUser(final String email, final Context context) throws LdapException {
         String sql = "SELECT id FROM user WHERE cid=? AND mail LIKE ?";
         Connection con;
         try {
             con = DBPool.pickup(context);
-        } catch (DBPoolingException e) {
+        } catch (final DBPoolingException e) {
             throw new LdapException(EnumComponent.USER, Code.NO_CONNECTION, e);
         }
         try {
-            String pattern = StringCollection.prepareForSearch(email, false, true);
+            final String pattern = StringCollection.prepareForSearch(email, false, true);
             PreparedStatement stmt = null;
             ResultSet result = null;
             int userId = -1;
@@ -488,7 +496,7 @@ public class RdbUserStorage extends UserStorage {
                 if (result.next()) {
                     userId = result.getInt(1);
                 }
-            } catch (SQLException e) {
+            } catch (final SQLException e) {
                 throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e, e.getMessage());
             } finally {
                 closeSQLStuff(result, stmt);
@@ -510,12 +518,12 @@ public class RdbUserStorage extends UserStorage {
                     throw new LdapException(EnumComponent.USER, Code.NO_USER_BY_MAIL, email);
                 }
                 return getUser(context, con, new int[] { userId })[0];
-            } catch (SQLException e) {
+            } catch (final SQLException e) {
                 throw new LdapException(EnumComponent.USER, Code.SQL_ERROR, e, e.getMessage());
             } finally {
                 closeSQLStuff(result, stmt);
             }
-        } catch (UserException e) {
+        } catch (final UserException e) {
             throw new LdapException(e);
         } finally {
             DBPool.closeReaderSilent(context, con);
@@ -548,7 +556,7 @@ public class RdbUserStorage extends UserStorage {
             result = stmt.executeQuery();
             final List<Integer> tmp = new ArrayList<Integer>();
             while (result.next()) {
-                tmp.add(Integer.valueOf(result.getInt(1)));
+                tmp.add(I(result.getInt(1)));
             }
             users = new int[tmp.size()];
             for (int i = 0; i < users.length; i++) {
@@ -568,7 +576,7 @@ public class RdbUserStorage extends UserStorage {
      * {@inheritDoc}
      */
     @Override
-    public int[] listAllUser(Context context) throws UserException {
+    public int[] listAllUser(final Context context) throws UserException {
         Connection con = null;
         try {
             con = DBPool.pickup(context);
@@ -582,7 +590,7 @@ public class RdbUserStorage extends UserStorage {
         }
     }
 
-    private int[] listAllUser(Context ctx, Connection con) throws UserException {
+    private int[] listAllUser(final Context ctx, final Connection con) throws UserException {
         final int[] users;
         PreparedStatement stmt = null;
         ResultSet result = null;
@@ -590,7 +598,7 @@ public class RdbUserStorage extends UserStorage {
             stmt = con.prepareStatement("SELECT id FROM user WHERE user.cid=?");
             stmt.setInt(1, ctx.getContextId());
             result = stmt.executeQuery();
-            List<Integer> tmp = new ArrayList<Integer>();
+            final List<Integer> tmp = new ArrayList<Integer>();
             while (result.next()) {
                 tmp.add(I(result.getInt(1)));
             }
@@ -598,7 +606,7 @@ public class RdbUserStorage extends UserStorage {
             for (int i = 0; i < users.length; i++) {
                 users[i] = tmp.get(i).intValue();
             }
-        } catch (SQLException e) {
+        } catch (final SQLException e) {
             throw new UserException(UserException.Code.SQL_ERROR, e, e.getMessage());
         } finally {
             closeSQLStuff(result, stmt);
@@ -633,7 +641,7 @@ public class RdbUserStorage extends UserStorage {
                 } while (result.next());
             } else {
                 throw new UserException(UserException.Code.USER_NOT_FOUND,
-                        imapLogin, Integer.valueOf(cid));
+                        imapLogin, I(cid));
             }
             users = sia.toArray();
         } catch (final SQLException e) {
@@ -655,10 +663,12 @@ public class RdbUserStorage extends UserStorage {
     }
 
     @Override
-    protected void startInternal() throws UserException {
+    protected void startInternal() {
+        // Nothing to set up.
     }
 
     @Override
-    protected void stopInternal() throws UserException {
+    protected void stopInternal() {
+        // Nothing to tear down.
     }
 }
