@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,15 +49,19 @@
 
 package com.openexchange.groupware.notify;
 
+import java.io.IOException;
+import java.io.InputStream;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
 import java.util.Collections;
 import java.util.Date;
+import java.util.EnumSet;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
@@ -65,21 +69,39 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TimeZone;
 import java.util.TreeSet;
+import javax.activation.DataHandler;
+import javax.mail.BodyPart;
+import javax.mail.MessagingException;
+import javax.mail.Multipart;
+import javax.mail.internet.MimeBodyPart;
+import javax.mail.internet.MimeMultipart;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.api2.OXException;
+import com.openexchange.data.conversion.ical.ConversionError;
+import com.openexchange.data.conversion.ical.ConversionWarning;
+import com.openexchange.data.conversion.ical.ICalEmitter;
+import com.openexchange.data.conversion.ical.ICalItem;
+import com.openexchange.data.conversion.ical.ICalSession;
+import com.openexchange.data.conversion.ical.ITipContainer;
+import com.openexchange.data.conversion.ical.ITipMethod;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.event.impl.AppointmentEventInterface2;
 import com.openexchange.event.impl.TaskEventInterface2;
+import com.openexchange.folderstorage.internal.CalculatePermission;
 import com.openexchange.group.Group;
 import com.openexchange.group.GroupStorage;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.Types;
 import com.openexchange.groupware.calendar.CalendarCollectionService;
+import com.openexchange.groupware.calendar.CalendarDataObject;
 import com.openexchange.groupware.calendar.Constants;
 import com.openexchange.groupware.calendar.OXCalendarException;
+import com.openexchange.groupware.calendar.RecurringResultInterface;
+import com.openexchange.groupware.calendar.RecurringResultsInterface;
 import com.openexchange.groupware.container.Appointment;
 import com.openexchange.groupware.container.CalendarObject;
+import com.openexchange.groupware.container.ExternalUserParticipant;
 import com.openexchange.groupware.container.FolderObject;
 import com.openexchange.groupware.container.Participant;
 import com.openexchange.groupware.container.UserParticipant;
@@ -92,6 +114,8 @@ import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.ldap.UserException;
 import com.openexchange.groupware.ldap.UserStorage;
 import com.openexchange.groupware.notify.NotificationConfig.NotificationProperty;
+import com.openexchange.groupware.notify.State.Type;
+import com.openexchange.groupware.tasks.ExternalParticipant;
 import com.openexchange.groupware.tasks.Task;
 import com.openexchange.groupware.userconfiguration.RdbUserConfigurationStorage;
 import com.openexchange.groupware.userconfiguration.UserConfiguration;
@@ -120,6 +144,10 @@ import com.openexchange.i18n.tools.replacement.TaskActionReplacement;
 import com.openexchange.i18n.tools.replacement.TaskPriorityReplacement;
 import com.openexchange.i18n.tools.replacement.TaskStatusReplacement;
 import com.openexchange.mail.MailException;
+import com.openexchange.mail.mime.MessageHeaders;
+import com.openexchange.mail.mime.datasource.FileDataSource;
+import com.openexchange.mail.mime.datasource.MessageDataSource;
+import com.openexchange.mail.mime.utils.MIMEMessageUtility;
 import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 import com.openexchange.resource.Resource;
@@ -132,6 +160,8 @@ import com.openexchange.tools.exceptions.LoggingLogic;
 import com.openexchange.tools.oxfolder.OXFolderAccess;
 import com.openexchange.tools.session.ServerSession;
 import com.openexchange.tools.session.ServerSessionAdapter;
+import com.openexchange.tools.stream.UnsynchronizedByteArrayInputStream;
+import com.openexchange.tools.stream.UnsynchronizedByteArrayOutputStream;
 
 public class ParticipantNotify implements AppointmentEventInterface2, TaskEventInterface2 {
 
@@ -163,16 +193,16 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
      * @param state The state
      */
     protected static void sendMessage(final MailMessage mmsg, final ServerSession session, final CalendarObject obj, final State state) {
-        messageSender.sendMessage(mmsg.title, mmsg.message, mmsg.addresses, session, obj, mmsg.folderId, state, false, mmsg.internal);
+        messageSender.sendMessage(mmsg, session, obj, state, false);
     }
 
-    protected void sendMessage(final String messageTitle, final String message, final List<String> name, final ServerSession session, final CalendarObject obj, final int folderId, final State state, final boolean suppressOXReminderHeader, final boolean internal) {
+    protected void sendMessage(MailMessage msg, final ServerSession session, final CalendarObject obj, final State state, final boolean suppressOXReminderHeader) {
         if (DEBUG) {
-            LOG.debug(new StringBuilder(message.length() + 64).append("Sending message to: ").append(name).append("\n=====[").append(
-                messageTitle).append("]====\n\n").append(message).append("\n\n"));
+            LOG.debug(new StringBuilder().append("Sending message to: ").append(msg.addresses).append("\n=====[").append(
+                msg.title).append("]====\n\n").append(msg.message).append("\n\n"));
         }
 
-        int fuid = folderId;
+        int fuid = msg.folderId;
         if (fuid == -1) {
             fuid = obj.getParentFolderID();
         }
@@ -180,21 +210,27 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         if (suppressOXReminderHeader) {
             fuid = MailObject.DONT_SET;
         }
-
-        final MailObject mail = new MailObject(session, obj.getObjectID(), fuid, state.getModule(), state.getType().toString());
+        String type = (msg.overrideType != null) ? msg.overrideType.toString() : state.getType().toString();
+        final MailObject mail = new MailObject(session, obj.getObjectID(), fuid, state.getModule(), type);
         mail.setFromAddr(UserStorage.getStorageUser(session.getUserId(), session.getContext()).getMail());
-        mail.setToAddrs(name.toArray(new String[name.size()]));
-        mail.setText(message);
-        mail.setSubject(messageTitle);
-        mail.setContentType("text/plain; charset=UTF-8");
-
-        if (internal) {
-            state.modifyInternal(mail, obj, session);
+        mail.setToAddrs(msg.addresses.toArray(new String[msg.addresses.size()]));
+        mail.setText(msg.message);
+        mail.setSubject(msg.title);
+        
+        if (Multipart.class.isInstance(msg.message)) {
+            mail.setContentType("multipart/alternative");
         } else {
-            state.modifyExternal(mail, obj, session);
+            mail.setContentType("text/plain; charset=UTF-8");
+        }
+
+        if (state.getModule() == Types.TASK) {
+            if (msg.internal) {
+                state.modifyInternal(mail, obj, session);
+            } else {
+                state.modifyExternal(mail, obj, session);
+            }
         }
 
-        // System.out.println(folderId);
         try {
             mail.send();
         } catch (final MailException e) {
@@ -467,7 +503,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         /*
          * Generate a render map filled with object-specific information
          */
-        final RenderMap renderMap = createRenderMap(newObj, oldObj, isUpdate, title, state.getModule(), receivers, serverSession);
+        final RenderMap renderMap = createRenderMap(newObj, oldObj, isUpdate, title, state, receivers, serverSession);
 
         /*
          * Add confirmation action replacement to render map if non-null
@@ -497,16 +533,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
          * Send messages
          */
         for (final MailMessage mmsg : messages) {
-            sendMessage(
-                mmsg.title,
-                mmsg.message,
-                mmsg.addresses,
-                serverSession,
-                newObj,
-                mmsg.folderId,
-                state,
-                suppressOXReminderHeader,
-                mmsg.internal);
+            sendMessage(mmsg, serverSession, newObj, state, suppressOXReminderHeader);
         }
     }
 
@@ -608,20 +635,13 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
                         /*
                          * Compose message
                          */
+                        MailMessage message = null;
                         if (Participant.USER == p.type) {
-                            messages.add(createUserMessage(
-                                p,
-                                (userCanReadObject(p, newObj, session)),
-                                title,
-                                actionRepl,
-                                state,
-                                locale,
-                                renderMap,
-                                isUpdate,
-                                b));
+                            message = createUserMessage(session, newObj, p, (userCanReadObject(p, newObj, session)), title, actionRepl, state, locale, renderMap, isUpdate, b);
                         } else {
-                            messages.add(createParticipantMessage(p, title, actionRepl, state, locale, renderMap, isUpdate, b));
+                            message = createParticipantMessage(session, newObj, p, title, actionRepl, state, locale, renderMap, isUpdate, b);
                         }
+                        messages.add(message);
                         if (DEBUG) {
                             LOG.debug(new StringBuilder(128).append((Types.APPOINTMENT == state.getModule() ? "Appointment" : "Task")).append(
                                 " (id = ").append(newObj.getObjectID()).append(") \"").append(
@@ -692,6 +712,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
 
     /**
      * Creates a message for specified user.
+     * @param session 
      * 
      * @param p The participant
      * @param canRead <code>true</code> if provided participant has read permission; otherwise <code>false</code>
@@ -704,12 +725,13 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
      * @param b A string builder
      * @return The created message
      */
-    protected static MailMessage createUserMessage(final EmailableParticipant p, final boolean canRead, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
-        return createParticipantMessage0(p, canRead, title, actionRepl, state, locale, renderMap, isUpdate, b);
+    protected static MailMessage createUserMessage(ServerSession session, CalendarObject cal, final EmailableParticipant p, final boolean canRead, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
+        return createParticipantMessage0(session, cal, p, canRead, title, actionRepl, state, locale, renderMap, isUpdate, b);
     }
 
     /**
      * Creates a message for specified participant.
+     * @param session 
      * 
      * @param p The participant
      * @param title The object's title
@@ -721,11 +743,11 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
      * @param b A string builder
      * @return The created message
      */
-    protected static MailMessage createParticipantMessage(final EmailableParticipant p, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
-        return createParticipantMessage0(p, true, title, actionRepl, state, locale, renderMap, isUpdate, b);
+    protected static MailMessage createParticipantMessage(ServerSession session, CalendarObject cal, final EmailableParticipant p, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
+        return createParticipantMessage0(session, cal, p, true, title, actionRepl, state, locale, renderMap, isUpdate, b);
     }
 
-    private static MailMessage createParticipantMessage0(final EmailableParticipant p, final boolean canRead, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
+    private static MailMessage createParticipantMessage0(ServerSession session, CalendarObject cal, final EmailableParticipant p, final boolean canRead, final String title, final TemplateReplacement actionRepl, final State state, final Locale locale, final RenderMap renderMap, final boolean isUpdate, final StringBuilder b) {
         final MailMessage msg = new MailMessage();
         final Template createTemplate = state.getTemplate();
         final StringHelper strings = new StringHelper(locale);
@@ -738,6 +760,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
                 /*
                  * Current participant is removed by caught update event
                  */
+                msg.overrideType = State.Type.DELETED;
                 /*
                  * Get cloned version of render map to apply changed status
                  */
@@ -765,6 +788,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
                 /*
                  * Current participant is added by caught update event
                  */
+                msg.overrideType = State.Type.NEW;
                 /*
                  * Get cloned version of render map to apply changed status
                  */
@@ -801,14 +825,40 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
             }
         } else {
             if (State.Type.NEW.equals(state.getType())) {
+                String textMessage = "";
                 if ((p.type == Participant.EXTERNAL_USER || p.type == Participant.RESOURCE)) {
                     final String template = strings.getString(Types.APPOINTMENT == state.getModule() ? Notifications.APPOINTMENT_CREATE_MAIL_EXT : Notifications.TASK_CREATE_MAIL_EXT);
-                    msg.message = new StringTemplate(template).render(p.getLocale(), renderMap);
+                    textMessage = new StringTemplate(template).render(p.getLocale(), renderMap);
                 } else if (!canRead) {
                     final String template = strings.getString(state.getModule() == Types.APPOINTMENT ? Notifications.APPOINTMENT_CREATE_MAIL_NO_ACCESS : Notifications.TASK_CREATE_MAIL_NO_ACCESS);
-                    msg.message = new StringTemplate(template).render(p.getLocale(), renderMap);
+                    textMessage = new StringTemplate(template).render(p.getLocale(), renderMap);
                 } else {
+                    textMessage = createTemplate.render(p.getLocale(), renderMap);
+                }
+                if (p.type == Participant.USER && !NotificationConfig.getPropertyAsBoolean(NotificationProperty.INTERNAL_IMIP, false)) {
+                    msg.message = textMessage;
+                } else {
+                    msg.message = generateMessageMultipart(session, cal, textMessage, state.getModule(), state.getType(), ITipMethod.REQUEST);
+                }
+            } else if (EnumSet.of(State.Type.ACCEPTED, State.Type.DECLINED, State.Type.TENTATIVELY_ACCEPTED).contains(state.getType())) {
+                String textMessage = "";
+                if ((p.type == Participant.EXTERNAL_USER || p.type == Participant.RESOURCE)) {
+                    final String template = strings.getString(Types.APPOINTMENT == state.getModule() ? Notifications.APPOINTMENT_CONFIRMATION_MAIL_EXT : Notifications.TASK_CONFIRMATION_MAIL_EXT);
+                    textMessage = new StringTemplate(template).render(p.getLocale(), renderMap);
+                } else {
+                    textMessage = createTemplate.render(p.getLocale(), renderMap);
+                }
+                // Attach IMIP Magic only for external users on secondary events, to tell them the state of the appointment, but don't bother with internal users.
+                if(p.type == Participant.EXTERNAL_USER) {
+                    msg.message = generateMessageMultipart(session, cal, textMessage, state.getModule(), state.getType(), ITipMethod.REPLY);
+                } else {
+                    msg.message = textMessage;
+                }
+            } else  if (state.getType() == State.Type.DELETED) {
+                if (p.type == Participant.USER && !NotificationConfig.getPropertyAsBoolean(NotificationProperty.INTERNAL_IMIP, false)) {
                     msg.message = createTemplate.render(p.getLocale(), renderMap);
+                } else {
+                    msg.message = generateMessageMultipart(session, cal, createTemplate.render(p.getLocale(), renderMap), state.getModule(), state.getType(), ITipMethod.CANCEL);
                 }
             } else {
                 msg.message = createTemplate.render(p.getLocale(), renderMap);
@@ -830,6 +880,72 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         return msg;
     }
 
+    /**
+     * 
+     * @param session
+     * @param cal
+     * @param text
+     * @param module 
+     * @param type
+     * @param method
+     * @return
+     */
+    private static Object generateMessageMultipart(ServerSession session, CalendarObject cal, String text, int module, State.Type type, ITipMethod method) {
+        if (module == Types.TASK) {
+            return text;
+        }
+        
+        Appointment app = (Appointment) cal;
+        
+        ITipContainer iTip = new ITipContainer(method, type, session.getUserId());
+        ICalEmitter emitter = ServerServiceRegistry.getInstance().getService(ICalEmitter.class);
+        Multipart mp = new MimeMultipart("alternative");
+        
+        try {
+            BodyPart textPart = new MimeBodyPart();
+            textPart.setContent(text, "text/plain; charset=UTF-8");
+            
+            ICalSession icalSession = emitter.createSession();
+            Date until = null;
+            if (Appointment.NO_RECURRENCE != app.getRecurrenceType()) {
+                until = app.getEndDate();
+                app.setEndDate(computeFirstOccurrenceEnd(app));
+            }
+            boolean hasAlarm = app.containsAlarm();
+            int alarm = app.getAlarm();
+            app.removeAlarm();
+            emitter.writeAppointment(icalSession, app, session.getContext(), iTip, new ArrayList<ConversionError>(), new ArrayList<ConversionWarning>());
+            if (null != until) {
+                app.setEndDate(until);
+            }
+            if (hasAlarm) {
+                app.setAlarm(alarm);
+            }
+            UnsynchronizedByteArrayOutputStream byteArrayOutputStream = new UnsynchronizedByteArrayOutputStream();
+            emitter.writeSession(icalSession, byteArrayOutputStream);
+            InputStream icalFile = new UnsynchronizedByteArrayInputStream(byteArrayOutputStream.toByteArray());
+            
+            BodyPart iCalPart = new MimeBodyPart();
+            
+            String contentType = "text/calendar; " + method.getMethod();
+            iCalPart.setDataHandler(new DataHandler(new MessageDataSource(icalFile, contentType)));
+            iCalPart.setHeader(MessageHeaders.HDR_CONTENT_TYPE, MIMEMessageUtility.foldContentType(contentType));
+            iCalPart.setHeader(MessageHeaders.HDR_CONTENT_TRANSFER_ENC, "base64");
+            
+            mp.addBodyPart(textPart);
+            mp.addBodyPart(iCalPart);
+        } catch (MessagingException e) {
+            LOG.error("Unable to compose message", e);
+        } catch (ConversionError e) {
+            LOG.error("Unable to compose message", e);
+        } catch (IOException e) {
+            LOG.error("Unable to compose message", e);
+        } catch (OXException e) {
+            LOG.error("Unable to compose message", e);
+        } 
+        return mp;
+    }
+
     private static String getTaskCreateMessage(final EmailableParticipant p, final boolean canRead) {
         if (p.type == Participant.EXTERNAL_USER || p.type == Participant.RESOURCE) {
             return Notifications.TASK_CREATE_MAIL_EXT;
@@ -850,7 +966,8 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         }
     }
 
-    private RenderMap createRenderMap(final CalendarObject newObj, final CalendarObject oldObj, final boolean isUpdate, final String title, final int module, final Map<Locale, List<EmailableParticipant>> receivers, final ServerSession session) {
+    private RenderMap createRenderMap(final CalendarObject newObj, final CalendarObject oldObj, final boolean isUpdate, final String title, final State state, final Map<Locale, List<EmailableParticipant>> receivers, final ServerSession session) {
+        final int module = state.getModule();
         /*
          * Containers for traversed participants
          */
@@ -885,7 +1002,8 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
                 resourceSet,
                 receivers,
                 session,
-                all);
+                all,
+                newObj.getOrganizer());
         }
         /*
          * Generate a render map
@@ -1134,16 +1252,20 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         }
     }
 
-    private void sortExternalParticipantsAndResources(final Participant[] oldParticipants, final Participant[] newParticipants, final Set<EmailableParticipant> participantSet, final Set<EmailableParticipant> resourceSet, final Map<Locale, List<EmailableParticipant>> receivers, final ServerSession session, final Map<String, EmailableParticipant> all) {
+    private void sortExternalParticipantsAndResources(final Participant[] oldParticipants, final Participant[] newParticipants, final Set<EmailableParticipant> participantSet, final Set<EmailableParticipant> resourceSet, final Map<Locale, List<EmailableParticipant>> receivers, final ServerSession session, final Map<String, EmailableParticipant> all, String organizer) {
         sortNewExternalParticipantsAndResources(newParticipants, participantSet, resourceSet, receivers, session, all, oldParticipants);
-        sortOldExternalParticipantsAndResources(oldParticipants, participantSet, resourceSet, receivers, all, session, newParticipants);
+        sortOldExternalParticipantsAndResources(oldParticipants, participantSet, resourceSet, receivers, all, session, newParticipants, organizer);
     }
 
-    private void sortOldExternalParticipantsAndResources(final Participant[] oldParticipants, final Set<EmailableParticipant> participantSet, final Set<EmailableParticipant> resourceSet, final Map<Locale, List<EmailableParticipant>> receivers, final Map<String, EmailableParticipant> all, final ServerSession session, final Participant[] newParticipants) {
+    private void sortOldExternalParticipantsAndResources(final Participant[] oldParticipants, final Set<EmailableParticipant> participantSet, final Set<EmailableParticipant> resourceSet, final Map<Locale, List<EmailableParticipant>> receivers, final Map<String, EmailableParticipant> all, final ServerSession session, final Participant[] newParticipants, String organizer) {
         if (oldParticipants == null) {
             return;
         }
         final Context ctx = session.getContext();
+//        List<Participant> mergedWithOrganizer = new ArrayList<Participant>(Arrays.asList(oldParticipants));
+//        if (organizer != null && !organizer.trim().equals(""))
+//            mergedWithOrganizer.add(new ExternalUserParticipant(organizer));
+        
         for (final Participant participant : oldParticipants) {
             switch (participant.getType()) {
             case Participant.USER:
@@ -1172,6 +1294,9 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
                 throw new IllegalArgumentException("Unknown Participant Type: " + participant.getType());
             }
         }
+        
+        if (organizer != null)
+            addSingleParticipant(getExternalParticipant(new ExternalUserParticipant(organizer), session), participantSet, resourceSet, receivers, all, false);
     }
 
     private void sortNewExternalParticipantsAndResources(final Participant[] newParticipants, final Set<EmailableParticipant> participantSet, final Set<EmailableParticipant> resourceSet, final Map<Locale, List<EmailableParticipant>> receivers, final ServerSession session, final Map<String, EmailableParticipant> all, final Participant[] oldParticipants) {
@@ -1575,6 +1700,8 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
 
     static final class MailMessage {
 
+        public Type overrideType;
+
         /**
          * Initializes a new MailMessage
          */
@@ -1582,7 +1709,7 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
             super();
         }
 
-        public String message;
+        public Object message;
 
         public String title;
 
@@ -1821,6 +1948,13 @@ public class ParticipantNotify implements AppointmentEventInterface2, TaskEventI
         return cal.getTime();
     }
 
+    private static Date computeFirstOccurrenceEnd(CalendarObject app) throws OXException {
+        CalendarCollectionService service = ServerServiceRegistry.getInstance().getService(CalendarCollectionService.class);
+        RecurringResultsInterface recurrences = service.calculateFirstRecurring(app);
+        RecurringResultInterface recurringResult = recurrences.getRecurringResult(0);
+        return new Date(recurringResult.getEnd());
+    }
+
     /**
      * Checks if specified appointment is a change exception.
      * 
