@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -58,17 +58,15 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Timestamp;
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Date;
 import java.util.LinkedList;
-import java.util.List;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.api.OXConflictException;
 import com.openexchange.api.OXMandatoryFieldException;
 import com.openexchange.api2.OXException;
-import com.openexchange.api2.ReminderSQLInterface;
+import com.openexchange.api2.ReminderService;
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.EnumComponent;
@@ -76,12 +74,15 @@ import com.openexchange.groupware.Types;
 import com.openexchange.groupware.AbstractOXException.Category;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.impl.IDGenerator;
+import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.reminder.ReminderException.Code;
+import com.openexchange.groupware.reminder.internal.GetArisingReminder;
 import com.openexchange.groupware.reminder.internal.SQL;
+import com.openexchange.groupware.reminder.internal.TargetRegistry;
 import com.openexchange.server.impl.DBPool;
+import com.openexchange.session.Session;
 import com.openexchange.tools.iterator.SearchIterator;
 import com.openexchange.tools.iterator.SearchIteratorException;
-import com.openexchange.tools.iterator.SearchIteratorException.SearchIteratorCode;
 import com.openexchange.tools.sql.DBUtils;
 
 /**
@@ -89,29 +90,18 @@ import com.openexchange.tools.sql.DBUtils;
  *
  * @author <a href="mailto:sebastian.kauss@open-xchange.org">Sebastian Kauss</a>
  */
-public class ReminderHandler implements Types, ReminderSQLInterface {
+public class ReminderHandler implements ReminderService {
 
     /**
      * Logger.
      */
     private static final Log LOG = LogFactory.getLog(ReminderHandler.class);
 
-    private final Context context;
-
-    private ReminderDeleteInterface reminderDeleteInterface;
+    final Context context;
 
     public ReminderHandler(final Context context) {
+        super();
         this.context = context;
-        reminderDeleteInterface = new EmptyReminderDeleteImpl();
-    }
-
-    public ReminderHandler(final Context context, final ReminderDeleteInterface reminderDeleteInterface) {
-        this.context = context;
-        this.reminderDeleteInterface = reminderDeleteInterface;
-    }
-
-    public void setReminderDeleteInterface(final ReminderDeleteInterface reminderDeleteInterface) {
-        this.reminderDeleteInterface = reminderDeleteInterface;
     }
 
     public int insertReminder( final ReminderObject reminderObj) throws OXException {
@@ -281,14 +271,14 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
         try {
             writeCon = DBPool.pickupWriteable(context);
             int a = 0;
-            ps = writeCon.prepareStatement(SQL.sqlDeleteWithId);
+            ps = writeCon.prepareStatement(SQL.DELETE_WITH_ID);
             ps.setInt(++a, contextId);
             ps.setInt(++a, reminder.getObjectId());
             final int deleted = ps.executeUpdate();
             if (deleted == 0) {
                 throw new ReminderException(Code.NOT_FOUND, reminder, I(contextId));
             }
-            reminderDeleteInterface.updateTargetObject(context, writeCon, reminder.getTargetId(), reminder.getUser());
+            TargetRegistry.getInstance().getService(reminder.getModule()).updateTargetObject(context, writeCon, reminder.getTargetId(), reminder.getUser());
         } catch (final SQLException exc) {
             throw new ReminderException(Code.DELETE_EXCEPTION, exc);
         } catch (final DBPoolingException exc) {
@@ -338,7 +328,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
         }
     }
 
-    public void deleteReminder(int targetId, int userId, int module, Connection con) throws OXMandatoryFieldException, OXConflictException, OXException {
+    public void deleteReminder(int targetId, int userId, int module, Connection con) throws OXException {
         final int contextId = context.getContextId();
         if (userId == 0) {
             throw new ReminderException(Code.MANDATORY_FIELD_USER, "missing user id");
@@ -354,7 +344,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             if (0 == stmt.executeUpdate()) {
                 throw new ReminderException(Code.NOT_FOUND, I(targetId), I(contextId));
             }
-            reminderDeleteInterface.updateTargetObject(context, con, targetId, userId);
+            TargetRegistry.getInstance().getService(module).updateTargetObject(context, con, targetId, userId);
         } catch (SQLException e) {
             throw new ReminderException(Code.DELETE_EXCEPTION, e);
         } catch (ReminderException e) {
@@ -386,7 +376,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
         }
     }
 
-    public void deleteReminder(int targetId, int module, Connection con) throws OXMandatoryFieldException, OXConflictException, OXException {
+    public void deleteReminder(int targetId, int module, Connection con) throws OXException {
         final int contextId = context.getContextId();
         PreparedStatement stmt = null;
         try {
@@ -398,7 +388,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             if (0 == stmt.executeUpdate()) {
                 throw new ReminderException(Code.NOT_FOUND, I(targetId), I(contextId));
             }
-            reminderDeleteInterface.updateTargetObject(context, con, targetId);
+            TargetRegistry.getInstance().getService(module).updateTargetObject(context, con, targetId);
         } catch (SQLException e) {
             throw new ReminderException(Code.SQL_ERROR, e, e.getMessage());
         } catch (ReminderException e) {
@@ -455,7 +445,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             stmt.setInt(pos++, module);
             stmt.setInt(pos++, userId);
             result = stmt.executeQuery();
-            return result2Object(result, stmt, false);
+            return result2Object(context, result, stmt, false);
         } catch (SQLException e) {
             throw new ReminderException(Code.LOAD_EXCEPTION, e);
         } finally {
@@ -567,7 +557,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             ps.setInt(++a, objectId);
 
             final ResultSet rs = ps.executeQuery();
-            final ReminderObject reminderObj = result2Object(rs, ps, true);
+            final ReminderObject reminderObj = result2Object(context, rs, ps, true);
 
             if (reminderObj != null) {
                 return reminderObj;
@@ -578,7 +568,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
         }
     }
 
-    public ReminderObject result2Object(ResultSet result, PreparedStatement stmt, boolean closeStatements) throws SQLException, ReminderException {
+    public static ReminderObject result2Object(Context ctx, ResultSet result, PreparedStatement stmt, boolean closeStatements) throws SQLException, ReminderException {
         try {
             if (result.next()) {
                 int pos = 1;
@@ -594,7 +584,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
                 reminderObj.setLastModified(new Date(result.getLong(pos++)));
                 return reminderObj;
             }
-            throw new ReminderException(Code.NOT_FOUND, I(-1), I(context.getContextId()));
+            throw new ReminderException(Code.NOT_FOUND, I(-1), I(ctx.getContextId()));
         } finally {
             if (closeStatements) {
                 closeSQLStuff(result, stmt);
@@ -616,7 +606,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             ps.setInt(pos++, module);
             ps.setString(pos++, String.valueOf(targetId));
             final ResultSet rs = ps.executeQuery();
-            return new ReminderSearchIterator(rs, ps, con);
+            return new ReminderSearchIterator(context, ps, rs, con);
         } catch (SQLException e) {
             throw new ReminderException(Code.SQL_ERROR, e, e.getMessage());
         } catch (SearchIteratorException e) {
@@ -624,26 +614,9 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
         }
     }
 
-    public ReminderSearchIterator listReminder(final int userId, final Date end) throws OXException {
-        Connection readCon = null;
-
-        try {
-            readCon = DBPool.pickup(context);
-
-            final PreparedStatement ps = readCon.prepareStatement(SQL.sqlRange);
-            ps.setInt(1, context.getContextId());
-            ps.setInt(2, userId);
-            ps.setTimestamp(3, new Timestamp(end.getTime()));
-
-            final ResultSet rs = ps.executeQuery();
-            return new ReminderSearchIterator(rs, ps, readCon);
-        } catch (final SearchIteratorException exc) {
-            throw new OXException(exc);
-        } catch (final SQLException exc) {
-            throw new OXException(EnumComponent.REMINDER, Category.CODE_ERROR, -1, "SQL Problem.", exc);
-        } catch (final DBPoolingException exc) {
-            throw new OXException(exc);
-        }
+    public SearchIterator<ReminderObject> getArisingReminder(Session session, Context ctx, final User user, final Date end) throws OXException {
+        GetArisingReminder arising = new GetArisingReminder(session, ctx, user, end);
+        return arising.loadWithIterator();
     }
 
     public SearchIterator<ReminderObject> listModifiedReminder(final int userId, final Date lastModified) throws OXException {
@@ -658,7 +631,7 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             ps.setTimestamp(3, new Timestamp(lastModified.getTime()));
 
             final ResultSet rs = ps.executeQuery();
-                return new ReminderSearchIterator(rs, ps, readCon);
+                return new ReminderSearchIterator(context, ps, rs, readCon);
         } catch (final SearchIteratorException exc) {
             throw new OXException(exc);
         } catch (final SQLException exc) {
@@ -667,83 +640,4 @@ public class ReminderHandler implements Types, ReminderSQLInterface {
             throw new OXException(exc);
         }
     }
-
-    private class ReminderSearchIterator implements SearchIterator<ReminderObject> {
-
-        private ReminderObject next;
-
-        private final ResultSet rs;
-
-        private final PreparedStatement preparedStatement;
-
-        private final Connection readCon;
-
-        private final List<AbstractOXException> warnings;
-
-        private ReminderSearchIterator(final ResultSet rs, final PreparedStatement preparedStatement, final Connection readCon) throws SearchIteratorException {
-            this.warnings =  new ArrayList<AbstractOXException>(2);
-            this.rs = rs;
-            this.readCon = readCon;
-            this.preparedStatement = preparedStatement;
-            try {
-                next = result2Object(rs, preparedStatement, false);
-            } catch (final ReminderException exc) {
-                next = null;
-            } catch (final SQLException exc) {
-                throw new SearchIteratorException(SearchIteratorCode.SQL_ERROR, exc, EnumComponent.REMINDER);
-            }
-        }
-
-        public boolean hasNext() {
-            return next != null;
-        }
-
-        public ReminderObject next() throws SearchIteratorException {
-            final ReminderObject reminderObj = next;
-            try {
-                next = result2Object(rs, preparedStatement, false);
-            } catch (final ReminderException exc) {
-                next = null;
-            } catch (final SQLException exc) {
-                throw new SearchIteratorException(SearchIteratorCode.SQL_ERROR, exc, EnumComponent.REMINDER);
-            }
-            return reminderObj;
-        }
-
-        public void close() throws SearchIteratorException {
-            try {
-                if (rs != null) {
-                    rs.close();
-                }
-
-                if (preparedStatement != null) {
-                    preparedStatement.close();
-                }
-
-                DBPool.closeReaderSilent(context,readCon);
-            } catch (final SQLException exc) {
-                throw new SearchIteratorException(SearchIteratorCode.SQL_ERROR, exc, EnumComponent.REMINDER);
-            }
-        }
-
-        public int size() {
-            throw new UnsupportedOperationException("Method size() not implemented");
-        }
-
-        public boolean hasSize() {
-            return false;
-        }
-
-        public void addWarning(final AbstractOXException warning) {
-            warnings.add(warning);
-        }
-
-        public AbstractOXException[] getWarnings() {
-            return warnings.isEmpty() ? null : warnings.toArray(new AbstractOXException[warnings.size()]);
-        }
-
-        public boolean hasWarnings() {
-            return !warnings.isEmpty();
-        }
-    }
 }
