@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -67,8 +67,7 @@ public final class SQL {
         "UPDATE reminder SET alarm=?,recurrence=?,description=?,last_modified=?,"
         + "folder=? WHERE cid=? AND target_id=? AND module=? AND userid=?";
 
-    public static final String sqlDeleteWithId =
-        "DELETE FROM reminder WHERE cid=? AND object_id=?";
+    public static final String DELETE_WITH_ID = "DELETE FROM reminder WHERE cid=? AND object_id=?";
 
     public static final String sqlDelete =
         "DELETE FROM reminder WHERE cid=? AND target_id=? AND module=? AND userid=?";
@@ -94,9 +93,7 @@ public final class SQL {
         "SELECT object_id,target_id,module,userid,alarm,recurrence,description,"
         + "folder,last_modified FROM reminder WHERE cid=? AND module=? AND target_id=?";
 
-    public static final String sqlRange =
-        "SELECT object_id,target_id,module,userid,alarm,recurrence,description,"
-        + "folder,last_modified FROM reminder WHERE cid=? AND userid=? AND alarm<=?";
+    public static final String SELECT_RANGE = "SELECT object_id,target_id,module,userid,alarm,recurrence,description,folder,last_modified FROM reminder WHERE cid=? AND userid=? AND alarm<=?";
 
     public static final String sqlModified =
         "SELECT object_id,target_id,module,userid,alarm,recurrence,description,"
