@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,6 +57,7 @@ import com.openexchange.groupware.settings.IValueHandler;
 import com.openexchange.groupware.settings.PreferencesItemService;
 import com.openexchange.groupware.settings.Setting;
 import com.openexchange.groupware.settings.SettingException;
+import com.openexchange.groupware.settings.SharedNode;
 import com.openexchange.groupware.settings.SettingException.Code;
 
 /**
@@ -258,8 +259,6 @@ public final class ConfigTree {
             com.openexchange.groupware.settings.tree.modules.infostore.Module.class,
             com.openexchange.groupware.settings.tree.modules.interfaces.ICal.class,
             com.openexchange.groupware.settings.tree.modules.interfaces.SyncML.class,
-            com.openexchange.groupware.settings.tree.modules.interfaces.EAS.class,
-            com.openexchange.groupware.settings.tree.modules.interfaces.USM.class,
             com.openexchange.groupware.settings.tree.modules.interfaces.VCard.class,
             com.openexchange.groupware.settings.tree.modules.mail.Addresses.class,
             com.openexchange.groupware.settings.tree.modules.mail.AllowHTMLImages.class,
@@ -288,6 +287,7 @@ public final class ConfigTree {
             com.openexchange.groupware.settings.tree.modules.mailaccount.Module.class,
             com.openexchange.groupware.settings.tree.modules.passwordchange.Module.class,
             com.openexchange.groupware.settings.tree.modules.personaldata.Module.class,
+            com.openexchange.groupware.settings.tree.modules.personaldata.InternalUserEditEnabled.class,
             com.openexchange.groupware.settings.tree.modules.portal.GUI.class,
             com.openexchange.groupware.settings.tree.modules.portal.Module.class,
             com.openexchange.groupware.settings.tree.modules.tasks.GUI.class,
