@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -94,7 +94,7 @@ public class DefaultStatusPublic implements PreferencesItemService {
                 return true;
             }
 
-            public void writeValue(final Context ctx, final User user, final Setting setting) throws SettingException {
+            public void writeValue(final Session session, final Context ctx, final User user, final Setting setting) throws SettingException {
                 Integer value;
                 try {
                     value = new Integer(String.valueOf(setting.getSingleValue()));
