@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,12 +49,12 @@
 
 package com.openexchange.groupware.tasks;
 
+import static com.openexchange.java.Autoboxing.I;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.EnumMap;
 import java.util.List;
 import java.util.Map;
-
 import com.openexchange.groupware.search.TaskSearchObject;
 import com.openexchange.groupware.tasks.TaskException.Code;
 
@@ -225,7 +225,9 @@ public final class SQL {
             final Mapper<?> mapper = Mapping.getMapping(i);
             if (null == mapper) {
                 switch (i) {
+                case Task.LAST_MODIFIED_OF_NEWEST_ATTACHMENT:
                 case Task.PARTICIPANTS:
+                case Task.USERS:
                 case Task.ALARM:
                     break;
                 case Task.FOLDER_ID:
@@ -234,7 +236,7 @@ public final class SQL {
                     }
                     break;
                 default:
-                    throw new TaskException(Code.UNKNOWN_ATTRIBUTE, Integer.valueOf(i));
+                    throw new TaskException(Code.UNKNOWN_ATTRIBUTE, I(i));
                 }
             } else {
                 sql.append(mapper.getDBColumnName());
