@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.groupware.update.tasks;
 
+import static com.openexchange.java.Autoboxing.I;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.sql.Connection;
@@ -59,35 +60,24 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.database.DBPoolingException;
 import com.openexchange.databaseold.Database;
 import com.openexchange.groupware.AbstractOXException;
-import com.openexchange.groupware.EnumComponent;
-import com.openexchange.groupware.OXExceptionSource;
 import com.openexchange.groupware.OXThrowsMultiple;
 import com.openexchange.groupware.update.Schema;
+import com.openexchange.groupware.update.UpdateException;
+import com.openexchange.groupware.update.UpdateExceptionCodes;
 import com.openexchange.groupware.update.UpdateTask;
-import com.openexchange.groupware.update.exception.Classes;
-import com.openexchange.groupware.update.exception.SchemaException;
-import com.openexchange.groupware.update.exception.SchemaExceptionFactory;
 import com.openexchange.tools.file.FileStorageException;
 import com.openexchange.tools.file.LocalFileStorage;
 
-@OXExceptionSource(
-	    classId = Classes.UPDATE_TASK,
-	    component = EnumComponent.UPDATE
-	)
 public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
 
     private final ThreadLocal<Map<Integer,LocalFileStorage>> filestorages = new ThreadLocal<Map<Integer,LocalFileStorage>>();
 
     private static final Log LOG = LogFactory.getLog(ClearLeftoverAttachmentsUpdateTask.class);
-    private static final SchemaExceptionFactory EXCEPTIONS =
-        new SchemaExceptionFactory(ClearLeftoverAttachmentsUpdateTask.class);
 
     public int addedWithVersion() {
         return 11;
@@ -97,86 +87,91 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
         return UpdateTaskPriority.NORMAL.priority;
     }
 
-    @OXThrowsMultiple(category = { AbstractOXException.Category.CODE_ERROR,AbstractOXException.Category.SETUP_ERROR }, desc = { "" }, exceptionId = { 1,2 }, msg = { "An SQL error occurred while performing task ClearLeftoverAttachmentsUpdateTask: %1$s.", "Can't resolve filestore." })
+    @OXThrowsMultiple(
+        category = { AbstractOXException.Category.CODE_ERROR,AbstractOXException.Category.SETUP_ERROR },
+        desc = { "" },
+        exceptionId = { 1,2 },
+        msg = { "An SQL error occurred while performing task ClearLeftoverAttachmentsUpdateTask: %1$s.", "Can't resolve filestore." }
+    )
     public void perform(final Schema schema, final int contextId) throws AbstractOXException {
         try {
             filestorages.set(new HashMap<Integer,LocalFileStorage>());
-            for(final LeftoverAttachment att : getLeftoverAttachmentsInSchema(contextId, schema)){
+            for(final LeftoverAttachment att : getLeftoverAttachmentsInSchema(contextId)){
                 removeFile(att.getFileId(), att.getContextId()); //FIXME will not work during update
                 try {
                     removeDatabaseEntry(att.getId(),att.getContextId());
-                } catch (final SQLException e) {
-                    throw EXCEPTIONS.create(1, e, e.getMessage());
+                } catch (SQLException e) {
+                    throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
                 }
             }
-        } catch (final SQLException e) {
-            throw EXCEPTIONS.create(1, e, e.getMessage());
+        } catch (SQLException e) {
+            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
         } finally {
             filestorages.set(null);
         }
     }
 
     private void removeDatabaseEntry(final int id, final int contextId) throws DBPoolingException, SQLException {
-        update(contextId, "DELETE FROM prg_attachment WHERE id = ? and cid = ?", id, contextId);
+        update(contextId, "DELETE FROM prg_attachment WHERE id = ? and cid = ?", I(id), I(contextId));
     }
 
     private void update(final int contextId, final String sql, final Object...args) throws DBPoolingException, SQLException {
         Connection writeCon = null;
-		PreparedStatement stmt = null;
+        PreparedStatement stmt = null;
 
-		try {
-			writeCon = Database.get(contextId, true);
-			writeCon.setAutoCommit(false);
-			stmt = writeCon.prepareStatement(sql);
-			for(int i = 0; i < args.length; i++) {
+        try {
+            writeCon = Database.get(contextId, true);
+            writeCon.setAutoCommit(false);
+            stmt = writeCon.prepareStatement(sql);
+            for(int i = 0; i < args.length; i++) {
                 stmt.setObject(i+1, args[i]);
             }
             stmt.executeUpdate();
 
 
-		} catch (final SQLException x) {
-			try {
-				writeCon.rollback();
-			} catch (final SQLException x2) {
-				LOG.error("Can't execute rollback.", x2);
-			}
-			throw x;
-		} finally {
-			if(stmt != null) {
-				try {
-					stmt.close();
-				} catch (final SQLException x) {
-					LOG.warn("Couldn't close statement", x);
-				}
-			}
+        } catch (final SQLException x) {
+            try {
+                writeCon.rollback();
+            } catch (final SQLException x2) {
+                LOG.error("Can't execute rollback.", x2);
+            }
+            throw x;
+        } finally {
+            if(stmt != null) {
+                try {
+                    stmt.close();
+                } catch (final SQLException x) {
+                    LOG.warn("Couldn't close statement", x);
+                }
+            }
 
-			if(writeCon != null) {
-				try {
-					writeCon.setAutoCommit(true);
-				} catch (final SQLException x){
-					LOG.warn("Can't reset auto commit", x);
-				}
+            if(writeCon != null) {
+                try {
+                    writeCon.setAutoCommit(true);
+                } catch (final SQLException x){
+                    LOG.warn("Can't reset auto commit", x);
+                }
 
-				if(writeCon != null) {
-					Database.back(contextId, true, writeCon);
-				}
-			}   
+                if(writeCon != null) {
+                    Database.back(contextId, true, writeCon);
+                }
+            }   
         }
     }
 
-    private void removeFile(final String fileId,final int ctx_id) throws SQLException, DBPoolingException, FileStorageException, SchemaException {
+    private void removeFile(final String fileId,final int ctx_id) throws SQLException, DBPoolingException, FileStorageException, UpdateException {
         // We have to use the local file storage to bypass quota handling, which must remain
         // unaffected by these operations
 
-        LocalFileStorage fs = filestorages.get().get(ctx_id);
+        LocalFileStorage fs = filestorages.get().get(I(ctx_id));
         if(fs == null) {
             final URI uri = createURI(ctx_id);
-            if(uri == null) {
-                throw EXCEPTIONS.create(2);
+            if (uri == null) {
+                throw UpdateExceptionCodes.OTHER_PROBLEM.create("Can not determine filestore for context " + ctx_id + ".");
             }
 
-            fs = new LocalFileStorage(3,256,uri);  //FIXME: It's very dangerous to just copy these values (3 and 256)!
-            filestorages.get().put(ctx_id, fs);
+            fs = new LocalFileStorage(I(3),I(256),uri);  //FIXME: It's very dangerous to just copy these values (3 and 256)!
+            filestorages.get().put(I(ctx_id), fs);
         }
         try {
             fs.deleteFile(fileId);
@@ -190,11 +185,11 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
         // We can't use the API, because the ContextStorage will throw exceptions
         // when we try to load a Context during the update process;
         Connection readCon = null;
-		PreparedStatement stmt = null;
+        PreparedStatement stmt = null;
         ResultSet rs = null;
 
         try {
-			readCon = Database.get(false);
+            readCon = Database.get(false);
             stmt = readCon.prepareStatement("SELECT filestore_id, filestore_name FROM context WHERE cid = ?");
             stmt.setInt(1, ctx_id);
 
@@ -233,13 +228,13 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
             LOG.error(e);
             return null;
         } finally {
-			if(stmt != null) {
-				try {
-					stmt.close();
-				} catch (final SQLException x) {
-					LOG.warn("Couldn't close statement", x);
-				}
-			}
+            if(stmt != null) {
+                try {
+                    stmt.close();
+                } catch (final SQLException x) {
+                    LOG.warn("Couldn't close statement", x);
+                }
+            }
 
             if(rs != null) {
                 try {
@@ -250,14 +245,14 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
             }
 
             if(readCon != null) {
-				if(readCon != null) {
-					Database.back(false, readCon);
-				}
-			}
+                if(readCon != null) {
+                    Database.back(false, readCon);
+                }
+            }
         }
     }
 
-    private List<LeftoverAttachment> getLeftoverAttachmentsInSchema(final int contextId, final Schema schema) throws SQLException, DBPoolingException {
+    private List<LeftoverAttachment> getLeftoverAttachmentsInSchema(final int contextId) throws SQLException, DBPoolingException {
 
         final String query = "SELECT prg_attachment.cid, prg_attachment.id, prg_attachment.file_id FROM prg_attachment " +
                 "JOIN sequence_attachment ON prg_attachment.cid = sequence_attachment.cid  WHERE prg_attachment.id > sequence_attachment.id";
@@ -265,11 +260,11 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
         final List<LeftoverAttachment> attachments = new ArrayList<LeftoverAttachment>();
         
         Connection readCon = null;
-		PreparedStatement stmt = null;
+        PreparedStatement stmt = null;
         ResultSet rs = null;
 
         try {
-			readCon = Database.get(contextId, false);
+            readCon = Database.get(contextId, false);
             stmt = readCon.prepareStatement(query);
             rs = stmt.executeQuery();
             while(rs.next()) {
@@ -278,13 +273,13 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
 
             return attachments;
         } finally {
-			if(stmt != null) {
-				try {
-					stmt.close();
-				} catch (final SQLException x) {
-					LOG.warn("Couldn't close statement", x);
-				}
-			}
+            if(stmt != null) {
+                try {
+                    stmt.close();
+                } catch (final SQLException x) {
+                    LOG.warn("Couldn't close statement", x);
+                }
+            }
 
             if(rs != null) {
                 try {
@@ -295,10 +290,10 @@ public class ClearLeftoverAttachmentsUpdateTask implements UpdateTask {
             }
 
             if(readCon != null) {
-				if(readCon != null) {
-					Database.back(contextId, false, readCon);
-				}
-			}
+                if(readCon != null) {
+                    Database.back(contextId, false, readCon);
+                }
+            }
         }
     }
 
