@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,6 +52,8 @@ package com.openexchange.groupware.update.tasks;
 import static com.openexchange.tools.sql.DBUtils.autocommit;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
 import static com.openexchange.tools.sql.DBUtils.rollback;
+import gnu.trove.TIntObjectHashMap;
+import gnu.trove.TIntObjectProcedure;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -70,8 +72,6 @@ import com.openexchange.database.DBPoolingException;
 import com.openexchange.databaseold.Database;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.EnumComponent;
-import com.openexchange.groupware.OXExceptionSource;
-import com.openexchange.groupware.OXThrowsMultiple;
 import com.openexchange.groupware.AbstractOXException.Category;
 import com.openexchange.groupware.calendar.CalendarCache;
 import com.openexchange.groupware.contact.Contacts;
@@ -81,10 +81,9 @@ import com.openexchange.groupware.contexts.impl.ContextImpl;
 import com.openexchange.groupware.i18n.FolderStrings;
 import com.openexchange.groupware.update.PerformParameters;
 import com.openexchange.groupware.update.ProgressState;
+import com.openexchange.groupware.update.UpdateException;
+import com.openexchange.groupware.update.UpdateExceptionCodes;
 import com.openexchange.groupware.update.UpdateTaskAdapter;
-import com.openexchange.groupware.update.exception.Classes;
-import com.openexchange.groupware.update.exception.UpdateException;
-import com.openexchange.groupware.update.exception.UpdateExceptionFactory;
 import com.openexchange.i18n.LocaleTools;
 import com.openexchange.i18n.tools.StringHelper;
 import com.openexchange.tools.Collections.SmartIntArray;
@@ -96,24 +95,28 @@ import com.openexchange.tools.oxfolder.OXFolderSQL;
  * 
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-@OXExceptionSource(classId = Classes.UPDATE_TASK, component = EnumComponent.UPDATE)
 public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAdapter {
 
-    /**
-     * Initializes a new {@link DuplicateContactCollectFolderRemoverTask}.
-     */
+    private static final String[] DEPENDENCIES = { MailAccountAddPersonalTask.class.getName() };
+
     public DuplicateContactCollectFolderRemoverTask() {
         super();
     }
 
+    @Override
     public int addedWithVersion() {
         return 98;
     }
 
+    @Override
     public int getPriority() {
         return UpdateTaskPriority.HIGH.priority;
     }
 
+    public String[] getDependencies() {
+        return DEPENDENCIES;
+    }
+
     public void perform(final PerformParameters params) throws AbstractOXException {
         /*
          * Logger
@@ -126,29 +129,32 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
         /*
          * Get all contexts with contained users
          */
-        final Map<Integer, List<Integer>> m = new HashMap<Integer, List<Integer>>();
+        final TIntObjectHashMap<List<Integer>> m = new TIntObjectHashMap<List<Integer>>();
         final int total = getAllUsers(params.getContextId(), m);
         status.setTotal(total);
         /*
          * Iterate per context
          */
         final Map<Locale, String> names = new HashMap<Locale, String>(4);
-        for (final Map.Entry<Integer, List<Integer>> me : m.entrySet()) {
-            final int currentContextId = me.getKey().intValue();
-            try {
-                iterateUsersPerContext(me.getValue(), names, currentContextId, status, log);
-            } catch (final AbstractOXException e) {
-                final StringBuilder sb = new StringBuilder(128);
-                sb.append("DuplicateContactCollectFolderRemoverTask experienced an error while removing duplicate contact collect folders for users in context ");
-                sb.append(currentContextId);
-                sb.append(":\n");
-                sb.append(e.getMessage());
-                log.error(sb.toString(), e);
+        m.forEachEntry(new TIntObjectProcedure<List<Integer>>() {
+            
+            public boolean execute(final int currentContextId, final List<Integer> list) {
+                try {
+                    iterateUsersPerContext(list, names, currentContextId, status, log);
+                } catch (final AbstractOXException e) {
+                    final StringBuilder sb = new StringBuilder(128);
+                    sb.append("DuplicateContactCollectFolderRemoverTask experienced an error while removing duplicate contact collect folders for users in context ");
+                    sb.append(currentContextId);
+                    sb.append(":\n");
+                    sb.append(e.getMessage());
+                    log.error(sb.toString(), e);
+                }
+                return true;
             }
-        }
+        });
     }
 
-    private static int getAllUsers(final int contextId, final Map<Integer, List<Integer>> m) throws UpdateException {
+    private static int getAllUsers(final int contextId, final TIntObjectHashMap<List<Integer>> m) throws UpdateException {
         final Connection con;
         try {
             con = Database.getNoTimeout(contextId, true);
@@ -165,7 +171,7 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
             }
             int total = 0;
             do {
-                final Integer cid = Integer.valueOf(rs.getInt(1));
+                final int cid = rs.getInt(1);
                 final Integer user = Integer.valueOf(rs.getInt(2));
                 final List<Integer> l;
                 if (!m.containsKey(cid)) {
@@ -179,14 +185,14 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
             } while (rs.next());
             return total;
         } catch (final SQLException e) {
-            throw createSQLError(e);
+            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
         } finally {
             closeSQLStuff(rs, stmt);
             Database.backNoTimeout(contextId, true, con);
         }
     }
 
-    private static void iterateUsersPerContext(final List<Integer> users, final Map<Locale, String> names, final int contextId, final ProgressState status, final Log log) throws UpdateException {
+    static void iterateUsersPerContext(final List<Integer> users, final Map<Locale, String> names, final int contextId, final ProgressState status, final Log log) throws UpdateException {
         /*
          * Create context instance
          */
@@ -211,7 +217,7 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
                 throw new UpdateException(e);
             } catch (final SQLException e) {
                 // Auto-Commit mode could not be changed
-                throw createSQLError(e);
+                throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
             }
             try {
                 checkDuplicates4User(user, ctx, names, writeCon, log);
@@ -219,7 +225,7 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
                 writeCon.commit(); // COMMIT
             } catch (final SQLException e) {
                 rollback(writeCon);
-                throw createSQLError(e);
+                throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
             }
             /*
              * catch (final UpdateException e) { rollback(writeCon); throw e; }
@@ -366,7 +372,7 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
                 }
                 return -1;
             } catch (final SQLException e) {
-                throw createSQLError(e);
+                throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
             } finally {
                 closeSQLStuff(rs, stmt);
             }
@@ -515,10 +521,4 @@ public final class DuplicateContactCollectFolderRemoverTask extends UpdateTaskAd
             log.error(e.getMessage(), e);
         }
     }
-
-    @OXThrowsMultiple(category = { Category.CODE_ERROR }, desc = { "" }, exceptionId = { 1 }, msg = { "A SQL error occurred while performing task DuplicateContactCollectFolderRemoverTask: %1$s." })
-    private static UpdateException createSQLError(final SQLException e) {
-        return new UpdateExceptionFactory(DuplicateContactCollectFolderRemoverTask.class).create(1, e, e.getMessage());
-    }
-
 }
