@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,10 +53,8 @@ import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-
 import com.openexchange.databaseold.Database;
 import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.EnumComponent;
@@ -65,16 +63,16 @@ import com.openexchange.groupware.OXThrows;
 import com.openexchange.groupware.update.Schema;
 import com.openexchange.groupware.update.UpdateTask;
 import com.openexchange.groupware.update.exception.Classes;
-import com.openexchange.groupware.update.exception.SchemaExceptionFactory;
+import com.openexchange.groupware.update.exception.UpdateExceptionFactory;
+
 @OXExceptionSource(
-	    classId = Classes.UPDATE_TASK,
-	    component = EnumComponent.UPDATE
-	)
+        classId = Classes.UPDATE_TASK,
+        component = EnumComponent.UPDATE
+    )
 public class InfostoreLongerURLFieldTask  implements UpdateTask {
 
     private final Log LOG = LogFactory.getLog(InfostoreLongerURLFieldTask.class);
-    private static final SchemaExceptionFactory EXCEPTIONS =
-        new SchemaExceptionFactory(InfostoreLongerURLFieldTask.class);
+    private static final UpdateExceptionFactory EXCEPTIONS = new UpdateExceptionFactory(InfostoreLongerURLFieldTask.class);
 
     public int addedWithVersion() {
         return 12;
