@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -68,9 +68,10 @@ import com.openexchange.groupware.AbstractOXException.Category;
 import com.openexchange.groupware.container.FolderObject;
 import com.openexchange.groupware.update.PerformParameters;
 import com.openexchange.groupware.update.ProgressState;
+import com.openexchange.groupware.update.UpdateException;
+import com.openexchange.groupware.update.UpdateExceptionCodes;
 import com.openexchange.groupware.update.UpdateTaskAdapter;
 import com.openexchange.groupware.update.exception.Classes;
-import com.openexchange.groupware.update.exception.UpdateException;
 import com.openexchange.groupware.update.exception.UpdateExceptionFactory;
 
 /**
@@ -93,14 +94,22 @@ public class RemoveAdminPermissionOnInfostoreTask extends UpdateTaskAdapter {
         exceptionFactory = new UpdateExceptionFactory(RemoveAdminPermissionOnInfostoreTask.class);
     }
 
+    @Override
     public int addedWithVersion() {
         return 76;
     }
 
+    @Override
     public int getPriority() {
         return UpdateTaskPriority.HIGH.priority;
     }
 
+    private static final String[] DEPENDENCIES = { ContactsAddIndex4AutoCompleteSearch.class.getName() };
+
+    public String[] getDependencies() {
+        return DEPENDENCIES;
+    }
+
     public void perform(final PerformParameters params) throws AbstractOXException {
         final int triggeringContextId = params.getContextId();
         final int[] ctxIds = Database.getContextsInSameSchema(triggeringContextId);
@@ -134,7 +143,7 @@ public class RemoveAdminPermissionOnInfostoreTask extends UpdateTaskAdapter {
             con.commit();
         } catch (SQLException e) {
             rollback(con);
-            throw createSQLError(e);
+            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
         } finally {
             autocommit(con);
             Database.backNoTimeout(triggeringContextId, true, con);
@@ -160,17 +169,12 @@ public class RemoveAdminPermissionOnInfostoreTask extends UpdateTaskAdapter {
             stmt.setInt(3, mailAdmin);
             stmt.executeUpdate();
         } catch (final SQLException e) {
-            throw createSQLError(e);
+            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
         } finally {
             closeSQLStuff(stmt);
         }
     }
 
-    @OXThrowsMultiple(category = { Category.CODE_ERROR }, desc = { "" }, exceptionId = { 1 }, msg = { "A SQL error occurred while performing task RemoveAdminPermissionOnInfostoreTask: %1$s." })
-    private UpdateException createSQLError(final SQLException e) {
-        return exceptionFactory.create(1, e, e.getMessage());
-    }
-
     @OXThrowsMultiple(category = { Category.CODE_ERROR }, desc = { "" }, exceptionId = { 2 }, msg = { "Error while performing task RemoveAdminPermissionOnInfostoreTask: No context admin exists for context %1$s." })
     private UpdateException missingAdminError(final int contextId) {
         return exceptionFactory.create(2, Integer.valueOf(contextId));
@@ -188,7 +192,7 @@ public class RemoveAdminPermissionOnInfostoreTask extends UpdateTaskAdapter {
             }
             return -1;
         } catch (final SQLException e) {
-            throw createSQLError(e);
+            throw UpdateExceptionCodes.SQL_PROBLEM.create(e, e.getMessage());
         } finally {
             closeSQLStuff(rs, stmt);
         }
