@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,7 +49,6 @@
 
 package com.openexchange.groupware.update.tools;
 
-import java.util.Collection;
 import java.util.List;
 
 /**
@@ -228,14 +227,4 @@ public final class Utility {
         }
         sb.append('\n');
     }
-
-    private static int getMaxLen(final Collection<? extends Object> c, final int startLen) {
-        int max = startLen;
-        for (final Object obj : c) {
-            final int b = obj.toString().length();
-            max = (max >= b) ? max : b;
-        }
-        return max;
-    }
-
 }
