@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,8 +51,9 @@ package com.openexchange.image.internal;
 
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
+import com.openexchange.conversion.ConversionService;
 import com.openexchange.conversion.DataArguments;
-import com.openexchange.conversion.DataSource;
+import com.openexchange.image.ImageDataSource;
 import com.openexchange.server.services.ServerServiceRegistry;
 import com.openexchange.session.Session;
 import com.openexchange.timer.ScheduledTimerTask;
@@ -101,8 +102,12 @@ public final class ImageRegistry {
 
     private final ConcurrentMap<String, ConcurrentMap<String, ImageData>> sessionBoundImagesMap;
 
+    private final Object sessionLock;
+
     private final ConcurrentMap<Integer, ConcurrentMap<String, ImageData>> contextBoundImagesMap;
 
+    private final Object contextLock;
+
     private ScheduledTimerTask[] tasks;
 
     /**
@@ -111,7 +116,9 @@ public final class ImageRegistry {
     private ImageRegistry() {
         super();
         sessionBoundImagesMap = new ConcurrentHashMap<String, ConcurrentMap<String, ImageData>>();
+        sessionLock = new Object();
         contextBoundImagesMap = new ConcurrentHashMap<Integer, ConcurrentMap<String, ImageData>>();
+        contextLock = new Object();
     }
 
     /**
@@ -179,7 +186,7 @@ public final class ImageRegistry {
      * @param imageArguments The image arguments
      * @return Either the new image data from specified data source and data arguments or the existing one if already contained in registry.
      */
-    public ImageData addImageData(final Session session, final DataSource imageSource, final DataArguments imageArguments) {
+    public ImageData addImageData(final Session session, final ImageDataSource imageSource, final DataArguments imageArguments) {
         return addImageData(session, imageSource, imageArguments, ImageData.DEFAULT_TTL);
     }
 
@@ -193,7 +200,7 @@ public final class ImageRegistry {
      * @param timeToLive The time-to-live for the new image data
      * @return Either the new image data from specified data source and data arguments or the existing one if already contained in registry.
      */
-    public ImageData addImageData(final Session session, final DataSource imageSource, final DataArguments imageArguments, final int timeToLive) {
+    public ImageData addImageData(final Session session, final ImageDataSource imageSource, final DataArguments imageArguments, final int timeToLive) {
         final String sessionId = session.getSessionID();
         ConcurrentMap<String, ImageData> m = sessionBoundImagesMap.get(sessionId);
         boolean check = true;
@@ -206,14 +213,14 @@ public final class ImageRegistry {
             }
         }
         ImageData imageData;
-        final String id = imageArguments.getID();
+        final String id = ImageIDGenerator.generateId(imageSource, imageArguments);
         if (check && (imageData = m.get(id)) != null) {
             if (DEBUG) {
                 LOG.debug("Image data fetched from registry for UID: " + id);
             }
             return imageData.touch();
         }
-        imageData = new ImageData(imageSource, imageArguments, timeToLive);
+        imageData = new ImageData(imageSource, imageArguments, timeToLive, id);
         m.put(id, imageData);
         if (DEBUG) {
             LOG.debug("Image data put into registry with UID: " + id);
@@ -230,7 +237,7 @@ public final class ImageRegistry {
      * @param imageArguments The image arguments
      * @return Either the new image data from specified data source and data arguments or the existing one if already contained in registry.
      */
-    public ImageData addImageData(final int contextId, final DataSource imageSource, final DataArguments imageArguments) {
+    public ImageData addImageData(final int contextId, final ImageDataSource imageSource, final DataArguments imageArguments) {
         return addImageData(contextId, imageSource, imageArguments, ImageData.DEFAULT_TTL);
     }
 
@@ -244,7 +251,7 @@ public final class ImageRegistry {
      * @param timeToLive The time-to-live for the new image data
      * @return Either the new image data from specified data source and data arguments or the existing one if already contained in registry.
      */
-    public ImageData addImageData(final int contextId, final DataSource imageSource, final DataArguments imageArguments, final int timeToLive) {
+    public ImageData addImageData(final int contextId, final ImageDataSource imageSource, final DataArguments imageArguments, final int timeToLive) {
         final Integer cid = Integer.valueOf(contextId);
         ConcurrentMap<String, ImageData> m = contextBoundImagesMap.get(cid);
         boolean check = true;
@@ -257,14 +264,14 @@ public final class ImageRegistry {
             }
         }
         ImageData imageData;
-        final String id = imageArguments.getID();
+        final String id = ImageIDGenerator.generateId(imageSource, imageArguments);
         if (check && (imageData = m.get(id)) != null) {
             if (DEBUG) {
                 LOG.debug("Image data fetched from registry for UID: " + id);
             }
             return imageData.touch();
         }
-        imageData = new ImageData(imageSource, imageArguments, timeToLive);
+        imageData = new ImageData(imageSource, imageArguments, timeToLive, id);
         m.put(id, imageData);
         if (DEBUG) {
             LOG.debug("Image data put into registry with UID: " + id);
@@ -282,7 +289,7 @@ public final class ImageRegistry {
         final Integer cid = Integer.valueOf(contextId);
         ConcurrentMap<String, ImageData> m = contextBoundImagesMap.get(cid);
         if (m == null) {
-            synchronized (contextBoundImagesMap) {
+            synchronized (contextLock) {
                 m = contextBoundImagesMap.get(cid);
                 if (m == null) {
                     m = new ConcurrentHashMap<String, ImageData>();
@@ -303,7 +310,7 @@ public final class ImageRegistry {
         final String sessionId = session.getSessionID();
         ConcurrentMap<String, ImageData> m = sessionBoundImagesMap.get(sessionId);
         if (m == null) {
-            synchronized (sessionBoundImagesMap) {
+            synchronized (sessionLock) {
                 m = sessionBoundImagesMap.get(sessionId);
                 if (m == null) {
                     m = new ConcurrentHashMap<String, ImageData>();
@@ -395,15 +402,27 @@ public final class ImageRegistry {
     public ImageData getImageData(final Session session, final String uniqueId) {
         final ConcurrentMap<String, ImageData> m = sessionBoundImagesMap.get(session.getSessionID());
         if (m == null) {
-            return null;
+            return onMissingImageData(session, uniqueId);
         }
         final ImageData imageData = m.get(uniqueId);
         if (imageData == null) {
-            return null;
+            return onMissingImageData(session, uniqueId);
         }
         return imageData.touch();
     }
 
+    private ImageData onMissingImageData(final Session session, final String uniqueId) {
+        final ConversionService service = ServerServiceRegistry.getInstance().getService(ConversionService.class);
+        if (null == service) {
+            return null;
+        }
+        final Object[] objects = ImageIDGenerator.parseId(uniqueId, service);
+        if (null == objects) {
+            return null;
+        }
+        return addImageData(session, (ImageDataSource) objects[0], (DataArguments) objects[1]);
+    }
+
     /**
      * Gets the image data bound to specified context and registered to specified unique ID.
      * 
@@ -414,15 +433,27 @@ public final class ImageRegistry {
     public ImageData getImageData(final int contextId, final String uniqueId) {
         final ConcurrentMap<String, ImageData> m = contextBoundImagesMap.get(Integer.valueOf(contextId));
         if (m == null) {
-            return null;
+            return onMissingImageData(contextId, uniqueId);
         }
         final ImageData imageData = m.get(uniqueId);
         if (imageData == null) {
-            return null;
+            return onMissingImageData(contextId, uniqueId);
         }
         return imageData.touch();
     }
 
+    private ImageData onMissingImageData(final int contextId, final String uniqueId) {
+        final ConversionService service = ServerServiceRegistry.getInstance().getService(ConversionService.class);
+        if (null == service) {
+            return null;
+        }
+        final Object[] objects = ImageIDGenerator.parseId(uniqueId, service);
+        if (null == objects) {
+            return null;
+        }
+        return addImageData(contextId, (ImageDataSource) objects[0], (DataArguments) objects[1]);
+    }
+
     /**
      * Clears the registry.
      */
@@ -430,4 +461,5 @@ public final class ImageRegistry {
         sessionBoundImagesMap.clear();
         contextBoundImagesMap.clear();
     }
+
 }
