@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.mail;
 
+import java.util.Collection;
 import com.openexchange.api2.MailInterfaceMonitor;
 import com.openexchange.filemanagement.ManagedFile;
 import com.openexchange.mail.api.MailConfig;
@@ -202,7 +203,7 @@ public abstract class MailServletInterface {
      * Returns the an array of messages located in given folder. If <code>fromToUID</code> is not <code>null</code> only messages fitting
      * into uid range will be returned.
      */
-    public abstract MailMessage[] getMessageList(String folder, String[] uids, int[] fields) throws MailException;
+    public abstract MailMessage[] getMessageList(String folder, String[] uids, int[] fields, String[] headerFields) throws MailException;
 
     /**
      * Gets the mail identified through given ID from store located in given folder.
@@ -263,6 +264,17 @@ public abstract class MailServletInterface {
      */
     public abstract String sendMessage(ComposedMailMessage transportMail, ComposeType sendType, int accountId) throws MailException;
 
+    /**
+     * Appends given messages to given folder.
+     * 
+     * @param destFolder The destination folder
+     * @param msgs - The messages to append (<b>must</b> be completely pre-filled incl. content references)
+     * @param force <code>true</code> to enforce append and to omit checks; otherwise <code>false</code>
+     * @return The corresponding mail IDs in destination folder
+     * @throws MailException If messages cannot be appended.
+     */
+    public abstract String[] appendMessages(String destFolder, MailMessage[] msgs, boolean force) throws MailException;
+
     /**
      * Creates an instance of <code>JSONMessageObject</code> which contains the initial reply content of the message identifed through
      * <code>replyMsgUID</code>. <code>replyToAll</code> defines whether to reply to all involved entities or just to main sender.
@@ -408,4 +420,12 @@ public abstract class MailServletInterface {
      * @return The account ID
      */
     public abstract int getAccountID();
+
+    /**
+     * Gets possible warnings.
+     * 
+     * @return Possible warnings
+     */
+    public abstract Collection<MailException> getWarnings();
+    
 }
