@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,9 @@
 package com.openexchange.mail.api;
 
 import java.io.Serializable;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.Properties;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.Condition;
@@ -100,6 +103,8 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
 
     protected final int accountId;
 
+    protected final Collection<MailException> warnings;
+
     protected boolean cacheable;
 
     private transient MailConfig mailConfig;
@@ -127,11 +132,30 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      */
     protected MailAccess(final Session session, final int accountId) {
         super();
+        warnings = new ArrayList<MailException>(2);
         this.session = session;
         this.accountId = accountId;
         cacheable = true;
     }
 
+    /**
+     * Adds given warnings.
+     * 
+     * @param warnings The warnings to add
+     */
+    protected void addWarnings(final Collection<MailException> warnings) {
+        this.warnings.addAll(warnings);
+    }
+
+    /**
+     * Gets possible warnings.
+     * 
+     * @return Possible warnings.
+     */
+    public Collection<MailException> getWarnings() {
+        return Collections.unmodifiableCollection(warnings);
+    }
+
     /**
      * Resets this access' settings.
      */
@@ -256,9 +280,6 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
                 LOCK_CON.unlock();
             }
         }
-        /*
-         * Create a new mail access through user's mail provider
-         */
         return MailProviderRegistry.getMailProviderBySession(session, accountId).createNewMailAccess(session, accountId);
     }
 
@@ -288,7 +309,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      * 
      * @return the mailProperties
      */
-    public final Properties getMailProperties() {
+    public Properties getMailProperties() {
         return mailProperties;
     }
 
@@ -297,7 +318,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      * 
      * @param mailProperties The properties
      */
-    public final void setMailProperties(final Properties mailProperties) {
+    public void setMailProperties(final Properties mailProperties) {
         this.mailProperties = mailProperties;
     }
 
@@ -316,7 +337,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
          */
         if (mailConfig.getServer() == null) {
             throw new MailException(MailException.Code.MISSING_CONNECT_PARAM, "mail server");
-        } else if (checkMailServerPort() && (mailConfig.getPort() <= 0)) {
+        } else if (delegateCheckMailServerPort() && (mailConfig.getPort() <= 0)) {
             throw new MailException(MailException.Code.MISSING_CONNECT_PARAM, "mail server port");
         } else if (mailConfig.getLogin() == null) {
             throw new MailException(MailException.Code.MISSING_CONNECT_PARAM, "login");
@@ -427,7 +448,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
             }
         } else {
             checkFieldsBeforeConnect(getMailConfig());
-            connectInternal();
+            delegateConnectInternal();
             if (checkDefaultFolder) {
                 checkDefaultFolderOnConnect();
             }
@@ -452,7 +473,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
                     Integer.valueOf(session.getContextId()),
                     e.getMessage());
             LOG.error(mailExc.getMessage(), mailExc);
-            closeInternal();
+            delegateCloseInternal();
             throw mailExc;
         }
     }
@@ -465,6 +486,13 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      */
     protected abstract void connectInternal() throws MailException;
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public void delegateConnectInternal() throws MailException {
+        connectInternal();
+    }
+
     /**
      * Closes this access.
      * <p>
@@ -482,7 +510,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
                 /*
                  * Release all used, non-cachable resources
                  */
-                releaseResources();
+                delegateReleaseResources();
             } catch (final Throwable t) {
                 /*
                  * Dropping
@@ -508,7 +536,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
             /*
              * Close mail connection
              */
-            closeInternal();
+            delegateCloseInternal();
             signalAvailableConnection();
         } finally {
             /*
@@ -555,7 +583,7 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      * 
      * @return The mail configuration
      */
-    public final MailConfig getMailConfig() throws MailException {
+    public MailConfig getMailConfig() throws MailException {
         if (null == mailConfig) {
             mailConfig = createMailConfig();
         }
@@ -578,8 +606,8 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      * @throws MailException If creating a new mail configuration fails
      */
     private final MailConfig createMailConfig() throws MailException {
-        final MailConfig instance = createNewMailConfig();
-        instance.setMailProperties(createNewMailProperties());
+        final MailConfig instance = delegateCreateNewMailConfig();
+        instance.setMailProperties(delegateCreateNewMailProperties());
         return MailConfig.getConfig(instance.getClass(), instance, session, accountId);
     }
 
@@ -679,6 +707,13 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      */
     protected abstract MailConfig createNewMailConfig();
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public MailConfig delegateCreateNewMailConfig() {
+        return createNewMailConfig();
+    }
+
     /**
      * Gets an implementation-specific new instance of {@link IMailProperties}.
      * 
@@ -687,6 +722,13 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      */
     protected abstract IMailProperties createNewMailProperties() throws MailException;
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public IMailProperties delegateCreateNewMailProperties() throws MailException {
+        return createNewMailProperties();
+    }
+
     /**
      * Defines if mail server port has to be present in provided mail configuration before establishing any connection.
      * 
@@ -694,16 +736,37 @@ public abstract class MailAccess<F extends IMailFolderStorage, M extends IMailMe
      */
     protected abstract boolean checkMailServerPort();
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public boolean delegateCheckMailServerPort() {
+        return checkMailServerPort();
+    }
+
     /**
      * Releases all used resources prior to caching or closing a connection.
      */
     protected abstract void releaseResources();
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public void delegateReleaseResources() {
+        releaseResources();
+    }
+
     /**
      * Internal close method to drop a mail connection.
      */
     protected abstract void closeInternal();
 
+    /**
+     * For delegating purpose. Don't invoke.
+     */
+    public void delegateCloseInternal() {
+        closeInternal();
+    }
+
     /**
      * Gets the appropriate {@link IMailFolderStorage} implementation that is considered as the main entry point to a user's mailbox.
      * 
