@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -459,7 +459,7 @@ public final class MailMessageCache {
                  * TODO: Return cloned version ???
                  */
                 retval[i] = map.getValue(entryKey, uids[i]);
-                if (retval[i] == null) {
+                if (null == retval[i]) {
                     /*
                      * Not all desired messages can be served from cache
                      */
