@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -551,7 +551,7 @@ public final class MailProperties implements IMailProperties {
             String javaMailPropertiesStr = configuration.getProperty("com.openexchange.mail.JavaMailProperties");
             if (null != javaMailPropertiesStr) {
                 javaMailPropertiesStr = javaMailPropertiesStr.trim();
-                if (javaMailPropertiesStr.indexOf("@oxgroupwaresysconfdir@") != -1) {
+                if (javaMailPropertiesStr.indexOf("@oxgroupwaresysconfdir@") >= 0) {
                     final String configPath = configuration.getProperty("CONFIGPATH");
                     if (null == configPath) {
                         throw new MailConfigException("Missing property \"CONFIGPATH\" in system.properties");
