@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -65,6 +65,7 @@ import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.PlainTextAddress;
 import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.converters.MIMEMessageConverter;
+import com.openexchange.mail.mime.utils.MIMEMessageUtility;
 import com.openexchange.mail.utils.DateUtils;
 
 /**
@@ -370,14 +371,14 @@ public abstract class MailMessage extends MailPart {
     /**
      * The color label (set through an user flag).
      */
-    private int colorLabel = COLOR_LABEL_NONE;
+    private int colorLabel;
 
     private boolean b_colorLabel;
 
     /**
      * The priority (the <code>X-Priority</code> header).
      */
-    private int priority = PRIORITY_NORMAL;
+    private int priority;
 
     private boolean b_priority;
 
@@ -428,6 +429,8 @@ public abstract class MailMessage extends MailPart {
      */
     protected MailMessage() {
         super();
+        priority = PRIORITY_NORMAL;
+        colorLabel = COLOR_LABEL_NONE;
     }
 
     /**
@@ -898,7 +901,7 @@ public abstract class MailMessage extends MailPart {
      */
     public String getSubject() {
         if (!b_subject) {
-            final String subjectStr = getFirstHeader(MessageHeaders.HDR_SUBJECT);
+            final String subjectStr = MIMEMessageUtility.checkNonAscii(getFirstHeader(MessageHeaders.HDR_SUBJECT));
             if (subjectStr != null) {
                 setSubject(decodeMultiEncodedHeader(subjectStr));
             }
@@ -976,12 +979,21 @@ public abstract class MailMessage extends MailPart {
     /**
      * Gets the received date which represents the internal timestamp set by mail server on arrival.
      * 
-     * @return the received date
+     * @return The received date
      */
     public Date getReceivedDate() {
         return receivedDate == null ? null : new Date(receivedDate.getTime());
     }
 
+    /**
+     * Gets the received date directly which represents the internal timestamp set by mail server on arrival.
+     * 
+     * @return The received date
+     */
+    public Date getReceivedDateDirect() {
+        return receivedDate;
+    }
+
     /**
      * @return <code>true</code> if received date is set; otherwise <code>false</code>
      */
@@ -1063,6 +1075,9 @@ public abstract class MailMessage extends MailPart {
      */
     public String[] getUserFlags() {
         if (containsUserFlags() && (null != userFlags)) {
+            if (userFlags.isEmpty()) {
+                return EMPTY_UF;
+            }
             final int size = userFlags.size();
             final List<String> retval = new ArrayList<String>(size);
             final Iterator<HeaderName> iter = userFlags.iterator();
@@ -1075,23 +1090,23 @@ public abstract class MailMessage extends MailPart {
     }
 
     /**
-     * Gets the colorLabel
+     * Gets the color label
      * 
-     * @return the colorLabel
+     * @return the color label
      */
     public int getColorLabel() {
         return colorLabel;
     }
 
     /**
-     * @return <code>true</code> if colorLabel is set; otherwise <code>false</code>
+     * @return <code>true</code> if color label is set; otherwise <code>false</code>
      */
     public boolean containsColorLabel() {
         return b_colorLabel;
     }
 
     /**
-     * Removes the colorLabel
+     * Removes the color label
      */
     public void removeColorLabel() {
         colorLabel = COLOR_LABEL_NONE;
@@ -1099,9 +1114,9 @@ public abstract class MailMessage extends MailPart {
     }
 
     /**
-     * Sets the colorLabel
+     * Sets the color label
      * 
-     * @param colorLabel the colorLabel to set
+     * @param colorLabel the color label to set
      */
     public void setColorLabel(final int colorLabel) {
         this.colorLabel = colorLabel;
