@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -175,15 +175,11 @@ public abstract class MailPart implements Serializable, Cloneable {
      */
     protected MailPart() {
         super();
-        try {
-            contentType = new ContentType("text/plain; charset=us-ascii");
-            contentDisposition = new ContentDisposition();
-        } catch (final MailException e) {
-            /*
-             * Cannot occur
-             */
-            LOG.error(e.getMessage(), e);
-        }
+        contentType = new ContentType();
+        contentType.setPrimaryType("text");
+        contentType.setSubType("plain");
+        contentType.setCharsetParameter("us-ascii");
+        contentDisposition = new ContentDisposition();
     }
 
     /**
@@ -541,6 +537,20 @@ public abstract class MailPart implements Serializable, Cloneable {
         }
     }
 
+    /**
+     * Checks if this part contains all of specified headers.
+     * 
+     * @param names The names of the headers to check
+     * @return <code>true</code> if this part contains all of specified headers; otherwise <code>false</code>
+     */
+    public boolean hasHeaders(final String... names) {
+        boolean ret = true;
+        for (int i = 0; ret && i < names.length; i++) {
+            ret = headers.containsHeader(names[i]);
+        }
+        return ret;
+    }
+
     /**
      * Gets the size
      * 
