@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,6 +52,7 @@ package com.openexchange.mail.json.parser;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.parseAddressList;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.quotePersonal;
 import java.io.InputStream;
+import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -74,7 +75,9 @@ import com.openexchange.conversion.Data;
 import com.openexchange.conversion.DataArguments;
 import com.openexchange.conversion.DataException;
 import com.openexchange.conversion.DataExceptionCodes;
+import com.openexchange.conversion.DataProperties;
 import com.openexchange.conversion.DataSource;
+import com.openexchange.conversion.SimpleData;
 import com.openexchange.filemanagement.ManagedFile;
 import com.openexchange.filemanagement.ManagedFileException;
 import com.openexchange.filemanagement.ManagedFileManagement;
@@ -106,6 +109,9 @@ import com.openexchange.mail.mime.MIMETypes;
 import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.parser.MailMessageParser;
 import com.openexchange.mail.parser.handlers.MultipleMailPartHandler;
+import com.openexchange.mail.text.HTMLProcessing;
+import com.openexchange.mail.text.parser.HTMLParser;
+import com.openexchange.mail.text.parser.handler.HTML2TextHandler;
 import com.openexchange.mail.transport.TransportProvider;
 import com.openexchange.mail.transport.TransportProviderRegistry;
 import com.openexchange.mail.transport.config.TransportProperties;
@@ -230,46 +236,50 @@ public final class MessageParser {
              * Attached data sources
              */
             if (jsonObj.hasAndNotNull(MailJSONField.DATASOURCES.getKey())) {
-                final ConversionService conversionService = ServerServiceRegistry.getInstance().getService(ConversionService.class);
-                if (conversionService == null) {
-                    throw new MailException(new ServiceException(
-                        ServiceException.Code.SERVICE_UNAVAILABLE,
-                        ConversionService.class.getName()));
-                }
                 final JSONArray datasourceArray = jsonObj.getJSONArray(MailJSONField.DATASOURCES.getKey());
                 final int length = datasourceArray.length();
-                final Set<Class<?>> types = new HashSet<Class<?>>(4);
-                for (int i = 0; i < length; i++) {
-                    final JSONObject dataSourceObject = datasourceArray.getJSONObject(i);
-                    if (!dataSourceObject.hasAndNotNull(JSON_IDENTIFIER)) {
-                        throw new MailException(MailException.Code.MISSING_PARAM, JSON_IDENTIFIER);
+                if (length > 0) {
+                    final ConversionService conversionService = ServerServiceRegistry.getInstance().getService(ConversionService.class);
+                    if (conversionService == null) {
+                        throw new MailException(new ServiceException(
+                            ServiceException.Code.SERVICE_UNAVAILABLE,
+                            ConversionService.class.getName()));
                     }
-                    final DataSource dataSource = conversionService.getDataSource(dataSourceObject.getString(JSON_IDENTIFIER));
-                    if (dataSource == null) {
-                        throw new MailException(DataExceptionCodes.UNKNOWN_DATA_SOURCE.create(dataSourceObject.getString(JSON_IDENTIFIER)));
-                    }
-                    if (!types.isEmpty()) {
-                        types.clear();
-                    }
-                    types.addAll(Arrays.asList(dataSource.getTypes()));
-                    final Data<?> data;
-                    if (types.contains(InputStream.class)) {
-                        try {
-                            data = dataSource.getData(InputStream.class, parseDataSourceArguments(dataSourceObject), session);
-                        } catch (final DataException e) {
-                            throw new MailException(e);
+                    final Set<Class<?>> types = new HashSet<Class<?>>(4);
+                    for (int i = 0; i < length; i++) {
+                        final JSONObject dataSourceObject = datasourceArray.getJSONObject(i);
+                        if (!dataSourceObject.hasAndNotNull(JSON_IDENTIFIER)) {
+                            throw new MailException(MailException.Code.MISSING_PARAM, JSON_IDENTIFIER);
                         }
-                    } else if (types.contains(byte[].class)) {
-                        try {
-                            data = dataSource.getData(byte[].class, parseDataSourceArguments(dataSourceObject), session);
-                        } catch (final DataException e) {
-                            throw new MailException(e);
+                        final DataSource dataSource = conversionService.getDataSource(dataSourceObject.getString(JSON_IDENTIFIER));
+                        if (dataSource == null) {
+                            throw new MailException(
+                                DataExceptionCodes.UNKNOWN_DATA_SOURCE.create(dataSourceObject.getString(JSON_IDENTIFIER)));
                         }
-                    } else {
-                        throw new MailException(MailException.Code.UNSUPPORTED_DATASOURCE);
+                        if (!types.isEmpty()) {
+                            types.clear();
+                        }
+                        types.addAll(Arrays.asList(dataSource.getTypes()));
+                        final Data<?> data;
+                        if (types.contains(InputStream.class)) {
+                            try {
+                                data = dataSource.getData(InputStream.class, parseDataSourceArguments(dataSourceObject), session);
+                            } catch (final DataException e) {
+                                throw new MailException(e);
+                            }
+                        } else if (types.contains(byte[].class)) {
+                            try {
+                                data = dataSource.getData(byte[].class, parseDataSourceArguments(dataSourceObject), session);
+                            } catch (final DataException e) {
+                                throw new MailException(e);
+                            }
+                        } else {
+                            throw new MailException(MailException.Code.UNSUPPORTED_DATASOURCE);
+                        }
+                        final DataMailPart dataMailPart =
+                            provider.getNewDataPart(data.getData(), data.getDataProperties().toMap(), session);
+                        attachmentHandler.addAttachment(dataMailPart);
                     }
-                    final DataMailPart dataMailPart = provider.getNewDataPart(data.getData(), data.getDataProperties().toMap(), session);
-                    attachmentHandler.addAttachment(dataMailPart);
                 }
             }
             /*
@@ -463,7 +473,9 @@ public final class MessageParser {
             final Iterator<String> iter = obj.keys();
             for (int i = 0; i < size; i++) {
                 final String key = iter.next();
-                headers.addHeader(key, obj.getString(key));
+                if (isCustomHeader(key)) {
+                    headers.addHeader(key, obj.getString(key));
+                }
             }
             mail.addHeaders(headers);
         }
@@ -562,6 +574,20 @@ public final class MessageParser {
         }
     }
 
+    /**
+     * Checks if specified header name is a custom header that is it starts ignore-case with <code>"X-"</code>.
+     * 
+     * @param headerName The header name to check
+     * @return <code>true</code> if specified header name is a custom header; otherwise <code>false</code>
+     */
+    private static boolean isCustomHeader(final String headerName) {
+        if (null == headerName || headerName.length() < 2) {
+            return false;
+        }
+        final char first = headerName.charAt(0);
+        return (('X' == first) || ('x' == first)) && ('-' == headerName.charAt(1));
+    }
+
     private static final String ROOT = "0";
 
     private static final String FILE_PREFIX = "file://";
@@ -589,7 +615,42 @@ public final class MessageParser {
                 final JSONObject attachment = attachmentArray.getJSONObject(i);
                 final String seqId =
                     attachment.hasAndNotNull(MailListField.ID.getKey()) ? attachment.getString(MailListField.ID.getKey()) : null;
-                if (seqId != null && seqId.startsWith(FILE_PREFIX, 0)) {
+                if (null == seqId && attachment.hasAndNotNull(MailJSONField.CONTENT.getKey())) {
+                    /*
+                     * A direct attachment, as data part
+                     */
+                    final String contentType = parseContentType(attachment.getString(MailJSONField.CONTENT_TYPE.getKey()));
+                    final String charsetName = "UTF-8";
+                    final byte[] content;
+                    try {
+                        /*
+                         * UI delivers HTML content in any case. Generate well-formed HTML for further processing dependent on given content
+                         * type.
+                         */
+                        final String validHtml =
+                            HTMLProcessing.getConformHTML(attachment.getString(MailJSONField.CONTENT.getKey()), "US-ASCII");
+                        if (MIMETypes.MIME_TEXT_PLAIN.equals(contentType)) {
+                            final HTML2TextHandler html2textHandler = new HTML2TextHandler(4096, true);
+                            HTMLParser.parse(validHtml, html2textHandler);
+                            content = html2textHandler.getText().getBytes(charsetName);
+                        } else {
+                            content = validHtml.getBytes(charsetName);
+                        }
+
+                    } catch (final UnsupportedEncodingException e) {
+                        throw new MailException(MailException.Code.ENCODING_ERROR, e, e.getMessage());
+                    }
+                    /*
+                     * As data object
+                     */
+                    final DataProperties properties = new DataProperties();
+                    properties.put(DataProperties.PROPERTY_CONTENT_TYPE, contentType);
+                    properties.put(DataProperties.PROPERTY_SIZE, String.valueOf(content.length));
+                    properties.put(DataProperties.PROPERTY_CHARSET, charsetName);
+                    final Data<byte[]> data = new SimpleData<byte[]>(content, properties);
+                    final DataMailPart dataMailPart = provider.getNewDataPart(data.getData(), data.getDataProperties().toMap(), session);
+                    attachmentHandler.addAttachment(dataMailPart);
+                } else if (null != seqId && seqId.startsWith(FILE_PREFIX, 0)) {
                     /*
                      * A file reference
                      */
@@ -656,7 +717,7 @@ public final class MessageParser {
             final JSONObject attachment = attachmentArray.getJSONObject(i);
             final String seqId =
                 attachment.hasAndNotNull(MailListField.ID.getKey()) ? attachment.getString(MailListField.ID.getKey()) : null;
-            if (seqId != null && seqId.startsWith(FILE_PREFIX, 0)) {
+            if (seqId == null || seqId.startsWith(FILE_PREFIX, 0)) {
                 /*
                  * A file reference
                  */
