@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -47,7 +47,6 @@
  *
  */
 
-
 package com.openexchange.mail.json.parser;
 
 import static com.openexchange.groupware.upload.impl.UploadUtility.getSize;
@@ -81,6 +80,7 @@ import com.openexchange.groupware.infostore.database.impl.DocumentMetadataImpl;
 import com.openexchange.groupware.ldap.LdapException;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.ldap.UserException;
+import com.openexchange.groupware.ldap.UserStorage;
 import com.openexchange.groupware.tx.TransactionException;
 import com.openexchange.i18n.tools.StringHelper;
 import com.openexchange.mail.MailException;
@@ -90,6 +90,7 @@ import com.openexchange.mail.dataobjects.compose.ComposedMailMessage;
 import com.openexchange.mail.dataobjects.compose.TextBodyMailPart;
 import com.openexchange.mail.mime.MIMEMailException;
 import com.openexchange.mail.mime.MessageHeaders;
+import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.utils.MIMEMessageUtility;
 import com.openexchange.mail.transport.TransportProvider;
 import com.openexchange.mail.transport.config.TransportProperties;
@@ -159,11 +160,12 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
             if (uploadQuotaPerFile > 0 && size > uploadQuotaPerFile) {
                 if (LOG.isDebugEnabled()) {
                     final String fileName = attachment.getFileName();
-                    final MailException e = new MailException(
-                        MailException.Code.UPLOAD_QUOTA_EXCEEDED_FOR_FILE,
-                        Long.valueOf(uploadQuotaPerFile),
-                        null == fileName ? "" : fileName,
-                        Long.valueOf(size));
+                    final MailException e =
+                        new MailException(
+                            MailException.Code.UPLOAD_QUOTA_EXCEEDED_FOR_FILE,
+                            Long.valueOf(uploadQuotaPerFile),
+                            null == fileName ? "" : fileName,
+                            Long.valueOf(size));
                     LOG.debug(new StringBuilder(64).append("Per-file quota (").append(getSize(uploadQuotaPerFile, 2, false, true)).append(
                         ") exceeded. Message is going to be sent with links to publishing infostore folder.").toString(), e);
                 }
@@ -217,9 +219,8 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
             /*
              * Get discovery service
              */
-            final PublicationTargetDiscoveryService discoveryService = ServerServiceRegistry.getInstance().getService(
-                PublicationTargetDiscoveryService.class,
-                true);
+            final PublicationTargetDiscoveryService discoveryService =
+                ServerServiceRegistry.getInstance().getService(PublicationTargetDiscoveryService.class, true);
             /*
              * Get discovery service's target
              */
@@ -285,7 +286,10 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
         /*
          * Iterate recipients and split them to internal vs. external recipients
          */
-        final Date elapsedDate = new Date(System.currentTimeMillis() + TransportProperties.getInstance().getPublishedDocumentTimeToLive());
+        Date elapsedDate = null;
+        if(TransportProperties.getInstance().publishedDocumentsExpire()) {
+            elapsedDate = new Date(System.currentTimeMillis() + TransportProperties.getInstance().getPublishedDocumentTimeToLive());
+        }
         final UserService userService = ServerServiceRegistry.getInstance().getService(UserService.class);
         final Map<Locale, ComposedMailMessage> internalMessages = new HashMap<Locale, ComposedMailMessage>(addresses.size());
         ComposedMailMessage externalMessage = null;
@@ -301,16 +305,27 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
                 if (LdapException.Code.NO_USER_BY_MAIL.getDetailNumber() != e.getDetailNumber()) {
                     throw new MailException(e);
                 }
+                /*
+                 * Retry
+                 */
+                try {
+                    user = userService.searchUser(QuotedInternetAddress.toIDN(address.getAddress()), ctx);
+                } catch (final UserException inner) {
+                    if (LdapException.Code.NO_USER_BY_MAIL.getDetailNumber() != inner.getDetailNumber()) {
+                        throw new MailException(inner);
+                    }
+                }
             }
             if (null == user) {
                 // External user
                 if (null == externalMessage) {
-                    externalMessage = generateExternalVersion(
-                        source,
-                        ctx,
-                        links,
-                        TransportProperties.getInstance().isProvideLinksInAttachment(),
-                        elapsedDate);
+                    externalMessage =
+                        generateExternalVersion(
+                            source,
+                            ctx,
+                            links,
+                            TransportProperties.getInstance().isProvideLinksInAttachment(),
+                            elapsedDate);
                 }
                 externalMessage.addRecipient(address);
             } else {
@@ -318,13 +333,14 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
                 final Locale locale = user.getLocale();
                 ComposedMailMessage localedMessage = internalMessages.get(locale);
                 if (null == localedMessage) {
-                    localedMessage = generateInternalVersion(
-                        source,
-                        ctx,
-                        links,
-                        TransportProperties.getInstance().isProvideLinksInAttachment(),
-                        elapsedDate,
-                        locale);
+                    localedMessage =
+                        generateInternalVersion(
+                            source,
+                            ctx,
+                            links,
+                            TransportProperties.getInstance().isProvideLinksInAttachment(),
+                            elapsedDate,
+                            locale);
                     internalMessages.put(locale, localedMessage);
                 }
                 localedMessage.addRecipient(address);
@@ -338,6 +354,18 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
         if (null != externalMessage) {
             mails.add(externalMessage);
         }
+        /*
+         * Any version available?
+         */
+        if (mails.isEmpty()) {
+            mails.add(generateInternalVersion(
+                source,
+                ctx,
+                links,
+                TransportProperties.getInstance().isProvideLinksInAttachment(),
+                elapsedDate,
+                getSessionUser().getLocale()));
+        }
         return mails.toArray(new ComposedMailMessage[mails.size()]);
     }
 
@@ -352,6 +380,17 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
         }
     }
 
+    private User getSessionUser() throws MailException {
+        if (session instanceof ServerSession) {
+            return ((ServerSession) session).getUser();
+        }
+        try {
+            return UserStorage.getInstance().getUser(session.getUserId(), getContext());
+        } catch (final LdapException e) {
+            throw new MailException(e);
+        }
+    }
+
     private static final Pattern PATTERN_DATE = Pattern.compile(Pattern.quote("#DATE#"));
 
     private ComposedMailMessage generateInternalVersion(final ComposedMailMessage source, final Context ctx, final List<LinkAndNamePair> links, final boolean appendLinksAsAttachment, final Date elapsedDate, final Locale locale) throws MailException {
@@ -371,9 +410,11 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
             final StringBuilder textBuilder = new StringBuilder(text.length() + 512);
             textBuilder.append(htmlFormat(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_PREFIX))).append("<br />");
             appendLinks(links, textBuilder);
-            textBuilder.append(
-                htmlFormat(PATTERN_DATE.matcher(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_APPENDIX)).replaceFirst(
-                    DateFormat.getDateInstance(DateFormat.LONG, locale).format(elapsedDate)))).append("<br /><br />");
+            if(elapsedDate != null) {
+                textBuilder.append(
+                    htmlFormat(PATTERN_DATE.matcher(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_APPENDIX)).replaceFirst(
+                        DateFormat.getDateInstance(DateFormat.LONG, locale).format(elapsedDate)))).append("<br /><br />");
+            }
             textBuilder.append(text);
             textPart.setText(textBuilder.toString());
             internalVersion.setBodyPart(textPart);
@@ -405,9 +446,11 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
                 final StringBuilder textBuilder = new StringBuilder(text.length() + 512);
                 textBuilder.append(htmlFormat(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_PREFIX))).append("<br />");
                 appendLinks(links, textBuilder);
-                textBuilder.append(
-                    htmlFormat(PATTERN_DATE.matcher(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_APPENDIX)).replaceFirst(
-                        DateFormat.getDateInstance(DateFormat.LONG, locale).format(elapsedDate)))).append("<br /><br />");
+                if(elapsedDate != null) {
+                    textBuilder.append(
+                        htmlFormat(PATTERN_DATE.matcher(stringHelper.getString(MailStrings.PUBLISHED_ATTACHMENTS_APPENDIX)).replaceFirst(
+                            DateFormat.getDateInstance(DateFormat.LONG, locale).format(elapsedDate)))).append("<br /><br />");
+                }
                 textBuilder.append(text);
                 textPart.setText(textBuilder.toString());
                 externalVersion.setBodyPart(textPart);
@@ -486,9 +529,13 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
             final MimeBodyPart bodyPart = new MimeBodyPart();
             bodyPart.setText(getConformHTML(text, "UTF-8"), "UTF-8", "html");
             bodyPart.setHeader(MessageHeaders.HDR_MIME_VERSION, "1.0");
-            bodyPart.setHeader(MessageHeaders.HDR_CONTENT_TYPE, MIMEMessageUtility.foldContentType("text/html; charset=UTF-8; name=links.html"));
+            bodyPart.setHeader(
+                MessageHeaders.HDR_CONTENT_TYPE,
+                MIMEMessageUtility.foldContentType("text/html; charset=UTF-8; name=links.html"));
             bodyPart.setHeader(MessageHeaders.HDR_CONTENT_TRANSFER_ENC, "base64");
-            bodyPart.setHeader(MessageHeaders.HDR_CONTENT_DISPOSITION, MIMEMessageUtility.foldContentDisposition("attachment; filename=links.html"));
+            bodyPart.setHeader(
+                MessageHeaders.HDR_CONTENT_DISPOSITION,
+                MIMEMessageUtility.foldContentDisposition("attachment; filename=links.html"));
             return convertPart(bodyPart, false);
         } catch (final MessagingException e) {
             throw MIMEMailException.handleMessagingException(e);
@@ -535,7 +582,8 @@ final class PublishAttachmentHandler extends AbstractAttachmentHandler {
                     final int pos = name.lastIndexOf('.');
                     final String newName;
                     if (pos >= 0) {
-                        newName = hlp.append(name.substring(0, pos)).append("_(").append(++count).append(')').append(name.substring(pos)).toString();
+                        newName =
+                            hlp.append(name.substring(0, pos)).append("_(").append(++count).append(')').append(name.substring(pos)).toString();
                     } else {
                         newName = hlp.append(name).append("_(").append(++count).append(')').toString();
                     }
