@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.mail.json.writer;
 
+import static com.openexchange.mail.mime.QuotedInternetAddress.toIDN;
 import static com.openexchange.mail.utils.MailFolderUtility.prepareFullname;
 import java.util.EnumMap;
 import java.util.TimeZone;
@@ -99,6 +100,15 @@ public final class MessageWriter {
 
     /**
      * Writes specified mail's structure as a JSON object.
+     * <p>
+     * Optionally a prepared version can be returned, this includes following actions:
+     * <ol>
+     * <li>Header names are inserted to JSON lower-case</li>
+     * <li>Mail-safe encoded header values as per RFC 2047 are decoded;<br>
+     * e.g.&nbsp;<code><i>To:&nbsp;=?iso-8859-1?q?Keld_J=F8rn?=&nbsp;&lt;keld@xyz.dk&gt;</i></code></li>
+     * <li>Address headers are delivered as JSON objects with a <code>"personal"</code> and an <code>"address"</code> field</li>
+     * <li>Parameterized headers are delivered as JSON objects with a <code>"type"</code> and a <code>"params"</code> field</li>
+     * </ol>
      * 
      * @param accountId The mail's account ID
      * @param mail The mail to write
@@ -172,6 +182,47 @@ public final class MessageWriter {
         public void writeField(JSONValue jsonContainer, MailMessage mail, int level, boolean withKey, int accountId, int user, int cid) throws MailException;
     }
 
+    private static final class HeaderFieldWriter implements MailFieldWriter {
+
+        private final String headerName;
+
+        HeaderFieldWriter(final String headerName) {
+            super();
+            this.headerName = headerName;
+        }
+
+        public void writeField(final JSONValue jsonContainer, final MailMessage mail, final int level, final boolean withKey, final int accountId, final int user, final int cid) throws MailException {
+            final Object value = getHeaderValue(mail);
+            if (withKey) {
+                if (null != value) {
+                    try {
+                        ((JSONObject) jsonContainer).put(headerName, value);
+                    } catch (final JSONException e) {
+                        throw new MailException(MailException.Code.JSON_ERROR, e, e.getMessage());
+                    }
+                }
+            } else {
+                ((JSONArray) jsonContainer).put(null == value ? JSONObject.NULL : value);
+            }
+        }
+
+        private Object getHeaderValue(final MailMessage mail) {
+            final String[] headerValues = mail.getHeader(headerName);
+            if (null == headerValues || 0 == headerValues.length) {
+                return null;
+            }
+            if (1 == headerValues.length) {
+                return headerValues[0];
+            }
+            final JSONArray ja = new JSONArray();
+            for (int j = 0; j < headerValues.length; j++) {
+                ja.put(headerValues[j]);
+            }
+            return ja;
+        }
+
+    }
+
     private static final EnumMap<MailListField, MailFieldWriter> WRITERS = new EnumMap<MailListField, MailFieldWriter>(MailListField.class);
 
     static {
@@ -569,6 +620,23 @@ public final class MessageWriter {
         return retval;
     }
 
+    /**
+     * Gets writers for specified header names.
+     * 
+     * @param headers The header names
+     * @return The writers for specified header names
+     */
+    public static MailFieldWriter[] getHeaderFieldWriter(final String[] headers) {
+        if (null == headers) {
+            return new MailFieldWriter[0];
+        }
+        final MailFieldWriter[] retval = new MailFieldWriter[headers.length];
+        for (int i = 0; i < headers.length; i++) {
+            retval[i] = new HeaderFieldWriter(headers[i]);
+        }
+        return retval;
+    }
+
     /**
      * Adds the user time zone offset to given date time
      * 
@@ -610,7 +678,7 @@ public final class MessageWriter {
         retval.put(personal == null || personal.length() == 0 ? JSONObject.NULL : preparePersonal(personal));
         // Address
         final String address = addr.getAddress();
-        retval.put(address == null || address.length() == 0 ? JSONObject.NULL : prepareAddress(address));
+        retval.put(address == null || address.length() == 0 ? JSONObject.NULL : prepareAddress(toIDN(address)));
 
         return retval;
     }
