@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,31 +49,193 @@
 
 package com.openexchange.mail.mime;
 
+import java.util.Iterator;
 import java.util.Locale;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.mime.utils.MIMEMessageUtility;
+import com.openexchange.tools.Collections;
 
 /**
  * {@link ContentType} - Parses value of MIME header <code>Content-Type</code>
  * 
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public final class ContentType extends ParameterizedHeader {
+public class ContentType extends ParameterizedHeader {
 
     private static final long serialVersionUID = -9197784872892324694L;
 
     /**
-     * The default content type: <code>text/plain; charset=us-ascii</code>
+     * {@link UnmodifiableContentType} - An unmodifiable content type.
+     *
+     * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+     * @since Open-Xchange v6.16
+     */
+    public static final class UnmodifiableContentType extends ContentType {
+
+        private static final long serialVersionUID = 2473639344400699520L;
+
+        private final ContentType contentType;
+
+        /**
+         * Initializes a new {@link UnmodifiableContentType}.
+         * 
+         * @param contentType The backing content type
+         */
+        public UnmodifiableContentType(final ContentType contentType) {
+            super();
+            this.contentType = contentType;
+        }
+
+        @Override
+        public void addParameter(final String key, final String value) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.addParameter()");
+        }
+
+        @Override
+        public int compareTo(final ParameterizedHeader other) {
+            return contentType.compareTo(other);
+        }
+
+        @Override
+        public boolean containsCharsetParameter() {
+            return contentType.containsCharsetParameter();
+        }
+
+        @Override
+        public boolean containsNameParameter() {
+            return contentType.containsNameParameter();
+        }
+
+        @Override
+        public boolean containsParameter(final String key) {
+            return contentType.containsParameter(key);
+        }
+
+        @Override
+        public boolean equals(final Object obj) {
+            return contentType.equals(obj);
+        }
+
+        @Override
+        public String getBaseType() {
+            return contentType.getBaseType();
+        }
+
+        @Override
+        public String getCharsetParameter() {
+            return contentType.getCharsetParameter();
+        }
+
+        @Override
+        public String getNameParameter() {
+            return contentType.getNameParameter();
+        }
+
+        @Override
+        public String getParameter(final String key) {
+            return contentType.getParameter(key);
+        }
+
+        @Override
+        public Iterator<String> getParameterNames() {
+            return Collections.unmodifiableIterator(contentType.getParameterNames());
+        }
+
+        @Override
+        public String getPrimaryType() {
+            return contentType.getPrimaryType();
+        }
+
+        @Override
+        public String getSubType() {
+            return contentType.getSubType();
+        }
+
+        @Override
+        public int hashCode() {
+            return contentType.hashCode();
+        }
+
+        @Override
+        public boolean isMimeType(final String pattern) {
+            return contentType.isMimeType(pattern);
+        }
+
+        @Override
+        public String removeParameter(final String key) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.removeParameter()");
+        }
+
+        @Override
+        public ContentType setBaseType(final String baseType) throws MailException {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setCharsetParameter()");
+        }
+
+        @Override
+        public ContentType setCharsetParameter(final String charset) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setCharsetParameter()");
+        }
+
+        @Override
+        public void setContentType(final ContentType contentType) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setContentType()");
+        }
+
+        @Override
+        public void setContentType(final String contentType) throws MailException {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setContentType()");
+        }
+
+        @Override
+        public ContentType setNameParameter(final String filename) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setNameParameter()");
+        }
+
+        @Override
+        public void setParameter(final String key, final String value) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setParameter()");
+        }
+
+        @Override
+        public ContentType setPrimaryType(final String primaryType) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setPrimaryType()");
+        }
+
+        @Override
+        public ContentType setSubType(final String subType) {
+            throw new UnsupportedOperationException("ContentType.UnmodifiableContentType.setSubType()");
+        }
+
+        @Override
+        public boolean startsWith(final String prefix) {
+            return contentType.startsWith(prefix);
+        }
+
+        @Override
+        public String toString() {
+            return contentType.toString();
+        }
+
+        @Override
+        public String toString(final boolean skipEmptyParams) {
+            return contentType.toString(skipEmptyParams);
+        }
+
+    } // End of UnmodifiableContentType
+
+    /**
+     * The (unmodifiable) default content type: <code>text/plain; charset=us-ascii</code>
      */
     public static final ContentType DEFAULT_CONTENT_TYPE;
 
     static {
-        DEFAULT_CONTENT_TYPE = new ContentType();
-        DEFAULT_CONTENT_TYPE.setPrimaryType("text");
-        DEFAULT_CONTENT_TYPE.setSubType("plain");
-        DEFAULT_CONTENT_TYPE.setCharsetParameter("us-ascii");
+        final ContentType tmp = new ContentType();
+        tmp.setPrimaryType("text");
+        tmp.setSubType("plain");
+        tmp.setCharsetParameter("us-ascii");
+        DEFAULT_CONTENT_TYPE = new UnmodifiableContentType(tmp);
     }
 
     /**
@@ -88,6 +250,8 @@ public final class ContentType extends ParameterizedHeader {
      */
     private static final char DELIMITER = '/';
 
+    // private static final String DEFAULT_PRIMTYPE = "APPLICATION";
+
     private static final String DEFAULT_SUBTYPE = "OCTET-STREAM";
 
     private static final String PARAM_CHARSET = "charset";
@@ -181,18 +345,41 @@ public final class ContentType extends ParameterizedHeader {
             return;
         }
         final String contentType = prepareParameterizedHeader(contentTypeArg);
-        final Matcher ctMatcher = PATTERN_CONTENT_TYPE.matcher(contentType);
-        if (!ctMatcher.find() || (ctMatcher.start() != 0)) {
-            throw new MailException(MailException.Code.INVALID_CONTENT_TYPE, contentTypeArg);
-        }
-        primaryType = ctMatcher.group(1);
-        subType = ctMatcher.group(2);
-        if ((subType == null) || (subType.length() == 0)) {
-            subType = DEFAULT_SUBTYPE;
-        }
-        baseType = new StringBuilder(16).append(primaryType).append(DELIMITER).append(subType).toString();
-        if (paramList) {
-            parameterList = new ParameterList(contentType.substring(ctMatcher.end()));
+        final int pos = contentType.indexOf(';');
+        final Matcher ctMatcher = PATTERN_CONTENT_TYPE.matcher(pos < 0 ? contentType : contentType.substring(0, pos));
+        if (ctMatcher.find()) {
+            if (ctMatcher.start() != 0) {
+                throw new MailException(MailException.Code.INVALID_CONTENT_TYPE, contentTypeArg);
+            }
+            primaryType = ctMatcher.group(1);
+            subType = ctMatcher.group(2);
+            if ((subType == null) || (subType.length() == 0)) {
+                subType = DEFAULT_SUBTYPE;
+            }
+            baseType = new StringBuilder(16).append(primaryType).append(DELIMITER).append(subType).toString();
+            if (paramList) {
+                parameterList = new ParameterList(contentType.substring(ctMatcher.end()));
+            }
+        } else {
+            primaryType = "text";
+            subType = "plain";
+            baseType = new StringBuilder(16).append(primaryType).append(DELIMITER).append(subType).toString();
+            if (paramList) {
+                parameterList = new ParameterList(pos < 0 ? contentType : contentType.substring(pos));
+                final String name = parameterList.getParameter("name");
+                if (null != name) {
+                    final String byName = MIMEType2ExtMap.getContentType(name);
+                    if (null != byName) {
+                        final int slash = byName.indexOf('/');
+                        primaryType = byName.substring(0, slash);
+                        subType = byName.substring(slash + 1);
+                        if ((subType == null) || (subType.length() == 0)) {
+                            subType = DEFAULT_SUBTYPE;
+                        }
+                        baseType = new StringBuilder(16).append(primaryType).append(DELIMITER).append(subType).toString();
+                    }
+                }
+            }
         }
     }
 
@@ -227,10 +414,13 @@ public final class ContentType extends ParameterizedHeader {
 
     /**
      * Sets primary type
+     * 
+     * @return This content type with new primary type applied
      */
-    public void setPrimaryType(final String primaryType) {
+    public ContentType setPrimaryType(final String primaryType) {
         this.primaryType = primaryType;
         baseType = null;
+        return this;
     }
 
     /**
@@ -242,10 +432,13 @@ public final class ContentType extends ParameterizedHeader {
 
     /**
      * Sets sub-type
+     * 
+     * @return This content type with new sub-type applied
      */
-    public void setSubType(final String subType) {
+    public ContentType setSubType(final String subType) {
         this.subType = subType;
         baseType = null;
+        return this;
     }
 
     /**
@@ -260,57 +453,69 @@ public final class ContentType extends ParameterizedHeader {
 
     /**
      * Sets base type (e.g. text/plain)
+     * 
+     * @return This content type with new base type applied
      */
-    public void setBaseType(final String baseType) throws MailException {
+    public ContentType setBaseType(final String baseType) throws MailException {
         parseBaseType(baseType);
+        return this;
     }
 
     /**
-     * Sets charset parameter
+     * Sets <code>"charset"</code> parameter
+     * 
+     * @param charset The charset parameter value; e.g. <code>"UTF-8"</code>
+     * @return This content type with new <code>"charset"</code> parameter applied
      */
-    public void setCharsetParameter(final String charset) {
+    public ContentType setCharsetParameter(final String charset) {
         setParameter(PARAM_CHARSET, charset);
+        return this;
     }
 
     /**
-     * @return the charset value or <code>null</code> if not present
+     * @return the <code>"charset"</code> value or <code>null</code> if not present
      */
     public String getCharsetParameter() {
         return getParameter(PARAM_CHARSET);
     }
 
     /**
-     * @return <code>true</code> if charset parameter is present, <code>false</code> otherwise
+     * @return <code>true</code> if <code>"charset"</code> parameter is present, <code>false</code> otherwise
      */
     public boolean containsCharsetParameter() {
         return containsParameter(PARAM_CHARSET);
     }
 
     /**
-     * Sets name parameter
+     * Sets <code>"name"</code> parameter
      * 
      * @param filename The name parameter
+     * @return This content type with new <code>"name"</code> parameter applied
      */
-    public void setNameParameter(final String filename) {
+    public ContentType setNameParameter(final String filename) {
         setParameter(PARAM_NAME, filename);
+        return this;
     }
 
     /**
-     * @return the name value or <code>null</code> if not present
+     * @return the <code>"name"</code> value or <code>null</code> if not present
      */
     public String getNameParameter() {
         return getParameter(PARAM_NAME);
     }
 
     /**
-     * @return <code>true</code> if name parameter is present, <code>false</code> otherwise
+     * @return <code>true</code> if <code>"name"</code> parameter is present, <code>false</code> otherwise
      */
     public boolean containsNameParameter() {
         return containsParameter(PARAM_NAME);
     }
 
     /**
-     * Sets Content-Type
+     * Sets the content type to specified content type string; e.g. "text/plain; charset=US-ASCII"
+     * 
+     * @param contentType The content type string
+     * @throws MailException If specified content type string cannot be parsed
      */
     public void setContentType(final String contentType) throws MailException {
         parseContentType(contentType);
@@ -432,10 +637,20 @@ public final class ContentType extends ParameterizedHeader {
 
     @Override
     public String toString() {
+        return toString(false);
+    }
+
+    /**
+     * Returns a RFC2045 style (ASCII-only) string representation of this content type.
+     * 
+     * @param skipEmptyParams <code>true</code> to skip empty parameters; otherwise <code>false</code>
+     * @return A RFC2045 style (ASCII-only) string representation of this content type
+     */
+    public String toString(final boolean skipEmptyParams) {
         final StringBuilder sb = new StringBuilder(64);
         sb.append(primaryType).append(DELIMITER).append(subType);
         if (null != parameterList) {
-            parameterList.appendUnicodeString(sb);
+            parameterList.appendRFC2045String(sb, skipEmptyParams);
         }
         return sb.toString();
     }
