@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -58,7 +58,8 @@ import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MimeUtility;
 
 /**
- * {@link QuotedInternetAddress} - A quoted version of {@link InternetAddress}.
+ * {@link QuotedInternetAddress} - A quoted version of {@link InternetAddress} originally written by <b>Bill Shannon</b> and <b>John
+ * Mani</b>. Moreover this class supports <a href="http://en.wikipedia.org/wiki/Punycode">punycode</a>.
  * <p>
  * Quotes are added to encoded personal names to maintain them when converting to mail-safe version. Parental {@link InternetAddress} class
  * ignores quotes when when converting to mail-safe version:
@@ -81,12 +82,30 @@ public final class QuotedInternetAddress extends InternetAddress {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(QuotedInternetAddress.class);
 
-    private static final boolean ignoreBogusGroupName = getBooleanSystemProperty("mail.mime.address.ignorebogusgroupname", true);
+    private static final boolean IGNORE_BOGUS_GROUP_NAME = getBooleanSystemProperty("mail.mime.address.ignorebogusgroupname", true);
 
     private static boolean getBooleanSystemProperty(final String name, final boolean def) {
         return Boolean.parseBoolean(System.getProperty(name, def ? "true" : "false"));
     }
 
+    /**
+     * Converts given array of {@link InternetAddress} to quoted addresses
+     * 
+     * @param addrs The addresses to convert
+     * @return The quoted addresses
+     * @throws AddressException If conversion fails
+     */
+    public static InternetAddress[] toQuotedAddresses(final InternetAddress[] addrs) throws AddressException {
+        if (null == addrs) {
+            return null;
+        }
+        final InternetAddress[] ret = new InternetAddress[addrs.length];
+        for (int i = 0; i < ret.length; i++) {
+            ret[i] = new QuotedInternetAddress(addrs[i]);
+        }
+        return ret;
+    }
+
     /**
      * Parse the given comma separated sequence of addresses into {@link InternetAddress} objects. Addresses must follow RFC822 syntax.
      * 
@@ -253,7 +272,7 @@ public final class QuotedInternetAddress extends InternetAddress {
                         index++; // skip both '\' and the escaped char
                         break;
                     case '"':
-                        inquote = !inquote;
+                        inquote ^= true;
                         break;
                     case '>':
                         if (inquote) {
@@ -404,6 +423,7 @@ public final class QuotedInternetAddress extends InternetAddress {
 
                 // otherwise, parsing a header; treat semicolon like comma
                 // fall through to comma case...
+                //$FALL-THROUGH$
 
             case ',': // end of an address, probably
                 if (start == -1) {
@@ -425,7 +445,7 @@ public final class QuotedInternetAddress extends InternetAddress {
                 String pers = null;
                 if (rfc822 && start_personal >= 0) {
                     pers = unquote(s.substring(start_personal, end_personal).trim());
-                    if (pers.trim().length() == 0) {
+                    if (isEmpty(pers)) {
                         pers = null;
                     }
                 }
@@ -513,7 +533,7 @@ public final class QuotedInternetAddress extends InternetAddress {
                     // ignore bogus "mailto:" prefix in front of an address,
                     // or bogus mail header name included in the address field
                     final String gname = s.substring(start, index);
-                    if (ignoreBogusGroupName && (gname.equalsIgnoreCase("mailto") || gname.equalsIgnoreCase("From") || gname.equalsIgnoreCase("To") || gname.equalsIgnoreCase("Cc") || gname.equalsIgnoreCase("Subject") || gname.equalsIgnoreCase("Re"))) {
+                    if (IGNORE_BOGUS_GROUP_NAME && (gname.equalsIgnoreCase("mailto") || gname.equalsIgnoreCase("From") || gname.equalsIgnoreCase("To") || gname.equalsIgnoreCase("Cc") || gname.equalsIgnoreCase("Subject") || gname.equalsIgnoreCase("Re"))) {
                         start = -1; // we're not really in a group
                     } else {
                         in_group = true;
@@ -551,7 +571,7 @@ public final class QuotedInternetAddress extends InternetAddress {
             String pers = null;
             if (rfc822 && start_personal >= 0) {
                 pers = unquote(s.substring(start_personal, end_personal).trim());
-                if (pers.trim().length() == 0) {
+                if (isEmpty(pers)) {
                     pers = null;
                 }
             }
@@ -667,7 +687,7 @@ public final class QuotedInternetAddress extends InternetAddress {
             if (c <= 040 || c >= 0177) {
                 throw new AddressException("Local address contains control or whitespace", addr);
             }
-            if (specialsNoDot.indexOf(c) >= 0) {
+            if (SPECIALS_NO_DOT.indexOf(c) >= 0) {
                 throw new AddressException("Local address contains illegal character", addr);
             }
         }
@@ -707,7 +727,7 @@ public final class QuotedInternetAddress extends InternetAddress {
             if (c <= 040 || c >= 0177) {
                 throw new AddressException("Domain contains control or whitespace", addr);
             }
-            if (specialsNoDot.indexOf(c) >= 0) {
+            if (SPECIALS_NO_DOT.indexOf(c) >= 0) {
                 throw new AddressException("Domain contains illegal character", addr);
             }
             if (c == '.' && lastc == '.') {
@@ -720,32 +740,51 @@ public final class QuotedInternetAddress extends InternetAddress {
         }
     }
 
-    private static String toACE(final String idnString) throws AddressException {
-        if (null == idnString) {
+    /**
+     * Converts a unicode representation of an internet address to ASCII using the procedure in RFC3490 section 4.1. Unassigned characters
+     * are not allowed and STD3 ASCII rules are enforced.
+     * <p>
+     * <code>"someone@m&uuml;ller.de"</code> is converted to <code>"someone@xn--mller-kva.de"</code>
+     * 
+     * @param idnAddress The unicode representation of an internet address
+     * @return The ASCII-encoded (punycode) of given internet address
+     * @throws AddressException If ASCII representation of given internet address cannot be created
+     */
+    public static String toACE(final String idnAddress) throws AddressException {
+        if (null == idnAddress) {
             return null;
         }
         try {
-            final int pos = idnString.indexOf('@');
+            final int pos = idnAddress.indexOf('@');
             if (pos < 0) {
-                return idnString;
+                return idnAddress;
             }
-            return new StringBuilder(idnString.length() + 8).append(idnString.substring(0, pos)).append('@').append(
-                gnu.inet.encoding.IDNA.toASCII(idnString.substring(pos + 1))).toString();
+            return new StringBuilder(idnAddress.length() + 8).append(idnAddress.substring(0, pos)).append('@').append(
+                gnu.inet.encoding.IDNA.toASCII(idnAddress.substring(pos + 1))).toString();
         } catch (final gnu.inet.encoding.IDNAException e) {
-            throw new AddressException(new StringBuilder(e.getMessage()).append(": ").append(idnString).toString());
+            throw new AddressException(new StringBuilder(e.getMessage()).append(": ").append(idnAddress).toString());
         }
     }
 
-    private static String toIDN(final String aceString) {
-        if (null == aceString) {
+    /**
+     * Converts an ASCII-encoded address to its unicode representation. Unassigned characters are not allowed and STD3 hostnames are
+     * enforced.
+     * <p>
+     * <code>"someone@xn--mller-kva.de"</code> is converted to <code>"someone@m&uuml;ller.de"</code>
+     * 
+     * @param aceAddress The ASCII-encoded (punycode) address
+     * @return The unicode representation of given internet address
+     */
+    public static String toIDN(final String aceAddress) {
+        if (null == aceAddress) {
             return null;
         }
-        final int pos = aceString.indexOf('@');
+        final int pos = aceAddress.indexOf('@');
         if (pos < 0) {
-            return aceString;
+            return aceAddress;
         }
-        return new StringBuilder(aceString.length()).append(aceString.substring(0, pos)).append('@').append(
-            gnu.inet.encoding.IDNA.toUnicode(aceString.substring(pos + 1))).toString();
+        return new StringBuilder(aceAddress.length()).append(aceAddress.substring(0, pos)).append('@').append(
+            gnu.inet.encoding.IDNA.toUnicode(aceAddress.substring(pos + 1))).toString();
     }
 
     /**
@@ -755,6 +794,20 @@ public final class QuotedInternetAddress extends InternetAddress {
         super();
     }
 
+    /**
+     * Copy constructor.
+     */
+    private QuotedInternetAddress(final InternetAddress src) throws AddressException {
+        this();
+        this.address = toACE(src.getAddress());
+        try {
+            setPersonal(getPersonal(), null);
+        } catch (final UnsupportedEncodingException e) {
+            // Cannot occur
+            throw new IllegalStateException("Unsupported default charset.");
+        }
+    }
+
     /**
      * Initializes a new {@link QuotedInternetAddress}.
      * <p>
@@ -870,12 +923,11 @@ public final class QuotedInternetAddress extends InternetAddress {
     }
 
     /**
-     * Gets the email address.
+     * Gets the email address in Unicode characters.
      * 
-     * @return The email address
+     * @return The email address in Unicode characters
      */
-    @Override
-    public String getAddress() {
+    public String getUnicodeAddress() {
         return toIDN(address);
     }
 
@@ -958,15 +1010,34 @@ public final class QuotedInternetAddress extends InternetAddress {
         } else if (isGroup() || isSimple()) {
             return toIDN(address);
         } else {
-            return new StringBuilder().append('<').append(toIDN(address)).append('>').toString();
+            return new StringBuilder(32).append('<').append(toIDN(address)).append('>').toString();
         }
     }
 
+    // @Override
+    // public boolean equals(final Object a) {
+    // if (this == a) {
+    // return true;
+    // }
+    // if (!(a instanceof InternetAddress)) {
+    // return false;
+    // }
+    // final String s = ((InternetAddress) a).getAddress();
+    // if (address == null) {
+    // if (s != null) {
+    // return false;
+    // }
+    // } else if (!address.equalsIgnoreCase(s) && !toIDN(address).equalsIgnoreCase(s)) {
+    // return false;
+    // }
+    // return true;
+    // }
+
     /**
      * Is this a "simple" address? Simple addresses don't contain quotes or any RFC822 special characters other than '@' and '.'.
      */
     private boolean isSimple() {
-        return address == null || indexOfAny(address, specialsNoDotNoAt) < 0;
+        return null == address || indexOfAny(address, SPECIALS_NO_DOT_NO_AT) < 0;
     }
 
     /**
@@ -990,9 +1061,9 @@ public final class QuotedInternetAddress extends InternetAddress {
         }
     }
 
-    private static final String specialsNoDotNoAt = "()<>,;:\\\"[]";
+    private static final String SPECIALS_NO_DOT_NO_AT = "()<>,;:\\\"[]";
 
-    private static final String specialsNoDot = "()<>@,;:\\\"[]";
+    private static final String SPECIALS_NO_DOT = "()<>@,;:\\\"[]";
 
     private final static String RFC822 = "()<>@,;:\\\".[]";
 
@@ -1076,6 +1147,9 @@ public final class QuotedInternetAddress extends InternetAddress {
     }
 
     private static String unquote(final String str) {
+        if (isEmpty(str)) {
+            return str;
+        }
         String s = str;
         int length = s.length();
         if ('"' == s.charAt(0) && '"' == s.charAt(length - 1)) {
@@ -1097,4 +1171,16 @@ public final class QuotedInternetAddress extends InternetAddress {
         return s;
     }
 
+    private static boolean isEmpty(final String str) {
+        if (null == str || 0 == str.length()) {
+            return true;
+        }
+        final int len = str.length();
+        boolean ret = true;
+        for (int i = 0; ret && i < len; i++) {
+            ret = Character.isWhitespace(str.charAt(i));
+        }
+        return ret;
+    }
+
 }
