@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.mail.mime.converters;
 
+import static com.openexchange.mail.mime.utils.MIMEMessageUtility.checkNonAscii;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.decodeMultiEncodedHeader;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.getFileName;
 import static com.openexchange.mail.mime.utils.MIMEMessageUtility.hasAttachments;
@@ -57,10 +58,13 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.EnumMap;
 import java.util.EnumSet;
 import java.util.Enumeration;
+import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import javax.mail.Flags;
@@ -115,6 +119,29 @@ public final class MIMEMessageConverter {
     private static final EnumSet<MailField> ENUM_SET_FULL =
         EnumSet.complementOf(EnumSet.of(MailField.BODY, MailField.FULL, MailField.ACCOUNT_NAME));
 
+    /**
+     * {@link ExistenceChecker} - A checker to ensure existence of a certain field.
+     * 
+     * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+     * @since Open-Xchange v6.16
+     */
+    private interface ExistenceChecker {
+
+        /**
+         * Checks existence of a certain field in given mails.
+         * 
+         * @param mailMessages The mails to check
+         */
+        void check(MailMessage... mailMessages);
+
+        /**
+         * Checks existence of a certain field in given mails.
+         * 
+         * @param mailMessages The mail to check
+         */
+        void check(Collection<MailMessage> mailMessages);
+    }
+
     private static interface MailMessageFieldFiller {
 
         public static final String[] NON_MATCHING_HEADERS =
@@ -124,6 +151,9 @@ public final class MIMEMessageConverter {
                 MessageHeaders.HDR_X_PRIORITY, MessageHeaders.HDR_MESSAGE_ID, MessageHeaders.HDR_IN_REPLY_TO,
                 MessageHeaders.HDR_REFERENCES, MessageHeaders.HDR_X_OX_VCARD, MessageHeaders.HDR_X_OX_NOTIFICATION };
 
+        public static final String[] ALREADY_INSERTED_HEADERS =
+            { MessageHeaders.HDR_MESSAGE_ID, MessageHeaders.HDR_REPLY_TO, MessageHeaders.HDR_REFERENCES };
+
         public static final org.apache.commons.logging.Log LOG1 =
             org.apache.commons.logging.LogFactory.getLog(MailMessageFieldFiller.class);
 
@@ -138,6 +168,27 @@ public final class MIMEMessageConverter {
         public void fillField(final MailMessage mailMessage, final Message msg) throws MessagingException, MailException;
     }
 
+    private static final class HeaderFieldFiller implements MailMessageFieldFiller {
+
+        private final String headerName;
+
+        HeaderFieldFiller(final String headerName) {
+            super();
+            this.headerName = headerName;
+        }
+
+        public void fillField(final MailMessage mailMessage, final Message msg) throws MessagingException, MailException {
+            final String[] header = msg.getHeader(headerName);
+            if (null == header || 0 == header.length) {
+                return;
+            }
+            for (final String value : header) {
+                mailMessage.addHeader(headerName, checkNonAscii(value));
+            }
+        }
+
+    }
+
     private static abstract class ExtendedMailMessageFieldFiller implements MailMessageFieldFiller {
 
         final Folder folder;
@@ -400,7 +451,7 @@ public final class MIMEMessageConverter {
      * @throws MailException If conversion fails
      */
     public static MailMessage[] convertMessages(final Message[] msgs, final MailField[] fields) throws MailException {
-        return convertMessages(msgs, fields, false);
+        return convertMessages(msgs, fields, null, false);
     }
 
     /**
@@ -412,13 +463,23 @@ public final class MIMEMessageConverter {
      * @see #convertMessages(Message[], Folder, MailField[])
      * @param msgs The source messages
      * @param fields The fields to fill
+     * @param headerNames The header names
      * @param includeBody Whether to create mail messages with reference to content or not
      * @return The converted array of {@link Message} instances
      * @throws MailException If conversion fails
      */
-    public static MailMessage[] convertMessages(final Message[] msgs, final MailField[] fields, final boolean includeBody) throws MailException {
+    public static MailMessage[] convertMessages(final Message[] msgs, final MailField[] fields, final String[] headerNames, final boolean includeBody) throws MailException {
         try {
-            final MailMessageFieldFiller[] fillers = createFieldFillers(fields);
+            final MailMessageFieldFiller[] fillers;
+            {
+                final List<MailMessageFieldFiller> tmp = new ArrayList<MailMessageFieldFiller>(Arrays.asList(createFieldFillers(fields)));
+                if (null != headerNames) {
+                    for (final String headerName : headerNames) {
+                        tmp.add(new HeaderFieldFiller(headerName));
+                    }
+                }
+                fillers = tmp.toArray(new MailMessageFieldFiller[tmp.size()]);
+            }
             final MailMessage[] mails = new MIMEMailMessage[msgs.length];
             for (int i = 0; i < mails.length; i++) {
                 if (null != msgs[i]) {
@@ -477,7 +538,176 @@ public final class MIMEMessageConverter {
     private static final EnumMap<MailField, MailMessageFieldFiller> FILLER_MAP_EXT =
         new EnumMap<MailField, MailMessageFieldFiller>(MailField.class);
 
+    private static final EnumMap<MailField, ExistenceChecker> CHECKER_MAP;
+
     static {
+        final org.apache.commons.logging.Log logger = LOG;
+
+        CHECKER_MAP = new EnumMap<MailField, ExistenceChecker>(MailField.class);
+        final InternetAddress empty = null;
+        CHECKER_MAP.put(MailField.FROM, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsFrom()) {
+                        mailMessage.addFrom(empty);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsFrom()) {
+                        mailMessage.addFrom(empty);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.TO, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsTo()) {
+                        mailMessage.addTo(empty);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsTo()) {
+                        mailMessage.addTo(empty);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.CC, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsCc()) {
+                        mailMessage.addCc(empty);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsCc()) {
+                        mailMessage.addCc(empty);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.BCC, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsBcc()) {
+                        mailMessage.addBcc(empty);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsBcc()) {
+                        mailMessage.addBcc(empty);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.SUBJECT, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsSubject()) {
+                        mailMessage.setSubject(null);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsSubject()) {
+                        mailMessage.setSubject(null);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.SENT_DATE, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsSentDate()) {
+                        mailMessage.setSentDate(null);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsSentDate()) {
+                        mailMessage.setSentDate(null);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.DISPOSITION_NOTIFICATION_TO, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsDispositionNotification()) {
+                        mailMessage.setDispositionNotification(null);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsDispositionNotification()) {
+                        mailMessage.setDispositionNotification(null);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.PRIORITY, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsPriority()) {
+                        mailMessage.setPriority(MailMessage.PRIORITY_NORMAL);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsPriority()) {
+                        mailMessage.setPriority(MailMessage.PRIORITY_NORMAL);
+                    }
+                }
+            }
+        });
+        CHECKER_MAP.put(MailField.THREAD_LEVEL, new ExistenceChecker() {
+
+            public void check(final MailMessage... mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsThreadLevel()) {
+                        mailMessage.setThreadLevel(0);
+                    }
+                }
+            }
+
+            public void check(final Collection<MailMessage> mailMessages) {
+                for (final MailMessage mailMessage : mailMessages) {
+                    if (null != mailMessage && !mailMessage.containsThreadLevel()) {
+                        mailMessage.setThreadLevel(0);
+                    }
+                }
+            }
+        });
+
         FILLER_MAP_EXT.put(MailField.HEADERS, new MailMessageFieldFiller() {
 
             public void fillField(final MailMessage mailMessage, final Message msg) throws MessagingException {
@@ -584,11 +814,15 @@ public final class MIMEMessageConverter {
                     }
                 }
                 /*
-                 * All other
+                 * Add all as regular headers; except previously inserted headers
                  */
-                for (final Enumeration<?> e = extMimeMessage.getNonMatchingHeaders(NON_MATCHING_HEADERS); e.hasMoreElements();) {
+                for (final Enumeration<?> e = msg.getNonMatchingHeaders(ALREADY_INSERTED_HEADERS); e.hasMoreElements();) {
                     final Header h = (Header) e.nextElement();
-                    mailMessage.addHeader(h.getName(), h.getValue());
+                    try {
+                        mailMessage.addHeader(h.getName(), h.getValue());
+                    } catch (final Exception exc) {
+                        logger.warn(exc.getMessage(), exc);
+                    }
                 }
             }
         });
@@ -723,7 +957,8 @@ public final class MIMEMessageConverter {
     private static MailMessageFieldFiller[] createFieldFillers(final MailField[] fields) throws MailException {
         final MailField[] arr;
         {
-            final EnumSet<MailField> fieldSet = EnumSet.copyOf(Arrays.asList(fields));
+            final List<MailField> list = Arrays.asList(fields);
+            final EnumSet<MailField> fieldSet = list.isEmpty() ? EnumSet.noneOf(MailField.class) : EnumSet.copyOf(list);
             if (fieldSet.contains(MailField.FULL)) {
                 arr = ENUM_SET_FULL.toArray(new MailField[ENUM_SET_FULL.size()]);
             } else {
@@ -754,6 +989,7 @@ public final class MIMEMessageConverter {
         new EnumMap<MailField, MailMessageFieldFiller>(MailField.class);
 
     static {
+        final org.apache.commons.logging.Log logger = LOG;
         FILLER_MAP.put(MailField.HEADERS, new MailMessageFieldFiller() {
 
             public void fillField(final MailMessage mailMessage, final Message msg) throws MessagingException {
@@ -866,11 +1102,15 @@ public final class MIMEMessageConverter {
                     }
                 }
                 /*
-                 * All other
+                 * Add all as regular headers; except previously inserted headers
                  */
-                for (final Enumeration<?> e = msg.getNonMatchingHeaders(NON_MATCHING_HEADERS); e.hasMoreElements();) {
+                for (final Enumeration<?> e = msg.getNonMatchingHeaders(ALREADY_INSERTED_HEADERS); e.hasMoreElements();) {
                     final Header h = (Header) e.nextElement();
-                    mailMessage.addHeader(h.getName(), h.getValue());
+                    try {
+                        mailMessage.addHeader(h.getName(), h.getValue());
+                    } catch (final Exception exc) {
+                        logger.warn(exc.getMessage(), exc);
+                    }
                 }
             }
         });
@@ -1028,6 +1268,24 @@ public final class MIMEMessageConverter {
         });
     }
 
+    /**
+     * Checks field existence.
+     * 
+     * @param mails The mails to checks
+     * @param fields The field to check for
+     */
+    public static void checkFieldExistence(final MailMessage[] mails, final MailField[] fields) {
+        if (null == mails) {
+            return;
+        }
+        for (final MailField field : fields) {
+            final ExistenceChecker checker = CHECKER_MAP.get(field);
+            if (null != checker) {
+                checker.check(mails);
+            }
+        }
+    }
+
     /**
      * Creates the field fillers and expects the messages to be common instances of {@link Message}.
      * 
@@ -1039,7 +1297,8 @@ public final class MIMEMessageConverter {
     private static MailMessageFieldFiller[] createFieldFillers(final Folder folder, final MailField[] fields) throws MailException {
         final MailField[] arr;
         {
-            final EnumSet<MailField> fieldSet = EnumSet.copyOf(Arrays.asList(fields));
+            final List<MailField> list = Arrays.asList(fields);
+            final EnumSet<MailField> fieldSet = list.isEmpty() ? EnumSet.noneOf(MailField.class) : EnumSet.copyOf(list);
             if (fieldSet.contains(MailField.FULL)) {
                 arr = ENUM_SET_FULL.toArray(new MailField[ENUM_SET_FULL.size()]);
             } else {
@@ -1767,13 +2026,13 @@ public final class MIMEMessageConverter {
         }
         final String values;
         if ('\0' != delimiter && valueArr.length > 1) {
-            final StringBuilder sb = new StringBuilder(valueArr[0]);
+            final StringBuilder sb = new StringBuilder(checkNonAscii(valueArr[0]));
             for (int i = 1; i < valueArr.length; i++) {
-                sb.append(delimiter).append(valueArr[i]);
+                sb.append(delimiter).append(checkNonAscii(valueArr[i]));
             }
             values = sb.toString();
         } else {
-            values = valueArr[0];
+            values = checkNonAscii(valueArr[0]);
         }
         return decodeMultiEncodedHeader(values);
     }
@@ -1817,6 +2076,26 @@ public final class MIMEMessageConverter {
         }
     }
 
+    /**
+     * Gets the address header from given address header value.
+     * 
+     * @param addresses The address header value
+     * @return The parsed addresses
+     */
+    public static InternetAddress[] getAddressHeader(final String addresses) {
+        try {
+            return QuotedInternetAddress.parseHeader(addresses, true);
+        } catch (final AddressException e) {
+            if (DEBUG) {
+                LOG.debug(
+                    new StringBuilder(128).append("Internet addresses could not be properly parsed: \"").append(e.getMessage()).append(
+                        "\". Using plain addresses' string representation instead.").toString(),
+                    e);
+            }
+            return new InternetAddress[] { new PlainTextAddress(addresses) };
+        }
+    }
+
     private static InternetAddress[] getAddressesOnParseError(final String[] addrs) {
         final InternetAddress[] retval = new InternetAddress[addrs.length];
         for (int i = 0; i < addrs.length; i++) {
