@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -113,6 +113,8 @@ public final class MIMEMessageUtility {
 
     private static final Set<HeaderName> ENCODINGS;
 
+    private static final MailDateFormat MAIL_DATE_FORMAT;
+
     static {
         final Set<HeaderName> tmp = new HashSet<HeaderName>(4);
         tmp.add(HeaderName.valueOf("iso-8859-1"));
@@ -120,6 +122,9 @@ public final class MIMEMessageUtility {
         tmp.add(HeaderName.valueOf("UTF-8"));
         tmp.add(HeaderName.valueOf("us-ascii"));
         ENCODINGS = java.util.Collections.unmodifiableSet(tmp);
+        final MailDateFormat mdf = new MailDateFormat();
+        mdf.setTimeZone(TimeZoneUtils.getTimeZone("UTC"));
+        MAIL_DATE_FORMAT = mdf;
     }
 
     private static final ConcurrentMap<String, Future<MailDateFormat>> MDF_MAP = new ConcurrentHashMap<String, Future<MailDateFormat>>();
@@ -131,6 +136,27 @@ public final class MIMEMessageUtility {
         super();
     }
 
+    /**
+     * Gets the default {@link MailDateFormat}.
+     * <p>
+     * Note that returned instance of {@link MailDateFormat} is shared, therefore use a surrounding synchronized block to preserve thread
+     * safety:
+     * 
+     * <pre>
+     * ...
+     * final MailDateFormat mdf = MIMEMessageUtility.getMailDateFormat(session);
+     * synchronized(mdf) {
+     *  mimeMessage.setHeader(&quot;Date&quot;, mdf.format(sendDate));
+     * }
+     * ...
+     * </pre>
+     * 
+     * @return The {@link MailDateFormat} for specified session
+     */
+    public static MailDateFormat getDefaultMailDateFormat() {
+        return MAIL_DATE_FORMAT;
+    }
+
     /**
      * Gets the {@link MailDateFormat} for specified session.
      * <p>
@@ -609,6 +635,47 @@ public final class MIMEMessageUtility {
         }
     }
 
+    /**
+     * Checks if given raw header contains non-ascii characters.
+     * 
+     * @param rawHeader The raw header
+     * @return The proper unicode string
+     */
+    public static String checkNonAscii(final String rawHeader) {
+        if (null == rawHeader) {
+            return null;
+        }
+        if (isAscii(rawHeader)) {
+            return rawHeader;
+        }
+        final int length = rawHeader.length();
+        final byte[] bytes = new byte[length];
+        for (int i = 0; i < length; i++) {
+            bytes[i] = (byte) rawHeader.charAt(i);
+        }
+        try {
+            return new String(bytes, MailProperties.getInstance().getDefaultMimeCharset());
+        } catch (final UnsupportedEncodingException e) {
+            // Cannot occur
+            return rawHeader;
+        }
+    }
+
+    /**
+     * Checks whether the specified string's characters are ASCII 7 bit
+     * 
+     * @param s The string to check
+     * @return <code>true</code> if string's characters are ASCII 7 bit; otherwise <code>false</code>
+     */
+    private static boolean isAscii(final String s) {
+        final char[] chars = s.toCharArray();
+        boolean isAscci = true;
+        for (int i = 0; (i < chars.length) && isAscci; i++) {
+            isAscci &= (chars[i] < 128);
+        }
+        return isAscci;
+    }
+
     /**
      * Decodes a multi-mime-encoded header value using the algorithm specified in RFC 2047, Section 6.1 in a safe manner.
      * 
@@ -776,9 +843,8 @@ public final class MIMEMessageUtility {
      * @param addresslist - comma separated address strings
      * @param strict - <code>true</code> to enforce RFC822 syntax; otherwise <code>false</code>
      * @return An array of <code>InternetAddress</code> objects
-     * @throws AddressException - if parsing fails
      */
-    public static InternetAddress[] parseAddressList(final String addresslist, final boolean strict) throws AddressException {
+    public static InternetAddress[] parseAddressList(final String addresslist, final boolean strict) {
         if (null == addresslist) {
             return new InternetAddress[0];
         }
