@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,14 +52,17 @@ package com.openexchange.mail.mime.utils;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.EnumMap;
 import java.util.EnumSet;
-import java.util.Iterator;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 import javax.mail.FetchProfile;
 import javax.mail.UIDFolder;
 import javax.mail.FetchProfile.Item;
 import com.openexchange.mail.MailField;
+import com.openexchange.mail.mime.HeaderName;
 import com.openexchange.mail.mime.MessageHeaders;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.UIDSet;
@@ -91,7 +94,8 @@ public final class MIMEStorageUtility {
         CACHE_FETCH_PROFILE.add(FetchProfile.Item.CONTENT_INFO);
         CACHE_FETCH_PROFILE.add(UIDFolder.FetchProfileItem.UID);
         CACHE_FETCH_PROFILE.add(IMAPFolder.FetchProfileItem.SIZE);
-        CACHE_FETCH_PROFILE.add(MessageHeaders.HDR_X_PRIORITY);
+        // CACHE_FETCH_PROFILE.add(IMAPFolder.FetchProfileItem.HEADERS);
+
         // Cache fields
         final Collection<MailField> fields = fetchProfile2MailListFields(CACHE_FETCH_PROFILE);
         fields.add(MailField.ACCOUNT_NAME);
@@ -324,10 +328,11 @@ public final class MIMEStorageUtility {
         // ENV_FIELDS.add(MailListField.SIZE);
     }
 
-    private static final EnumSet<MailField> ENUM_SET_FULL = EnumSet.complementOf(EnumSet.of(
-        MailField.BODY,
-        MailField.FULL,
-        MailField.ACCOUNT_NAME));
+    private static final EnumSet<MailField> ENUM_SET_FULL =
+        EnumSet.complementOf(EnumSet.of(MailField.BODY, MailField.FULL, MailField.ACCOUNT_NAME));
+
+    private static final List<HeaderName> ENV_LIST =
+        Arrays.asList(HeaderName.valuesOf("From", "To", "Cc", "Bcc", "Subject", "Date", "ReplyTo"));
 
     /**
      * Gets the appropriate fetch profile
@@ -342,16 +347,34 @@ public final class MIMEStorageUtility {
      * @return The appropriate IMAP fetch profile
      */
     public static FetchProfile getFetchProfile(final MailField[] fields, final MailField[] searchFields, final MailField sortField, final boolean preferEnvelope) {
+        return getFetchProfile(fields, null, searchFields, sortField, preferEnvelope);
+    }
+
+    /**
+     * Gets the appropriate fetch profile
+     * <p>
+     * <b>Note</b> that {@link MailField#BODY} and {@link MailField#FULL} are discarded since no corresponding fetch profile item exists and
+     * therefore should be handled separately.
+     * 
+     * @param fields The fields
+     * @param headerNames The header names
+     * @param searchFields The search fields
+     * @param sortField The sort field
+     * @param preferEnvelope <code>true</code> to prefer ENVELOPE instead of single fetch items; otherwise <code>false</code>
+     * @return The appropriate IMAP fetch profile
+     */
+    public static FetchProfile getFetchProfile(final MailField[] fields, final String[] headerNames, final MailField[] searchFields, final MailField sortField, final boolean preferEnvelope) {
         final MailField[] arr;
         {
-            final EnumSet<MailField> fieldSet = EnumSet.copyOf(Arrays.asList(fields));
+            final List<MailField> list = Arrays.asList(fields);
+            final EnumSet<MailField> fieldSet = list.isEmpty() ? EnumSet.noneOf(MailField.class) : EnumSet.copyOf(list);
             if (fieldSet.contains(MailField.FULL)) {
                 arr = ENUM_SET_FULL.toArray(new MailField[ENUM_SET_FULL.size()]);
             } else {
                 arr = fields;
             }
         }
-        final FetchProfile retval = new FetchProfile();
+        final FetchProfile fetchProfile = new FetchProfile();
         /*
          * Use a set to avoid duplicate entries
          */
@@ -365,6 +388,15 @@ public final class MIMEStorageUtility {
         if (sortField != null) {
             set.add(sortField);
         }
+        /*
+         * Set of header names
+         */
+        final Set<HeaderName> names;
+        if (null == headerNames) {
+            names = Collections.emptySet();
+        } else {
+            names = new HashSet<HeaderName>(Arrays.asList(HeaderName.valuesOf(headerNames)));
+        }
         /*
          * Check which fields are contained in fetch profile item "ENVELOPE"
          */
@@ -372,44 +404,85 @@ public final class MIMEStorageUtility {
             /*
              * Add ENVELOPE since set of fields has changed
              */
-            retval.add(FetchProfile.Item.ENVELOPE);
+            fetchProfile.add(FetchProfile.Item.ENVELOPE);
+            /*
+             * Remove header names covered by ENVELOPE
+             */
+            names.removeAll(ENV_LIST);
         }
+        /*
+         * Proceed
+         */
         if (!set.isEmpty()) {
-            final int size = set.size();
-            final Iterator<MailField> iter = set.iterator();
-            for (int i = 0; i < size; i++) {
-                addFetchItem(retval, iter.next());
+            /*
+             * Check set against header names
+             */
+            if (set.contains(MailField.FROM)) {
+                names.remove(HeaderName.valueOf("From"));
+            }
+            if (set.contains(MailField.TO)) {
+                names.remove(HeaderName.valueOf("To"));
+            }
+            if (set.contains(MailField.CC)) {
+                names.remove(HeaderName.valueOf("Cc"));
+            }
+            if (set.contains(MailField.BCC)) {
+                names.remove(HeaderName.valueOf("Bcc"));
+            }
+            if (set.contains(MailField.SUBJECT)) {
+                names.remove(HeaderName.valueOf("Subject"));
+            }
+            if (set.contains(MailField.SENT_DATE)) {
+                names.remove(HeaderName.valueOf("Date"));
+            }
+            if (set.contains(MailField.DISPOSITION_NOTIFICATION_TO)) {
+                names.remove(HeaderName.valueOf("Disposition-Notification-To"));
+            }
+            if (set.contains(MailField.PRIORITY)) {
+                names.remove(HeaderName.valueOf("X-Priority"));
+            }
+            /*
+             * Iterate fields
+             */
+            for (final MailField mailField : set) {
+                addFetchItem(fetchProfile, mailField);
+            }
+            /*
+             * Iterate header names
+             */
+            for (final HeaderName headerName : names) {
+                fetchProfile.add(headerName.toString());
             }
         }
-        return retval;
+        return fetchProfile;
     }
 
-    private static final EnumMap<MailField, FetchProfile.Item> field2item;
+    private static final EnumMap<MailField, FetchProfile.Item> FIELD2ITEM;
 
-    private static final EnumMap<MailField, String> field2string;
+    private static final EnumMap<MailField, String> FIELD2STRING;
 
     static {
         /*
          * Item map
          */
-        field2item = new EnumMap<MailField, FetchProfile.Item>(MailField.class);
-        field2item.put(MailField.HEADERS, IMAPFolder.FetchProfileItem.HEADERS);
-        field2item.put(MailField.ID, UIDFolder.FetchProfileItem.UID);
-        field2item.put(MailField.CONTENT_TYPE, FetchProfile.Item.CONTENT_INFO);
-        field2item.put(MailField.SIZE, IMAPFolder.FetchProfileItem.SIZE);
-        field2item.put(MailField.FLAGS, FetchProfile.Item.FLAGS);
+        FIELD2ITEM = new EnumMap<MailField, FetchProfile.Item>(MailField.class);
+        FIELD2ITEM.put(MailField.HEADERS, IMAPFolder.FetchProfileItem.HEADERS);
+        FIELD2ITEM.put(MailField.ID, UIDFolder.FetchProfileItem.UID);
+        FIELD2ITEM.put(MailField.CONTENT_TYPE, FetchProfile.Item.CONTENT_INFO);
+        FIELD2ITEM.put(MailField.SIZE, IMAPFolder.FetchProfileItem.SIZE);
+        FIELD2ITEM.put(MailField.FLAGS, FetchProfile.Item.FLAGS);
         /*
          * String map
          */
-        field2string = new EnumMap<MailField, String>(MailField.class);
-        field2string.put(MailField.FROM, MessageHeaders.HDR_FROM);
-        field2string.put(MailField.TO, MessageHeaders.HDR_TO);
-        field2string.put(MailField.CC, MessageHeaders.HDR_CC);
-        field2string.put(MailField.BCC, MessageHeaders.HDR_BCC);
-        field2string.put(MailField.SUBJECT, MessageHeaders.HDR_SUBJECT);
-        field2string.put(MailField.SENT_DATE, MessageHeaders.HDR_DATE);
-        field2string.put(MailField.DISPOSITION_NOTIFICATION_TO, MessageHeaders.HDR_DISP_NOT_TO);
-        field2string.put(MailField.PRIORITY, MessageHeaders.HDR_X_PRIORITY);
+        FIELD2STRING = new EnumMap<MailField, String>(MailField.class);
+        FIELD2STRING.put(MailField.FROM, MessageHeaders.HDR_FROM);
+        FIELD2STRING.put(MailField.TO, MessageHeaders.HDR_TO);
+        FIELD2STRING.put(MailField.CC, MessageHeaders.HDR_CC);
+        FIELD2STRING.put(MailField.BCC, MessageHeaders.HDR_BCC);
+        FIELD2STRING.put(MailField.SUBJECT, MessageHeaders.HDR_SUBJECT);
+        FIELD2STRING.put(MailField.SENT_DATE, MessageHeaders.HDR_DATE);
+        FIELD2STRING.put(MailField.DISPOSITION_NOTIFICATION_TO, MessageHeaders.HDR_DISP_NOT_TO);
+        FIELD2STRING.put(MailField.PRIORITY, MessageHeaders.HDR_X_PRIORITY);
     }
 
     private static void addFetchItem(final FetchProfile fp, final MailField field) {
@@ -419,12 +492,15 @@ public final class MIMEStorageUtility {
             }
             return;
         }
-        if (field2item.containsKey(field)) {
-            fp.add(field2item.get(field));
+        final Item item = FIELD2ITEM.get(field);
+        if (null != item) {
+            fp.add(item);
             return;
         }
-        if (field2string.containsKey(field)) {
-            fp.add(field2string.get(field));
+        final String string = FIELD2STRING.get(field);
+        if (null != string) {
+            fp.add(string);
         }
     }
+
 }
