@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -158,7 +158,15 @@ public final class MailMessageParser {
 
     private static final String TNEF_IPM_MS_READ_RECEIPT = "IPM.Microsoft Mail.Read Receipt";
 
-    // private static final String TNEF_IPM_MS_SCHEDULE = "IPM.Microsoft Schedule.MtgCncl";
+    // private static final String TNEF_IPM_MS_SCHEDULE_CANCELED = "IPM.Microsoft Schedule.MtgCncl";
+
+    // private static final String TNEF_IPM_MS_SCHEDULE_REQUEST = "IPM.Microsoft Schedule.MtgReq";
+
+    // private static final String TNEF_IPM_MS_SCHEDULE_ACCEPTED = "IPM.Microsoft Schedule.MtgRespP";
+
+    // private static final String TNEF_IPM_MS_SCHEDULE_DECLINED = "IPM.Microsoft Schedule.MtgRespN";
+
+    // private static final String TNEF_IPM_MS_SCHEDULE_TENTATIVE = "IPM.Microsoft Schedule.MtgRespA";
 
     /*
      * +++++++++++++++++++ MEMBERS +++++++++++++++++++
@@ -270,7 +278,31 @@ public final class MailMessageParser {
          * final boolean isInline = ((disposition == null
          *     || disposition.equalsIgnoreCase(Part.INLINE)) && mailPart.getFileName() == null);
          */
-        if (isText(lcct)) {
+        if (isMultipart(lcct)) {
+            final int count = mailPart.getEnclosedCount();
+            if (count == -1) {
+                throw new MailException(MailException.Code.INVALID_MULTIPART_CONTENT);
+            }
+            final String mpId = null == prefix && !multipartDetected ? "" : getSequenceId(prefix, partCount);
+            if (!mailPart.containsSequenceId()) {
+                mailPart.setSequenceId(mpId);
+            }
+            if (!handler.handleMultipart(mailPart, count, mpId)) {
+                stop = true;
+                return;
+            }
+            final String mpPrefix;
+            if (multipartDetected) {
+                mpPrefix = mpId;
+            } else {
+                mpPrefix = prefix;
+                multipartDetected = true;
+            }
+            for (int i = 0; i < count; i++) {
+                final MailPart enclosedContent = mailPart.getEnclosedMailPart(i);
+                parseMailContent(enclosedContent, handler, mpPrefix, i + 1);
+            }
+        } else if (isText(lcct)) {
             if (isInline) {
                 final String content = readContent(mailPart, contentType);
                 final UUEncodedMultiPart uuencodedMP = new UUEncodedMultiPart(content);
@@ -341,30 +373,6 @@ public final class MailMessageParser {
                     return;
                 }
             }
-        } else if (isMultipart(lcct)) {
-            final int count = mailPart.getEnclosedCount();
-            if (count == -1) {
-                throw new MailException(MailException.Code.INVALID_MULTIPART_CONTENT);
-            }
-            final String mpId = null == prefix && !multipartDetected ? "" : getSequenceId(prefix, partCount);
-            if (!mailPart.containsSequenceId()) {
-                mailPart.setSequenceId(mpId);
-            }
-            if (!handler.handleMultipart(mailPart, count, mpId)) {
-                stop = true;
-                return;
-            }
-            final String mpPrefix;
-            if (multipartDetected) {
-                mpPrefix = mpId;
-            } else {
-                mpPrefix = prefix;
-                multipartDetected = true;
-            }
-            for (int i = 0; i < count; i++) {
-                final MailPart enclosedContent = mailPart.getEnclosedMailPart(i);
-                parseMailContent(enclosedContent, handler, mpPrefix, i + 1);
-            }
         } else if (isImage(lcct)) {
             if (!mailPart.containsSequenceId()) {
                 mailPart.setSequenceId(getSequenceId(prefix, partCount));
@@ -527,7 +535,9 @@ public final class MailMessageParser {
                             if (attachFilename != null) {
                                 final ContentDisposition cd = new ContentDisposition(Part.ATTACHMENT);
                                 cd.setFilenameParameter(attachFilename);
-                                bodyPart.setHeader(MessageHeaders.HDR_CONTENT_DISPOSITION, MIMEMessageUtility.foldContentDisposition(cd.toString()));
+                                bodyPart.setHeader(
+                                    MessageHeaders.HDR_CONTENT_DISPOSITION,
+                                    MIMEMessageUtility.foldContentDisposition(cd.toString()));
                             }
                             os.reset();
                             attachment.writeTo(os);
@@ -574,7 +584,9 @@ public final class MailMessageParser {
                         if (attachFilename != null) {
                             final ContentDisposition cd = new ContentDisposition(Part.ATTACHMENT);
                             cd.setFilenameParameter(attachFilename);
-                            bodyPart.setHeader(MessageHeaders.HDR_CONTENT_DISPOSITION, MIMEMessageUtility.foldContentDisposition(cd.toString()));
+                            bodyPart.setHeader(
+                                MessageHeaders.HDR_CONTENT_DISPOSITION,
+                                MIMEMessageUtility.foldContentDisposition(cd.toString()));
                         }
                         bodyPart.setSize(messageClass.getLength());
                         parseMailContent(MIMEMessageConverter.convertPart(bodyPart), handler, prefix, partCount++);
