@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -115,6 +115,23 @@ public final class JSONMessageHandler implements MailMessageHandler {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(JSONMessageHandler.class);
 
+    private static final class PlainTextContent {
+
+        final String id;
+
+        final String contentType;
+
+        final String content;
+
+        public PlainTextContent(final String id, final String contentType, final String content) {
+            super();
+            this.id = id;
+            this.contentType = contentType;
+            this.content = content;
+        }
+
+    }
+
     private final Session session;
 
     private final Context ctx;
@@ -149,6 +166,8 @@ public final class JSONMessageHandler implements MailMessageHandler {
 
     private final boolean[] modified;
 
+    private PlainTextContent plainText;
+
     /**
      * Initializes a new {@link JSONMessageHandler}
      * 
@@ -537,10 +556,20 @@ public final class JSONMessageHandler implements MailMessageHandler {
     // private static final RTF2HtmlConverter RTFCONV = new RTF2HtmlConverter();
 
     public boolean handleInlinePlainText(final String plainTextContentArg, final ContentType contentType, final long size, final String fileName, final String id) throws MailException {
-        if (isAlternative && usm.isDisplayHtmlInlineContent() && (DisplayMode.RAW.getMode() < displayMode.getMode()) && contentType.isMimeType(MIMETypes.MIME_TEXT_PLAIN)) {
+        if (isAlternative && usm.isDisplayHtmlInlineContent() && (DisplayMode.RAW.getMode() < displayMode.getMode()) && contentType.startsWith(MIMETypes.MIME_TEXT_PLAIN)) {
             /*
              * User wants to see message's alternative content
              */
+            if (null == plainText) {
+                /*
+                 * Remember plain-text content
+                 */
+                plainText =
+                    new PlainTextContent(id, contentType.getBaseType(), HTMLProcessing.formatTextForDisplay(
+                        plainTextContentArg,
+                        usm,
+                        displayMode));
+            }
             /* textAppended = true; */
             return true;
         }
@@ -698,11 +727,17 @@ public final class JSONMessageHandler implements MailMessageHandler {
          * Since we obviously touched message's content, mark its corresponding message object as seen
          */
         mail.setFlags(mail.getFlags() | MailMessage.FLAG_SEEN);
+        if (!textAppended && plainText != null) {
+            /*
+             * No text present
+             */
+            asRawContent(plainText.id, plainText.contentType, plainText.content);
+        }
+        /*-
+         *
         if (!textAppended) {
             try {
-                /*
-                 * Ensure at least one text part is present
-                 */
+                // Ensure at least one text part is present
                 final JSONObject jsonObject = new JSONObject();
                 jsonObject.put(MailJSONField.CONTENT_TYPE.getKey(), MIMETypes.MIME_TEXT_PLAIN);
                 jsonObject.put(MailJSONField.SIZE.getKey(), 0);
@@ -713,6 +748,7 @@ public final class JSONMessageHandler implements MailMessageHandler {
                 throw new MailException(MailException.Code.JSON_ERROR, e, e.getMessage());
             }
         }
+         */
     }
 
     public boolean handleMultipart(final MailPart mp, final int bodyPartCount, final String id) throws MailException {
