@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -118,6 +118,8 @@ public final class RawJSONMessageHandler implements MailMessageHandler {
 
     private JSONArray nestedMsgsArr;
 
+    private boolean bodyAdded;
+
     private boolean textWasEmpty;
 
     private boolean isAlternative;
@@ -125,7 +127,7 @@ public final class RawJSONMessageHandler implements MailMessageHandler {
     private String altId;
 
     /**
-     * Initializes a new {@link RawJSONMessageHandler}
+     * Initializes a new {@link RawJSONMessageHandler}.
      * 
      * @param accountId The account ID
      * @param mailPath The unique mail path
@@ -139,7 +141,7 @@ public final class RawJSONMessageHandler implements MailMessageHandler {
     }
 
     /**
-     * Initializes a new {@link RawJSONMessageHandler} for internal usage
+     * Initializes a new {@link RawJSONMessageHandler}.
      * 
      * @param accountId The account ID
      * @param mailPath The unique mail path
@@ -407,37 +409,20 @@ public final class RawJSONMessageHandler implements MailMessageHandler {
         /*
          * Append HTML content as-is
          */
-        if (containsContent("text/htm")) {
-            asAttachment(id, contentType.getBaseType(), size, fileName);
-        } else {
-            /*
-             * Check size
-             */
-            if (addConsumedBodySize(size > 0 ? size : htmlContent.length()) > MAX_BODY_SIZE) {
-                discardJSONObject();
-                return false;
-            }
-            asRawContent(id, contentType.getBaseType(), htmlContent, fileName);
-        }
-        return true;
-
-    }
-
-    public boolean handleInlinePlainText(final String plainTextContentArg, final ContentType contentType, final long size, final String fileName, final String id) throws MailException {
-        /*
-         * Append plain-text content as-is
-         */
-        if (containsContent(contentType.getBaseType())) {
-            if (textWasEmpty) {
-                /*
-                 * Check size
-                 */
-                if (addConsumedBodySize(size > 0 ? size : plainTextContentArg.length()) > MAX_BODY_SIZE) {
-                    discardJSONObject();
-                    return false;
+        if (bodyAdded) {
+            if (isAlternative) {
+                if (containsContent("text/htm")) {
+                    asAttachment(id, contentType.getBaseType(), size, fileName);
+                } else {
+                    /*
+                     * Check size
+                     */
+                    if (addConsumedBodySize(htmlContent.length()) > MAX_BODY_SIZE) {
+                        discardJSONObject();
+                        return false;
+                    }
+                    asRawContent(id, contentType.getBaseType(), htmlContent, fileName);
                 }
-                replaceEmptyContent(id, contentType.getBaseType(), plainTextContentArg);
-                textWasEmpty = (null == plainTextContentArg || 0 == plainTextContentArg.length());
             } else {
                 asAttachment(id, contentType.getBaseType(), size, fileName);
             }
@@ -445,12 +430,73 @@ public final class RawJSONMessageHandler implements MailMessageHandler {
             /*
              * Check size
              */
-            if (addConsumedBodySize(size > 0 ? size : plainTextContentArg.length()) > MAX_BODY_SIZE) {
+            if (addConsumedBodySize(htmlContent.length()) > MAX_BODY_SIZE) {
+                discardJSONObject();
+                return false;
+            }
+            asRawContent(id, contentType.getBaseType(), htmlContent, fileName);
+            bodyAdded = true;
+        }
+        return true;
+    }
+
+    public boolean handleInlinePlainText(final String plainTextContentArg, final ContentType contentType, final long size, final String fileName, final String id) throws MailException {
+        /*
+         * Append plain-text content as-is
+         */
+        if (bodyAdded) {
+            if (isAlternative) {
+                if (containsContent(contentType.getBaseType())) {
+                    if (textWasEmpty) {
+                        /*
+                         * Check size
+                         */
+                        if (addConsumedBodySize(null == plainTextContentArg ? 0 : plainTextContentArg.length()) > MAX_BODY_SIZE) {
+                            discardJSONObject();
+                            return false;
+                        }
+                        replaceEmptyContent(id, contentType.getBaseType(), plainTextContentArg);
+                        textWasEmpty = (null == plainTextContentArg || 0 == plainTextContentArg.length());
+                    } else {
+                        asAttachment(id, contentType.getBaseType(), size, fileName);
+                    }
+                } else {
+                    /*
+                     * Check size
+                     */
+                    if (addConsumedBodySize(null == plainTextContentArg ? 0 : plainTextContentArg.length()) > MAX_BODY_SIZE) {
+                        discardJSONObject();
+                        return false;
+                    }
+                    asRawContent(id, contentType.getBaseType(), plainTextContentArg, fileName);
+                    textWasEmpty = (null == plainTextContentArg || 0 == plainTextContentArg.length());
+                }
+            } else {
+                if (textWasEmpty) {
+                    /*
+                     * Check size
+                     */
+                    if (addConsumedBodySize(null == plainTextContentArg ? 0 : plainTextContentArg.length()) > MAX_BODY_SIZE) {
+                        discardJSONObject();
+                        return false;
+                    }
+                    replaceEmptyContent(id, contentType.getBaseType(), plainTextContentArg);
+                    textWasEmpty = (null == plainTextContentArg || 0 == plainTextContentArg.length());
+                } else {
+                    asAttachment(id, contentType.getBaseType(), size, fileName);
+                }
+            }
+        } else {
+            /*
+             * Check size
+             */
+            if (addConsumedBodySize(null == plainTextContentArg ? 0 : plainTextContentArg.length()) > MAX_BODY_SIZE) {
                 discardJSONObject();
                 return false;
             }
             asRawContent(id, contentType.getBaseType(), plainTextContentArg, fileName);
             textWasEmpty = (null == plainTextContentArg || 0 == plainTextContentArg.length());
+            bodyAdded = true;
         }
         return true;
     }
