@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,10 +49,9 @@
 
 package com.openexchange.mailaccount;
 
+import gnu.trove.TIntObjectHashMap;
 import java.util.Collections;
 import java.util.EnumSet;
-import java.util.HashMap;
-import java.util.Map;
 import java.util.Set;
 import com.openexchange.mailaccount.servlet.fields.MailAccountFields;
 
@@ -205,23 +204,40 @@ public enum Attribute {
         }
     }
 
+    /**
+     * Gets the attribute name.
+     * 
+     * @return The name
+     */
     public String getName() {
         return attrName;
     }
 
+    /**
+     * Gets the field identifier associated with this attribute.
+     * 
+     * @return The field identifier
+     */
     public int getId() {
         return id;
     }
 
-    private static Map<Integer, Attribute> byId = new HashMap<Integer, Attribute>();
+    private static TIntObjectHashMap<Attribute> byId = new TIntObjectHashMap<Attribute>();
 
     static {
         for (final Attribute attribute : Attribute.values()) {
-            byId.put(Integer.valueOf(attribute.getId()), attribute);
+            byId.put(attribute.getId(), attribute);
         }
     }
 
+    /**
+     * Gets the attribute by given field identifier.
+     * 
+     * @param col The field identifier
+     * @return The attribute by given field identifier or <code>null</code> if no such attribute is present
+     */
     public static Attribute getById(final int col) {
-        return byId.get(Integer.valueOf(col));
+        return byId.get(col);
     }
+
 }
