@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -103,6 +103,9 @@ public final class MailAccountExceptionStrings implements LocalizableStrings {
     // Multiple mail accounts not enabled for user %1$s in context %2$s.
     public static final String NOT_ENABLED_MSG = "Multiple mail accounts not enabled for user %1$s in context %2$s.";
 
+    // Invalid mail account name: %1$s
+    public static final String INVALID_NAME_MSG = "Invalid mail account name: %1$s";
+
     /**
      * Initializes a new {@link MailAccountExceptionStrings}.
      */
