@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -259,6 +259,9 @@ public class MailAccountParser extends DataParser {
         if (json.hasAndNotNull(MailAccountFields.POP3_PATH)) {
             props.put("pop3.path", json.getString(MailAccountFields.POP3_PATH).trim());
             attributes.add(Attribute.POP3_PATH_LITERAL);
+        } else if ("pop3".equalsIgnoreCase(account.getMailProtocol())) {
+            props.put("pop3.path", "mailaccount");
+            attributes.add(Attribute.POP3_PATH_LITERAL);
         }
         account.setProperties(props);
         return attributes;
