@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.multiple;
 
+import java.util.Collection;
 import java.util.Date;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -63,17 +64,23 @@ import com.openexchange.tools.session.ServerSession;
  */
 public interface MultipleHandler {
 
+    /**
+     * The constant for the key to get the data element from passed {@link JSONObject JSON object} instance.
+     */
+    public static final String DATA = "data";
+
     /**
      * Performs the multiple request identified by specified action string.
      * 
      * @param action The action string denoting the request to perform
      * @param jsonObject The JSON object providing request parameters and/or body
      * @param session The session providing needed user data
+     * @param secure <code>true</code> for a secure connection such as HTTPS; otherwise <code>false</code>
      * @return A {@link JSONValue} as a result of the performed request
      * @throws AbstractOXException If performing the request fails
      * @throws JSONException If a JSON error occurs
      */
-    public Object performRequest(String action, JSONObject jsonObject, ServerSession session) throws AbstractOXException, JSONException;
+    public Object performRequest(String action, JSONObject jsonObject, ServerSession session, boolean secure) throws AbstractOXException, JSONException;
 
     /**
      * Gets the time stamp when {@link #performRequest()} has been called.
@@ -82,6 +89,13 @@ public interface MultipleHandler {
      */
     public Date getTimestamp();
 
+    /**
+     * Gets the warnings.
+     * 
+     * @return The warnings
+     */
+    public Collection<AbstractOXException> getWarnings();
+
     /**
      * Closes/frees all associated resources.
      */
