@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,7 +55,6 @@ import java.util.List;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.ReentrantLock;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
@@ -66,7 +65,7 @@ import org.apache.commons.logging.LogFactory;
  */
 public class ReentrantLockPool<T> implements Pool<T>, Runnable {
 
-    private static final Log LOG = LogFactory.getLog(ReentrantLockPool.class);
+    static final Log LOG = LogFactory.getLog(ReentrantLockPool.class);
 
     /* --- Constants --- */
 
@@ -633,8 +632,7 @@ public class ReentrantLockPool<T> implements Pool<T>, Runnable {
                     sb.append(metaData.getIdentifier());
                     sb.append(", Object: ");
                     sb.append(metaData.getPooled());
-                    final PoolingException e = new PoolingException(sb
-                        .toString());
+                    final PoolingException e = new PoolingException(sb.toString());
                     if (testThreads && null != metaData.getTrace()) {
                         e.setStackTrace(metaData.getTrace());
                     }
