@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -63,7 +63,6 @@ import javax.management.Attribute;
 import javax.management.AttributeList;
 import javax.management.AttributeNotFoundException;
 import javax.management.DynamicMBean;
-import javax.management.InvalidAttributeValueException;
 import javax.management.MBeanException;
 import javax.management.MBeanInfo;
 import javax.management.ReflectionException;
@@ -188,7 +187,9 @@ public class ReportingMBean implements DynamicMBean {
                     sb.append(loginInfo);
                     sb.append(',');
                 }
-                sb.setLength(sb.length() - 1);
+                if (sb.length() > 0) {
+                    sb.setLength(sb.length() - 1);
+                }
                 TabularDataSupport moduleAccessCombinations = new TabularDataSupport(moduleAccessCombinationsType);
                 consolidateAccessCombinations(configs, moduleAccessCombinations);
                 CompositeDataSupport value = new CompositeDataSupport(detailRow, detailNames, new Object[] {
@@ -287,13 +288,13 @@ public class ReportingMBean implements DynamicMBean {
         return mbeanInfo;
     }
 
-    public Object invoke(String actionName, Object[] params, String[] signature) throws MBeanException, ReflectionException {
+    public Object invoke(String actionName, Object[] params, String[] signature) {
         throw new RuntimeOperationsException(
             new UnsupportedOperationException("invoke is not supported"),
             "The method invoke is not supported.");
     }
 
-    public void setAttribute(Attribute attribute) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
+    public void setAttribute(Attribute attribute) throws AttributeNotFoundException {
         throw new AttributeNotFoundException("No attribute can be set in this MBean");
     }
 
