@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -113,6 +113,7 @@ import com.openexchange.groupware.contact.ContactInterfaceDiscoveryService;
 import com.openexchange.groupware.contact.ContactInterfaceProvider;
 import com.openexchange.groupware.contact.datahandler.ContactInsertDataHandler;
 import com.openexchange.groupware.contact.datasource.ContactDataSource;
+import com.openexchange.groupware.contact.datasource.ContactImageDataSource;
 import com.openexchange.groupware.contact.internal.ContactInterfaceDiscoveryServiceImpl;
 import com.openexchange.groupware.datahandler.ICalInsertDataHandler;
 import com.openexchange.groupware.datahandler.ICalJSONDataHandler;
@@ -120,14 +121,11 @@ import com.openexchange.groupware.delete.DeleteListener;
 import com.openexchange.groupware.importexport.importers.ExtraneousSeriesMasterRecoveryParser;
 import com.openexchange.groupware.infostore.InfostoreFacade;
 import com.openexchange.groupware.notify.hostname.HostnameService;
-import com.openexchange.groupware.reminder.ReminderDeleteInterface;
+import com.openexchange.groupware.reminder.TargetService;
 import com.openexchange.groupware.settings.PreferencesItemService;
 import com.openexchange.groupware.tx.DBPoolProvider;
 import com.openexchange.groupware.tx.DBProvider;
 import com.openexchange.groupware.tx.osgi.WhiteboardDBProvider;
-import com.openexchange.groupware.update.UpdateTaskProviderService;
-import com.openexchange.groupware.update.UpdateTaskServiceTrackerCustomizer;
-import com.openexchange.groupware.update.tasks.MALPollCreateTableTask;
 import com.openexchange.i18n.I18nService;
 import com.openexchange.image.ImageService;
 import com.openexchange.login.LoginHandlerService;
@@ -135,6 +133,7 @@ import com.openexchange.mail.api.MailProvider;
 import com.openexchange.mail.cache.MailAccessCacheEventListener;
 import com.openexchange.mail.cache.MailSessionEventHandler;
 import com.openexchange.mail.conversion.ICalMailPartDataSource;
+import com.openexchange.mail.conversion.InlineImageDataSource;
 import com.openexchange.mail.conversion.VCardAttachMailDataHandler;
 import com.openexchange.mail.conversion.VCardMailPartDataSource;
 import com.openexchange.mail.loginhandler.MailLoginHandler;
@@ -150,6 +149,7 @@ import com.openexchange.mailaccount.MailAccountStorageService;
 import com.openexchange.mailaccount.UnifiedINBOXManagement;
 import com.openexchange.mailaccount.internal.CreateMailAccountTables;
 import com.openexchange.management.ManagementService;
+import com.openexchange.messaging.registry.MessagingServiceRegistry;
 import com.openexchange.multiple.MultipleHandlerFactoryService;
 import com.openexchange.multiple.internal.MultipleHandlerServiceTracker;
 import com.openexchange.passwordchange.PasswordChangeService;
@@ -211,7 +211,7 @@ public final class ServerActivator extends DeferredActivator {
         {
             ConfigurationService.class, CacheService.class, EventAdmin.class, SessiondService.class, SpringParser.class, JDOMParser.class,
             TimerService.class, ThreadPoolService.class, CalendarAdministrationService.class, AppointmentSqlFactoryService.class,
-            CalendarCollectionService.class, ReminderDeleteInterface.class
+            CalendarCollectionService.class, TargetService.class, MessagingServiceRegistry.class
         };
 
     private final List<ServiceRegistration> registrationList;
@@ -332,8 +332,6 @@ public final class ServerActivator extends DeferredActivator {
         serviceTrackerList.add(new ServiceTracker(context, DatabaseService.class.getName(), new DatabaseCustomizer(context)));
         // I18n service load
         serviceTrackerList.add(new ServiceTracker(context, I18nService.class.getName(), new I18nServiceListener(context)));
-        // Update task service tracker
-        serviceTrackerList.add(new ServiceTracker(context, UpdateTaskProviderService.class.getName(), new UpdateTaskServiceTrackerCustomizer(context)));
 
         // Mail account delete listener
         serviceTrackerList.add(new ServiceTracker(
@@ -503,7 +501,6 @@ public final class ServerActivator extends DeferredActivator {
         registrationList.add(context.registerService(LoginHandlerService.class.getName(), new TransportLoginHandler(), null));
         // Register table creation for mail account storage.
         registrationList.add(context.registerService(CreateTableService.class.getName(), new CreateMailAccountTables(), null));
-        registrationList.add(context.registerService(CreateTableService.class.getName(), new MALPollCreateTableTask(), null));
         // TODO: Register server's mail account storage here until its encapsulated in an own bundle
         registrationList.add(context.registerService(
             MailAccountStorageService.class.getName(),
@@ -532,6 +529,18 @@ public final class ServerActivator extends DeferredActivator {
             props.put(STR_IDENTIFIER, "com.openexchange.contact");
             registrationList.add(context.registerService(DataSource.class.getName(), new ContactDataSource(), props));
         }
+        {
+            final InlineImageDataSource dataSource = new InlineImageDataSource();
+            final Dictionary<Object, Object> props = new Hashtable<Object, Object>();
+            props.put(STR_IDENTIFIER, dataSource.getRegistrationName());
+            registrationList.add(context.registerService(DataSource.class.getName(), dataSource, props));
+        }
+        {
+            final ContactImageDataSource dataSource = new ContactImageDataSource();
+            final Dictionary<Object, Object> props = new Hashtable<Object, Object>();
+            props.put(STR_IDENTIFIER, dataSource.getRegistrationName());
+            registrationList.add(context.registerService(DataSource.class.getName(), dataSource, props));
+        }
         /*
          * Register data handlers
          */
