@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -141,7 +141,7 @@ public class TableWriter {
             retval.append(format.getWidth());
             switch (format.getConversion()) {
             case DATE:
-                retval.append('t');
+                retval.append("tc");
                 break;
             case STRING:
             default:
