@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,6 +50,7 @@
 package com.openexchange.tools.iterator;
 
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 import com.openexchange.groupware.AbstractOXException;
@@ -97,6 +98,15 @@ public class SearchIteratorDelegator<T> implements SearchIterator<T> {
         warnings = new ArrayList<AbstractOXException>(2);
     }
 
+    /**
+     * Initializes a new {@link SearchIteratorDelegator}.
+     * 
+     * @param collection The collection to iterate
+     */
+    public SearchIteratorDelegator(final Collection<T> collection) {
+        this(collection.iterator(), collection.size());
+    }
+
     public boolean hasNext() {
         return delegate.hasNext();
     }
