@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -51,16 +51,15 @@ package com.openexchange.tools.oxfolder;
 
 import static com.openexchange.tools.sql.DBUtils.closeResources;
 import static com.openexchange.tools.sql.DBUtils.closeSQLStuff;
+import gnu.trove.TIntArrayList;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.List;
 import java.util.Set;
 import com.openexchange.api2.OXException;
 import com.openexchange.cache.impl.FolderCacheManager;
@@ -127,6 +126,9 @@ public final class OXFolderAdminHelper {
      * 'projects/projects_list_all', null, 'folder/item_projects.png');
      */
 
+    private static final int[] CHANGEABLE_PUBLIC_FOLDERS =
+        { FolderObject.SYSTEM_PUBLIC_FOLDER_ID, FolderObject.SYSTEM_INFOSTORE_FOLDER_ID, FolderObject.SYSTEM_PUBLIC_INFOSTORE_FOLDER_ID };
+
     /**
      * Checks if context's admin has administer permission on public folder(s).
      * <p>
@@ -153,7 +155,7 @@ public final class OXFolderAdminHelper {
         ResultSet rs = null;
         try {
             stmt =
-                readCon.prepareStatement("SELECT admin_flag FROM oxfolder_permissions WHERE cid = ? AND permission_id = ? AND fuid IN " + StringCollection.getSqlInString(new int[] { FolderObject.SYSTEM_PUBLIC_INFOSTORE_FOLDER_ID }));
+                readCon.prepareStatement("SELECT admin_flag FROM oxfolder_permissions WHERE cid = ? AND permission_id = ? AND fuid IN " + StringCollection.getSqlInString(CHANGEABLE_PUBLIC_FOLDERS));
             stmt.setInt(1, cid);
             stmt.setInt(2, admin);
             rs = stmt.executeQuery();
@@ -191,96 +193,92 @@ public final class OXFolderAdminHelper {
         if (userId != admin) {
             return;
         }
-        final int publicInfostoreFolderId = FolderObject.SYSTEM_PUBLIC_INFOSTORE_FOLDER_ID;
-        final int[] ids = { publicInfostoreFolderId };
-        for (final int id : ids) {
-            if (publicInfostoreFolderId == id) {
-                try {
-                    /*
-                     * Check if folder has already been created for given context
-                     */
-                    if (!checkFolderExistence(cid, id, writeCon)) {
-                        createPublicInfostoreFolder(cid, admin, writeCon, System.currentTimeMillis());
-                    }
-                } catch (final SQLException e) {
-                    throw new OXFolderException(FolderCode.SQL_ERROR, e, e.getMessage());
+        for (final int id : CHANGEABLE_PUBLIC_FOLDERS) {
+            try {
+                /*
+                 * Check if folder has already been created for given context
+                 */
+                if (!checkFolderExistence(cid, id, writeCon)) {
+                    createPublicFolder(id, cid, admin, writeCon, System.currentTimeMillis());
                 }
-                PreparedStatement stmt = null;
-                ResultSet rs = null;
-                try {
+            } catch (final SQLException e) {
+                throw new OXFolderException(FolderCode.SQL_ERROR, e, e.getMessage());
+            }
+            PreparedStatement stmt = null;
+            ResultSet rs = null;
+            try {
+                stmt =
+                    writeCon.prepareStatement("SELECT permission_id FROM oxfolder_permissions WHERE cid = ? AND fuid = ? AND permission_id = ?");
+                int pos = 1;
+                stmt.setInt(pos++, cid);
+                stmt.setInt(pos++, id);
+                stmt.setInt(pos++, admin);
+                rs = stmt.executeQuery();
+                final boolean update = rs.next();
+                DBUtils.closeSQLStuff(rs, stmt);
+                rs = null;
+                /*
+                 * Insert/Update
+                 */
+                if (update) {
                     stmt =
-                        writeCon.prepareStatement("SELECT permission_id FROM oxfolder_permissions WHERE cid = ? AND fuid = ? AND permission_id = ?");
-                    int pos = 1;
+                        writeCon.prepareStatement("UPDATE oxfolder_permissions SET fp = ?, orp = ?, owp = ?, admin_flag = ?, odp = ? WHERE cid = ? AND fuid = ? AND permission_id = ?");
+                    pos = 1;
+                    stmt.setInt(pos++, OCLPermission.CREATE_SUB_FOLDERS); // fp
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // orp
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // owp
+                    stmt.setInt(pos++, editable ? 1 : 0); // admin_flag
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // odp
                     stmt.setInt(pos++, cid);
-                    stmt.setInt(pos++, publicInfostoreFolderId);
+                    stmt.setInt(pos++, id);
                     stmt.setInt(pos++, admin);
-                    rs = stmt.executeQuery();
-                    final boolean update = rs.next();
-                    DBUtils.closeSQLStuff(rs, stmt);
-                    rs = null;
-                    /*
-                     * Insert/Update
-                     */
-                    if (update) {
-                        stmt =
-                            writeCon.prepareStatement("UPDATE oxfolder_permissions SET fp = ?, orp = ?, owp = ?, admin_flag = ?, odp = ? WHERE cid = ? AND fuid = ? AND permission_id = ?");
-                        pos = 1;
-                        stmt.setInt(pos++, OCLPermission.CREATE_SUB_FOLDERS); // fp
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // orp
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // owp
-                        stmt.setInt(pos++, editable ? 1 : 0); // admin_flag
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // odp
-                        stmt.setInt(pos++, cid);
-                        stmt.setInt(pos++, publicInfostoreFolderId);
-                        stmt.setInt(pos++, admin);
-                        stmt.executeUpdate();
-                    } else {
-                        stmt =
-                            writeCon.prepareStatement("INSERT INTO oxfolder_permissions (cid, fuid, permission_id, fp, orp, owp, odp, admin_flag, group_flag, system) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
-                        pos = 1;
-                        stmt.setInt(pos++, cid); // cid
-                        stmt.setInt(pos++, publicInfostoreFolderId); // fuid
-                        stmt.setInt(pos++, admin); // permission_id
-                        stmt.setInt(pos++, OCLPermission.CREATE_SUB_FOLDERS); // fp
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // orp
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // owp
-                        stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // odp
-                        stmt.setInt(pos++, editable ? 1 : 0); // admin_flag
-                        stmt.setInt(pos++, 0); // group_flag
-                        stmt.setInt(pos++, 0); // system
-                        stmt.executeUpdate();
-                    }
-                    DBUtils.closeSQLStuff(stmt);
-                    stmt = null;
-                    /*
-                     * Update last-modified of folder
-                     */
-                    final ContextImpl ctx = new ContextImpl(cid);
-                    ctx.setMailadmin(admin);
-                    OXFolderSQL.updateLastModified(publicInfostoreFolderId, System.currentTimeMillis(), admin, writeCon, ctx);
-                    /*
-                     * Update caches
-                     */
-                    try {
-                        if (FolderCacheManager.isEnabled()) {
-                            FolderCacheManager.getInstance().removeFolderObject(publicInfostoreFolderId, ctx);
-                        }
-                        if (FolderQueryCacheManager.isInitialized()) {
-                            FolderQueryCacheManager.getInstance().invalidateContextQueries(cid);
-                        }
-                        if (CalendarCache.isInitialized()) {
-                            CalendarCache.getInstance().invalidateGroup(cid);
-                        }
-                    } catch (final AbstractOXException e) {
-                        LOG.error(e.getMessage(), e);
-                    }
-                } catch (final SQLException e) {
-                    throw new OXFolderException(FolderCode.SQL_ERROR, e, e.getMessage());
-                } catch (final DBPoolingException e) {
-                    throw new OXException(e);
-                } finally {
-                    DBUtils.closeSQLStuff(rs, stmt);
+                    stmt.executeUpdate();
+                } else {
+                    stmt =
+                        writeCon.prepareStatement("INSERT INTO oxfolder_permissions (cid, fuid, permission_id, fp, orp, owp, odp, admin_flag, group_flag, system) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+                    pos = 1;
+                    stmt.setInt(pos++, cid); // cid
+                    stmt.setInt(pos++, id); // fuid
+                    stmt.setInt(pos++, admin); // permission_id
+                    stmt.setInt(pos++, OCLPermission.CREATE_SUB_FOLDERS); // fp
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // orp
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // owp
+                    stmt.setInt(pos++, OCLPermission.NO_PERMISSIONS); // odp
+                    stmt.setInt(pos++, editable ? 1 : 0); // admin_flag
+                    stmt.setInt(pos++, 0); // group_flag
+                    stmt.setInt(pos++, 0); // system
+                    stmt.executeUpdate();
                 }
+                DBUtils.closeSQLStuff(stmt);
+                stmt = null;
+                /*
+                 * Update last-modified of folder
+                 */
+                final ContextImpl ctx = new ContextImpl(cid);
+                ctx.setMailadmin(admin);
+                OXFolderSQL.updateLastModified(id, System.currentTimeMillis(), admin, writeCon, ctx);
+                /*
+                 * Update caches
+                 */
+                try {
+                    if (FolderCacheManager.isEnabled()) {
+                        FolderCacheManager.getInstance().removeFolderObject(id, ctx);
+                    }
+                    if (FolderQueryCacheManager.isInitialized()) {
+                        FolderQueryCacheManager.getInstance().invalidateContextQueries(cid);
+                    }
+                    if (CalendarCache.isInitialized()) {
+                        CalendarCache.getInstance().invalidateGroup(cid);
+                    }
+                } catch (final AbstractOXException e) {
+                    LOG.error(e.getMessage(), e);
+                }
+            } catch (final SQLException e) {
+                throw new OXFolderException(FolderCode.SQL_ERROR, e, e.getMessage());
+            } catch (final DBPoolingException e) {
+                throw new OXException(e);
+            } finally {
+                DBUtils.closeSQLStuff(rs, stmt);
             }
         }
     }
@@ -318,7 +316,7 @@ public final class OXFolderAdminHelper {
             /*
              * Get context's users
              */
-            final List<Integer> users;
+            final TIntArrayList users;
             {
                 PreparedStatement stmt = null;
                 ResultSet rs = null;
@@ -327,9 +325,9 @@ public final class OXFolderAdminHelper {
                     final int pos = 1;
                     stmt.setInt(pos, cid);
                     rs = stmt.executeQuery();
-                    users = new ArrayList<Integer>();
+                    users = new TIntArrayList();
                     while (rs.next()) {
-                        users.add(Integer.valueOf(rs.getInt(pos)));
+                        users.add(rs.getInt(pos));
                     }
                 } catch (final SQLException e) {
                     throw new OXFolderException(FolderCode.SQL_ERROR, e, e.getMessage());
@@ -344,12 +342,12 @@ public final class OXFolderAdminHelper {
                 final Integer admin = Integer.valueOf(getContextAdminID(cid, writeCon));
                 final int size = users.size();
                 for (int i = 1; i < size; i++) {
-                    setGlobalAddressBookDisabled(cid, users.get(i).intValue(), !enable, writeCon, admin, false);
+                    setGlobalAddressBookDisabled(cid, users.getQuick(i), !enable, writeCon, admin, false);
                 }
                 /*
                  * Propagate with last update
                  */
-                setGlobalAddressBookDisabled(cid, users.get(0).intValue(), !enable, writeCon, admin, true);
+                setGlobalAddressBookDisabled(cid, users.getQuick(0), !enable, writeCon, admin, true);
             }
         } finally {
             Database.back(cid, true, writeCon);
@@ -613,24 +611,9 @@ public final class OXFolderAdminHelper {
         /*
          * Insert system public folder
          */
-        systemPermission.setAllPermission(
-            OCLPermission.CREATE_SUB_FOLDERS,
-            OCLPermission.NO_PERMISSIONS,
-            OCLPermission.NO_PERMISSIONS,
-            OCLPermission.NO_PERMISSIONS);
-        systemPermission.setFolderAdmin(false);
-        createSystemFolder(
-            FolderObject.SYSTEM_PUBLIC_FOLDER_ID,
-            FolderObject.SYSTEM_PUBLIC_FOLDER_NAME,
-            systemPermission,
-            FolderObject.SYSTEM_ROOT_FOLDER_ID,
-            FolderObject.SYSTEM_MODULE,
-            true,
-            creatingTime,
-            mailAdmin,
-            true,
-            cid,
-            writeCon);
+        if (!checkFolderExistence(cid, FolderObject.SYSTEM_PUBLIC_FOLDER_ID, writeCon)) {
+            createSystemPublicFolder(cid, mailAdmin, writeCon, creatingTime);
+        }
         /*
          * Insert system shared folder
          */
@@ -676,24 +659,9 @@ public final class OXFolderAdminHelper {
         /*
          * Insert system infostore folder
          */
-        systemPermission.setAllPermission(
-            OCLPermission.READ_FOLDER,
-            OCLPermission.NO_PERMISSIONS,
-            OCLPermission.NO_PERMISSIONS,
-            OCLPermission.NO_PERMISSIONS);
-        systemPermission.setFolderAdmin(false);
-        createSystemFolder(
-            FolderObject.SYSTEM_INFOSTORE_FOLDER_ID,
-            FolderObject.SYSTEM_INFOSTORE_FOLDER_NAME,
-            systemPermission,
-            FolderObject.SYSTEM_ROOT_FOLDER_ID,
-            FolderObject.SYSTEM_MODULE,
-            true,
-            creatingTime,
-            mailAdmin,
-            true,
-            cid,
-            writeCon);
+        if (!checkFolderExistence(cid, FolderObject.SYSTEM_INFOSTORE_FOLDER_ID, writeCon)) {
+            createSystemInfostoreFolder(cid, mailAdmin, writeCon, creatingTime);
+        }
         /*
          * Insert system system_global folder aka 'Shared Address Book'
          */
@@ -800,7 +768,7 @@ public final class OXFolderAdminHelper {
          * Insert system public infostore folder
          */
         if (!checkFolderExistence(cid, FolderObject.SYSTEM_PUBLIC_INFOSTORE_FOLDER_ID, writeCon)) {
-            createPublicInfostoreFolder(cid, mailAdmin, writeCon, creatingTime);
+            createSystemPublicInfostoreFolder(cid, mailAdmin, writeCon, creatingTime);
         }
         if (LOG.isInfoEnabled()) {
             LOG.info(new StringBuilder("All System folders successfully created for context ").append(cid).toString());
@@ -876,6 +844,93 @@ public final class OXFolderAdminHelper {
         }
     }
 
+    /**
+     * Creates a public system folder with default permission for all-groups-and-users entity.
+     * 
+     * @param cid The context ID
+     * @param mailAdmin The context admin
+     * @param writeCon A writable connection
+     * @param creatingTime The creation date
+     * @throws SQLException If a SQL error occurs
+     */
+    private static void createPublicFolder(final int folderId, final int cid, final int mailAdmin, final Connection writeCon, final long creatingTime) throws SQLException {
+        if (FolderObject.SYSTEM_PUBLIC_FOLDER_ID == folderId) {
+            createSystemPublicFolder(cid, mailAdmin, writeCon, creatingTime);
+        } else if (FolderObject.SYSTEM_INFOSTORE_FOLDER_ID == folderId) {
+            createSystemInfostoreFolder(cid, mailAdmin, writeCon, creatingTime);
+        } else if (FolderObject.SYSTEM_PUBLIC_INFOSTORE_FOLDER_ID == folderId) {
+            createSystemPublicInfostoreFolder(cid, mailAdmin, writeCon, creatingTime);
+        } else {
+            throw new IllegalArgumentException("Specified folder ID is not a public folder ID: " + folderId);
+        }
+    }
+
+    /**
+     * Creates the public folder with default permission for all-groups-and-users entity.
+     * 
+     * @param cid The context ID
+     * @param mailAdmin The context admin
+     * @param writeCon A writable connection
+     * @param creatingTime The creation date
+     * @throws SQLException If a SQL error occurs
+     */
+    private static void createSystemPublicFolder(final int cid, final int mailAdmin, final Connection writeCon, final long creatingTime) throws SQLException {
+        final OCLPermission systemPermission = new OCLPermission();
+        systemPermission.setEntity(OCLPermission.ALL_GROUPS_AND_USERS);
+        systemPermission.setGroupPermission(true);
+        systemPermission.setAllPermission(
+            OCLPermission.CREATE_SUB_FOLDERS,
+            OCLPermission.NO_PERMISSIONS,
+            OCLPermission.NO_PERMISSIONS,
+            OCLPermission.NO_PERMISSIONS);
+        systemPermission.setFolderAdmin(false);
+        createSystemFolder(
+            FolderObject.SYSTEM_PUBLIC_FOLDER_ID,
+            FolderObject.SYSTEM_PUBLIC_FOLDER_NAME,
+            systemPermission,
+            FolderObject.SYSTEM_ROOT_FOLDER_ID,
+            FolderObject.SYSTEM_MODULE,
+            true,
+            creatingTime,
+            mailAdmin,
+            true,
+            cid,
+            writeCon);
+    }
+
+    /**
+     * Creates the infostore folder with default permission for all-groups-and-users entity.
+     * 
+     * @param cid The context ID
+     * @param mailAdmin The context admin
+     * @param writeCon A writable connection
+     * @param creatingTime The creation date
+     * @throws SQLException If a SQL error occurs
+     */
+    private static void createSystemInfostoreFolder(final int cid, final int mailAdmin, final Connection writeCon, final long creatingTime) throws SQLException {
+        final OCLPermission systemPermission = new OCLPermission();
+        systemPermission.setEntity(OCLPermission.ALL_GROUPS_AND_USERS);
+        systemPermission.setGroupPermission(true);
+        systemPermission.setAllPermission(
+            OCLPermission.READ_FOLDER,
+            OCLPermission.NO_PERMISSIONS,
+            OCLPermission.NO_PERMISSIONS,
+            OCLPermission.NO_PERMISSIONS);
+        systemPermission.setFolderAdmin(false);
+        createSystemFolder(
+            FolderObject.SYSTEM_INFOSTORE_FOLDER_ID,
+            FolderObject.SYSTEM_INFOSTORE_FOLDER_NAME,
+            systemPermission,
+            FolderObject.SYSTEM_ROOT_FOLDER_ID,
+            FolderObject.SYSTEM_MODULE,
+            true,
+            creatingTime,
+            mailAdmin,
+            true,
+            cid,
+            writeCon);
+    }
+
     /**
      * Creates the public infostore folder with default permission for all-groups-and-users entity.
      * 
@@ -885,7 +940,7 @@ public final class OXFolderAdminHelper {
      * @param creatingTime The creation date
      * @throws SQLException If a SQL error occurs
      */
-    private void createPublicInfostoreFolder(final int cid, final int mailAdmin, final Connection writeCon, final long creatingTime) throws SQLException {
+    private static void createSystemPublicInfostoreFolder(final int cid, final int mailAdmin, final Connection writeCon, final long creatingTime) throws SQLException {
         final OCLPermission systemPermission = new OCLPermission();
         systemPermission.setEntity(OCLPermission.ALL_GROUPS_AND_USERS);
         systemPermission.setGroupPermission(true);
@@ -1032,7 +1087,7 @@ public final class OXFolderAdminHelper {
 
     private static final String SQL_INSERT_SPECIAL_FOLDER = "INSERT INTO oxfolder_specialfolders " + "(tag, cid, fuid) VALUES (?,?,?)";
 
-    private void createSystemFolder(final int systemFolderId, final String systemFolderName, final OCLPermission systemPermission, final int parentId, final int module, final boolean insertIntoSpecialFolders, final long creatingTime, final int mailAdminId, final boolean isPublic, final int cid, final Connection writeCon) throws SQLException {
+    private static void createSystemFolder(final int systemFolderId, final String systemFolderName, final OCLPermission systemPermission, final int parentId, final int module, final boolean insertIntoSpecialFolders, final long creatingTime, final int mailAdminId, final boolean isPublic, final int cid, final Connection writeCon) throws SQLException {
         PreparedStatement stmt = null;
         try {
             stmt = writeCon.prepareStatement(SQL_INSERT_SYSTEM_FOLDER);
@@ -1265,9 +1320,9 @@ public final class OXFolderAdminHelper {
             stmt.setInt(2, cid);
             stmt.setInt(3, group);
             rs = stmt.executeQuery();
-            final List<Integer> list = new ArrayList<Integer>();
+            final TIntArrayList list = new TIntArrayList();
             while (rs.next()) {
-                list.add(Integer.valueOf(rs.getInt(1)));
+                list.add(rs.getInt(1));
             }
             closeSQLStuff(rs, stmt);
             rs = null;
@@ -1275,7 +1330,7 @@ public final class OXFolderAdminHelper {
             if (!list.isEmpty()) {
                 stmt = writeCon.prepareStatement(SQL_UPDATE_FOLDER_TIMESTAMP.replaceFirst("#FT#", STR_OXFOLDERTREE));
                 do {
-                    final int fuid = list.remove(0).intValue();
+                    final int fuid = list.remove(0);
                     stmt.setLong(1, lastModified);
                     stmt.setInt(2, cid);
                     stmt.setInt(3, fuid);
