@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -502,7 +502,15 @@ public class OXFolderException extends OXException {
         /**
          * No individual user permissions are allowed for folder %1$s in context %2$s.
          */
-        NO_INDIVIDUAL_PERMISSION("No individual user permissions are allowed for folder %1$s in context %2$s.", Category.PERMISSION, 83);
+        NO_INDIVIDUAL_PERMISSION("No individual user permissions are allowed for folder %1$s in context %2$s.", Category.PERMISSION, 83),
+        /**
+         * Folder module must not be updated.
+         */
+        DENY_FOLDER_MODULE_UPDATE("Folder module must not be updated.", Category.USER_INPUT, 84),
+        /**
+         * The module of a default folder must not be changed.
+         */
+        NO_DEFAULT_FOLDER_MODULE_UPDATE("The module of a default folder must not be changed.", Category.USER_INPUT, 85);
 
         /**
          * Message of the exception.
