@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,10 +49,12 @@
 
 package com.openexchange.tools.oxfolder;
 
+import gnu.trove.TIntArrayList;
+import gnu.trove.TIntObjectHashMap;
+import gnu.trove.TIntObjectProcedure;
 import java.sql.Connection;
 import java.sql.DataTruncation;
 import java.sql.SQLException;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
@@ -117,6 +119,16 @@ final class OXFolderManagerImpl extends OXFolderManager {
 
     private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(OXFolderManagerImpl.class);
 
+    /**
+     * No options.
+     */
+    static final int OPTION_NONE = 0;
+
+    /**
+     * The option to deny updating a folder's module (provided that folder is empty).
+     */
+    static final int OPTION_DENY_MODULE_UPDATE = 1;
+
     private static final String TABLE_OXFOLDER_TREE = "oxfolder_tree";
 
     private final Connection readCon;
@@ -519,7 +531,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
             if (performMove) {
                 move(fo.getObjectID(), fo.getParentFolderID(), lastModified);
             }
-            update(fo, lastModified);
+            update(fo, OPTION_NONE, lastModified);
         } else if (fo.containsFolderName()) {
             if (performMove) {
                 move(fo.getObjectID(), fo.getParentFolderID(), lastModified);
@@ -582,7 +594,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
         }
     }
 
-    private void update(final FolderObject folderObj, final long lastModified) throws OXException {
+    private void update(final FolderObject folderObj, final int options, final long lastModified) throws OXException {
         if (folderObj.getObjectID() <= 0) {
             throw new OXFolderException(FolderCode.INVALID_OBJECT_ID, OXFolderUtility.getFolderName(folderObj));
         }
@@ -642,7 +654,20 @@ final class OXFolderManagerImpl extends OXFolderManager {
                     OXFolderUtility.folderModule2String(folderObj.getModule()),
                     Integer.valueOf(ctx.getContextId()));
             }
-            if (!isFolderEmpty(storageObj.getObjectID(), storageObj.getModule())) {
+            if (storageObj.isDefaultFolder()) {
+                /*
+                 * A default folder's module must not be changed
+                 */
+                throw new OXFolderException(FolderCode.NO_DEFAULT_FOLDER_MODULE_UPDATE);
+            } else if (!isFolderEmpty(storageObj.getObjectID(), storageObj.getModule())) {
+                /*
+                 * Module cannot be updated since folder already contains elements
+                 */
+                throw new OXFolderException(FolderCode.DENY_FOLDER_MODULE_UPDATE);
+            } else if ((options & OPTION_DENY_MODULE_UPDATE) > 0) {
+                /*
+                 * Folder module must not be updated
+                 */
                 throw new OXFolderException(FolderCode.NO_FOLDER_MODULE_UPDATE);
             }
             final FolderObject parent = getFolderFromMaster(storageObj.getParentFolderID());
@@ -1070,8 +1095,8 @@ final class OXFolderManagerImpl extends OXFolderManager {
                 /*
                  * Check if target is a descendant folder
                  */
-                final List<Integer> parentIDList = new ArrayList<Integer>(1);
-                parentIDList.add(Integer.valueOf(storageSrc.getObjectID()));
+                final TIntArrayList parentIDList = new TIntArrayList(1);
+                parentIDList.add(storageSrc.getObjectID());
                 if (OXFolderUtility.isDescendentFolder(parentIDList, targetFolderId, readCon, ctx)) {
                     throw new OXFolderException(
                         FolderCode.NO_SUBFOLDER_MOVE,
@@ -1147,22 +1172,28 @@ final class OXFolderManagerImpl extends OXFolderManager {
          * Update OLD parent in cache, cause this can only be done here
          */
         if (FolderCacheManager.isEnabled()) {
-            try {
-                Connection wc = writeCon;
-                final boolean create = (wc == null);
+            Connection wc = writeCon;
+            final boolean create = (wc == null);
+            if (create) {
                 try {
-                    if (create) {
-                        wc = DBPool.pickupWriteable(ctx);
-                    }
-                    FolderCacheManager.getInstance().loadFolderObject(storageSrc.getParentFolderID(), ctx, wc);
-                    FolderCacheManager.getInstance().loadFolderObject(storageDest.getParentFolderID(), ctx, wc);
-                } finally {
-                    if (create && wc != null) {
-                        DBPool.closeWriterSilent(ctx, wc);
-                    }
+                    wc = DBPool.pickupWriteable(ctx);
+                } catch (final DBPoolingException e) {
+                    throw new OXFolderException(FolderCode.DBPOOLING_ERROR, e, Integer.valueOf(ctx.getContextId()));
+                }
+            }
+            try {
+                final int srcParentId = storageSrc.getParentFolderID();
+                if (srcParentId > 0) {
+                    FolderCacheManager.getInstance().loadFolderObject(srcParentId, ctx, wc);
+                }
+                final int destParentId = storageDest.getParentFolderID();
+                if (destParentId > 0) {
+                    FolderCacheManager.getInstance().loadFolderObject(destParentId, ctx, wc);
+                }
+            } finally {
+                if (create && wc != null) {
+                    DBPool.closeWriterSilent(ctx, wc);
                 }
-            } catch (final DBPoolingException e) {
-                throw new OXFolderException(FolderCode.DBPOOLING_ERROR, e, Integer.valueOf(ctx.getContextId()));
             }
         }
     }
@@ -1336,7 +1367,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
         /*
          * Gather all deletable subfolders
          */
-        final HashMap<Integer, HashMap<?, ?>> deleteableFolders;
+        final TIntObjectHashMap<TIntObjectHashMap<?>> deleteableFolders;
         try {
             deleteableFolders =
                 gatherDeleteableFolders(fo.getObjectID(), user.getId(), userConfig, StringCollection.getSqlInString(
@@ -1377,7 +1408,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
                 /*
                  * Check parent subfolder flag
                  */
-                final boolean hasSubfolders = (OXFolderSQL.getSubfolderIDs(parentObj.getObjectID(), wc, ctx).size() > 0);
+                final boolean hasSubfolders = !OXFolderSQL.getSubfolderIDs(parentObj.getObjectID(), wc, ctx).isEmpty();
                 OXFolderSQL.updateSubfolderFlag(parentObj.getObjectID(), hasSubfolders, lastModified, wc, ctx);
                 /*
                  * Update cache
@@ -1409,25 +1440,76 @@ final class OXFolderManagerImpl extends OXFolderManager {
         }
     }
 
-    private void deleteValidatedFolders(final HashMap<Integer, HashMap<?, ?>> deleteableIDs, final long lastModified, final int type) throws OXException {
-        final int deleteableIDsSize = deleteableIDs.size();
-        final Iterator<Map.Entry<Integer, HashMap<?, ?>>> iter = deleteableIDs.entrySet().iterator();
-        for (int i = 0; i < deleteableIDsSize; i++) {
-            final Map.Entry<Integer, HashMap<?, ?>> entry = iter.next();
-            final Integer folderID = entry.getKey();
-            final @SuppressWarnings("unchecked") HashMap<Integer, HashMap<?, ?>> hashMap =
-                (HashMap<Integer, HashMap<?, ?>>) entry.getValue();
-            /*
-             * Delete subfolders first, if any exist
-             */
-            if (hashMap != null) {
-                deleteValidatedFolders(hashMap, lastModified, type);
+    /**
+     * Deletes the validated folders.
+     * 
+     * @param deleteableIDs The gathered IDs of validated folders
+     * @param lastModified The last-modified time stamp
+     * @param type The folder type
+     * @throws OXException If deletion fails for any folder
+     */
+    void deleteValidatedFolders(final TIntObjectHashMap<TIntObjectHashMap<?>> deleteableIDs, final long lastModified, final int type) throws OXException {
+        final DeleteValidatedFoldersProcedure procedure = new DeleteValidatedFoldersProcedure(lastModified, type);
+        if (!deleteableIDs.forEachEntry(procedure)) {
+            final OXException error = procedure.error;
+            if (null != error) {
+                throw error;
             }
-            deleteValidatedFolder(folderID.intValue(), lastModified, type);
         }
     }
 
-    private void deleteValidatedFolder(final int folderID, final long lastModified, final int type) throws OXException {
+    private final class DeleteValidatedFoldersProcedure implements TIntObjectProcedure<TIntObjectHashMap<?>> {
+
+        public OXException error;
+
+        private final long lastModified;
+
+        private final int type;
+
+        public DeleteValidatedFoldersProcedure(final long lastModified, final int type) {
+            super();
+            this.lastModified = lastModified;
+            this.type = type;
+        }
+
+        public boolean execute(final int folderId, final TIntObjectHashMap<?> hashMap) {
+            if (null == error) {
+                try {
+                    if (null != hashMap) {
+                        final @SuppressWarnings("unchecked") TIntObjectHashMap<TIntObjectHashMap<?>> tmp =
+                            (TIntObjectHashMap<TIntObjectHashMap<?>>) hashMap;
+                        deleteValidatedFolders(tmp, lastModified, type);
+                    }
+                    deleteValidatedFolder(folderId, lastModified, type);
+                    /*
+                     * Allow further executions
+                     */
+                    return true;
+                } catch (final OXException e) {
+                    error = e;
+                    /*
+                     * Deny further executions
+                     */
+                    return false;
+                }
+            }
+            /*
+             * Deny further executions
+             */
+            return false;
+        }
+
+    }
+
+    /**
+     * Deletes the validated folder.
+     * 
+     * @param folderID The folder ID
+     * @param lastModified The last-modified time stamp
+     * @param type The folder type
+     * @throws OXException If deletion fails
+     */
+    void deleteValidatedFolder(final int folderID, final long lastModified, final int type) throws OXException {
         /*
          * Iterate possibly listening folder delete listeners
          */
@@ -1520,7 +1602,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
                 final ServerUserSetting sus = ServerUserSetting.getInstance(wc);
                 final Integer collectFolder = sus.getIContactCollectionFolder(ctx.getContextId(), user.getId());
                 if (null != collectFolder && folderID == collectFolder.intValue()) {
-                    sus.setIContactColletion(ctx.getContextId(), user.getId(), false);
+                    sus.setIContactColletion(ctx, session, user.getId(), false);
                     sus.setIContactCollectionFolder(ctx.getContextId(), user.getId(), null);
                 }
             } catch (final SettingException e) {
@@ -1627,8 +1709,8 @@ final class OXFolderManagerImpl extends OXFolderManager {
     /**
      * Gathers all folders which are allowed to be deleted
      */
-    private HashMap<Integer, HashMap<?, ?>> gatherDeleteableFolders(final int folderID, final int userId, final UserConfiguration userConfig, final String permissionIDs) throws OXException, DBPoolingException, SQLException {
-        final HashMap<Integer, HashMap<?, ?>> deleteableIDs = new HashMap<Integer, HashMap<?, ?>>();
+    private TIntObjectHashMap<TIntObjectHashMap<?>> gatherDeleteableFolders(final int folderID, final int userId, final UserConfiguration userConfig, final String permissionIDs) throws OXException, DBPoolingException, SQLException {
+        final TIntObjectHashMap<TIntObjectHashMap<?>> deleteableIDs = new TIntObjectHashMap<TIntObjectHashMap<?>>();
         gatherDeleteableSubfoldersRecursively(folderID, userId, userConfig, permissionIDs, deleteableIDs, folderID);
         return deleteableIDs;
     }
@@ -1636,7 +1718,7 @@ final class OXFolderManagerImpl extends OXFolderManager {
     /**
      * Gathers all folders which are allowed to be deleted in a recursive manner
      */
-    private void gatherDeleteableSubfoldersRecursively(final int folderID, final int userId, final UserConfiguration userConfig, final String permissionIDs, final HashMap<Integer, HashMap<?, ?>> deleteableIDs, final int initParent) throws OXException, DBPoolingException, SQLException {
+    private void gatherDeleteableSubfoldersRecursively(final int folderID, final int userId, final UserConfiguration userConfig, final String permissionIDs, final TIntObjectHashMap<TIntObjectHashMap<?>> deleteableIDs, final int initParent) throws OXException, DBPoolingException, SQLException {
         final FolderObject delFolder = getOXFolderAccess().getFolderObject(folderID);
         /*
          * Check if shared
@@ -1716,25 +1798,23 @@ final class OXFolderManagerImpl extends OXFolderManager {
          * Check, if folder has subfolders
          */
         if (!delFolder.hasSubfolders()) {
-            deleteableIDs.put(Integer.valueOf(folderID), null);
+            deleteableIDs.put(folderID, null);
             return;
         }
         /*
          * No subfolders detected
          */
-        final List<Integer> subfolders = OXFolderSQL.getSubfolderIDs(delFolder.getObjectID(), readCon, ctx);
+        final TIntArrayList subfolders = OXFolderSQL.getSubfolderIDs(delFolder.getObjectID(), readCon, ctx);
         if (subfolders.isEmpty()) {
-            deleteableIDs.put(Integer.valueOf(folderID), null);
+            deleteableIDs.put(folderID, null);
             return;
         }
-        final HashMap<Integer, HashMap<?, ?>> subMap = new HashMap<Integer, HashMap<?, ?>>();
+        final TIntObjectHashMap<TIntObjectHashMap<?>> subMap = new TIntObjectHashMap<TIntObjectHashMap<?>>();
         final int size = subfolders.size();
-        final Iterator<Integer> it = subfolders.iterator();
         for (int i = 0; i < size; i++) {
-            final int fuid = it.next().intValue();
-            gatherDeleteableSubfoldersRecursively(fuid, userId, userConfig, permissionIDs, subMap, initParent);
+            gatherDeleteableSubfoldersRecursively(subfolders.getQuick(i), userId, userConfig, permissionIDs, subMap, initParent);
         }
-        deleteableIDs.put(Integer.valueOf(folderID), subMap);
+        deleteableIDs.put(folderID, subMap);
     }
 
     /**
