@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -141,7 +141,23 @@ public class AjaxException extends AbstractOXException {
         /**
          * Invalid parameter "%1$s": %2$s
          */
-        InvalidParameterValue("Invalid parameter \"%1$s\": %2$s", Category.CODE_ERROR, 10);
+        InvalidParameterValue("Invalid parameter \"%1$s\": %2$s", Category.CODE_ERROR, 10),
+        /**
+         * Unexpected error: %1$s
+         */
+        UnexpectedError("Unexpected error: %1$s", Category.CODE_ERROR, 11),
+        /**
+         * A parameter conflict occurred.
+         */
+        ParameterConflict("A parameter conflict occurred.", Category.CODE_ERROR, 12),
+        /**
+         * Parameter "%1$s" conflicts with parameter "%2$s".
+         */
+        EitherParameterConflict("Parameter \"%1$s\" conflicts with parameter \"%2$s\".", Category.CODE_ERROR, 13),
+        /**
+         * Action "%1$s" on request path "%2$s" is not permitted via a non-secure connection.
+         */
+        NonSecureDenied("Action \"%1$s\" on request path \"%2$s\" is not permitted via a non-secure connection.", Category.CODE_ERROR, 14);
 
         /**
          * Message of the exception.
