@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,8 +50,6 @@
 package com.openexchange.tools.servlet.http.manager;
 
 import java.lang.reflect.Constructor;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.util.ConcurrentModificationException;
 import java.util.Dictionary;
 import java.util.Iterator;
@@ -79,9 +77,6 @@ import com.openexchange.tools.servlet.http.SingletonServletQueue;
  */
 public final class ConcurrentHttpServletManager extends AbstractHttpServletManager {
 
-    private static final org.apache.commons.logging.Log LOG =
-        org.apache.commons.logging.LogFactory.getLog(ConcurrentHttpServletManager.class);
-
     // private final ReadWriteLock readWriteLock;
 
     private final Lock readLock;
@@ -165,12 +160,16 @@ public final class ConcurrentHttpServletManager extends AbstractHttpServletManag
                      * Not available in implier cache
                      */
                     class Implier {
+
                         ServletQueue queue;
+
                         String implier;
+
                         Implier(final String implier, final ServletQueue queue) {
                             this.implier = implier;
                             this.queue = queue;
                         }
+
                         void setIfLonger(final String implier, final ServletQueue queue) {
                             if (implier.length() > this.implier.length()) {
                                 this.implier = implier;
@@ -192,7 +191,7 @@ public final class ConcurrentHttpServletManager extends AbstractHttpServletManag
                     }
                     if (null == implier) {
                         final ServletQueue errServletQueue =
-                            new SingletonServletQueue(new HttpErrorServlet("No servlet bound to path/alias: " + path), null, path);
+                            new SingletonServletQueue(new HttpErrorServlet("No servlet bound to path/alias: " + path), path);
                         implierCache.put(path, errServletQueue);
                     } else {
                         pathStorage.append(implier.implier);
@@ -222,8 +221,6 @@ public final class ConcurrentHttpServletManager extends AbstractHttpServletManag
         }
     }
 
-    private final static Class<?>[] CLASS_ARR = new Class[] {};
-
     public void putServlet(final String path, final HttpServlet servlet) {
         if (implierCache.containsKey(path)) {
             /*
@@ -273,56 +270,8 @@ public final class ConcurrentHttpServletManager extends AbstractHttpServletManag
     public void registerServlet(final String id, final HttpServlet servlet, final Dictionary<String, String> initParams) throws ServletException {
         writeLock.lock();
         try {
-            final String path = new URI(prependSlash(id)).normalize().toString();
-            if (servletPool.containsKey(path)) {
-                throw new ServletException(new StringBuilder(256).append("A servlet with alias \"").append(path).append(
-                    "\" has already been registered before.").toString());
-            }
-            final ServletConfigLoader configLoader = ServletConfigLoader.getDefaultInstance();
-            if (null == configLoader) {
-                throw new ServletException(
-                    "Aborting servlet registration: HTTP service has not been initialized since default servlet configuration loader is null.");
-            }
-            if ((null != initParams) && !initParams.isEmpty()) {
-                configLoader.setConfig(servlet.getClass().getCanonicalName(), initParams);
-            }
-            /*
-             * Try to determine default constructor for later instantiations
-             */
-            final FiFoServletQueue servletQueue;
-            try {
-                servletQueue =
-                    new FiFoServletQueue(1, servlet.getClass().getConstructor(CLASS_ARR), !(servlet instanceof SingleThreadModel), path);
-            } catch (final SecurityException e) {
-                final ServletException se =
-                    new ServletException("Default constructor could not be found for servlet class: " + servlet.getClass().getName(), e);
-                se.initCause(e);
-                throw se;
-            } catch (final NoSuchMethodException e) {
-                final ServletException se =
-                    new ServletException("Default constructor could not be found for servlet class: " + servlet.getClass().getName(), e);
-                se.initCause(e);
-                throw se;
-            }
-            final ServletConfig conf = configLoader.getConfig(servlet.getClass().getCanonicalName(), path);
-            servlet.init(conf);
-            servletQueue.enqueue(servlet);
-            /*
-             * Put into servlet pool for being accessible
-             */
-            if (servletPool.putIfAbsent(path, servletQueue) != null) {
-                throw new ServletException(new StringBuilder(256).append("A servlet with alias \"").append(path).append(
-                    "\" has already been registered before.").toString());
-            }
+            registerServlet0(id, servlet, initParams);
             implierCache.clear();
-            if (LOG.isInfoEnabled()) {
-                LOG.info(new StringBuilder(64).append("New servlet \"").append(servlet.getClass().getCanonicalName()).append(
-                    "\" successfully registered to \"").append(path).append('"'));
-            }
-        } catch (final URISyntaxException e) {
-            final ServletException se = new ServletException("Servlet path is not a valid URI", e);
-            se.initCause(e);
-            throw se;
         } finally {
             writeLock.unlock();
         }
@@ -331,26 +280,8 @@ public final class ConcurrentHttpServletManager extends AbstractHttpServletManag
     public void unregisterServlet(final String id) {
         writeLock.lock();
         try {
-            final String path = new URI(prependSlash(id)).normalize().toString();
-            final ServletConfigLoader configLoader = ServletConfigLoader.getDefaultInstance();
-            if (null == configLoader) {
-                LOG.error("Aborting servlet un-registration: HTTP service has not been initialized since default servlet configuration loader is null.");
-                return;
-            }
-            final ServletQueue servletQueue = servletPool.get(path);
-            if (null == servletQueue) {
-                if (LOG.isWarnEnabled()) {
-                    LOG.warn("Servlet un-registration failed. No servlet is bound to path: " + path);
-                }
-                return;
-            }
-            configLoader.removeConfig(servletQueue.dequeue().getClass().getCanonicalName());
-            servletPool.remove(path);
+            unregisterServlet0(id);
             implierCache.clear();
-        } catch (final URISyntaxException e) {
-            final ServletException se = new ServletException("Servlet path is not a valid URI", e);
-            se.initCause(e);
-            LOG.error("Unregistering servlet failed. Servlet path is not a valid URI: " + id, se);
         } finally {
             writeLock.unlock();
         }
