@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -60,6 +60,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -271,17 +272,30 @@ public final class Tools {
      * @param con writable database connection.
      * @param table name of the table that should get a new primary key.
      * @param columns names of the columns the primary key should cover.
+     * @param lengths The column lengths; <code>-1</code> for full column
      * @throws SQLException if some SQL problem occurs.
      */
-    public static final void createPrimaryKey(final Connection con, final String table, final String[] columns) throws SQLException {
+    public static final void createPrimaryKey(final Connection con, final String table, final String[] columns, final int[] lengths) throws SQLException {
         final StringBuilder sql = new StringBuilder("ALTER TABLE `");
         sql.append(table);
-        sql.append("` ADD PRIMARY KEY (`");
-        for (final String column : columns) {
-            sql.append(column);
-            sql.append("`,`");
+        sql.append("` ADD PRIMARY KEY (");
+        {
+            final String column = columns[0];
+            sql.append('`').append(column).append('`');
+            final int len = lengths[0];
+            if (len > 0) {
+                sql.append('(').append(len).append(')');
+            }
+        }
+        for (int i = 1; i < columns.length; i++) {
+            final String column = columns[i];
+            sql.append(',');
+            sql.append('`').append(column).append('`');
+            final int len = lengths[i];
+            if (len > 0) {
+                sql.append('(').append(len).append(')');
+            }
         }
-        sql.setLength(sql.length() - 2);
         sql.append(')');
         Statement stmt = null;
         try {
@@ -292,6 +306,21 @@ public final class Tools {
         }
     }
 
+    /**
+     * This method creates a new primary key on a table. Beware, this method is vulnerable to SQL injection because table and column names
+     * can not be set through a {@link PreparedStatement}.
+     *
+     * @param con writable database connection.
+     * @param table name of the table that should get a new primary key.
+     * @param columns names of the columns the primary key should cover.
+     * @throws SQLException if some SQL problem occurs.
+     */
+    public static final void createPrimaryKey(final Connection con, final String table, final String[] columns) throws SQLException {
+        final int[] lengths = new int[columns.length];
+        Arrays.fill(lengths, -1);
+        createPrimaryKey(con, table, columns, lengths);
+    }
+
     /**
      * This method creates a new index on a table. Beware, this method is vulnerable to SQL injection because table and column names can not
      * be set through a {@link PreparedStatement}.
@@ -382,6 +411,79 @@ public final class Tools {
         }
     }
 
+    /**
+     * Checks if denoted table has any primary key set.
+     * 
+     * @param con The connection
+     * @param table The table name
+     * @return <code>true</code> if denoted table has any primary key set; otherwise <code>false</code>
+     * @throws SQLException If a SQL error occurs
+     */
+    public static final boolean hasPrimaryKey(final Connection con, final String table) throws SQLException {
+        final DatabaseMetaData metaData = con.getMetaData();
+        // Get primary keys
+        final ResultSet primaryKeys = metaData.getPrimaryKeys(null, null, table);
+        try {
+            return primaryKeys.next();
+        } finally {
+            closeSQLStuff(primaryKeys);
+        }
+    }
+
+    /**
+     * Checks if denoted column in given table is of type {@link java.sql.Types#VARCHAR}.
+     * 
+     * @param con The connection
+     * @param table The table name
+     * @param column The column name
+     * @return <code>true</code> if denoted column in given table is of type {@link java.sql.Types#VARCHAR}; otherwise <code>false</code>
+     * @throws SQLException If a SQL error occurs
+     */
+    public static final boolean isVARCHAR(final Connection con, final String table, final String column) throws SQLException {
+        return isType(con, table, column, java.sql.Types.VARCHAR);
+    }
+
+    /**
+     * Checks if denoted column in given table is of specified type from {@link java.sql.Types}.
+     * 
+     * @param con The connection
+     * @param table The table name
+     * @param column The column name
+     * @param type The type to check against
+     * @return <code>true</code> if denoted column in given table is of specified type; otherwise <code>false</code>
+     * @throws SQLException If a SQL error occurs
+     */
+    public static final boolean isType(final Connection con, final String table, final String column, final int type) throws SQLException {
+        return type == getColumnType(con, table, column);
+    }
+
+    /**
+     * Gets the type of specified column in given table from {@link java.sql.Types}.
+     * 
+     * @param con The connection
+     * @param table The table name
+     * @param column The column name
+     * @return The type of specified column in given table from {@link java.sql.Types} or <code>-1</code> if column does not exist
+     * @throws SQLException If a SQL error occurs
+     */
+    public static final int getColumnType(final Connection con, final String table, final String column) throws SQLException {
+        if (!columnExists(con, table, column)) {
+            return -1;
+        }
+        final DatabaseMetaData metaData = con.getMetaData();
+        ResultSet rs = null;
+        int type = -1;
+        try {
+            rs = metaData.getColumns(null, null, table, column);
+            while (rs.next()) {
+                type = rs.getInt(5);
+            }
+        } finally {
+            closeSQLStuff(rs);
+        }
+        return type;
+    }
+
     public static final boolean tableExists(final Connection con, final String table) throws SQLException {
         final DatabaseMetaData metaData = con.getMetaData();
         ResultSet rs = null;
@@ -437,16 +539,12 @@ public final class Tools {
         PreparedStatement stmt = null;
         ResultSet rs = null;
         try {
-            stmt = con.prepareStatement("SELECT 1 FROM " + sequenceTable + " WHERE cid = "+ctxId);
+            stmt = con.prepareStatement("SELECT 1 FROM " + sequenceTable + " WHERE cid=?");
+            stmt.setInt(1, ctxId);
             rs = stmt.executeQuery();
             return rs.next();
         } finally {
-            if(rs != null) {
-                rs.close();
-            }
-            if(stmt != null) {
-                stmt.close();
-            }
+            closeSQLStuff(rs, stmt);
         }
     }
 
@@ -462,29 +560,48 @@ public final class Tools {
             }
             return contextIds;
         } finally {
-            if(rs != null) {
-                rs.close();
-            }
-            if(stmt != null) {
-                stmt.close();
-            }
+            closeSQLStuff(rs, stmt);
         }
     }
 
     public static void exec(final Connection con, final String sql, final Object...args) throws SQLException {
-        PreparedStatement statement = null;
+        PreparedStatement stmt = null;
         try {
-            statement = con.prepareStatement(sql);
+            stmt = con.prepareStatement(sql);
             int i = 1;
             for(final Object arg : args) {
-                statement.setObject(i++, arg);
+                stmt.setObject(i++, arg);
             }
-            statement.execute();
+            stmt.execute();
         } finally {
-            if(statement != null) {
-                statement.close();
+            closeSQLStuff(stmt);
+        }
+    }
+
+    public static void addColumns(Connection con, String tableName, Column... cols) throws SQLException {
+        StringBuffer sql = new StringBuffer("ALTER TABLE ");
+        sql.append(tableName);
+        for (Column column : cols) {
+            if (!columnExists(con, tableName, column.getName())) {
+                sql.append(" ADD ");
+                sql.append(column.getName());
+                sql.append(' ');
+                sql.append(column.getDefinition());
+                sql.append(',');
             }
         }
-        
+        if (sql.charAt(sql.length() - 1) == ',') {
+            sql.setLength(sql.length() - 1);
+        }
+        if (sql.length() == 12 + tableName.length()) {
+            return;
+        }
+        Statement stmt = null;
+        try {
+            stmt = con.createStatement();
+            stmt.execute(sql.toString());
+        } finally {
+            closeSQLStuff(stmt);
+        }
     }
 }
