@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,33 +50,30 @@
 package com.openexchange.tools.webdav;
 
 import java.io.IOException;
+import java.util.UUID;
 import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.jdom.Document;
 import org.jdom.JDOMException;
-import com.openexchange.authentication.Authenticated;
+import com.openexchange.ajax.fields.LoginFields;
 import com.openexchange.authentication.LoginException;
-import com.openexchange.authentication.LoginExceptionCodes;
-import com.openexchange.authentication.service.Authentication;
-import com.openexchange.groupware.AbstractOXException;
-import com.openexchange.groupware.contexts.Context;
-import com.openexchange.groupware.contexts.impl.ContextException;
-import com.openexchange.groupware.contexts.impl.ContextStorage;
-import com.openexchange.groupware.impl.PasswordExpiredException;
-import com.openexchange.groupware.impl.UserNotActivatedException;
-import com.openexchange.groupware.impl.UserNotFoundException;
-import com.openexchange.groupware.ldap.LdapException;
-import com.openexchange.groupware.ldap.User;
-import com.openexchange.groupware.ldap.UserStorage;
+import com.openexchange.groupware.AbstractOXException.Category;
+import com.openexchange.java.util.UUIDs;
+import com.openexchange.login.Interface;
+import com.openexchange.login.LoginRequest;
+import com.openexchange.login.LoginResult;
+import com.openexchange.login.internal.LoginPerformer;
 import com.openexchange.server.ServiceException;
 import com.openexchange.server.services.ServerServiceRegistry;
 import com.openexchange.session.Session;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.StringCollection;
 import com.openexchange.tools.encoding.Base64;
+import com.openexchange.webdav.WebdavException;
 import com.openexchange.xml.jdom.JDOMParser;
 
 /**
@@ -86,14 +83,8 @@ import com.openexchange.xml.jdom.JDOMParser;
  */
 public abstract class OXServlet extends WebDavServlet {
 
-    /**
-     * For serialization.
-     */
     private static final long serialVersionUID = 301910346402779362L;
 
-    /**
-     * Logger.
-     */
     private static final transient Log LOG = LogFactory.getLog(OXServlet.class);
 
     /**
@@ -121,9 +112,8 @@ public abstract class OXServlet extends WebDavServlet {
         return true;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    protected abstract Interface getInterface();
+
     @Override
     protected void service(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
         if (!"TRACE".equals(req.getMethod()) && useHttpAuth() && !doAuth(req, resp)) {
@@ -141,7 +131,6 @@ public abstract class OXServlet extends WebDavServlet {
         } catch (final Exception e) {
             LOG.error(e.getMessage(), e);
             final ServletException se = new ServletException(e.getMessage(), e);
-            se.initCause(e);
             throw se;
         }
     }
@@ -153,62 +142,39 @@ public abstract class OXServlet extends WebDavServlet {
      * @param resp http servlet response.
      * @return <code>true</code> if the authentication can be done correctly.
      * @throws IOException if a communication problem occurs.
-     * @throws ServletException if adding a session fails.
      */
     protected boolean doAuth(final HttpServletRequest req, final HttpServletResponse resp) throws IOException {
-        final String sessionId = getSessionId(req);
-        Session session = null;
-        if (null != sessionId) {
-            session = getSession(sessionId);
+        Session session;
+        try {
+            session = findSessionByCookie(req, resp);
+        } catch (ServiceException e) {
+            LOG.error(e.getMessage(), e);
+            resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
+            return false;
         }
-        if (null == sessionId || null == session) {
-            if (LOG.isTraceEnabled()) {
-                LOG.trace("No sessionId cookie found.");
-            }
-            final String auth = req.getHeader(AUTH_HEADER);
-            if (!checkForBasicAuthorization(auth)) {
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug("Authentication header missing.");
-                }
-                addUnauthorizedHeader(resp);
-                resp.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization Required!");
-                return false;
-            }
-            if (LOG.isTraceEnabled()) {
-                LOG.trace("Authorization header found.");
-            }
-            final String[] userpass = OXServlet.decodeAuthorization(auth);
-            final String login = userpass[0];
-            final String pass = userpass[1];
-            if (!checkLogin(login, pass, req.getRemoteAddr())) {
+        if (null == session) {
+            final LoginRequest loginRequest;
+            try {
+                loginRequest = parseLogin(req);
+            } catch (WebdavException e) {
+                LOG.debug(e.getMessage(), e);
                 addUnauthorizedHeader(resp);
                 resp.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization Required!");
                 return false;
             }
             try {
-                session = addSession(login, pass, req.getRemoteAddr());
-            } catch (final AbstractOXException e) {
+                session = addSession(loginRequest);
+            } catch (LoginException e) {
+                if (e.getCategory() == Category.USER_INPUT) {
+                    addUnauthorizedHeader(resp);
+                    resp.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization Required!");
+                    return false;
+                }
                 LOG.error(e.getMessage(), e);
                 resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, e.getMessage());
                 return false;
             }
-            if (null == session) {
-                if (LOG.isDebugEnabled()) {
-                    LOG.debug("Cannot authenticate user.");
-                }
-                addUnauthorizedHeader(resp);
-                resp.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Authorization Required!");
-                return false;
-            }
-            if (LOG.isTraceEnabled()) {
-                LOG.trace("Session created.");
-            }
             resp.addCookie(new Cookie(COOKIE_SESSIONID, session.getSessionID()));
-            if (null != sessionId) {
-                final Cookie cookie = new Cookie(COOKIE_SESSIONID, sessionId);
-                cookie.setMaxAge(0);
-                resp.addCookie(cookie);
-            }
         } else {
             final String address = req.getRemoteAddr();
             if (null == address || !address.equals(session.getLocalIp())) {
@@ -224,26 +190,6 @@ public abstract class OXServlet extends WebDavServlet {
         return true;
     }
 
-    /**
-     * This methods reads the session identifier from the cookies. TODO Fix handling if request contains old session identifier.
-     * 
-     * @param req http request.
-     * @return the session identifier or <code>null</code> if it can't be found.
-     */
-    private static String getSessionId(final HttpServletRequest req) {
-        final Cookie[] cookies = req.getCookies();
-        String sessionId = null;
-        if (null != cookies) {
-            for (int i = 0; i < cookies.length; i++) {
-                if (COOKIE_SESSIONID.equals(cookies[i].getName())) {
-                    sessionId = cookies[i].getValue();
-                    break;
-                }
-            }
-        }
-        return sessionId;
-    }
-
     /**
      * Checks if the client sends a correct basic authorization header.
      * 
@@ -288,14 +234,11 @@ public abstract class OXServlet extends WebDavServlet {
 
     /**
      * Checks if the login contains only valid values.
-     * 
-     * @param login login name of the user
      * @param pass password of the user
-     * @param ipAddress Client IP.
+     * 
      * @return false if the login contains illegal values.
      */
-    protected static boolean checkLogin(final String login, final String pass, final String ipAddress) {
-        // made by stefan - !!!login without password bugfix - moped!!!
+    protected static boolean checkLogin(final String pass) {
         // check if the user wants to login without password.
         // ldap bind doesn't fail with empty password. so check it here.
         if (pass == null || StringCollection.isEmpty(pass)) {
@@ -313,6 +256,48 @@ public abstract class OXServlet extends WebDavServlet {
         resp.setHeader("WWW-Authenticate", "Basic realm=\"" + authIdentifier + "\"");
     }
 
+    private LoginRequest parseLogin(final HttpServletRequest req) throws WebdavException, IOException {
+        final String auth = req.getHeader(AUTH_HEADER);
+        if (!checkForBasicAuthorization(auth)) {
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Authentication header missing.");
+            }
+            throw new WebdavException(WebdavException.Code.MISSING_HEADER_FIELD, AUTH_HEADER);
+        }
+        final String[] userpass = OXServlet.decodeAuthorization(auth);
+        final String login = userpass[0];
+        final String pass = userpass[1];
+        if (!checkLogin(pass)) {
+            throw new WebdavException(WebdavException.Code.EMPTY_PASSWORD);
+        }
+        return new LoginRequest() {
+            public String getUserAgent() {
+                return req.getHeader("user-agent");
+            }
+            public String getPassword() {
+                return pass;
+            }
+            public String getLogin() {
+                return login;
+            }
+            public Interface getInterface() {
+                return OXServlet.this.getInterface();
+            }
+            public String getClientIP() {
+                return req.getRemoteAddr();
+            }
+            public String getAuthId() {
+                return UUIDs.getUnformattedString(UUID.randomUUID());
+            }
+            public String getClient() {
+                return null;
+            }
+            public String getVersion() {
+                return null;
+            }
+        };
+    }
+
     /**
      * This method tries to create a session for the given user.
      * 
@@ -322,80 +307,32 @@ public abstract class OXServlet extends WebDavServlet {
      * @return the initilized session or <code>null</code>.
      * @throws LoginException if an error occurs while creating the session.
      */
-    private Session addSession(final String login, final String pass, final String ipAddress) throws AbstractOXException {
-        final SessiondService sessiondCon = ServerServiceRegistry.getInstance().getService(SessiondService.class);
-        if (null == sessiondCon) {
-            throw new ServiceException(ServiceException.Code.SERVICE_UNAVAILABLE, SessiondService.class.getName());
-        }
-        Session session = null;
-        try {
-            final Authenticated authed = Authentication.login(login, pass);
-
-            final String contextname = authed.getContextInfo();
-            final String username = authed.getUserInfo();
-
-            final ContextStorage contextStor = ContextStorage.getInstance();
-            final int contextId = contextStor.getContextId(contextname);
-            if (ContextStorage.NOT_FOUND == contextId) {
-                throw new ContextException(ContextException.Code.NO_MAPPING, contextname);
-            }
-            final Context context = contextStor.getContext(contextId);
-            if (null == context) {
-                throw new ContextException(ContextException.Code.NOT_FOUND, Integer.valueOf(contextId));
-            }
-
-            int userId = -1;
-            User u = null;
-
-            try {
-                final UserStorage us = UserStorage.getInstance();
-                userId = us.getUserId(username, context);
-                u = us.getUser(userId, context);
-            } catch (final LdapException ex) {
-                switch (ex.getDetail()) {
-                case ERROR:
-                    throw LoginExceptionCodes.UNKNOWN.create(ex);
-                case NOT_FOUND:
-                    throw new UserNotFoundException("User not found.", ex);
-                }
-            }
-
-            // is user active
-            if (u.isMailEnabled()) {
-                if (u.getShadowLastChange() == 0) {
-                    throw new PasswordExpiredException("user password is expired!");
-                }
-            } else {
-                throw new UserNotActivatedException("user is not activated!");
-            }
-
-            final String sessionId = sessiondCon.addSession(userId, username, pass, context, ipAddress, login);
-            session = sessiondCon.getSession(sessionId);
-        } catch (final LoginException e) {
-            if (AbstractOXException.Category.USER_INPUT == e.getCategory()) {
-                LOG.debug(e.getMessage(), e);
-            } else {
-                LOG.error(e.getMessage(), e);
-            }
-        } catch (final UserNotFoundException e) {
-            throw LoginExceptionCodes.INVALID_CREDENTIALS.create(e);
-        } catch (final PasswordExpiredException e) {
-            throw LoginExceptionCodes.INVALID_CREDENTIALS.create(e);
-        } catch (final UserNotActivatedException e) {
-            throw LoginExceptionCodes.INVALID_CREDENTIALS.create(e);
-        }
-        return session;
+    private Session addSession(LoginRequest request) throws LoginException {
+        LoginResult result = LoginPerformer.getInstance().doLogin(request);
+        return result.getSession();
     }
 
-    /**
-     * This method tries to get the session for the given session identifier.
-     * 
-     * @param sessionId session identifier.
-     * @return the session object or <code>null</code> if the session doesn't exist.
-     */
-    private Session getSession(final String sessionId) {
-        final SessiondService sessiondCon = ServerServiceRegistry.getInstance().getService(SessiondService.class);
-        return sessiondCon.getSession(sessionId);
+    private Session findSessionByCookie(HttpServletRequest req, HttpServletResponse resp) throws ServiceException {
+        final Cookie[] cookies = req.getCookies();
+        String sessionId = null;
+        if (null != cookies) {
+            for (Cookie cookie : cookies) {
+                if (COOKIE_SESSIONID.equals(cookie.getName())) {
+                    sessionId = cookie.getValue();
+                    break;
+                }
+            }
+        }
+        if (null == sessionId) {
+            return null;
+        }
+        final Session session = ServerServiceRegistry.getInstance().getService(SessiondService.class, true).getSession(sessionId);
+        if (null == session) {
+            final Cookie cookie = new Cookie(COOKIE_SESSIONID, sessionId);
+            cookie.setMaxAge(0);
+            resp.addCookie(cookie);
+        }
+        return session;
     }
 
     /**
@@ -418,7 +355,7 @@ public abstract class OXServlet extends WebDavServlet {
      * @throws JDOMException if JDOM gets an exception
      * @throws IOException if an exception occurs while reading the body.
      */
-    protected org.jdom.Document getJDOMDocument(final HttpServletRequest req) throws JDOMException, IOException {
+    protected Document getJDOMDocument(final HttpServletRequest req) throws JDOMException, IOException {
         org.jdom.Document doc = null;
         if (req.getContentLength() > 0) {
             doc = ServerServiceRegistry.getInstance().getService(JDOMParser.class).parse(req.getInputStream());
