@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2010 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,8 @@
 
 package com.openexchange.webdav;
 
+import gnu.trove.TIntObjectHashMap;
+import gnu.trove.TObjectProcedure;
 import java.io.IOException;
 import java.io.Writer;
 import java.sql.Connection;
@@ -58,10 +60,8 @@ import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.Enumeration;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
-import javax.servlet.ServletException;
+import java.util.UUID;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.apache.commons.logging.Log;
@@ -98,6 +98,7 @@ import com.openexchange.groupware.tasks.Task;
 import com.openexchange.groupware.tasks.TasksSQLImpl;
 import com.openexchange.groupware.userconfiguration.UserConfiguration;
 import com.openexchange.groupware.userconfiguration.UserConfigurationStorage;
+import com.openexchange.login.Interface;
 import com.openexchange.server.ServiceException;
 import com.openexchange.server.impl.DBPool;
 import com.openexchange.server.services.ServerServiceRegistry;
@@ -159,11 +160,16 @@ public final class ical extends PermissionServlet {
     // private static final String SQL_ENTRY_DELETE =
     // "DELETE FROM ical_ids WHERE cid=? AND principal_id=? AND target_object_id=? AND module=?";
 
+    @Override
+    protected Interface getInterface() {
+        return Interface.WEBDAV_ICAL;
+    }
+
     /**
      * {@inheritDoc}
      */
     @Override
-    public void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
+    public void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws IOException {
         if (LOG.isDebugEnabled()) {
             LOG.debug("GET");
         }
@@ -202,10 +208,10 @@ public final class ical extends PermissionServlet {
             final List<ConversionError> errors = new ArrayList<ConversionError>();
 
             final AppointmentSQLInterface appointmentSql = ServerServiceRegistry.getInstance().getService(AppointmentSqlFactoryService.class).createAppointmentSql(sessionObj);
-            CalendarCollectionService recColl = ServerServiceRegistry.getInstance().getService(CalendarCollectionService.class);
+            final CalendarCollectionService recColl = ServerServiceRegistry.getInstance().getService(CalendarCollectionService.class);
             SearchIterator<Appointment> iter = null;
             try {
-                final Map<Integer, SeriesUIDPatcher> patchers = new HashMap<Integer, SeriesUIDPatcher>();
+                final TIntObjectHashMap<SeriesUIDPatcher> patchers = new TIntObjectHashMap<SeriesUIDPatcher>();
                 iter = appointmentSql.getModifiedAppointmentsInFolder(calendarfolderId, APPOINTMENT_FIELDS, new Date(0), true);
                 while (iter.hasNext()) {
                     final Appointment appointment = iter.next();
@@ -227,7 +233,7 @@ public final class ical extends PermissionServlet {
                     // }
                     // Patch UID if change exceptions to be the same ID as of the series.
                     if (appointment.isMaster() || appointment.isException()) {
-                        final Integer recurrenceId = Integer.valueOf(appointment.getRecurrenceID());
+                        final int recurrenceId = appointment.getRecurrenceID();
                         SeriesUIDPatcher patcher = patchers.get(recurrenceId);
                         if (null == patcher) {
                             patcher = new SeriesUIDPatcher();
@@ -240,9 +246,7 @@ public final class ical extends PermissionServlet {
                         }
                     }
                 }
-                for (final SeriesUIDPatcher patcher : patchers.values()) {
-                    patcher.patchUIDs();
-                }
+                patchers.forEachValue(PATCH_PROCEDURE);
             } catch (final SearchIteratorException e) {
                 LOG.error(e.getMessage(), e);
             } catch (final OXException e) {
@@ -331,6 +335,14 @@ public final class ical extends PermissionServlet {
         }
     }
 
+    private static final TObjectProcedure<SeriesUIDPatcher> PATCH_PROCEDURE = new TObjectProcedure<SeriesUIDPatcher>() {
+        
+        public boolean execute(final SeriesUIDPatcher patcher) {
+            patcher.patchUIDs();
+            return true;
+        }
+    };
+
     private static final class SeriesUIDPatcher {
 
         private ICalItem series;
@@ -353,7 +365,11 @@ public final class ical extends PermissionServlet {
             if (null == series) {
                 return;
             }
-            final String uid = series.getUID();
+            String uid = series.getUID();
+            if (null == uid) {
+                uid = UUID.randomUUID().toString();
+                series.setUID(uid);
+            }
             for (final ICalItem changeException : changeExceptions) {
                 changeException.setUID(uid);
             }
